package pneumaticCraft.client;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import net.minecraft.client.Minecraft;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.client.event.RenderWorldLastEvent;
import net.minecraftforge.fml.client.FMLClientHandler;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.TickEvent;

import org.lwjgl.opengl.GL11;

import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.client.render.pneumaticArmor.DroneDebugUpgradeHandler;
import pneumaticCraft.client.render.pneumaticArmor.HUDHandler;
import pneumaticCraft.common.item.ItemGPSTool;
import pneumaticCraft.common.item.Itemss;
import pneumaticCraft.common.util.PneumaticCraftUtils;

public class AreaShowManager{
    private static AreaShowManager INSTANCE = new AreaShowManager();
    private final Map<BlockPos, AreaShowHandler> showHandlers = new HashMap<BlockPos, AreaShowHandler>();
    private World world;
    private DroneDebugUpgradeHandler droneDebugger;

    public static AreaShowManager getInstance(){
        return INSTANCE;
    }

    @SubscribeEvent
    public void renderWorldLastEvent(RenderWorldLastEvent event){
        Minecraft mc = FMLClientHandler.instance().getClient();
        EntityPlayer player = mc.field_71439_g;
        double playerX = player.field_70169_q + (player.field_70165_t - player.field_70169_q) * event.partialTicks;
        double playerY = player.field_70167_r + (player.field_70163_u - player.field_70167_r) * event.partialTicks;
        double playerZ = player.field_70166_s + (player.field_70161_v - player.field_70166_s) * event.partialTicks;

        GL11.glPushMatrix();
        GL11.glTranslated(-playerX, -playerY, -playerZ);

        GL11.glDisable(GL11.GL_TEXTURE_2D);
        GL11.glEnable(GL11.GL_BLEND);
        GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
        //    GL11.glDisable(GL11.GL_DEPTH_TEST);
        for(AreaShowHandler handler : showHandlers.values()) {
            handler.render();
        }

        ItemStack curItem = player.func_71045_bC();
        if(curItem != null && curItem.func_77973_b() == Itemss.GPSTool) {
            BlockPos gpsLocation = ItemGPSTool.getGPSLocation(curItem);
            if(gpsLocation != null) {
                Set<BlockPos> set = new HashSet<BlockPos>();
                set.add(gpsLocation);
                GL11.glDisable(GL11.GL_DEPTH_TEST);
                new AreaShowHandler(set, 0xFFFF00).render();
                GL11.glEnable(GL11.GL_DEPTH_TEST);
            }
        }

        ItemStack helmet = player.func_82169_q(3);
        if(helmet != null && helmet.func_77973_b() == Itemss.pneumaticHelmet) {
            if(droneDebugger == null) droneDebugger = HUDHandler.instance().getSpecificRenderer(DroneDebugUpgradeHandler.class);
            Set<BlockPos> set = droneDebugger.getShowingPositions();
            GL11.glDisable(GL11.GL_DEPTH_TEST);
            new AreaShowHandler(set, 0xFF0000).render();
            GL11.glEnable(GL11.GL_DEPTH_TEST);
        }

        // GL11.glEnable(GL11.GL_DEPTH_TEST);
        GL11.glEnable(GL11.GL_TEXTURE_2D);
        GL11.glDisable(GL11.GL_BLEND);
        GL11.glPopMatrix();
    }

    public AreaShowHandler showArea(BlockPos[] area, int color, TileEntity areaShower){
        return showArea(new HashSet<BlockPos>(Arrays.asList(area)), color, areaShower);
    }

    public AreaShowHandler showArea(Set<BlockPos> area, int color, TileEntity areaShower){
        if(areaShower == null) return null;
        removeHandlers(areaShower);
        AreaShowHandler handler = new AreaShowHandler(area, color);
        showHandlers.put(new BlockPos(areaShower.func_174877_v().func_177958_n(), areaShower.func_174877_v().func_177956_o(), areaShower.func_174877_v().func_177952_p()), handler);
        return handler;
    }

    public boolean isShowing(TileEntity te){
        return showHandlers.containsKey(new BlockPos(te.func_174877_v().func_177958_n(), te.func_174877_v().func_177956_o(), te.func_174877_v().func_177952_p()));
    }

    public void removeHandlers(TileEntity te){
        showHandlers.remove(new BlockPos(te.func_174877_v().func_177958_n(), te.func_174877_v().func_177956_o(), te.func_174877_v().func_177952_p()));
    }

    @SubscribeEvent
    public void tickEnd(TickEvent.ClientTickEvent event){
        EntityPlayer player = PneumaticCraft.proxy.getPlayer();
        if(player != null) {
            if(player.field_70170_p != world) {
                world = player.field_70170_p;
                showHandlers.clear();
            } else {
                if(event.phase == TickEvent.Phase.END) {
                    Iterator<BlockPos> iterator = showHandlers.keySet().iterator();
                    while(iterator.hasNext()) {
                        BlockPos pos = iterator.next();
                        if(PneumaticCraftUtils.distBetween(pos, player.field_70165_t, player.field_70163_u, player.field_70161_v) < 32 && world.func_175623_d(pos)) iterator.remove();
                    }
                }
            }
        }
    }
}
