package pneumaticCraft.client.gui;

import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

import net.minecraft.client.resources.I18n;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumChatFormatting;
import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.client.gui.widget.IGuiWidget;
import pneumaticCraft.client.gui.widget.WidgetAmadronOffer;
import pneumaticCraft.client.gui.widget.WidgetTextField;
import pneumaticCraft.client.gui.widget.WidgetVerticalScrollbar;
import pneumaticCraft.common.inventory.ContainerAmadron;
import pneumaticCraft.common.inventory.ContainerAmadron.EnumProblemState;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketAmadronOrderUpdate;
import pneumaticCraft.common.recipes.AmadronOffer;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.Textures;

public class GuiAmadron extends GuiPneumaticContainerBase{
    private WidgetTextField searchBar;
    private WidgetVerticalScrollbar scrollbar;
    private int page;
    private final List<WidgetAmadronOffer> widgetOffers = new ArrayList<WidgetAmadronOffer>();
    private boolean needsRefreshing;
    private boolean hadProblem = false;
    private GuiButtonSpecial addTradeButton;

    public GuiAmadron(InventoryPlayer playerInventory){
        super(new ContainerAmadron(playerInventory.field_70458_d), null, Textures.GUI_AMADRON);
        field_146999_f = 176;
        field_147000_g = 202;
    }

    @Override
    public void func_73866_w_(){
        super.func_73866_w_();
        String amadron = I18n.func_135052_a("gui.amadron");
        addLabel(amadron, field_147003_i + field_146999_f / 2 - field_146297_k.field_71466_p.func_78256_a(amadron) / 2, field_147009_r + 5);
        addLabel(I18n.func_135052_a("gui.search"), field_147003_i + 76 - field_146297_k.field_71466_p.func_78256_a(I18n.func_135052_a("gui.search")), field_147009_r + 41);

        addInfoTab(I18n.func_135052_a("gui.tooltip.item.amadronTablet"));
        addAnimatedStat("gui.tab.info.ghostSlotInteraction.title", new ItemStack(Blocks.field_150438_bZ), 0xFF00AAFF, true).setText("gui.tab.info.ghostSlotInteraction");
        addAnimatedStat("gui.tab.amadron.disclaimer.title", new ItemStack(Items.field_151099_bA), 0xFF0000FF, true).setText("gui.tab.amadron.disclaimer");

        searchBar = new WidgetTextField(field_146297_k.field_71466_p, field_147003_i + 79, field_147009_r + 40, 73, field_146297_k.field_71466_p.field_78288_b);
        addWidget(searchBar);

        scrollbar = new WidgetVerticalScrollbar(-1, field_147003_i + 156, field_147009_r + 54, 142);
        scrollbar.setStates(1);
        scrollbar.setListening(true);
        addWidget(scrollbar);

        List<String> tooltip = PneumaticCraftUtils.convertStringIntoList(I18n.func_135052_a("gui.amadron.button.order.tooltip"), 40);
        addWidget(new GuiButtonSpecial(1, field_147003_i + 6, field_147009_r + 15, 72, 20, I18n.func_135052_a("gui.amadron.button.order")).setTooltipText(tooltip));
        addTradeButton = new GuiButtonSpecial(2, field_147003_i + 80, field_147009_r + 15, 72, 20, I18n.func_135052_a("gui.amadron.button.addTrade"));
        addWidget(addTradeButton);

        updateVisibleOffers();
    }

    @Override
    public void func_73876_c(){
        super.func_73876_c();
        ContainerAmadron container = (ContainerAmadron)field_147002_h;
        if(needsRefreshing || page != scrollbar.getState()) {
            setPage(scrollbar.getState());
        }
        for(WidgetAmadronOffer offer : widgetOffers) {
            offer.setCanBuy(container.buyableOffers[container.offers.indexOf(offer.getOffer())]);
            offer.setShoppingAmount(container.getShoppingCartAmount(offer.getOffer()));
        }
        if(!hadProblem && container.problemState != EnumProblemState.NO_PROBLEMS) {
            problemTab.openWindow();
        }
        hadProblem = container.problemState != EnumProblemState.NO_PROBLEMS;
        addTradeButton.field_146124_l = container.currentOffers < container.maxOffers;
        List<String> tooltip = PneumaticCraftUtils.convertStringIntoList(I18n.func_135052_a("gui.amadron.button.addTrade.tooltip"), 40);
        tooltip.add((addTradeButton.field_146124_l ? EnumChatFormatting.GRAY : EnumChatFormatting.RED) + I18n.func_135052_a("gui.amadron.button.addTrade.tooltip.offerCount", container.currentOffers, container.maxOffers == Integer.MAX_VALUE ? "\u221E" : container.maxOffers));
        addTradeButton.setTooltipText(tooltip);
    }

    public void setPage(int page){
        this.page = page;
        updateVisibleOffers();
    }

    public void updateVisibleOffers(){
        needsRefreshing = false;
        final ContainerAmadron container = (ContainerAmadron)field_147002_h;
        int invSize = ContainerAmadron.ROWS * 2;
        container.clearStacks();
        List<AmadronOffer> offers = container.offers;
        List<AmadronOffer> visibleOffers = new ArrayList<AmadronOffer>();
        int skippedOffers = 0;
        int applicableOffers = 0;
        for(AmadronOffer offer : offers) {
            if(offer.passesQuery(searchBar.func_146179_b())) {
                applicableOffers++;
                if(skippedOffers < page * invSize) {
                    skippedOffers++;
                } else if(visibleOffers.size() < invSize) {
                    visibleOffers.add(offer);
                }
            }
        }

        scrollbar.setStates(Math.max(1, (applicableOffers + invSize - 1) / invSize - 1));

        widgets.removeAll(widgetOffers);
        for(int i = 0; i < visibleOffers.size(); i++) {
            AmadronOffer offer = visibleOffers.get(i);
            if(offer.getInput() instanceof ItemStack) container.setStack(i * 2, (ItemStack)offer.getInput());
            if(offer.getOutput() instanceof ItemStack) container.setStack(i * 2 + 1, (ItemStack)offer.getOutput());

            WidgetAmadronOffer widget = new WidgetAmadronOffer(i, field_147003_i + 6 + 73 * (i % 2), field_147009_r + 55 + 35 * (i / 2), offer){
                @Override
                public void onMouseClicked(int mouseX, int mouseY, int button){
                    NetworkHandler.sendToServer(new PacketAmadronOrderUpdate(container.offers.indexOf(getOffer()), button, PneumaticCraft.proxy.isSneakingInGui()));
                }
            };
            addWidget(widget);
            widgetOffers.add(widget);
        }
    }

    @Override
    public void onKeyTyped(IGuiWidget widget){
        super.onKeyTyped(widget);
        needsRefreshing = true;
        scrollbar.setCurrentState(0);
    }

    @Override
    public void actionPerformed(IGuiWidget widget){

        super.actionPerformed(widget);
    }

    @Override
    protected Point getInvTextOffset(){
        return null;
    }

    @Override
    protected void addProblems(List curInfo){
        super.addProblems(curInfo);
        EnumProblemState problemState = ((ContainerAmadron)field_147002_h).problemState;
        if(problemState != EnumProblemState.NO_PROBLEMS) {
            curInfo.add(problemState.getLocalizationKey());
        }
    }
}
