package pneumaticCraft.client.gui;

import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.Gui;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.renderer.RenderHelper;
import net.minecraft.client.renderer.entity.RenderItem;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ResourceLocation;

import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL12;

import pneumaticCraft.client.gui.widget.IGuiWidget;
import pneumaticCraft.client.gui.widget.IWidgetListener;

/**
 * Extension of GuiButton that allows a invisible clickable field. It can be added in Gui's like buttons (with the buttonList).
 */

public class GuiButtonSpecial extends GuiButton implements IGuiWidget{

    private ItemStack[] renderedStacks;
    private ResourceLocation resLoc;
    private List<String> tooltipText = new ArrayList<String>();
    private final RenderItem itemRenderer = Minecraft.func_71410_x().func_175599_af();
    private int invisibleHoverColor;
    private boolean thisVisible = true;
    private IWidgetListener listener;

    public GuiButtonSpecial(int buttonID, int startX, int startY, int xSize, int ySize, String buttonText){
        super(buttonID, startX, startY, xSize, ySize, buttonText);
    }

    public void setVisible(boolean visible){
        thisVisible = visible;
    }

    public void setInvisibleHoverColor(int color){
        invisibleHoverColor = color;
    }

    public GuiButtonSpecial setRenderStacks(ItemStack... renderedStacks){
        this.renderedStacks = renderedStacks;
        return this;
    }

    public void setRenderedIcon(ResourceLocation resLoc){
        this.resLoc = resLoc;
    }

    public GuiButtonSpecial setTooltipText(List<String> tooltip){
        tooltipText = tooltip;
        return this;
    }

    public GuiButtonSpecial setTooltipText(String tooltip){
        tooltipText.clear();
        if(tooltip != null && !tooltip.equals("")) {
            tooltipText.add(tooltip);
        }
        return this;
    }

    public void getTooltip(List<String> curTooltip){
        if(tooltipText != null) {
            curTooltip.addAll(tooltipText);
        }
    }

    public String getTooltip(){
        return tooltipText.size() > 0 ? tooltipText.get(0) : "";
    }

    public int getWidth(){
        return field_146120_f;
    }

    public int getHeight(){
        return field_146121_g;
    }

    @Override
    public void func_146112_a(Minecraft mc, int x, int y){
        if(thisVisible) super.func_146112_a(mc, x, y);

        if(field_146125_m) {
            if(renderedStacks != null) {
                int middleX = field_146128_h + field_146120_f / 2;
                int startX = middleX - renderedStacks.length * 9 + 1;
                GL11.glEnable(GL12.GL_RESCALE_NORMAL);
                RenderHelper.func_74520_c();
                for(int i = 0; i < renderedStacks.length; i++) {
                    itemRenderer.func_180450_b(renderedStacks[i], startX + i * 18, field_146129_i + 2);
                }
                RenderHelper.func_74518_a();
                GL11.glDisable(GL12.GL_RESCALE_NORMAL);
            }
            if(resLoc != null) {
                mc.func_110434_K().func_110577_a(resLoc);
                func_146110_a(field_146128_h + field_146120_f / 2 - 8, field_146129_i + 2, 0, 0, 16, 16, 16, 16);
            }
            if(field_146124_l && !thisVisible && x >= field_146128_h && y >= field_146129_i && x < field_146128_h + field_146120_f && y < field_146129_i + field_146121_g) {
                Gui.func_73734_a(field_146128_h, field_146129_i, field_146128_h + field_146120_f, field_146129_i + field_146121_g, invisibleHoverColor);
            }
        }
    }

    @Override
    public void setListener(IWidgetListener gui){
        listener = gui;
    }

    @Override
    public int getID(){
        return field_146127_k;
    }

    @Override
    public void render(int mouseX, int mouseY, float partialTick){
        func_146112_a(Minecraft.func_71410_x(), mouseX, mouseY);
    }

    @Override
    public void onMouseClicked(int mouseX, int mouseY, int button){
        if(func_146116_c(Minecraft.func_71410_x(), mouseX, mouseY)) {
            func_146113_a(Minecraft.func_71410_x().func_147118_V());
            listener.actionPerformed(this);
        }
    }

    @Override
    public void onMouseClickedOutsideBounds(int mouseX, int mouseY, int button){

    }

    @Override
    public Rectangle getBounds(){
        return new Rectangle(field_146128_h, field_146129_i, field_146120_f, field_146121_g);
    }

    @Override
    public void addTooltip(int mouseX, int mouseY, List<String> curTooltip, boolean shiftPressed){
        curTooltip.addAll(tooltipText);
    }

    @Override
    public boolean onKey(char key, int keyCode){
        return false;
    }

    @Override
    public void update(){

    }

    @Override
    public void handleMouseInput(){}

    @Override
    public void postRender(int mouseX, int mouseY, float partialTick){}

}
