package pneumaticCraft.client.gui;

import java.awt.Point;
import java.util.List;

import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.WorldRenderer;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.item.ItemStack;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.lwjgl.opengl.GL11;

import pneumaticCraft.api.item.IPressurizable;
import pneumaticCraft.common.inventory.ContainerChargingStation;
import pneumaticCraft.common.item.IChargingStationGUIHolderItem;
import pneumaticCraft.common.tileentity.TileEntityChargingStation;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.GuiConstants;
import pneumaticCraft.lib.PneumaticValues;
import pneumaticCraft.lib.Textures;

@SideOnly(Side.CLIENT)
public class GuiChargingStation extends GuiPneumaticContainerBase<TileEntityChargingStation>{
    private GuiButton guiSelectButton;

    public GuiChargingStation(InventoryPlayer player, TileEntityChargingStation te){

        super(new ContainerChargingStation(player, te), te, Textures.GUI_CHARGING_STATION_LOCATION);
    }

    @Override
    public void func_73866_w_(){
        super.func_73866_w_();

        int xStart = (field_146294_l - field_146999_f) / 2;
        int yStart = (field_146295_m - field_147000_g) / 2;
        guiSelectButton = new GuiButton(1, xStart + 90, yStart + 15, 25, 20, "inv.");
        field_146292_n.add(guiSelectButton);
    }

    @Override
    protected void func_146979_b(int x, int y){
        super.func_146979_b(x, y);
        field_146289_q.func_78276_b("Upgr.", 46, 19, 4210752);
    }

    @Override
    protected Point getInvTextOffset(){
        return new Point(20, 0);
    }

    @Override
    public String getRedstoneButtonText(int mode){
        switch(mode){
            case 0:
                return "gui.tab.redstoneBehaviour.button.never";
            case 1:
                return "gui.tab.redstoneBehaviour.chargingStation.button.doneDischarging";
            case 2:
                return "gui.tab.redstoneBehaviour.chargingStation.button.charging";
            case 3:
                return "gui.tab.redstoneBehaviour.chargingStation.button.discharging";

        }
        return "<ERROR>";
    }

    @Override
    protected void func_146976_a(float opacity, int x, int y){
        super.func_146976_a(opacity, x, y);
        GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);

        renderAir();
    }

    @Override
    public void func_73876_c(){
        super.func_73876_c();
        guiSelectButton.field_146124_l = te.func_70301_a(TileEntityChargingStation.CHARGE_INVENTORY_INDEX) != null && te.func_70301_a(TileEntityChargingStation.CHARGE_INVENTORY_INDEX).func_77973_b() instanceof IChargingStationGUIHolderItem;

    }

    @Override
    protected Point getGaugeLocation(){
        int xStart = (field_146294_l - field_146999_f) / 2;
        int yStart = (field_146295_m - field_147000_g) / 2;
        return new Point(xStart + field_146999_f * 3 / 4 + 10, yStart + field_147000_g * 1 / 4 + 4);
    }

    @Override
    protected void addPressureStatInfo(List<String> pressureStatText){
        super.addPressureStatInfo(pressureStatText);
        if(te.charging || te.disCharging) {
            pressureStatText.add("\u00a77" + (te.charging ? "C" : "Disc") + "harging at");
            pressureStatText.add("\u00a70" + (double)Math.round(PneumaticValues.CHARGING_STATION_CHARGE_RATE * te.getSpeedMultiplierFromUpgrades()) + "mL/tick");
        }
    }

    @Override
    protected void addProblems(List<String> textList){
        super.addProblems(textList);
        if(te.func_70301_a(TileEntityChargingStation.CHARGE_INVENTORY_INDEX) == null) {
            textList.add("\u00a77No items to (dis)charge");
            textList.addAll(PneumaticCraftUtils.convertStringIntoList("\u00a70Put a pneumatic item in the charge slot.", GuiConstants.maxCharPerLineLeft));
        } else if(!(te.func_70301_a(TileEntityChargingStation.CHARGE_INVENTORY_INDEX).func_77973_b() instanceof IPressurizable)) {
            textList.addAll(PneumaticCraftUtils.convertStringIntoList("\u00a77The put in item can't be (dis)charged", GuiConstants.maxCharPerLineLeft));
            textList.addAll(PneumaticCraftUtils.convertStringIntoList("\u00a70Put a pneumatic item in the charge slot.", GuiConstants.maxCharPerLineLeft));
        } else {
            ItemStack chargeStack = te.func_70301_a(TileEntityChargingStation.CHARGE_INVENTORY_INDEX);
            IPressurizable chargeItem = (IPressurizable)chargeStack.func_77973_b();
            if(chargeItem.getPressure(chargeStack) > te.getPressure() + 0.01F && chargeItem.getPressure(chargeStack) <= 0) {
                textList.addAll(PneumaticCraftUtils.convertStringIntoList("\u00a77The put in item can't be discharged", GuiConstants.maxCharPerLineLeft));
                textList.add("\u00a70The item is empty.");
            } else if(chargeItem.getPressure(chargeStack) < te.getPressure() - 0.01F && chargeItem.getPressure(chargeStack) >= chargeItem.maxPressure(chargeStack)) {
                textList.addAll(PneumaticCraftUtils.convertStringIntoList("\u00a77The put in item can't be charged", GuiConstants.maxCharPerLineLeft));
                textList.add("\u00a70The item is full.");
            } else if(!te.charging && !te.disCharging) {
                textList.addAll(PneumaticCraftUtils.convertStringIntoList("\u00a77The put in item can't be (dis)charged", GuiConstants.maxCharPerLineLeft));
                textList.add("\u00a70The pressures have equalized.");
            }
        }
    }

    private void renderAir(){
        GL11.glDisable(GL11.GL_TEXTURE_2D);
        GL11.glColor4d(1, 1, 1, 1);
        GL11.glLineWidth(2.0F);
        int particles = 10;
        for(int i = 0; i < particles; i++) {
            renderAirParticle(te.renderAirProgress % (1F / particles) + (float)i / particles);
        }

        GL11.glEnable(GL11.GL_TEXTURE_2D);
    }

    private void renderAirParticle(float particleProgress){
        int xStart = (field_146294_l - field_146999_f) / 2;
        int yStart = (field_146295_m - field_147000_g) / 2;
        float x = xStart + 117F;
        float y = yStart + 50.5F;
        if(particleProgress < 0.5F) {
            y += particleProgress * 56;
        } else if(particleProgress < 0.7F) {
            y += 28F;
            x -= (particleProgress - 0.5F) * 90;
        } else {
            y += 28F;
            x -= 18;
            y -= (particleProgress - 0.7F) * 70;
        }
        WorldRenderer wr = Tessellator.func_178181_a().func_178180_c();
        GL11.glPointSize(5);
        wr.func_181668_a(GL11.GL_POINTS, DefaultVertexFormats.field_181705_e);
        wr.func_181662_b(x, y, field_73735_i).func_181675_d();
        Tessellator.func_178181_a().func_78381_a();
    }
}
