package pneumaticCraft.client.gui;

import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

import net.minecraft.client.resources.I18n;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.util.MathHelper;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.client.gui.widget.GuiAnimatedStat;
import pneumaticCraft.client.gui.widget.GuiCheckBox;
import pneumaticCraft.client.gui.widget.WidgetTank;
import pneumaticCraft.client.gui.widget.WidgetTemperature;
import pneumaticCraft.common.inventory.ContainerPlasticMixer;
import pneumaticCraft.common.item.Itemss;
import pneumaticCraft.common.tileentity.TileEntityPlasticMixer;
import pneumaticCraft.lib.PneumaticValues;
import pneumaticCraft.lib.Textures;

@SideOnly(Side.CLIENT)
public class GuiPlasticMixer extends GuiPneumaticContainerBase<TileEntityPlasticMixer>{
    private GuiButtonSpecial[] buttons;
    private GuiCheckBox lockSelection;

    public GuiPlasticMixer(InventoryPlayer player, TileEntityPlasticMixer te){
        super(new ContainerPlasticMixer(player, te), te, Textures.GUI_PLASTIC_MIXER);
    }

    @Override
    public void func_73866_w_(){
        super.func_73866_w_();

        addWidget(new WidgetTemperature(0, field_147003_i + 55, field_147009_r + 25, 295, 500, te.getLogic(0)));
        addWidget(new WidgetTemperature(1, field_147003_i + 82, field_147009_r + 25, 295, 500, te.getLogic(1), PneumaticValues.PLASTIC_MIXER_MELTING_TEMP));
        addWidget(new WidgetTank(3, field_147003_i + 152, field_147009_r + 14, te.getFluidTank()));

        GuiAnimatedStat stat = addAnimatedStat("gui.tab.plasticMixer.plasticSelection", new ItemStack(Itemss.plastic, 1, 1), 0xFF005500, false);
        List<String> text = new ArrayList<String>();
        for(int i = 0; i < 12; i++) {
            text.add("                      ");
        }
        stat.setTextWithoutCuttingString(text);

        buttons = new GuiButtonSpecial[16];
        for(int x = 0; x < 4; x++) {
            for(int y = 0; y < 4; y++) {
                int index = y * 4 + x;
                ItemStack plastic = new ItemStack(Itemss.plastic, 1, index);
                buttons[index] = new GuiButtonSpecial(index + 1, x * 21 + 4, y * 21 + 30, 20, 20, "").setRenderStacks(plastic).setTooltipText(plastic.func_82833_r());
                stat.addWidget(buttons[index]);
            }
        }
        stat.addWidget(lockSelection = new GuiCheckBox(17, 4, 18, 0xFF000000, "gui.plasticMixer.lockSelection").setChecked(te.lockSelection).setTooltip(I18n.func_135052_a("gui.plasticMixer.lockSelection.tooltip")));
    }

    @Override
    public String getRedstoneButtonText(int mode){
        return mode == 3 ? "gui.tab.redstoneBehaviour.plasticMixer.button.selectOnSignal" : super.getRedstoneButtonText(mode);
    }

    @Override
    public void func_73876_c(){
        super.func_73876_c();
        for(int i = 0; i < buttons.length; i++) {
            buttons[i].field_146124_l = te.selectedPlastic != i;
        }
        lockSelection.checked = te.lockSelection;
    }

    @Override
    protected void func_146979_b(int x, int y){
        super.func_146979_b(x, y);

        field_146289_q.func_78276_b("Upgr.", 15, 19, 4210752);
        field_146289_q.func_78276_b("Hull", 56, 16, 4210752);
        field_146289_q.func_78276_b("Item", 88, 16, 4210752);
    }

    @Override
    protected void func_146976_a(float partialTicks, int x, int y){
        super.func_146976_a(partialTicks, x, y);
        for(int i = 0; i < 3; i++) {
            double percentage = (double)te.dyeBuffers[i] / TileEntityPlasticMixer.DYE_BUFFER_MAX;
            func_73728_b(field_147003_i + 123, field_147009_r + 37 + i * 18, field_147009_r + 37 - MathHelper.func_76125_a((int)(percentage * 16), 1, 15) + i * 18, 0xFF000000 | 0xFF0000 >> 8 * i);
        }
    }

    @Override
    protected Point getInvNameOffset(){
        return new Point(0, -1);
    }

    @Override
    protected Point getInvTextOffset(){
        return null;
    }

    @Override
    protected void addProblems(List<String> curInfo){
        super.addProblems(curInfo);
        if(te.getFluidTank().getFluidAmount() == 0) {
            if(te.func_70301_a(4) == null) {
                curInfo.add("gui.tab.problems.plasticMixer.noPlastic");
            } else {
                curInfo.add("gui.tab.problems.notEnoughHeat");
            }
        } else {
            if(te.func_70301_a(4) != null) {
                if(te.getLogic(1).getTemperature() >= PneumaticValues.PLASTIC_MIXER_MELTING_TEMP && te.getFluidTank().getCapacity() - te.getFluidTank().getFluidAmount() < 1000) {
                    curInfo.add("gui.tab.problems.plasticMixer.plasticLiquidOverflow");
                }
            }
        }
        if(te.func_70301_a(TileEntityPlasticMixer.INV_DYE_RED) == null) {
            curInfo.add(I18n.func_135052_a("gui.tab.problems.plasticMixer.noDye", new ItemStack(Items.field_151100_aR, 1, 1).func_82833_r()));
        }
        if(te.func_70301_a(TileEntityPlasticMixer.INV_DYE_GREEN) == null) {
            curInfo.add(I18n.func_135052_a("gui.tab.problems.plasticMixer.noDye", new ItemStack(Items.field_151100_aR, 1, 2).func_82833_r()));
        }
        if(te.func_70301_a(TileEntityPlasticMixer.INV_DYE_BLUE) == null) {
            curInfo.add(I18n.func_135052_a("gui.tab.problems.plasticMixer.noDye", new ItemStack(Items.field_151100_aR, 1, 4).func_82833_r()));
        }

        if(curInfo.size() == 0) {
            curInfo.add(I18n.func_135052_a("gui.tab.problems.plasticMixer.noProblems"));
        }
    }
}
