package pneumaticCraft.client.gui;

import java.awt.Point;
import java.awt.Rectangle;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.ScaledResolution;
import net.minecraft.client.gui.inventory.GuiContainer;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.WorldRenderer;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.client.resources.I18n;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.StatCollector;
import net.minecraftforge.fml.common.Loader;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.lwjgl.opengl.GL11;

import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.api.client.IGuiAnimatedStat;
import pneumaticCraft.api.tileentity.IAirHandler;
import pneumaticCraft.api.tileentity.IHeatExchanger;
import pneumaticCraft.client.gui.widget.GuiAnimatedStat;
import pneumaticCraft.client.gui.widget.IGuiWidget;
import pneumaticCraft.client.gui.widget.IWidgetListener;
import pneumaticCraft.client.gui.widget.WidgetLabel;
import pneumaticCraft.client.gui.widget.WidgetTextField;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketGuiButton;
import pneumaticCraft.common.tileentity.IMinWorkingPressure;
import pneumaticCraft.common.tileentity.IRedstoneControl;
import pneumaticCraft.common.tileentity.IRedstoneControlled;
import pneumaticCraft.common.tileentity.TileEntityBase;
import pneumaticCraft.common.tileentity.TileEntityPneumaticBase;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.ModIds;
import pneumaticCraft.lib.Textures;

@SideOnly(Side.CLIENT)
//TODO NEI dep @Optional.Interface(iface = "codechicken.nei.api.INEIGuiHandler", modid = ModIds.NEI)
public class GuiPneumaticContainerBase<Tile extends TileEntityBase> extends GuiContainer implements /*INEIGuiHandler,*/
IWidgetListener{

    public final Tile te;
    private final ResourceLocation guiTexture;
    /**
     * Any GuiAnimatedStat added to this list will be tracked for mouseclicks, tooltip renders, rendering,updating (resolution and expansion).
     */
    protected final List<IGuiWidget> widgets = new ArrayList<IGuiWidget>();
    private IGuiAnimatedStat lastLeftStat, lastRightStat;

    private GuiAnimatedStat pressureStat;
    protected GuiAnimatedStat problemTab;
    private GuiAnimatedStat redstoneTab;
    protected GuiButtonSpecial redstoneButton;
    private boolean hasInit; //Fix for some weird race condition occuring in 1.8 where drawing is called before initGui().

    public GuiPneumaticContainerBase(Container par1Container, Tile te, String guiTexture){
        super(par1Container);
        this.te = te;
        this.guiTexture = guiTexture != null ? new ResourceLocation(guiTexture) : null;
    }

    protected GuiAnimatedStat addAnimatedStat(String title, ItemStack icon, int color, boolean leftSided){
        int xStart = (field_146294_l - field_146999_f) / 2;
        int yStart = (field_146295_m - field_147000_g) / 2;

        GuiAnimatedStat stat = new GuiAnimatedStat(this, title, icon, xStart + (leftSided ? 0 : field_146999_f), leftSided && lastLeftStat != null || !leftSided && lastRightStat != null ? 3 : yStart + 5, color, leftSided ? lastLeftStat : lastRightStat, leftSided);
        addWidget(stat);
        if(leftSided) {
            lastLeftStat = stat;
        } else {
            lastRightStat = stat;
        }
        return stat;
    }

    protected GuiAnimatedStat addAnimatedStat(String title, String icon, int color, boolean leftSided){
        int xStart = (field_146294_l - field_146999_f) / 2;
        int yStart = (field_146295_m - field_147000_g) / 2;

        GuiAnimatedStat stat = new GuiAnimatedStat(this, title, icon, xStart + (leftSided ? 0 : field_146999_f), leftSided && lastLeftStat != null || !leftSided && lastRightStat != null ? 3 : yStart + 5, color, leftSided ? lastLeftStat : lastRightStat, leftSided);
        addWidget(stat);
        if(leftSided) {
            lastLeftStat = stat;
        } else {
            lastRightStat = stat;
        }
        return stat;
    }

    protected void addWidget(IGuiWidget widget){
        widgets.add(widget);
        widget.setListener(this);
    }

    protected void addWidgets(Iterable<IGuiWidget> widgets){
        for(IGuiWidget widget : widgets) {
            addWidget(widget);
        }
    }

    protected void addLabel(String text, int x, int y){
        addWidget(new WidgetLabel(x, y, text));
    }

    protected void removeWidget(IGuiWidget widget){
        widgets.remove(widget);
    }

    @Override
    public void func_73866_w_(){
        super.func_73866_w_();
        lastLeftStat = lastRightStat = null;
        if(shouldAddPressureTab() && te instanceof TileEntityPneumaticBase) {
            pressureStat = this.addAnimatedStat("gui.tab.pressure", new ItemStack(Blockss.pressureTube), 0xFF00AA00, false);
        }
        if(shouldAddProblemTab()) {
            problemTab = addAnimatedStat("gui.tab.problems", Textures.GUI_PROBLEMS_TEXTURE, 0xFFFF0000, false);
        }
        if(shouldAddRedstoneTab() && te instanceof IRedstoneControl) {
            redstoneTab = addAnimatedStat("gui.tab.redstoneBehaviour", new ItemStack(Items.field_151137_ax), 0xFFCC0000, true);
            List<String> curInfo = new ArrayList<String>();
            curInfo.add(I18n.func_135052_a(getRedstoneString()));
            for(int i = 0; i < 3; i++)
                curInfo.add("                                      ");// create some space for the button
            redstoneTab.setTextWithoutCuttingString(curInfo);
            Rectangle buttonRect = redstoneTab.getButtonScaledRectangle(-170, 24, 170, 20);
            redstoneButton = new GuiButtonSpecial(0, buttonRect.x, buttonRect.y, buttonRect.width, buttonRect.height, "-");//getButtonFromRectangle(0, buttonRect, "-");
            redstoneTab.addWidget(redstoneButton);
        }
        if(te instanceof IInventory) {
            if(shouldAddInfoTab()) {
                String info = "gui.tab.info." + ((IInventory)te).func_70005_c_();
                String translatedInfo = I18n.func_135052_a(info);
                if(!translatedInfo.equals(info)) {
                    addInfoTab(translatedInfo);
                }
            }
            if(te instanceof IHeatExchanger) {
                addAnimatedStat("gui.tab.info.heat.title", new ItemStack(Blocks.field_150480_ab), 0xFFFF5500, false).setText("gui.tab.info.heat");
            }
            if(shouldAddUpgradeTab()) {
                String upgrades = "gui.tab.upgrades." + ((IInventory)te).func_70005_c_();
                String translatedUpgrades = I18n.func_135052_a(upgrades);
                List<String> upgradeText = new ArrayList<String>();
                if(te instanceof TileEntityPneumaticBase) {
                    upgradeText.add("gui.tab.upgrades.volume");
                    upgradeText.add("gui.tab.upgrades.security");
                }
                if(te instanceof IHeatExchanger) {
                    upgradeText.add("gui.tab.upgrades.volumeCapacity");
                }
                if(!translatedUpgrades.equals(upgrades)) upgradeText.add(upgrades);

                if(upgradeText.size() > 0) addAnimatedStat("gui.tab.upgrades", Textures.GUI_UPGRADES_LOCATION, 0xFF0000FF, true).setText(upgradeText);
            }
        }
        hasInit = true;
    }

    protected void addInfoTab(String info){
        if(!Loader.isModLoaded(ModIds.IGWMOD)) info += " \\n \\n" + I18n.func_135052_a("gui.tab.info.assistIGW");
        addAnimatedStat("gui.tab.info", Textures.GUI_INFO_LOCATION, 0xFF8888FF, true).setText(info);
    }

    protected boolean shouldAddRedstoneTab(){
        return true;
    }

    protected boolean shouldAddPressureTab(){
        return true;
    }

    protected boolean shouldAddUpgradeTab(){
        return true;
    }

    protected boolean shouldAddInfoTab(){
        return true;
    }

    protected boolean shouldAddProblemTab(){
        return true;
    }

    @Override
    protected void func_146976_a(float partialTicks, int i, int j){
        if(shouldDrawBackground()) {
            GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
            bindGuiTexture();
            int xStart = (field_146294_l - field_146999_f) / 2;
            int yStart = (field_146295_m - field_147000_g) / 2;
            func_73729_b(xStart, yStart, 0, 0, field_146999_f, field_147000_g);
        }

        GL11.glColor4d(1, 1, 1, 1);
        GL11.glDisable(GL11.GL_LIGHTING);
        for(IGuiWidget widget : widgets) {
            widget.render(i, j, partialTicks);
        }
        for(IGuiWidget widget : widgets) {
            widget.postRender(i, j, partialTicks);
        }

        if(pressureStat != null) {
            TileEntityPneumaticBase pneu = (TileEntityPneumaticBase)te;
            Point gaugeLocation = getGaugeLocation();
            if(gaugeLocation != null) GuiUtils.drawPressureGauge(field_146289_q, -1, pneu.criticalPressure, pneu.dangerPressure, te instanceof IMinWorkingPressure ? ((IMinWorkingPressure)te).getMinWorkingPressure() : -1, pneu.getPressure(), gaugeLocation.x, gaugeLocation.y, field_73735_i);
        }
    }

    protected boolean shouldDrawBackground(){
        return true;
    }

    protected void bindGuiTexture(){
        if(guiTexture != null) {
            field_146297_k.func_110434_K().func_110577_a(guiTexture);
            GL11.glEnable(GL11.GL_TEXTURE_2D);
        }
    }

    protected Point getGaugeLocation(){
        int xStart = (field_146294_l - field_146999_f) / 2;
        int yStart = (field_146295_m - field_147000_g) / 2;
        return new Point(xStart + field_146999_f * 3 / 4, yStart + field_147000_g * 1 / 4 + 4);
    }

    @Override
    protected void func_146979_b(int x, int y){
        if(getInvNameOffset() != null && te instanceof IInventory) {
            IInventory inv = (IInventory)te;
            String containerName = inv.func_145818_k_() ? inv.func_70005_c_() : StatCollector.func_74838_a(inv.func_70005_c_() + ".name");
            field_146289_q.func_78276_b(containerName, field_146999_f / 2 - field_146289_q.func_78256_a(containerName) / 2 + getInvNameOffset().x, 6 + getInvNameOffset().y, 4210752);
        }
        if(getInvTextOffset() != null) field_146289_q.func_78276_b(StatCollector.func_74838_a("container.inventory"), 8 + getInvTextOffset().x, field_147000_g - 94 + getInvTextOffset().y, 4210752);
    }

    protected Point getInvNameOffset(){
        return new Point(0, 0);
    }

    protected Point getInvTextOffset(){
        return new Point(0, 0);
    }

    @Override
    public void func_73863_a(int x, int y, float partialTick){
        if(!hasInit) return;
        super.func_73863_a(x, y, partialTick);

        List<String> tooltip = new ArrayList<String>();
        for(Object obj : field_146292_n) {
            if(obj instanceof GuiButtonSpecial) {
                GuiButtonSpecial button = (GuiButtonSpecial)obj;
                if(button.field_146128_h < x && button.field_146128_h + button.getWidth() > x && button.field_146129_i < y && button.field_146129_i + button.getHeight() > y) {
                    button.getTooltip(tooltip);
                }
            }
        }

        GL11.glColor4d(1, 1, 1, 1);
        GL11.glDisable(GL11.GL_LIGHTING);
        for(IGuiWidget widget : widgets) {
            if(widget.getBounds().contains(x, y)) widget.addTooltip(x, y, tooltip, PneumaticCraft.proxy.isSneakingInGui());
        }

        if(tooltip.size() > 0) {
            drawHoveringString(tooltip, x, y, field_146289_q);
            tooltip.clear();
        }

        /* TODO boolean shift = PneumaticCraft.proxy.isSneakingInGui();
         for(IGuiWidget widget : widgets) {
             if(widget.getBounds().contains(x, y)) widget.addTooltip(tooltip, shift);
         }
         if(!tooltip.isEmpty()) {
             List<String> localizedTooltip = new ArrayList<String>();
             for(String line : tooltip) {
                 String localizedLine = I18n.format(line);
                 String[] lines = WordUtils.wrap(localizedLine, 50).split(System.getProperty("line.separator"));
                 for(String locLine : lines) {
                     localizedTooltip.add(locLine);
                 }
             }
             drawHoveringText(localizedTooltip, x, y, fontRendererObj);
         }*/
    }

    @Override
    public void func_73876_c(){
        super.func_73876_c();

        for(IGuiWidget widget : widgets)
            widget.update();

        if(pressureStat != null) {
            List<String> curInfo = new ArrayList<String>();
            addPressureStatInfo(curInfo);
            pressureStat.setText(curInfo);
        }
        if(problemTab != null) {
            List<String> curInfo = new ArrayList<String>();
            addProblems(curInfo);
            if(curInfo.size() == 0) {
                curInfo.add("gui.tab.problems.noProblems");
            }
            problemTab.setText(curInfo);
        }
        if(redstoneTab != null) {
            redstoneButton.field_146126_j = I18n.func_135052_a(getRedstoneButtonText(((IRedstoneControl)te).getRedstoneMode()));
        }
    }

    @Override
    protected void func_146284_a(GuiButton button){
        sendPacketToServer(button.field_146127_k);
    }

    public String getRedstoneButtonText(int mode){
        switch(mode){
            case 0:
                return "gui.tab.redstoneBehaviour.button.anySignal";
            case 1:
                return "gui.tab.redstoneBehaviour.button.highSignal";
            case 2:
                return "gui.tab.redstoneBehaviour.button.lowSignal";
        }
        return "<ERROR>";
    }

    public String getRedstoneString(){
        return te instanceof IRedstoneControlled ? "gui.tab.redstoneBehaviour.enableOn" : "gui.tab.redstoneBehaviour.emitRedstoneWhen";
    }

    protected void addPressureStatInfo(List<String> pressureStatText){
        TileEntityPneumaticBase pneumaticTile = (TileEntityPneumaticBase)te;
        IAirHandler airHandler = pneumaticTile.getAirHandler(null);
        pressureStatText.add("\u00a77Current Pressure:");
        pressureStatText.add("\u00a70" + PneumaticCraftUtils.roundNumberTo(pneumaticTile.getPressure(), 1) + " bar.");
        pressureStatText.add("\u00a77Current Air:");
        pressureStatText.add("\u00a70" + (airHandler.getAir() + airHandler.getVolume()) + " mL.");
        pressureStatText.add("\u00a77Volume:");
        pressureStatText.add("\u00a70" + pneumaticTile.defaultVolume + " mL.");
        int volumeLeft = airHandler.getVolume() - pneumaticTile.defaultVolume;
        if(volumeLeft > 0) {
            pressureStatText.add("\u00a70" + volumeLeft + " mL. (Volume Upgrades)");
            pressureStatText.add("\u00a70--------+");
            pressureStatText.add("\u00a70" + airHandler.getVolume() + " mL.");
        }
    }

    protected void addProblems(List<String> curInfo){
        if(te instanceof IRedstoneControlled && !te.redstoneAllows()) {
            IRedstoneControlled redstoneControlled = (IRedstoneControlled)te;
            curInfo.add("gui.tab.problems.redstoneDisallows");
            if(redstoneControlled.getRedstoneMode() == 1) {
                curInfo.add("gui.tab.problems.provideRedstone");
            } else {
                curInfo.add("gui.tab.problems.removeRedstone");
            }
        }
        if(te instanceof IMinWorkingPressure) {
            IMinWorkingPressure minWork = (IMinWorkingPressure)te;
            if(((TileEntityPneumaticBase)te).getPressure() < minWork.getMinWorkingPressure()) {
                curInfo.add("gui.tab.problems.notEnoughPressure");
                curInfo.add(I18n.func_135052_a("gui.tab.problems.applyPressure", minWork.getMinWorkingPressure()));
            }
        }
    }

    @Override
    protected void func_73864_a(int par1, int par2, int par3) throws IOException{
        super.func_73864_a(par1, par2, par3);
        for(IGuiWidget widget : widgets) {
            if(widget.getBounds().contains(par1, par2)) widget.onMouseClicked(par1, par2, par3);
            else widget.onMouseClickedOutsideBounds(par1, par2, par3);
        }
    }

    @Override
    public void actionPerformed(IGuiWidget widget){
        if(widget instanceof IGuiAnimatedStat) {
            boolean leftSided = ((IGuiAnimatedStat)widget).isLeftSided();
            for(IGuiWidget w : widgets) {
                if(w instanceof IGuiAnimatedStat) {
                    IGuiAnimatedStat stat = (IGuiAnimatedStat)w;
                    if(widget != stat && stat.isLeftSided() == leftSided) {//when the stat is on the same side, close it.
                        stat.closeWindow();
                    }
                }
            }
        }
        sendPacketToServer(widget.getID());
    }

    protected void sendPacketToServer(int id){
        NetworkHandler.sendToServer(new PacketGuiButton(id));
    }

    @Override
    public void func_146274_d() throws IOException{
        super.func_146274_d();
        for(IGuiWidget widget : widgets) {
            widget.handleMouseInput();
        }
    }

    @Override
    protected void func_73869_a(char key, int keyCode) throws IOException{
        for(IGuiWidget widget : widgets) {
            if(widget.onKey(key, keyCode)) return;
        }
        super.func_73869_a(key, keyCode);
    }

    @Override
    public void func_146280_a(Minecraft par1Minecraft, int par2, int par3){
        widgets.clear();
        super.func_146280_a(par1Minecraft, par2, par3);
    }

    public void drawHoveringString(List<String> text, int x, int y, FontRenderer fontRenderer){
        drawHoveringText(text, x, y, fontRenderer);
    }

    public static void drawTexture(String texture, int x, int y){
        Minecraft.func_71410_x().func_110434_K().func_110577_a(GuiUtils.getResourceLocation(texture));
        WorldRenderer wr = Tessellator.func_178181_a().func_178180_c();
        wr.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_181707_g);
        wr.func_181662_b(x, y + 16, 0).func_181673_a(0.0, 1.0).func_181675_d();
        wr.func_181662_b(x + 16, y + 16, 0).func_181673_a(1.0, 1.0).func_181675_d();
        wr.func_181662_b(x + 16, y, 0).func_181673_a(1.0, 0.0).func_181675_d();
        wr.func_181662_b(x, y, 0).func_181673_a(0.0, 0.0).func_181675_d();
        Tessellator.func_178181_a().func_78381_a();
        // this.drawTexturedModalRect(x, y, 0, 0, 16, 16);
    }

    public GuiButtonSpecial getButtonFromRectangle(int buttonID, Rectangle buttonSize, String buttonText){
        return new GuiButtonSpecial(buttonID, buttonSize.x, buttonSize.y, buttonSize.width, buttonSize.height, buttonText);
    }

    public GuiButtonSpecial getInvisibleButtonFromRectangle(int buttonID, Rectangle buttonSize){
        return new GuiButtonSpecial(buttonID, buttonSize.x, buttonSize.y, buttonSize.width, buttonSize.height, "");
    }

    public WidgetTextField getTextFieldFromRectangle(Rectangle textFieldSize){
        return new WidgetTextField(field_146289_q, textFieldSize.x, textFieldSize.y, textFieldSize.width, textFieldSize.height);
    }

    public int getGuiLeft(){
        return field_147003_i;
    }

    public int getGuiTop(){
        return field_147009_r;
    }

    //-----------NEI support

    /*@Override
    @Optional.Method(modid = ModIds.NEI)
    public VisiblityData modifyVisiblity(GuiContainer gui, VisiblityData currentVisibility){
        for(IGuiWidget w : widgets) {
            if(w instanceof IGuiAnimatedStat) {
                IGuiAnimatedStat stat = (IGuiAnimatedStat)w;
                if(stat.isLeftSided()) {
                    if(stat.getWidth() > 20) {
                        currentVisibility.showUtilityButtons = false;
                        currentVisibility.showStateButtons = false;
                    }
                } else {
                    if(stat.getAffectedY() < 10) {
                        currentVisibility.showWidgets = false;
                    }
                }
            }
        }
        return currentVisibility;
    }

    @Override
    public Iterable<Integer> getItemSpawnSlots(GuiContainer gui, ItemStack item){
        return new ArrayList<Integer>();
    }

    /**
     * @return A list of TaggedInventoryAreas that will be used with the savestates.
     */
    /* @Override
     @Optional.Method(modid = ModIds.NEI)
     public List<TaggedInventoryArea> getInventoryAreas(GuiContainer gui){
         return null;
     }

     /**
      * Handles clicks while an itemstack has been dragged from the item panel. Use this to set configurable slots and the like. 
      * Changes made to the stackSize of the dragged stack will be kept
      * @param gui The current gui instance
      * @param mousex The x position of the mouse
      * @param mousey The y position of the mouse
      * @param draggedStack The stack being dragged from the item panel
      * @param button The button presed
      * @return True if the drag n drop was handled. False to resume processing through other routes. The held stack will be deleted if draggedStack.stackSize == 0
      */
    /* @Override
     public boolean handleDragNDrop(GuiContainer gui, int mousex, int mousey, ItemStack draggedStack, int button){
         return false;
     }

     /**
      * Used to prevent the item panel from drawing on top of other gui elements.
      * @param x The x coordinate of the rectangle bounding the slot
      * @param y The y coordinate of the rectangle bounding the slot
      * @param w The w coordinate of the rectangle bounding the slot
      * @param h The h coordinate of the rectangle bounding the slot
      * @return true if the item panel slot within the specified rectangle should not be rendered.
      */
    /*  @Override
      public boolean hideItemPanelSlot(GuiContainer gui, int x, int y, int w, int h){
          for(IGuiWidget widget : widgets) {
              if(widget instanceof IGuiAnimatedStat) {
                  IGuiAnimatedStat stat = (IGuiAnimatedStat)widget;
                  if(stat.getBounds().intersects(new Rectangle(x, y, w, h))) return true;
              }
          }
          return false;
      }
    */
    @Override
    public void onKeyTyped(IGuiWidget widget){

    }

    protected void refreshScreen(){
        ScaledResolution scaledresolution = new ScaledResolution(Minecraft.func_71410_x());
        int i = scaledresolution.func_78326_a();
        int j = scaledresolution.func_78328_b();
        func_146280_a(Minecraft.func_71410_x(), i, j);
        for(IGuiWidget widget : widgets) {
            if(widget instanceof GuiAnimatedStat) {
                widget.update();
            }
        }
    }
}
