package pneumaticCraft.client.gui;

import java.awt.Point;
import java.util.List;

import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.resources.I18n;
import net.minecraft.item.ItemStack;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.item.IPressurizable;
import pneumaticCraft.common.inventory.ContainerChargingStationItemInventory;
import pneumaticCraft.common.item.ItemPneumaticArmor;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketGuiButton;
import pneumaticCraft.common.tileentity.TileEntityChargingStation;
import pneumaticCraft.lib.PneumaticValues;
import pneumaticCraft.lib.Textures;

@SideOnly(Side.CLIENT)
public abstract class GuiPneumaticInventoryItem extends GuiPneumaticContainerBase<TileEntityChargingStation>{

    protected ItemStack itemStack;

    private GuiButton guiSelectButton;

    public GuiPneumaticInventoryItem(ContainerChargingStationItemInventory container, TileEntityChargingStation te){
        super(container, te, Textures.GUI_PNEUMATIC_ARMOR_LOCATION);
        itemStack = te.func_70301_a(TileEntityChargingStation.CHARGE_INVENTORY_INDEX);
    }

    @Override
    public void func_73866_w_(){
        super.func_73866_w_();
        int xStart = (field_146294_l - field_146999_f) / 2;
        int yStart = (field_146295_m - field_147000_g) / 2;
        guiSelectButton = new GuiButton(2, xStart + 90, yStart + 15, 25, 20, "<--");
        field_146292_n.add(guiSelectButton);
    }

    @Override
    protected void addPressureStatInfo(List<String> pressureStatText){
        pressureStatText.add("\u00a77Current Pressure:");
        float curPressure = ((IPressurizable)itemStack.func_77973_b()).getPressure(te.func_70301_a(TileEntityChargingStation.CHARGE_INVENTORY_INDEX));
        int volume = ItemPneumaticArmor.getUpgrades(EnumUpgrade.VOLUME, te.func_70301_a(TileEntityChargingStation.CHARGE_INVENTORY_INDEX)) * PneumaticValues.VOLUME_VOLUME_UPGRADE + getDefaultVolume();
        pressureStatText.add("\u00a70" + (double)Math.round(curPressure * 10) / 10 + " bar.");
        pressureStatText.add("\u00a77Current Air:");
        pressureStatText.add("\u00a70" + (double)Math.round(curPressure * volume) + " mL.");
        pressureStatText.add("\u00a77Volume:");
        pressureStatText.add("\u00a70" + getDefaultVolume() + " mL.");
        if(volume > getDefaultVolume()) {
            pressureStatText.add("\u00a70" + (double)Math.round(volume - getDefaultVolume()) + " mL. (Volume Upgrades)");
            pressureStatText.add("\u00a70--------+");
            pressureStatText.add("\u00a70" + (double)Math.round(volume) + " mL.");
        }
    }

    @Override
    protected boolean shouldAddRedstoneTab(){
        return false;
    }

    @Override
    protected boolean shouldAddUpgradeTab(){
        return false;
    }

    @Override
    protected boolean shouldAddInfoTab(){
        return false;
    }

    @Override
    protected boolean shouldAddProblemTab(){
        return false;
    }

    protected abstract int getDefaultVolume();

    /**
     * Fired when a control is clicked. This is the equivalent of
     * ActionListener.actionPerformed(ActionEvent e).
     */
    @Override
    protected void func_146284_a(GuiButton button){
        NetworkHandler.sendToServer(new PacketGuiButton(button.field_146127_k));
    }

    @Override
    protected void func_146979_b(int x, int y){
        String containerName = itemStack.func_82833_r();
        field_146289_q.func_78276_b(containerName, field_146999_f / 2 - field_146289_q.func_78256_a(containerName) / 2, 4, 4210752);
        field_146289_q.func_78276_b(I18n.func_135052_a("gui.tab.upgrades"), 36, 14, 4210752);
    }

    @Override
    protected void func_146976_a(float opacity, int x, int y){
        super.func_146976_a(opacity, x, y);

        int xStart = (field_146294_l - field_146999_f) / 2;
        int yStart = (field_146295_m - field_147000_g) / 2;
        GuiUtils.drawPressureGauge(field_146289_q, 0, 10, 10, 0, ((IPressurizable)itemStack.func_77973_b()).getPressure(te.func_70301_a(TileEntityChargingStation.CHARGE_INVENTORY_INDEX)), xStart + field_146999_f * 3 / 4 + 8, yStart + field_147000_g * 1 / 4 + 4, field_73735_i);
    }

    @Override
    protected Point getGaugeLocation(){
        return null;
    }
}
