package pneumaticCraft.client.gui;

import java.awt.Rectangle;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.GuiScreen;
import net.minecraft.client.gui.GuiTextField;
import net.minecraft.client.resources.I18n;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.client.FMLClientHandler;

import org.apache.commons.lang3.text.WordUtils;
import org.lwjgl.opengl.GL11;

import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.api.client.IGuiAnimatedStat;
import pneumaticCraft.client.gui.widget.IGuiWidget;
import pneumaticCraft.client.gui.widget.IWidgetListener;
import pneumaticCraft.client.gui.widget.WidgetLabel;

public abstract class GuiPneumaticScreenBase extends GuiScreen implements IWidgetListener{

    protected final List<IGuiWidget> widgets = new ArrayList<IGuiWidget>();
    public int guiLeft, guiTop, xSize, ySize;

    @Override
    public void func_73866_w_(){
        super.func_73866_w_();
        widgets.clear();
        guiLeft = field_146294_l / 2 - xSize / 2;
        guiTop = field_146295_m / 2 - ySize / 2;
    }

    public void addWidgets(Iterable<IGuiWidget> widgets){
        for(IGuiWidget widget : widgets) {
            addWidget(widget);
        }
    }

    public void addWidget(IGuiWidget widget){
        widgets.add(widget);
        widget.setListener(this);
    }

    protected void addLabel(String text, int x, int y){
        addWidget(new WidgetLabel(x, y, text));
    }

    public void removeWidget(IGuiWidget widget){
        widgets.remove(widget);
    }

    protected abstract ResourceLocation getTexture();

    @Override
    public void func_73863_a(int x, int y, float partialTicks){
        if(getTexture() != null) {
            FMLClientHandler.instance().getClient().func_110434_K().func_110577_a(getTexture());
            func_73729_b(guiLeft, guiTop, 0, 0, xSize, ySize);
        }
        super.func_73863_a(x, y, partialTicks);

        for(IGuiWidget widget : widgets) {
            widget.render(x, y, partialTicks);
        }
        for(IGuiWidget widget : widgets) {
            widget.postRender(x, y, partialTicks);
        }
        GL11.glEnable(GL11.GL_TEXTURE_2D);
        GL11.glColor4d(1, 1, 1, 1);

        List<String> tooltip = new ArrayList<String>();
        for(Object obj : field_146292_n) {
            if(obj instanceof GuiButtonSpecial) {
                GuiButtonSpecial button = (GuiButtonSpecial)obj;
                if(button.field_146128_h < x && button.field_146128_h + button.getWidth() > x && button.field_146129_i < y && button.field_146129_i + button.getHeight() > y) {
                    button.getTooltip(tooltip);
                }
            }
        }
        boolean shift = PneumaticCraft.proxy.isSneakingInGui();
        for(IGuiWidget widget : widgets) {
            if(widget.getBounds().contains(x, y)) widget.addTooltip(x, y, tooltip, shift);
        }
        if(!tooltip.isEmpty()) {
            List<String> localizedTooltip = new ArrayList<String>();
            for(String line : tooltip) {
                String localizedLine = I18n.func_135052_a(line);
                for(String wrappedLine : localizedLine.split("\\\\n")) {
                    String[] lines = WordUtils.wrap(wrappedLine, 50).split(System.getProperty("line.separator"));
                    for(String locLine : lines) {
                        localizedTooltip.add(locLine);
                    }
                }
            }
            drawHoveringText(localizedTooltip, x, y, field_146289_q);
        }
    }

    @Override
    protected void func_73864_a(int par1, int par2, int par3) throws IOException{
        super.func_73864_a(par1, par2, par3);
        for(IGuiWidget widget : widgets) {
            if(widget.getBounds().contains(par1, par2)) widget.onMouseClicked(par1, par2, par3);
            else widget.onMouseClickedOutsideBounds(par1, par2, par3);
        }
    }

    @Override
    protected void func_73869_a(char key, int keyCode) throws IOException{
        if(keyCode == 1) {
            super.func_73869_a(key, keyCode);
        } else {
            for(IGuiWidget widget : widgets) {
                widget.onKey(key, keyCode);
            }
        }
    }

    @Override
    public void actionPerformed(IGuiWidget widget){
        if(widget instanceof IGuiAnimatedStat) {
            boolean leftSided = ((IGuiAnimatedStat)widget).isLeftSided();
            for(IGuiWidget w : widgets) {
                if(w instanceof IGuiAnimatedStat) {
                    IGuiAnimatedStat stat = (IGuiAnimatedStat)w;
                    if(widget != stat && stat.isLeftSided() == leftSided) {//when the stat is on the same side, close it.
                        stat.closeWindow();
                    }
                }
            }
        }
    }

    @Override
    public void func_73876_c(){
        super.func_73876_c();
        for(IGuiWidget widget : widgets) {
            widget.update();
        }
    }

    @Override
    public void func_146274_d() throws IOException{
        super.func_146274_d();
        for(IGuiWidget widget : widgets) {
            widget.handleMouseInput();
        }
    }

    @Override
    public void onKeyTyped(IGuiWidget widget){}

    @Override
    public void func_146280_a(Minecraft par1Minecraft, int par2, int par3){
        widgets.clear();
        super.func_146280_a(par1Minecraft, par2, par3);
    }

    public GuiButton getButtonFromRectangle(int buttonID, Rectangle buttonSize, String buttonText){
        return new GuiButton(buttonID, buttonSize.x, buttonSize.y, buttonSize.width, buttonSize.height, buttonText);
    }

    public GuiButtonSpecial getInvisibleButtonFromRectangle(int buttonID, Rectangle buttonSize){
        return new GuiButtonSpecial(buttonID, buttonSize.x, buttonSize.y, buttonSize.width, buttonSize.height, "");
    }

    public GuiTextField getTextFieldFromRectangle(Rectangle textFieldSize){
        return new GuiTextField(-1, field_146289_q, textFieldSize.x, textFieldSize.y, textFieldSize.width, textFieldSize.height);
    }

}
