package pneumaticCraft.client.gui;

import igwmod.gui.GuiWiki;

import java.awt.Point;
import java.awt.Rectangle;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.GuiScreen;
import net.minecraft.client.resources.I18n;
import net.minecraft.client.settings.GameSettings;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.StatCollector;
import net.minecraftforge.fml.client.FMLClientHandler;
import net.minecraftforge.fml.common.Loader;
import net.minecraftforge.fml.common.Optional;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.GL11;

import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.client.gui.widget.GuiCheckBox;
import pneumaticCraft.client.gui.widget.GuiRadioButton;
import pneumaticCraft.client.gui.widget.IGuiWidget;
import pneumaticCraft.client.gui.widget.WidgetTextField;
import pneumaticCraft.common.config.Config;
import pneumaticCraft.common.inventory.ContainerProgrammer;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketGuiButton;
import pneumaticCraft.common.network.PacketProgrammerUpdate;
import pneumaticCraft.common.network.PacketUpdateTextfield;
import pneumaticCraft.common.progwidgets.IProgWidget;
import pneumaticCraft.common.progwidgets.ProgWidgetArea;
import pneumaticCraft.common.progwidgets.ProgWidgetCoordinate;
import pneumaticCraft.common.progwidgets.ProgWidgetCoordinateOperator;
import pneumaticCraft.common.progwidgets.ProgWidgetStart;
import pneumaticCraft.common.progwidgets.WidgetRegistrator;
import pneumaticCraft.common.tileentity.TileEntityProgrammer;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.ModIds;
import pneumaticCraft.lib.Textures;

@SideOnly(Side.CLIENT)
public class GuiProgrammer extends GuiPneumaticContainerBase<TileEntityProgrammer>{
    private final EntityPlayer player;
    private GuiPastebin pastebinGui;

    //  private GuiButton redstoneButton;
    private GuiButtonSpecial importButton;
    private GuiButtonSpecial exportButton;
    private GuiButtonSpecial allWidgetsButton;
    private List<GuiRadioButton> difficultyButtons;
    private GuiCheckBox showInfo, showFlow;
    private WidgetTextField nameField;
    private GuiButtonSpecial undoButton, redoButton;
    private GuiButtonSpecial convertToRelativeButton;

    private final List<IProgWidget> visibleSpawnWidgets = new ArrayList<IProgWidget>();

    private GuiUnitProgrammer programmerUnit;
    private boolean wasClicking;
    private boolean wasFocused;
    private IProgWidget draggingWidget;
    private int lastMouseX, lastMouseY;
    private int dragMouseStartX, dragMouseStartY;
    private int dragWidgetStartX, dragWidgetStartY;
    private static final int FAULT_MARGIN = 4;
    private int widgetPage;
    private int maxPage;

    private boolean showingAllWidgets;
    private int showingWidgetProgress;
    private int oldShowingWidgetProgress;
    private static final int PROGRAMMING_START_Y = 17;
    private static final int PROGRAMMING_START_X = 5;
    private static final int PROGRAMMING_WIDTH = 294;
    private static final int PROGRAMMING_HEIGHT = 154;

    public GuiProgrammer(InventoryPlayer player, TileEntityProgrammer te){

        super(new ContainerProgrammer(player, te), te, Textures.GUI_PROGRAMMER);
        field_147000_g = 256;
        field_146999_f = 350;

        this.player = FMLClientHandler.instance().getClient().field_71439_g;
    }

    private void updateVisibleProgWidgets(){
        int y = 0, page = 0;
        int xSpacing = 22;
        int x = 322 - maxPage * xSpacing;
        boolean showAllWidgets = showingWidgetProgress == xSpacing * maxPage && showingAllWidgets;
        maxPage = 0;
        visibleSpawnWidgets.clear();
        int difficulty = 0;
        for(int i = 0; i < difficultyButtons.size(); i++) {
            if(difficultyButtons.get(i).checked) {
                difficulty = i;
                break;
            }
        }
        for(IProgWidget widget : WidgetRegistrator.registeredWidgets) {
            if(difficulty >= widget.getDifficulty().ordinal()) {
                widget.setY(y + 40);
                widget.setX(showAllWidgets ? x : 322);
                int widgetHeight = widget.getHeight() / 2 + (widget.hasStepOutput() ? 5 : 0) + 1;
                y += widgetHeight;

                if(showAllWidgets || page == widgetPage) visibleSpawnWidgets.add(widget);
                if(y > field_147000_g - 160) {
                    y = 0;
                    x += xSpacing;
                    page++;
                    maxPage++;
                }

            }
        }
        if(widgetPage > maxPage) {
            widgetPage = maxPage;
            updateVisibleProgWidgets();
        }
    }

    @Override
    protected boolean shouldAddInfoTab(){
        return false;
    }

    @Override
    public void func_73866_w_(){
        if(pastebinGui != null && pastebinGui.outputTag != null) {
            te.readProgWidgetsFromNBT(pastebinGui.outputTag);
            pastebinGui = null;
            NetworkHandler.sendToServer(new PacketProgrammerUpdate(te));
        }

        super.func_73866_w_();

        if(programmerUnit != null) {
            te.translatedX = programmerUnit.getTranslatedX();
            te.translatedY = programmerUnit.getTranslatedY();
            te.zoomState = programmerUnit.getLastZoom();
        }

        programmerUnit = new GuiUnitProgrammer(te.progWidgets, field_146289_q, field_147003_i, field_147009_r, field_146999_f, field_146294_l, field_146295_m, PROGRAMMING_START_X, PROGRAMMING_START_Y, PROGRAMMING_WIDTH, PROGRAMMING_HEIGHT, te.translatedX, te.translatedY, te.zoomState);
        addWidget(programmerUnit.getScrollBar());

        int xStart = (field_146294_l - field_146999_f) / 2;
        int yStart = (field_146295_m - field_147000_g) / 2;

        //    addProgWidgetTabs(xStart, yStart);

        importButton = new GuiButtonSpecial(1, xStart + 301, yStart + 3, 20, 15, "<--");
        importButton.setTooltipText("Import program");
        field_146292_n.add(importButton);

        exportButton = new GuiButtonSpecial(2, xStart + 301, yStart + 20, 20, 15, "-->");
        field_146292_n.add(exportButton);

        field_146292_n.add(new GuiButton(3, xStart + 294, yStart + 174, 10, 10, "-"));
        field_146292_n.add(new GuiButton(4, xStart + 335, yStart + 174, 10, 10, "+"));

        allWidgetsButton = new GuiButtonSpecial(8, xStart + 321, yStart + 159, 10, 10, "<");
        allWidgetsButton.setTooltipText(I18n.func_135052_a("gui.programmer.button.openPanel.tooltip"));
        addWidget(allWidgetsButton);

        difficultyButtons = new ArrayList<GuiRadioButton>();
        for(int i = 0; i < IProgWidget.WidgetDifficulty.values().length; i++) {
            GuiRadioButton radioButton = new GuiRadioButton(i, xStart + 263, yStart + 200 + i * 12, 0xFF000000, IProgWidget.WidgetDifficulty.values()[i].getLocalizedName());
            radioButton.checked = Config.getProgrammerDifficulty() == i;
            addWidget(radioButton);
            difficultyButtons.add(radioButton);
            radioButton.otherChoices = difficultyButtons;
            if(i == 1) radioButton.setTooltip(I18n.func_135052_a("gui.programmer.difficulty.medium.tooltip"));
            if(i == 2) radioButton.setTooltip(I18n.func_135052_a("gui.programmer.difficulty.advanced.tooltip"));
        }

        field_146292_n.add(new GuiButton(5, xStart + 5, yStart + 175, 87, 20, I18n.func_135052_a("gui.programmer.button.showStart")));
        field_146292_n.add(new GuiButton(6, xStart + 5, yStart + 197, 87, 20, I18n.func_135052_a("gui.programmer.button.showLatest")));
        addWidget(showInfo = new GuiCheckBox(-1, xStart + 5, yStart + 220, 0xFF000000, "gui.programmer.checkbox.showInfo").setChecked(te.showInfo));
        addWidget(showFlow = new GuiCheckBox(-1, xStart + 5, yStart + 232, 0xFF000000, "gui.programmer.checkbox.showFlow").setChecked(te.showFlow));

        GuiButtonSpecial pastebinButton = new GuiButtonSpecial(7, field_147003_i - 24, field_147009_r + 44, 20, 20, "");
        pastebinButton.setTooltipText(I18n.func_135052_a("gui.remote.button.pastebinButton"));
        pastebinButton.setRenderedIcon(Textures.GUI_PASTEBIN_ICON_LOCATION);
        field_146292_n.add(pastebinButton);

        undoButton = new GuiButtonSpecial(9, field_147003_i - 24, field_147009_r + 2, 20, 20, "");
        redoButton = new GuiButtonSpecial(10, field_147003_i - 24, field_147009_r + 23, 20, 20, "");
        GuiButtonSpecial clearAllButton = new GuiButtonSpecial(11, field_147003_i - 24, field_147009_r + 65, 20, 20, "");
        convertToRelativeButton = new GuiButtonSpecial(12, field_147003_i - 24, field_147009_r + 86, 20, 20, "Rel");

        undoButton.setRenderedIcon(Textures.GUI_UNDO_ICON_LOCATION);
        redoButton.setRenderedIcon(Textures.GUI_REDO_ICON_LOCATION);
        clearAllButton.setRenderedIcon(Textures.GUI_DELETE_ICON_LOCATION);

        undoButton.setTooltipText(I18n.func_135052_a("gui.programmer.button.undoButton.tooltip"));
        redoButton.setTooltipText(I18n.func_135052_a("gui.programmer.button.redoButton.tooltip"));
        clearAllButton.setTooltipText(I18n.func_135052_a("gui.programmer.button.clearAllButton.tooltip"));

        field_146292_n.add(undoButton);
        field_146292_n.add(redoButton);
        field_146292_n.add(clearAllButton);
        field_146292_n.add(convertToRelativeButton);

        String containerName = te.func_145818_k_() ? te.getName() : StatCollector.func_74838_a(te.getName() + ".name");
        addLabel(containerName, field_147003_i + 7, field_147009_r + 5);

        nameField = new WidgetTextField(field_146289_q, field_147003_i + 200, field_147009_r + 5, 98, field_146289_q.field_78288_b);
        addWidget(nameField);

        String name = I18n.func_135052_a("gui.programmer.name");
        addLabel(name, field_147003_i + 197 - field_146289_q.func_78256_a(name), field_147009_r + 5);

        updateVisibleProgWidgets();
    }

    @Override
    protected Point getInvNameOffset(){
        return null;
    }

    @Override
    protected Point getInvTextOffset(){
        return null;
    }

    @Override
    protected boolean shouldAddProblemTab(){
        return false;
    }

    @Override
    protected void func_146979_b(int x, int y){
        super.func_146979_b(x, y);

        boolean igwLoaded = Loader.isModLoaded(ModIds.IGWMOD);
        field_146289_q.func_78276_b(widgetPage + 1 + "/" + (maxPage + 1), 305, 175, 0xFF000000);
        field_146289_q.func_78276_b(I18n.func_135052_a("gui.programmer.difficulty"), 263, 190, 0xFF000000);

        if(showingWidgetProgress == 0) {
            programmerUnit.renderForeground(x, y, draggingWidget);
        }

        for(IProgWidget widget : visibleSpawnWidgets) {
            if(widget != draggingWidget && x - field_147003_i >= widget.getX() && y - field_147009_r >= widget.getY() && x - field_147003_i <= widget.getX() + widget.getWidth() / 2 && y - field_147009_r <= widget.getY() + widget.getHeight() / 2) {
                List<String> tooltip = new ArrayList<String>();
                widget.getTooltip(tooltip);
                if(igwLoaded) tooltip.add(I18n.func_135052_a("gui.programmer.pressIForInfo"));
                if(tooltip.size() > 0) drawHoveringString(tooltip, x - field_147003_i, y - field_147009_r, field_146289_q);
            }
        }

    }

    @Override
    protected void func_73869_a(char key, int keyCode) throws IOException{
        super.func_73869_a(key, keyCode);

        if(Keyboard.KEY_I == keyCode && Loader.isModLoaded(ModIds.IGWMOD)) {
            onIGWAction();
        }
        if(Keyboard.KEY_R == keyCode) {
            if(exportButton.getBounds().contains(lastMouseX, lastMouseY)) {
                NetworkHandler.sendToServer(new PacketGuiButton(0));
            }
        }
        if(Keyboard.KEY_SPACE == keyCode) {
            toggleShowWidgets();
        }
        if(Keyboard.KEY_DELETE == keyCode) {
            IProgWidget widget = programmerUnit.getHoveredWidget(lastMouseX, lastMouseY);
            if(widget != null) {
                te.progWidgets.remove(widget);
                NetworkHandler.sendToServer(new PacketProgrammerUpdate(te));
            }
        }
        if(Keyboard.KEY_Z == keyCode) {
            NetworkHandler.sendToServer(new PacketGuiButton(undoButton.field_146127_k));
        }
        if(Keyboard.KEY_Y == keyCode) {
            NetworkHandler.sendToServer(new PacketGuiButton(redoButton.field_146127_k));
        }
    }

    @Optional.Method(modid = ModIds.IGWMOD)
    private void onIGWAction(){
        int x = lastMouseX;
        int y = lastMouseY;

        IProgWidget hoveredWidget = programmerUnit.getHoveredWidget(x, y);
        if(hoveredWidget != null) {
            GuiWiki gui = new GuiWiki();
            FMLClientHandler.instance().showGuiScreen(gui);
            gui.setCurrentFile("pneumaticcraft:progwidget/" + hoveredWidget.getWidgetString());
        }

        for(IProgWidget widget : visibleSpawnWidgets) {
            if(widget != draggingWidget && x - field_147003_i >= widget.getX() && y - field_147009_r >= widget.getY() && x - field_147003_i <= widget.getX() + widget.getWidth() / 2 && y - field_147009_r <= widget.getY() + widget.getHeight() / 2) {
                GuiWiki gui = new GuiWiki();
                FMLClientHandler.instance().showGuiScreen(gui);
                gui.setCurrentFile("pneumaticcraft:progwidget/" + widget.getWidgetString());
            }
        }
    }

    @Override
    protected boolean shouldDrawBackground(){
        return false;
    }

    @Override
    protected void func_146976_a(float partialTicks, int x, int y){
        GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
        bindGuiTexture();
        int xStart = (field_146294_l - field_146999_f) / 2;
        int yStart = (field_146295_m - field_147000_g) / 2;
        func_146110_a(xStart, yStart, 0, 0, field_146999_f, field_147000_g, field_146999_f, field_147000_g);

        programmerUnit.getScrollBar().setEnabled(showingWidgetProgress == 0);
        super.func_146976_a(partialTicks, x, y);
        if(showingWidgetProgress > 0) programmerUnit.getScrollBar().setCurrentState(programmerUnit.getLastZoom());

        programmerUnit.render(x, y, showFlow.checked, showInfo.checked && showingWidgetProgress == 0, draggingWidget == null);

        int origX = x;
        int origY = y;
        x -= programmerUnit.getTranslatedX();
        y -= programmerUnit.getTranslatedY();
        float scale = programmerUnit.getScale();
        x = (int)(x / scale);
        y = (int)(y / scale);

        if(showingWidgetProgress > 0) {
            GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
            bindGuiTexture();
            int width = oldShowingWidgetProgress + (int)((showingWidgetProgress - oldShowingWidgetProgress) * partialTicks);
            for(int i = 0; i < width; i++) {
                func_146110_a(xStart + 320 - i, yStart + 36, 323, 36, 1, 136, field_146999_f, field_147000_g);
            }
            func_146110_a(xStart + 319 - width, yStart + 36, 319, 36, 2, 136, field_146999_f, field_147000_g);

            if(showingAllWidgets && draggingWidget != null) toggleShowWidgets();
        }
        GL11.glEnable(GL11.GL_TEXTURE_2D);
        for(IProgWidget widget : visibleSpawnWidgets) {
            GL11.glPushMatrix();
            GL11.glTranslated(widget.getX() + field_147003_i, widget.getY() + field_147009_r, 0);
            GL11.glScaled(0.5, 0.5, 1);
            widget.render();
            GL11.glPopMatrix();
        }

        GL11.glPushMatrix();
        GL11.glTranslated(programmerUnit.getTranslatedX(), programmerUnit.getTranslatedY(), 0);
        GL11.glScaled(scale, scale, 1);
        if(draggingWidget != null) {
            GL11.glPushMatrix();
            GL11.glTranslated(draggingWidget.getX() + field_147003_i, draggingWidget.getY() + field_147009_r, 0);
            GL11.glScaled(0.5, 0.5, 1);
            draggingWidget.render();
            GL11.glPopMatrix();
        }
        GL11.glPopMatrix();

        boolean isLeftClicking = Mouse.isButtonDown(0);
        boolean isMiddleClicking = GameSettings.func_100015_a(Minecraft.func_71410_x().field_71474_y.field_74322_I);

        if(draggingWidget != null) {
            setConnectingWidgetsToXY(draggingWidget, x - dragMouseStartX + dragWidgetStartX - field_147003_i, y - dragMouseStartY + dragWidgetStartY - field_147009_r);
        }

        if(isLeftClicking && !wasClicking) {
            for(IProgWidget widget : visibleSpawnWidgets) {
                if(origX >= widget.getX() + field_147003_i && origY >= widget.getY() + field_147009_r && origX <= widget.getX() + field_147003_i + widget.getWidth() / 2 && origY <= widget.getY() + field_147009_r + widget.getHeight() / 2) {
                    draggingWidget = widget.copy();
                    te.progWidgets.add(draggingWidget);
                    dragMouseStartX = x - (int)(field_147003_i / scale);
                    dragMouseStartY = y - (int)(field_147009_r / scale);
                    dragWidgetStartX = (int)((widget.getX() - programmerUnit.getTranslatedX()) / scale);
                    dragWidgetStartY = (int)((widget.getY() - programmerUnit.getTranslatedY()) / scale);
                    break;
                }
            }
            if(draggingWidget == null && showingWidgetProgress == 0) {
                IProgWidget widget = programmerUnit.getHoveredWidget(origX, origY);
                if(widget != null) {
                    draggingWidget = widget;
                    dragMouseStartX = x - field_147003_i;
                    dragMouseStartY = y - field_147009_r;
                    dragWidgetStartX = widget.getX();
                    dragWidgetStartY = widget.getY();
                }
            }
        } else if(isMiddleClicking && !wasClicking && showingWidgetProgress == 0) {
            IProgWidget widget = programmerUnit.getHoveredWidget(origX, origY);
            if(widget != null) {
                draggingWidget = widget.copy();
                te.progWidgets.add(draggingWidget);
                dragMouseStartX = 0;
                dragMouseStartY = 0;
                dragWidgetStartX = widget.getX() - (x - field_147003_i);
                dragWidgetStartY = widget.getY() - (y - field_147009_r);
                if(PneumaticCraft.proxy.isSneakingInGui()) copyAndConnectConnectingWidgets(widget, draggingWidget);
            }
        }

        if(!isLeftClicking && !isMiddleClicking && draggingWidget != null) {
            if(programmerUnit.isOutsideProgrammingArea(draggingWidget)) {
                deleteConnectingWidgets(draggingWidget);
            } else {
                handlePuzzleMargins();
                if(!isValidPlaced(draggingWidget)) {
                    setConnectingWidgetsToXY(draggingWidget, dragWidgetStartX, dragWidgetStartY);
                    if(programmerUnit.isOutsideProgrammingArea(draggingWidget)) deleteConnectingWidgets(draggingWidget);
                }
            }
            NetworkHandler.sendToServer(new PacketProgrammerUpdate(te));
            TileEntityProgrammer.updatePuzzleConnections(te.progWidgets);

            draggingWidget = null;
        }
        wasClicking = isLeftClicking || isMiddleClicking;
        lastMouseX = origX;
        lastMouseY = origY;
    }

    private boolean isValidPlaced(IProgWidget widget1){
        Rectangle draggingRect = new Rectangle(widget1.getX(), widget1.getY(), widget1.getWidth() / 2, widget1.getHeight() / 2);
        for(IProgWidget widget : te.progWidgets) {
            if(widget != widget1) {
                if(draggingRect.intersects(widget.getX(), widget.getY(), widget.getWidth() / 2, widget.getHeight() / 2)) {
                    return false;
                }
            }
        }
        IProgWidget[] parameters = widget1.getConnectedParameters();
        if(parameters != null) {
            for(IProgWidget widget : parameters) {
                if(widget != null && !isValidPlaced(widget)) return false;
            }
        }
        IProgWidget outputWidget = widget1.getOutputWidget();
        if(outputWidget != null && !isValidPlaced(outputWidget)) return false;
        return true;
    }

    private void handlePuzzleMargins(){
        //Check for connection to the left of the dragged widget.
        Class<? extends IProgWidget> returnValue = draggingWidget.returnType();
        if(returnValue != null) {
            for(IProgWidget widget : te.progWidgets) {
                if(widget != draggingWidget && Math.abs(widget.getX() + widget.getWidth() / 2 - draggingWidget.getX()) <= FAULT_MARGIN) {
                    Class<? extends IProgWidget>[] parameters = widget.getParameters();
                    if(parameters != null) {
                        for(int i = 0; i < parameters.length; i++) {
                            if(widget.canSetParameter(i) && parameters[i] == returnValue && Math.abs(widget.getY() + i * 11 - draggingWidget.getY()) <= FAULT_MARGIN) {
                                setConnectingWidgetsToXY(draggingWidget, widget.getX() + widget.getWidth() / 2, widget.getY() + i * 11);
                                return;
                            }
                        }
                    }
                }
            }
        }

        //check for connection to the right of the dragged widget.
        Class<? extends IProgWidget>[] parameters = draggingWidget.getParameters();
        if(parameters != null) {
            for(IProgWidget widget : te.progWidgets) {
                IProgWidget outerPiece = draggingWidget;
                if(outerPiece.returnType() != null) {//When the piece is a parameter pice (area, item filter, text).
                    while(outerPiece.getConnectedParameters()[0] != null) {
                        outerPiece = outerPiece.getConnectedParameters()[0];
                    }
                }
                if(widget != draggingWidget && Math.abs(outerPiece.getX() + outerPiece.getWidth() / 2 - widget.getX()) <= FAULT_MARGIN) {
                    if(widget.returnType() != null) {
                        for(int i = 0; i < parameters.length; i++) {
                            if(draggingWidget.canSetParameter(i) && parameters[i] == widget.returnType() && Math.abs(draggingWidget.getY() + i * 11 - widget.getY()) <= FAULT_MARGIN) {
                                setConnectingWidgetsToXY(draggingWidget, widget.getX() - draggingWidget.getWidth() / 2 - (outerPiece.getX() - draggingWidget.getX()), widget.getY() - i * 11);
                            }
                        }
                    } else {
                        Class<? extends IProgWidget>[] checkingPieceParms = widget.getParameters();
                        if(checkingPieceParms != null) {
                            for(int i = 0; i < checkingPieceParms.length; i++) {
                                if(widget.canSetParameter(i + parameters.length) && checkingPieceParms[i] == parameters[0] && Math.abs(widget.getY() + i * 11 - draggingWidget.getY()) <= FAULT_MARGIN) {
                                    setConnectingWidgetsToXY(draggingWidget, widget.getX() - draggingWidget.getWidth() / 2 - (outerPiece.getX() - draggingWidget.getX()), widget.getY() + i * 11);
                                }
                            }
                        }
                    }
                }
            }
        }

        //check for connection to the top of the dragged widget.
        if(draggingWidget.hasStepInput()) {
            for(IProgWidget widget : te.progWidgets) {
                if(widget.hasStepOutput() && Math.abs(widget.getX() - draggingWidget.getX()) <= FAULT_MARGIN && Math.abs(widget.getY() + widget.getHeight() / 2 - draggingWidget.getY()) <= FAULT_MARGIN) {
                    setConnectingWidgetsToXY(draggingWidget, widget.getX(), widget.getY() + widget.getHeight() / 2);
                }
            }
        }

        //check for connection to the bottom of the dragged widget.
        if(draggingWidget.hasStepOutput()) {
            for(IProgWidget widget : te.progWidgets) {
                if(widget.hasStepInput() && Math.abs(widget.getX() - draggingWidget.getX()) <= FAULT_MARGIN && Math.abs(widget.getY() - draggingWidget.getY() - draggingWidget.getHeight() / 2) <= FAULT_MARGIN) {
                    setConnectingWidgetsToXY(draggingWidget, widget.getX(), widget.getY() - draggingWidget.getHeight() / 2);
                }
            }
        }
    }

    private void setConnectingWidgetsToXY(IProgWidget widget, int x, int y){
        widget.setX(x);
        widget.setY(y);
        IProgWidget[] connectingWidgets = widget.getConnectedParameters();
        if(connectingWidgets != null) {
            for(int i = 0; i < connectingWidgets.length; i++) {
                if(connectingWidgets[i] != null) {
                    if(i < connectingWidgets.length / 2) {
                        setConnectingWidgetsToXY(connectingWidgets[i], x + widget.getWidth() / 2, y + i * 11);
                    } else {
                        int totalWidth = 0;
                        IProgWidget branch = connectingWidgets[i];
                        while(branch != null) {
                            totalWidth += branch.getWidth() / 2;
                            branch = branch.getConnectedParameters()[0];
                        }
                        setConnectingWidgetsToXY(connectingWidgets[i], x - totalWidth, y + (i - connectingWidgets.length / 2) * 11);
                    }
                }
            }
        }
        IProgWidget outputWidget = widget.getOutputWidget();
        if(outputWidget != null) setConnectingWidgetsToXY(outputWidget, x, y + widget.getHeight() / 2);
    }

    private void copyAndConnectConnectingWidgets(IProgWidget original, IProgWidget copy){
        IProgWidget[] connectingWidgets = original.getConnectedParameters();
        if(connectingWidgets != null) {
            for(int i = 0; i < connectingWidgets.length; i++) {
                if(connectingWidgets[i] != null) {
                    IProgWidget c = connectingWidgets[i].copy();
                    te.progWidgets.add(c);
                    copy.setParameter(i, c);
                    copyAndConnectConnectingWidgets(connectingWidgets[i], c);
                }
            }
        }
        IProgWidget outputWidget = original.getOutputWidget();
        if(outputWidget != null) {
            IProgWidget c = outputWidget.copy();
            te.progWidgets.add(c);
            copy.setOutputWidget(c);
            copyAndConnectConnectingWidgets(outputWidget, c);
        }
    }

    private void deleteConnectingWidgets(IProgWidget widget){
        te.progWidgets.remove(widget);
        IProgWidget[] connectingWidgets = widget.getConnectedParameters();
        if(connectingWidgets != null) {
            for(IProgWidget widg : connectingWidgets) {
                if(widg != null) deleteConnectingWidgets(widg);
            }
        }
        IProgWidget outputWidget = widget.getOutputWidget();
        if(outputWidget != null) deleteConnectingWidgets(outputWidget);
    }

    /**
     * Fired when a control is clicked. This is the equivalent of
     * ActionListener.actionPerformed(ActionEvent e).
     */
    @Override
    protected void func_146284_a(GuiButton button){
        switch(button.field_146127_k){
            case 0:// redstone button
                   //          redstoneBehaviourStat.closeWindow();
                break;
            case 3:
                if(--widgetPage < 0) widgetPage = maxPage;
                updateVisibleProgWidgets();
                return;
            case 4:
                if(++widgetPage > maxPage) widgetPage = 0;
                updateVisibleProgWidgets();
                return;
            case 5:
                for(IProgWidget widget : te.progWidgets) {
                    if(widget instanceof ProgWidgetStart) {
                        programmerUnit.gotoPiece(widget);
                        break;
                    }
                }
                return;
            case 6:
                if(te.progWidgets.size() > 0) {
                    programmerUnit.gotoPiece(te.progWidgets.get(te.progWidgets.size() - 1));
                }
                return;
            case 7:
                NBTTagCompound mainTag = new NBTTagCompound();
                te.writeProgWidgetsToNBT(mainTag);
                FMLClientHandler.instance().showGuiScreen(pastebinGui = new GuiPastebin(this, mainTag));
                break;
            case 11:
                te.progWidgets.clear();
                NetworkHandler.sendToServer(new PacketProgrammerUpdate(te));
                break;
            case 12:
                for(IProgWidget widget : te.progWidgets) {
                    if(widget instanceof ProgWidgetStart) {
                        generateRelativeOperators((ProgWidgetCoordinateOperator)widget.getOutputWidget(), null, false);
                        break;
                    }
                }
                break;
        }

        NetworkHandler.sendToServer(new PacketGuiButton(button.field_146127_k));
    }

    private void toggleShowWidgets(){
        showingAllWidgets = !showingAllWidgets;
        allWidgetsButton.field_146126_j = showingAllWidgets ? ">" : "<";
        updateVisibleProgWidgets();
    }

    @Override
    public void actionPerformed(IGuiWidget button){
        if(button == allWidgetsButton) {
            toggleShowWidgets();
        } else {
            for(int i = 0; i < difficultyButtons.size(); i++) {
                if(difficultyButtons.get(i).checked) {
                    Config.setProgrammerDifficulty(i);
                    break;
                }
            }
            if(showingAllWidgets) {
                toggleShowWidgets();
            }
            updateVisibleProgWidgets();
        }
    }

    @Override
    public void func_73876_c(){
        super.func_73876_c();

        undoButton.field_146124_l = te.canUndo;
        redoButton.field_146124_l = te.canRedo;

        updateConvertRelativeState();

        ItemStack programmedItem = te.func_70301_a(TileEntityProgrammer.PROGRAM_SLOT);
        oldShowingWidgetProgress = showingWidgetProgress;
        if(showingAllWidgets) {
            int maxProgress = maxPage * 22;
            if(showingWidgetProgress < maxProgress) {
                showingWidgetProgress += 30;
                if(showingWidgetProgress >= maxProgress) {
                    showingWidgetProgress = maxProgress;
                    updateVisibleProgWidgets();
                }
            }
        } else {
            showingWidgetProgress -= 30;
            if(showingWidgetProgress < 0) showingWidgetProgress = 0;
        }

        List<String> errors = new ArrayList<String>();
        List<String> warnings = new ArrayList<String>();
        for(IProgWidget w : te.progWidgets) {
            w.addErrors(errors, te.progWidgets);
            w.addWarnings(warnings, te.progWidgets);
        }

        boolean isDeviceInserted = programmedItem != null;
        importButton.field_146124_l = isDeviceInserted;
        exportButton.field_146124_l = isDeviceInserted && errors.size() == 0;

        List<String> exportButtonTooltip = new ArrayList<String>();
        exportButtonTooltip.add("Export program");
        exportButtonTooltip.add(I18n.func_135052_a("gui.programmer.button.export.programmingWhen", I18n.func_135052_a("gui.programmer.button.export." + (te.redstoneMode == 0 ? "pressingButton" : "onItemInsert"))));
        exportButtonTooltip.add(I18n.func_135052_a("gui.programmer.button.export.pressRToChange"));
        if(programmedItem != null) {
            List<ItemStack> requiredPieces = te.getRequiredPuzzleStacks();
            List<ItemStack> returnedPieces = te.getReturnedPuzzleStacks();
            if(!requiredPieces.isEmpty() || !returnedPieces.isEmpty()) exportButtonTooltip.add("");
            if(!requiredPieces.isEmpty()) {
                exportButtonTooltip.add("Required Programming Puzzles:");
                if(player.field_71075_bZ.field_75098_d) exportButtonTooltip.add("(Creative mode, so the following is free)");
                for(ItemStack stack : requiredPieces) {
                    String prefix;
                    if(te.hasEnoughPuzzleStacks(player, stack)) {
                        prefix = EnumChatFormatting.GREEN.toString();
                    } else {
                        prefix = EnumChatFormatting.RED.toString();
                        exportButton.field_146124_l = player.field_71075_bZ.field_75098_d && errors.size() == 0;
                    }
                    exportButtonTooltip.add(prefix + "-" + stack.field_77994_a + "x " + stack.func_82833_r());
                }
            }
            if(!returnedPieces.isEmpty()) {
                exportButtonTooltip.add("Returned Programming Puzzles:");
                if(player.field_71075_bZ.field_75098_d) exportButtonTooltip.add("(Creative mode, nothing's given)");
                for(ItemStack stack : returnedPieces) {
                    exportButtonTooltip.add("-" + stack.field_77994_a + "x " + stack.func_82833_r());
                }
            }
        } else {
            exportButtonTooltip.add("No programmable item inserted.");
        }

        if(errors.size() > 0) exportButtonTooltip.add(EnumChatFormatting.RED + I18n.func_135052_a("gui.programmer.errorCount", errors.size()));
        if(warnings.size() > 0) exportButtonTooltip.add(EnumChatFormatting.YELLOW + I18n.func_135052_a("gui.programmer.warningCount", warnings.size()));

        exportButton.setTooltipText(exportButtonTooltip);
        if(programmedItem != null) {
            nameField.func_146184_c(true);
            if(!nameField.func_146206_l()) {
                if(wasFocused) {
                    programmedItem.func_151001_c(nameField.func_146179_b());
                    NetworkHandler.sendToServer(new PacketUpdateTextfield(te, 0));
                }
                nameField.func_146180_a(programmedItem.func_82833_r());
                wasFocused = false;
            } else {
                wasFocused = true;
            }
        } else {
            nameField.func_146184_c(false);
            nameField.func_146180_a("");
            wasFocused = false;
        }
    }

    private void updateConvertRelativeState(){
        convertToRelativeButton.field_146124_l = false;
        List<String> tooltip = new ArrayList<String>();
        tooltip.add("gui.programmer.button.convertToRelative.desc");

        boolean startFound = false;
        for(IProgWidget startWidget : te.progWidgets) {
            if(startWidget instanceof ProgWidgetStart) {
                startFound = true;
                IProgWidget widget = startWidget.getOutputWidget();
                if(widget instanceof ProgWidgetCoordinateOperator) {
                    ProgWidgetCoordinateOperator operatorWidget = (ProgWidgetCoordinateOperator)widget;
                    if(!operatorWidget.getVariable().equals("")) {
                        try {
                            if(generateRelativeOperators(operatorWidget, tooltip, true)) {
                                convertToRelativeButton.field_146124_l = true;
                            } else {
                                tooltip.add("gui.programmer.button.convertToRelative.notEnoughRoom");
                            }
                        } catch(NullPointerException e) {
                            tooltip.add("gui.programmer.button.convertToRelative.cantHaveVariables");
                        }
                    } else {
                        tooltip.add("gui.programmer.button.convertToRelative.noVariableName");
                    }
                } else {
                    tooltip.add("gui.programmer.button.convertToRelative.noBaseCoordinate");
                }
            }
        }
        if(!startFound) tooltip.add("gui.programmer.button.convertToRelative.noStartPiece");

        List<String> localizedTooltip = new ArrayList<String>();
        for(String s : tooltip) {
            localizedTooltip.addAll(PneumaticCraftUtils.convertStringIntoList(I18n.func_135052_a(s), 40));
        }
        convertToRelativeButton.setTooltipText(localizedTooltip);
    }

    /**
     * 
     * @param baseWidget
     * @param simulate
     * @return true if successful
     */
    private boolean generateRelativeOperators(ProgWidgetCoordinateOperator baseWidget, List<String> tooltip, boolean simulate){
        BlockPos baseCoord = ProgWidgetCoordinateOperator.calculateCoordinate(baseWidget, 0, baseWidget.getOperator());
        Map<BlockPos, String> offsetToVariableNames = new HashMap<BlockPos, String>();
        for(IProgWidget widget : te.progWidgets) {
            if(widget instanceof ProgWidgetArea) {
                ProgWidgetArea area = (ProgWidgetArea)widget;
                if(area.getCoord1Variable().equals("") && (area.x1 != 0 || area.y1 != 0 || area.z1 != 0)) {
                    BlockPos offset = new BlockPos(area.x1 - baseCoord.func_177958_n(), area.y1 - baseCoord.func_177956_o(), area.z1 - baseCoord.func_177952_p());
                    String var = getOffsetVariable(offsetToVariableNames, baseWidget.getVariable(), offset);
                    if(!simulate) area.setCoord1Variable(var);
                }
                if(area.getCoord2Variable().equals("") && (area.x2 != 0 || area.y2 != 0 || area.z2 != 0)) {
                    BlockPos offset = new BlockPos(area.x2 - baseCoord.func_177958_n(), area.y2 - baseCoord.func_177956_o(), area.z2 - baseCoord.func_177952_p());
                    String var = getOffsetVariable(offsetToVariableNames, baseWidget.getVariable(), offset);
                    if(!simulate) area.setCoord2Variable(var);
                }
            } else if(widget instanceof ProgWidgetCoordinate && baseWidget.getConnectedParameters()[0] != widget) {
                ProgWidgetCoordinate coordinate = (ProgWidgetCoordinate)widget;
                if(!coordinate.isUsingVariable()) {
                    BlockPos c = coordinate.getCoordinate();
                    String chunkString = "(" + c.func_177958_n() + ", " + c.func_177956_o() + ", " + c.func_177952_p() + ")";
                    if(PneumaticCraftUtils.distBetween(c, 0, 0, 0) < 64) { //When the coordinate value is close to 0, there's a low chance it means a position, and rather an offset.
                        if(tooltip != null) tooltip.add(I18n.func_135052_a("gui.programmer.button.convertToRelative.coordIsNotChangedWarning", chunkString));
                    } else {
                        if(tooltip != null) tooltip.add(I18n.func_135052_a("gui.programmer.button.convertToRelative.coordIsChangedWarning", chunkString));
                        if(!simulate) {
                            BlockPos offset = new BlockPos(c.func_177958_n() - baseCoord.func_177958_n(), c.func_177956_o() - baseCoord.func_177956_o(), c.func_177952_p() - baseCoord.func_177952_p());
                            String var = getOffsetVariable(offsetToVariableNames, baseWidget.getVariable(), offset);
                            if(!simulate) {
                                coordinate.setVariable(var);
                                coordinate.setUsingVariable(true);
                            }
                        }
                    }
                }
            }
        }
        if(offsetToVariableNames.size() > 0) {
            ProgWidgetCoordinateOperator firstOperator = null;
            ProgWidgetCoordinateOperator prevOperator = baseWidget;
            int x = baseWidget.getX();
            for(Map.Entry<BlockPos, String> entry : offsetToVariableNames.entrySet()) {
                ProgWidgetCoordinateOperator operator = new ProgWidgetCoordinateOperator();
                operator.setVariable(entry.getValue());

                int y = prevOperator.getY() + prevOperator.getHeight() / 2;
                operator.setX(x);
                operator.setY(y);
                if(!isValidPlaced(operator)) return false;

                ProgWidgetCoordinate coordinatePiece1 = new ProgWidgetCoordinate();
                coordinatePiece1.setX(x + prevOperator.getWidth() / 2);
                coordinatePiece1.setY(y);
                coordinatePiece1.setVariable(baseWidget.getVariable());
                coordinatePiece1.setUsingVariable(true);
                if(!isValidPlaced(coordinatePiece1)) return false;

                ProgWidgetCoordinate coordinatePiece2 = new ProgWidgetCoordinate();
                coordinatePiece2.setX(x + prevOperator.getWidth() / 2 + coordinatePiece1.getWidth() / 2);
                coordinatePiece2.setY(y);
                coordinatePiece2.setCoordinate(entry.getKey());
                if(!isValidPlaced(coordinatePiece2)) return false;

                if(!simulate) {
                    te.progWidgets.add(operator);
                    te.progWidgets.add(coordinatePiece1);
                    te.progWidgets.add(coordinatePiece2);
                }
                if(firstOperator == null) firstOperator = operator;
                prevOperator = operator;
            }
            if(!simulate) {
                NetworkHandler.sendToServer(new PacketProgrammerUpdate(te));
                TileEntityProgrammer.updatePuzzleConnections(te.progWidgets);
            }
            return true;
        } else {
            return true; //When there's nothing to place there's always room.
        }
    }

    private String getOffsetVariable(Map<BlockPos, String> offsetToVariableNames, String baseVariable, BlockPos offset){
        if(offset.equals(new BlockPos(0, 0, 0))) return baseVariable;
        String var = offsetToVariableNames.get(offset);
        if(var == null) {
            var = "var" + (offsetToVariableNames.size() + 1);
            offsetToVariableNames.put(offset, var);
        }
        return var;
    }

    @Override
    protected void func_73864_a(int x, int y, int par3) throws IOException{
        ItemStack programmedItem = te.func_70301_a(TileEntityProgrammer.PROGRAM_SLOT);
        if(nameField.func_146206_l() && programmedItem != null) {
            programmedItem.func_151001_c(nameField.func_146179_b());
            NetworkHandler.sendToServer(new PacketUpdateTextfield(te, 0));
        }
        super.func_73864_a(x, y, par3);

        if(par3 == 1 && showingWidgetProgress == 0) {
            IProgWidget widget = programmerUnit.getHoveredWidget(x, y);
            if(widget != null) {
                GuiScreen screen = widget.getOptionWindow(this);
                if(screen != null) field_146297_k.func_147108_a(screen);
            }
        }
    }

    @Override
    public void func_146281_b(){
        te.translatedX = programmerUnit.getTranslatedX();
        te.translatedY = programmerUnit.getTranslatedY();
        te.zoomState = programmerUnit.getLastZoom();
        te.showFlow = showFlow.checked;
        te.showInfo = showInfo.checked;
        super.func_146281_b();
    }

    /* @Override
     * TODO NEI dep
     @Optional.Method(modid = ModIds.NEI)
     public VisiblityData modifyVisiblity(GuiContainer gui, VisiblityData currentVisibility){
         currentVisibility.showNEI = false;
         return currentVisibility;
     }*/

}
