package pneumaticCraft.client.gui;

import java.awt.Point;
import java.util.List;

import net.minecraft.client.resources.I18n;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.api.tileentity.IAirHandler;
import pneumaticCraft.common.inventory.ContainerVacuumPump;
import pneumaticCraft.common.tileentity.TileEntityVacuumPump;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.PneumaticValues;
import pneumaticCraft.lib.Textures;

@SideOnly(Side.CLIENT)
public class GuiVacuumPump extends GuiPneumaticContainerBase<TileEntityVacuumPump>{

    public GuiVacuumPump(InventoryPlayer player, TileEntityVacuumPump te){
        super(new ContainerVacuumPump(player, te), te, Textures.GUI_VACUUM_PUMP_LOCATION);
    }

    @Override
    protected void func_146979_b(int x, int y){
        super.func_146979_b(x, y);
        field_146289_q.func_78276_b("Upgr.", 76, 19, 4210752);

        field_146289_q.func_78276_b("+", 32, 47, 0xFF00AA00);
        field_146289_q.func_78276_b("-", 138, 47, 0xFFFF0000);
    }

    @Override
    protected void func_146976_a(float opacity, int x, int y){
        super.func_146976_a(opacity, x, y);

        int xStart = (field_146294_l - field_146999_f) / 2;
        int yStart = (field_146295_m - field_147000_g) / 2;
        GuiUtils.drawPressureGauge(field_146289_q, -1, PneumaticValues.MAX_PRESSURE_VACUUM_PUMP, PneumaticValues.DANGER_PRESSURE_VACUUM_PUMP, PneumaticValues.MIN_PRESSURE_VACUUM_PUMP, te.getAirHandler(te.getInputSide()).getPressure(), xStart + field_146999_f * 1 / 5, yStart + field_147000_g * 1 / 5 + 4, field_73735_i);
        GuiUtils.drawPressureGauge(field_146289_q, -1, PneumaticValues.MAX_PRESSURE_VACUUM_PUMP, PneumaticValues.DANGER_PRESSURE_VACUUM_PUMP, -1, te.getAirHandler(te.getVacuumSide()).getPressure(), xStart + field_146999_f * 4 / 5, yStart + field_147000_g * 1 / 5 + 4, field_73735_i);
    }

    @Override
    protected Point getGaugeLocation(){
        return null;
    }

    @Override
    protected void addPressureStatInfo(List<String> pressureStatText){
        IAirHandler inputHandler = te.getAirHandler(te.getInputSide());
        IAirHandler vacuumHandler = te.getAirHandler(te.getVacuumSide());
        pressureStatText.add("\u00a77Current Input Pressure:");
        pressureStatText.add("\u00a70" + PneumaticCraftUtils.roundNumberTo(inputHandler.getPressure(), 1) + " bar.");
        pressureStatText.add("\u00a77Current Input Air:");
        pressureStatText.add("\u00a70" + (inputHandler.getAir() + inputHandler.getVolume()) + " mL.");
        pressureStatText.add("\u00a77Current Vacuum Pressure:");
        pressureStatText.add("\u00a70" + PneumaticCraftUtils.roundNumberTo(vacuumHandler.getPressure(), 1) + " bar.");
        pressureStatText.add("\u00a77Current Vacuum Air:");
        pressureStatText.add("\u00a70" + (vacuumHandler.getAir() + vacuumHandler.getVolume()) + " mL.");
        pressureStatText.add("\u00a77Volume:");
        pressureStatText.add("\u00a70" + (double)Math.round(PneumaticValues.VOLUME_VACUUM_PUMP) + " mL.");
        int volumeLeft = inputHandler.getVolume() - PneumaticValues.VOLUME_VACUUM_PUMP;
        if(volumeLeft > 0) {
            pressureStatText.add("\u00a70" + volumeLeft + " mL. (Volume Upgrades)");
            pressureStatText.add("\u00a70--------+");
            pressureStatText.add("\u00a70" + inputHandler.getVolume() + " mL.");
        }

        if(te.turning) {
            pressureStatText.add("\u00a77Currently sucking at:");
            pressureStatText.add("\u00a70" + (double)Math.round(PneumaticValues.PRODUCTION_VACUUM_PUMP * te.getSpeedMultiplierFromUpgrades()) + " mL/tick.");
        }
    }

    @Override
    protected void addProblems(List<String> textList){
        super.addProblems(textList);
        if(te.getAirHandler(te.getInputSide()).getPressure() < PneumaticValues.MIN_PRESSURE_VACUUM_PUMP) {
            textList.add("gui.tab.problems.notEnoughPressure");
            textList.add(I18n.func_135052_a("gui.tab.problems.applyPressure", PneumaticValues.MIN_PRESSURE_VACUUM_PUMP));
        }
    }

}
