package pneumaticCraft.client.render;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.WorldRenderer;
import net.minecraft.client.renderer.texture.TextureManager;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.util.MathHelper;
import net.minecraftforge.fml.client.FMLClientHandler;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.lwjgl.opengl.GL11;

import pneumaticCraft.client.util.RenderUtils;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.Textures;

@SideOnly(Side.CLIENT)
public class RenderLaser{

    private int ticksExisted;
    private final int coreColor, glowColor;

    public RenderLaser(int color){
        this(color, color);
    }

    public RenderLaser(int coreColor, int glowColor){
        this.coreColor = coreColor;
        this.glowColor = glowColor;
    }

    public void update(){
        ticksExisted++;
    }

    public void render(float partialTicks, double x1, double y1, double z1, double x2, double y2, double z2){
        Minecraft mc = FMLClientHandler.instance().getClient();
        TextureManager textureManager = mc.field_71446_o;

        double laserLength = PneumaticCraftUtils.distBetween(x1, y1, z1, x2, y2, z2);
        double laserSize = 0.4;

        GL11.glDisable(GL11.GL_LIGHTING);
        GL11.glDisable(GL11.GL_CULL_FACE);
        GL11.glEnable(GL11.GL_BLEND);
        GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);

        GL11.glTranslated(x1, y1, z1);

        double dx = x2 - x1;
        double dy = y2 - y1;
        double dz = z2 - z1;
        float f3 = MathHelper.func_76133_a(dx * dx + dz * dz);
        double rotYaw = Math.atan2(dx, dz) * 180.0D / Math.PI;
        double rotPitch = 90 - (float)(Math.atan2(dy, f3) * 180.0D / Math.PI);

        GL11.glRotated(rotYaw, 0, 1, 0);
        GL11.glRotated(rotPitch, 1, 0, 0);

        GL11.glScaled(laserSize, laserSize, laserSize);
        GL11.glTranslated(0, 0.6, 0);
        GL11.glRotated((ticksExisted + partialTicks) * 200, 0, 1, 0);

        GL11.glPushMatrix();
        GL11.glScaled(1, laserLength / laserSize, 1);

        /*   GL11.glTranslated(0, -0.01, 0);
           textureManager.bindTexture(Textures.RENDER_LASER_ANIMATION);
           renderAnimation(partialTicks, laserLength / laserSize);
           GL11.glTranslated(0, 0.01, 0);*/

        textureManager.func_110577_a(Textures.RENDER_LASER);
        renderQuad(glowColor);
        textureManager.func_110577_a(Textures.RENDER_LASER_OVERLAY);
        renderQuad(coreColor);
        GL11.glPopMatrix();

        GL11.glRotated(180, 1, 0, 0);
        textureManager.func_110577_a(Textures.RENDER_LASER_START);
        renderQuad(glowColor);
        textureManager.func_110577_a(Textures.RENDER_LASER_START_OVERLAY);
        renderQuad(coreColor);

        GL11.glDisable(GL11.GL_BLEND);
        GL11.glEnable(GL11.GL_CULL_FACE);
        GL11.glEnable(GL11.GL_LIGHTING);

        GL11.glColor4d(1, 1, 1, 1);
    }

    private void renderQuad(int color){
        WorldRenderer wr = Tessellator.func_178181_a().func_178180_c();
        wr.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_181707_g);
        RenderUtils.glColorHex(color);
        wr.func_181662_b(-0.5, 0, 0).func_181673_a(0, 0).func_181675_d();
        wr.func_181662_b(-0.5, 1, 0).func_181673_a(0, 1).func_181675_d();
        wr.func_181662_b(0.5, 1, 0).func_181673_a(1, 1).func_181675_d();
        wr.func_181662_b(0.5, 0, 0).func_181673_a(1, 0).func_181675_d();
        Tessellator.func_178181_a().func_78381_a();
    }

    /*  private void renderAnimation(float partialTicks, double length){
          float p = (ticksExisted + partialTicks) % 100 / 100;
          WorldRenderer wr = Tessellator.getInstance().getWorldRenderer();
          wr.begin(GL11.GL_QUADS, DefaultVertexFormats.POSITION);
          t.addVertexWithUV(-0.5, 0, 0, 0, p);
          t.addVertexWithUV(-0.5, 1, 0, 0, length + p);
          t.addVertexWithUV(0.5, 1, 0, 1, length + p);
          t.addVertexWithUV(0.5, 0, 0, 1, p);
          Tessellator.getInstance().draw();
      }*/
}
