package pneumaticCraft.client.render.pneumaticArmor;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.monster.EntityMob;
import net.minecraft.entity.monster.EntitySlime;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;
import net.minecraftforge.client.event.MouseEvent;
import net.minecraftforge.fml.client.FMLClientHandler;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.lwjgl.opengl.GL11;

import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.api.client.pneumaticHelmet.IEntityTrackEntry;
import pneumaticCraft.api.client.pneumaticHelmet.IHackableEntity;
import pneumaticCraft.client.gui.widget.GuiAnimatedStat;
import pneumaticCraft.client.render.RenderProgressBar;
import pneumaticCraft.client.render.pneumaticArmor.entitytracker.EntityTrackHandler;
import pneumaticCraft.client.render.pneumaticArmor.hacking.HackableHandler;
import pneumaticCraft.common.NBTUtil;
import pneumaticCraft.common.entity.living.EntityDrone;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketHackingEntityStart;
import pneumaticCraft.common.network.PacketUpdateDebuggingDrone;
import pneumaticCraft.lib.NBTKeys;
import pneumaticCraft.lib.Sounds;

public class RenderTarget{

    public Entity entity;
    private final RenderTargetCircle circle1;
    private final RenderTargetCircle circle2;
    public int ticksExisted = 0;
    private float oldSize;
    @SideOnly(Side.CLIENT)
    private final GuiAnimatedStat stat;
    private boolean didMakeLockSound;
    public boolean isLookingAtTarget;
    private List<String> textList = new ArrayList<String>();
    private final List<IEntityTrackEntry> trackEntries;
    private int hackTime;

    public RenderTarget(Entity entity){
        this.entity = entity;
        trackEntries = EntityTrackHandler.getTrackersForEntity(entity);
        circle1 = new RenderTargetCircle();
        circle2 = new RenderTargetCircle();
        Item droppedItem = null;
        if(entity instanceof EntityLiving) {
            try {
                droppedItem = null;//TODO 1.8 EntityUtils.getLivingDrop((EntityLiving)entity);
            } catch(Throwable e) {}
        }
        if(droppedItem != null) {
            stat = new GuiAnimatedStat(null, entity.func_70005_c_(), new ItemStack(droppedItem, 1, 0), 20, -20, 0x3000AA00, null, false);
        } else {
            stat = new GuiAnimatedStat(null, entity.func_70005_c_(), "", 20, -20, 0x3000AA00, null, false);
        }
        stat.setMinDimensionsAndReset(0, 0);
    }

    public RenderDroneAI getDroneAIRenderer(){
        for(IEntityTrackEntry tracker : trackEntries) {
            if(tracker instanceof EntityTrackHandler.EntityTrackEntryDrone) {
                return ((EntityTrackHandler.EntityTrackEntryDrone)tracker).getDroneAIRenderer();
            }
        }
        throw new IllegalStateException("[RenderTarget] Drone entity, but no drone AI Renderer?");
    }

    public void update(){
        stat.update();
        stat.setTitle(entity.func_70005_c_());
        EntityPlayer player = FMLClientHandler.instance().getClient().field_71439_g;
        if(ticksExisted >= 30 && !didMakeLockSound) {
            didMakeLockSound = true;
            player.field_70170_p.func_72980_b(player.field_70165_t, player.field_70163_u, player.field_70161_v, Sounds.HUD_ENTITY_LOCK, 0.1F, 1.0F, true);
        }
        boolean tagged = NBTUtil.getInteger(player.func_82169_q(3), NBTKeys.PNEUMATIC_HELMET_DEBUGGING_DRONE) == entity.func_145782_y();
        circle1.setRenderingAsTagged(tagged);
        circle2.setRenderingAsTagged(tagged);
        circle1.update();
        circle2.update();
        for(IEntityTrackEntry tracker : trackEntries) {
            tracker.update(entity);
        }
        isLookingAtTarget = isPlayerLookingAtTarget();

        if(hackTime > 0) {
            IHackableEntity hackableEntity = HackableHandler.getHackableForEntity(entity, PneumaticCraft.proxy.getPlayer());
            if(hackableEntity != null) {
                hackTime++;// = Math.min(hackTime + 1, hackableEntity.getHackTime(entity, PneumaticCraft.proxy.getPlayer()));
            } else {
                hackTime = 0;
            }
        }
    }

    public boolean isInitialized(){
        return ticksExisted > 120;
    }

    public void render(float partialTicks, boolean justRenderWhenHovering){
        for(IEntityTrackEntry tracker : trackEntries) {
            tracker.render(entity, partialTicks);
        }
        double x = entity.field_70169_q + (entity.field_70165_t - entity.field_70169_q) * partialTicks;
        double y = entity.field_70167_r + (entity.field_70163_u - entity.field_70167_r) * partialTicks + entity.field_70131_O / 2D;
        double z = entity.field_70166_s + (entity.field_70161_v - entity.field_70166_s) * partialTicks;

        GL11.glDepthMask(false);
        GL11.glDisable(GL11.GL_DEPTH_TEST);
        GL11.glDisable(GL11.GL_CULL_FACE);
        GL11.glDisable(GL11.GL_TEXTURE_2D);

        GL11.glEnable(GL11.GL_BLEND);
        GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);

        GL11.glPushMatrix();

        GL11.glClear(GL11.GL_DEPTH_BUFFER_BIT);

        float red;
        float green;
        float blue;
        float alpha = 0.5F;
        if(entity instanceof EntityDrone) {
            red = 1;
            green = 1;
            blue = 0;
        } else if(entity instanceof EntityMob || entity instanceof EntitySlime) {
            red = 1;
            green = 0;
            blue = 0;
        } else {
            red = 0;
            green = 1;
            blue = 0;
        }

        float size = entity.field_70131_O * 0.5F;

        if(ticksExisted < 60) {
            size += 5 - Math.abs(ticksExisted) * 0.083F;
            alpha = Math.abs(ticksExisted) * 0.005F;
        }

        GL11.glTranslated(x, y, z);

        GL11.glRotatef(180.0F - Minecraft.func_71410_x().func_175598_ae().field_78735_i, 0.0F, 1.0F, 0.0F);
        GL11.glRotatef(180.0F - Minecraft.func_71410_x().func_175598_ae().field_78732_j, 1.0F, 0.0F, 0.0F);
        GL11.glColor4d(red, green, blue, alpha);
        float renderSize = oldSize + (size - oldSize) * partialTicks;
        circle1.render(renderSize, partialTicks);
        circle2.render(renderSize + 0.2D, partialTicks);
        int targetAcquireProgress = (int)((ticksExisted - 50) / 0.7F);
        if(ticksExisted <= 120 && ticksExisted > 50) {
            GL11.glColor4d(0, 1, 0, 0.8D);
            RenderProgressBar.render(0D, 0.4D, 1.8D, 0.9D, 0, targetAcquireProgress);
        }

        GL11.glEnable(GL11.GL_TEXTURE_2D);

        FontRenderer fontRenderer = Minecraft.func_71410_x().func_175598_ae().func_78716_a();
        GL11.glScaled(0.02D, 0.02D, 0.02D);
        GL11.glColor4d(red, green, blue, alpha);
        if(ticksExisted > 120) {
            if(justRenderWhenHovering && !isLookingAtTarget) {
                stat.closeWindow();
            } else {
                stat.openWindow();
            }
            textList = new ArrayList<String>();
            for(IEntityTrackEntry tracker : trackEntries) {
                tracker.addInfo(entity, textList);
            }
            stat.setText(textList);
            stat.render(-1, -1, partialTicks);
        } else if(ticksExisted > 50) {
            fontRenderer.func_78276_b("Acquiring Target...", 0, 0, 0x7F7F7F);
            fontRenderer.func_78276_b(targetAcquireProgress + "%", 37, 28, 0x002F00);
        } else if(ticksExisted < -30) {
            stat.closeWindow();

            //if(stat.getWidth() > stat.getMinWidth() || stat.getHeight() > stat.getMinHeight()) {
            //    stat.setText(new ArrayList<String>());
            stat.render(-1, -1, partialTicks);
            //            }
            fontRenderer.func_78276_b("Lost Target!", 0, 0, 0xFF0000);
        }

        GL11.glPopMatrix();
        GL11.glEnable(GL11.GL_CULL_FACE);
        GL11.glEnable(GL11.GL_DEPTH_TEST);
        GL11.glDisable(GL11.GL_BLEND);
        GL11.glDepthMask(true);

        oldSize = size;
    }

    public List<String> getEntityText(){
        return textList;
    }

    private boolean isPlayerLookingAtTarget(){
        // code used from the Enderman player looking code.
        EntityPlayer player = FMLClientHandler.instance().getClient().field_71439_g;
        World world = FMLClientHandler.instance().getClient().field_71441_e;
        Vec3 vec3 = player.func_70676_i(1.0F).func_72432_b();
        Vec3 vec31 = new Vec3(entity.field_70165_t - player.field_70165_t, entity.func_174813_aQ().field_72338_b + entity.field_70131_O / 2.0F - (player.field_70163_u + player.func_70047_e()), entity.field_70161_v - player.field_70161_v);
        double d0 = vec31.func_72433_c();
        vec31 = vec31.func_72432_b();
        double d1 = vec3.func_72430_b(vec31);
        return d1 > 1.0D - 0.050D / d0;
    }

    public void hack(){
        if(isInitialized() && isPlayerLookingAtTarget()) {
            IHackableEntity hackable = HackableHandler.getHackableForEntity(entity, PneumaticCraft.proxy.getPlayer());
            if(hackable != null && (hackTime == 0 || hackTime > hackable.getHackTime(entity, PneumaticCraft.proxy.getPlayer()))) NetworkHandler.sendToServer(new PacketHackingEntityStart(entity));
        }
    }

    public void selectAsDebuggingTarget(){
        if(isInitialized() && isPlayerLookingAtTarget() && entity instanceof EntityDrone) {
            NetworkHandler.sendToServer(new PacketUpdateDebuggingDrone(entity.func_145782_y()));
        }
    }

    public void onHackConfirmServer(){
        hackTime = 1;
    }

    public int getHackTime(){
        return hackTime;
    }

    public boolean scroll(MouseEvent event){
        if(isInitialized() && isPlayerLookingAtTarget()) {
            return stat.handleMouseWheel(event.dwheel);
        }
        return false;
    }
}
