package pneumaticCraft.client.semiblock;

import java.util.HashMap;
import java.util.Map;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.RenderHelper;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.BlockPos;
import net.minecraftforge.client.event.RenderWorldLastEvent;
import net.minecraftforge.fml.client.FMLClientHandler;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import org.lwjgl.opengl.GL11;

import pneumaticCraft.common.semiblock.ISemiBlock;
import pneumaticCraft.common.semiblock.SemiBlockHeatFrame;
import pneumaticCraft.common.semiblock.SemiBlockLogistics;
import pneumaticCraft.common.semiblock.SemiBlockManager;

public class ClientSemiBlockManager{
    private static final Map<Class<? extends ISemiBlock>, ISemiBlockRenderer> renderers = new HashMap<Class<? extends ISemiBlock>, ISemiBlockRenderer>();

    static {
        registerRenderer(SemiBlockLogistics.class, new SemiBlockRendererLogistics());
        registerRenderer(SemiBlockHeatFrame.class, new SemiBlockRendererHeatFrame());
    }

    public static void registerRenderer(Class<? extends ISemiBlock> semiBlock, ISemiBlockRenderer renderer){
        renderers.put(semiBlock, renderer);
    }

    @SubscribeEvent
    public void renderWorldLastEvent(RenderWorldLastEvent event){
        Minecraft mc = FMLClientHandler.instance().getClient();
        EntityPlayer player = mc.field_71439_g;
        double playerX = player.field_70169_q + (player.field_70165_t - player.field_70169_q) * event.partialTicks;
        double playerY = player.field_70167_r + (player.field_70163_u - player.field_70167_r) * event.partialTicks;
        double playerZ = player.field_70166_s + (player.field_70161_v - player.field_70166_s) * event.partialTicks;

        GL11.glPushMatrix();
        GL11.glTranslated(-playerX, -playerY, -playerZ);
        RenderHelper.func_74519_b();

        for(Map<BlockPos, ISemiBlock> map : SemiBlockManager.getInstance(player.field_70170_p).getSemiBlocks().values()) {
            for(ISemiBlock semiBlock : map.values()) {
                ISemiBlockRenderer renderer = getRenderer(semiBlock);
                if(renderer != null) {
                    GL11.glPushMatrix();
                    GL11.glTranslated(semiBlock.getPos().func_177958_n(), semiBlock.getPos().func_177956_o(), semiBlock.getPos().func_177952_p());
                    renderer.render(semiBlock, event.partialTicks);
                    GL11.glPopMatrix();
                }
            }
        }
        RenderHelper.func_74518_a();
        GL11.glPopMatrix();
    }

    public static ISemiBlockRenderer getRenderer(ISemiBlock semiBlock){
        Class clazz = semiBlock.getClass();
        while(clazz != Object.class && !renderers.containsKey(clazz)) {
            clazz = clazz.getSuperclass();
        }
        return renderers.get(clazz);
    }
}
