package pneumaticCraft.common;

import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.BlockPos;
import net.minecraft.util.Vec3i;

public class NBTUtil{
    /**
     * Initializes the {@link NBTTagCompound} for the given {@link ItemStack} if
     * it is null.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     */
    public static void initNBTTagCompound(ItemStack itemStack){
        if(itemStack.func_77978_p() == null) {
            itemStack.func_77982_d(new NBTTagCompound());
        }
    }

    /**
     * Checks if the {@link NBTTagCompound} of the given {@link ItemStack} has a
     * given tag.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag which should be checked.
     * @return True if the {@link NBTTagCompound} has the tag otherwise false.
     */
    public static boolean hasTag(ItemStack itemStack, String tagName){
        if(itemStack.func_77978_p() != null) {
            return itemStack.func_77978_p().func_74764_b(tagName);
        }
        return false;
    }

    /**
     * Removes the given tag from the {@link NBTTagCompound} of the given
     * {@link ItemStack}.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag which should be removed.
     */
    public static void removeTag(ItemStack itemStack, String tagName){
        if(itemStack.func_77978_p() != null) {
            itemStack.func_77978_p().func_82580_o(tagName);
        }
    }

    /**
     * Gets a String value of the given tag from the {@link NBTTagCompound} of
     * the given {@link ItemStack}. If the {@link NBTTagCompound} is null it
     * will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag for which the value should be determined.
     * @return The value of the given tag as a String or an empty String if the
     *         NBT Tag Compound has no such key.
     */
    public static String getString(ItemStack itemStack, String tagName){
        initNBTTagCompound(itemStack);

        if(!itemStack.func_77978_p().func_74764_b(tagName)) {
            setString(itemStack, tagName, "");
        }

        return itemStack.func_77978_p().func_74779_i(tagName);
    }

    /**
     * Sets the given String value for the given tag on the
     * {@link NBTTagCompound} of the given {@link ItemStack}. If the
     * {@link NBTTagCompound} is null it will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag which should be set.
     * @param tagValue
     *            The value which should be set to the given tag as a String.
     */
    public static void setString(ItemStack itemStack, String tagName, String tagValue){
        initNBTTagCompound(itemStack);
        itemStack.func_77978_p().func_74778_a(tagName, tagValue);
    }

    /**
     * Gets a Boolean value of the given tag from the {@link NBTTagCompound} of
     * the given {@link ItemStack}. If the {@link NBTTagCompound} is null it
     * will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag for which the value should be determined.
     * @return The value of the given tag as a Boolean or false if the NBT Tag
     *         Compound has no such key.
     */
    public static boolean getBoolean(ItemStack itemStack, String tagName){
        initNBTTagCompound(itemStack);

        if(!itemStack.func_77978_p().func_74764_b(tagName)) {
            setBoolean(itemStack, tagName, false);
        }

        return itemStack.func_77978_p().func_74767_n(tagName);
    }

    /**
     * Sets the given boolean value for the given tag on the
     * {@link NBTTagCompound} of the given {@link ItemStack}. If the
     * {@link NBTTagCompound} is null it will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag which should be set.
     * @param tagValue
     *            The value which should be set to the given tag as a boolean.
     */
    public static void setBoolean(ItemStack itemStack, String tagName, boolean tagValue){
        initNBTTagCompound(itemStack);
        itemStack.func_77978_p().func_74757_a(tagName, tagValue);
    }

    /**
     * Gets a Byte value of the given tag from the {@link NBTTagCompound} of the
     * given {@link ItemStack}. If the {@link NBTTagCompound} is null it will be
     * initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag for which the value should be determined.
     * @return The value of the given tag as a Byte or 0 if the NBT Tag Compound
     *         has no such key.
     */
    public static byte getByte(ItemStack itemStack, String tagName){
        initNBTTagCompound(itemStack);

        if(!itemStack.func_77978_p().func_74764_b(tagName)) {
            setByte(itemStack, tagName, (byte)0);
        }

        return itemStack.func_77978_p().func_74771_c(tagName);
    }

    /**
     * Sets the given byte value for the given tag on the {@link NBTTagCompound}
     * of the given {@link ItemStack}. If the {@link NBTTagCompound} is null it
     * will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag which should be set.
     * @param tagValue
     *            The value which should be set to the given tag as a byte.
     */
    public static void setByte(ItemStack itemStack, String tagName, byte tagValue){
        initNBTTagCompound(itemStack);
        itemStack.func_77978_p().func_74774_a(tagName, tagValue);
    }

    /**
     * Gets a Short value of the given tag from the {@link NBTTagCompound} of
     * the given {@link ItemStack}. If the {@link NBTTagCompound} is null it
     * will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag for which the value should be determined.
     * @return The value of the given tag as a Short or 0 if the NBT Tag
     *         Compound has no such key.
     */
    public static short getShort(ItemStack itemStack, String tagName){
        initNBTTagCompound(itemStack);

        if(!itemStack.func_77978_p().func_74764_b(tagName)) {
            setShort(itemStack, tagName, (short)0);
        }

        return itemStack.func_77978_p().func_74765_d(tagName);
    }

    /**
     * Sets the given short value for the given tag on the
     * {@link NBTTagCompound} of the given {@link ItemStack}. If the
     * {@link NBTTagCompound} is null it will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag which should be set.
     * @param tagValue
     *            The value which should be set to the given tag as a short.
     */
    public static void setShort(ItemStack itemStack, String tagName, short tagValue){
        initNBTTagCompound(itemStack);
        itemStack.func_77978_p().func_74777_a(tagName, tagValue);
    }

    /**
     * Gets an Integer value of the given tag from the {@link NBTTagCompound} of
     * the given {@link ItemStack}. If the {@link NBTTagCompound} is null it
     * will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag for which the value should be determined.
     * @return The value of the given tag as an Integer or 0 if the NBT Tag
     *         Compound has no such key.
     */
    public static int getInteger(ItemStack itemStack, String tagName){
        initNBTTagCompound(itemStack);

        if(!itemStack.func_77978_p().func_74764_b(tagName)) {
            setInteger(itemStack, tagName, 0);
        }

        return itemStack.func_77978_p().func_74762_e(tagName);
    }

    /**
     * Sets the given integer value for the given tag on the
     * {@link NBTTagCompound} of the given {@link ItemStack}. If the
     * {@link NBTTagCompound} is null it will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag which should be set.
     * @param tagValue
     *            The value which should be set to the given tag as an integer.
     */
    public static void setInteger(ItemStack itemStack, String tagName, int tagValue){
        initNBTTagCompound(itemStack);
        itemStack.func_77978_p().func_74768_a(tagName, tagValue);
    }

    /**
     * Gets a Long value of the given tag from the {@link NBTTagCompound} of the
     * given {@link ItemStack}. If the {@link NBTTagCompound} is null it will be
     * initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag for which the value should be determined.
     * @return The value of the given tag as a Long or 0 if the NBT Tag Compound
     *         has no such key.
     */
    public static long getLong(ItemStack itemStack, String tagName){
        initNBTTagCompound(itemStack);

        if(!itemStack.func_77978_p().func_74764_b(tagName)) {
            setLong(itemStack, tagName, 0);
        }

        return itemStack.func_77978_p().func_74763_f(tagName);
    }

    /**
     * Sets the given long value for the given tag on the {@link NBTTagCompound}
     * of the given {@link ItemStack}. If the {@link NBTTagCompound} is null it
     * will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag which should be set.
     * @param tagValue
     *            The value which should be set to the given tag as a long.
     */
    public static void setLong(ItemStack itemStack, String tagName, long tagValue){
        initNBTTagCompound(itemStack);
        itemStack.func_77978_p().func_74772_a(tagName, tagValue);
    }

    /**
     * Gets a Float value of the given tag from the {@link NBTTagCompound} of
     * the given {@link ItemStack}. If the {@link NBTTagCompound} is null it
     * will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag for which the value should be determined.
     * @return The value of the given tag as a Float or 0 if the NBT Tag
     *         Compound has no such key.
     */
    public static float getFloat(ItemStack itemStack, String tagName){
        initNBTTagCompound(itemStack);

        if(!itemStack.func_77978_p().func_74764_b(tagName)) {
            setFloat(itemStack, tagName, 0);
        }

        return itemStack.func_77978_p().func_74760_g(tagName);
    }

    /**
     * Sets the given float value for the given tag on the
     * {@link NBTTagCompound} of the given {@link ItemStack}. If the
     * {@link NBTTagCompound} is null it will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag which should be set.
     * @param tagValue
     *            The value which should be set to the given tag as a float.
     */
    public static void setFloat(ItemStack itemStack, String tagName, float tagValue){
        initNBTTagCompound(itemStack);
        itemStack.func_77978_p().func_74776_a(tagName, tagValue);
    }

    /**
     * Gets a Double value of the given tag from the {@link NBTTagCompound} of
     * the given {@link ItemStack}. If the {@link NBTTagCompound} is null it
     * will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag for which the value should be determined.
     * @return The value of the given tag as a Double or 0 if the NBT Tag
     *         Compound has no such key.
     */
    public static double getDouble(ItemStack itemStack, String tagName){
        initNBTTagCompound(itemStack);

        if(!itemStack.func_77978_p().func_74764_b(tagName)) {
            setDouble(itemStack, tagName, 0);
        }

        return itemStack.func_77978_p().func_74769_h(tagName);
    }

    /**
     * Sets the given double value for the given tag on the
     * {@link NBTTagCompound} of the given {@link ItemStack}. If the
     * {@link NBTTagCompound} is null it will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag which should be set.
     * @param tagValue
     *            The value which should be set to the given tag as a double.
     */
    public static void setDouble(ItemStack itemStack, String tagName, double tagValue){
        initNBTTagCompound(itemStack);
        itemStack.func_77978_p().func_74780_a(tagName, tagValue);
    }

    /**
     * Gets a {@link NBTTagCompound} value of the given tag from the
     * {@link NBTTagCompound} of the given {@link ItemStack}. If the
     * {@link NBTTagCompound} is null it will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag for which the value should be determined.
     * @return The value of the given tag as a {@link NBTTagCompound}.
     */
    public static NBTTagCompound getCompoundTag(ItemStack itemStack, String tagName){
        initNBTTagCompound(itemStack);

        if(!itemStack.func_77978_p().func_74764_b(tagName)) {
            itemStack.func_77978_p().func_74782_a(tagName, new NBTTagCompound());
        }

        return itemStack.func_77978_p().func_74775_l(tagName);
    }

    /**
     * Sets the given {@link NBTTagCompound} value for the given tag on the
     * {@link NBTTagCompound} of the given {@link ItemStack}. If the
     * {@link NBTTagCompound} is null it will be initialized.
     * 
     * @param itemStack
     *            The {@link ItemStack} which holds the {@link NBTTagCompound}.
     * @param tagName
     *            The name of the tag which should be set.
     * @param tagValue
     *            The value which should be set to the given tag as a
     *            {@link NBTTagCompound}.
     */
    public static void setCompoundTag(ItemStack itemStack, String tagName, NBTBase tagValue){
        initNBTTagCompound(itemStack);
        itemStack.func_77978_p().func_74782_a(tagName, tagValue);
    }

    public static void setPos(ItemStack stack, Vec3i vec){
        initNBTTagCompound(stack);
        setPos(stack.func_77978_p(), vec);
    }

    public static BlockPos getPos(ItemStack stack){
        initNBTTagCompound(stack);
        return getPos(stack.func_77978_p());
    }

    public static void setPos(NBTTagCompound tag, Vec3i vec){
        tag.func_74768_a("x", vec.func_177958_n());
        tag.func_74768_a("y", vec.func_177956_o());
        tag.func_74768_a("z", vec.func_177952_p());
    }

    public static BlockPos getPos(NBTTagCompound tag){
        return new BlockPos(tag.func_74762_e("x"), tag.func_74762_e("y"), tag.func_74762_e("z"));
    }
}
