package pneumaticCraft.common.ai;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.minecraft.entity.ai.EntityAIBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.Vec3;
import net.minecraft.world.IBlockAccess;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.item.IPressurizable;
import pneumaticCraft.common.item.ItemPneumaticArmor;
import pneumaticCraft.common.item.Itemss;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketSpawnParticle;
import pneumaticCraft.common.progwidgets.IBlockOrdered;
import pneumaticCraft.common.progwidgets.IBlockOrdered.EnumOrder;
import pneumaticCraft.common.progwidgets.ProgWidgetAreaItemBase;
import pneumaticCraft.common.progwidgets.ProgWidgetDigAndPlace;
import pneumaticCraft.common.progwidgets.ProgWidgetPlace;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.common.util.ThreadedSorter;

public abstract class DroneAIBlockInteraction<Widget extends ProgWidgetAreaItemBase> extends EntityAIBase{
    protected final IDroneBase drone;
    protected final Widget widget;
    private final EnumOrder order;
    protected BlockPos curPos;
    private final List<BlockPos> area;
    protected final IBlockAccess worldCache;
    private final List<BlockPos> blacklist = new ArrayList<BlockPos>();//a list of position which weren't allowed to be digged in the past.
    private int curY;
    private int lastSuccessfulY;
    private int minY, maxY;
    private ThreadedSorter<BlockPos> sorter;
    private boolean aborted;

    protected boolean searching; //true while the drone is searching for a coordinate, false if traveling/processing a coordinate.
    private int searchIndex;//The current index in the area list the drone is searching at.
    private static final int LOOKUPS_PER_SEARCH_TICK = 30; //How many blocks does the drone access per AI update.
    private int totalActions;
    private int maxActions = -1;

    /**
     * 
     * @param drone
     * @param speed
     * @param widget needs to implement IBlockOrdered
     */
    public DroneAIBlockInteraction(IDroneBase drone, Widget widget){
        this.drone = drone;
        func_75248_a(63);//binary 111111, so it won't run along with other AI tasks.
        this.widget = widget;
        order = widget instanceof IBlockOrdered ? ((IBlockOrdered)widget).getOrder() : EnumOrder.CLOSEST;
        area = widget.getCachedAreaList();
        worldCache = ProgWidgetAreaItemBase.getCache(area, drone.getWorld());
        if(area.size() > 0) {
            Iterator<BlockPos> iterator = area.iterator();
            BlockPos pos = iterator.next();
            minY = maxY = pos.func_177956_o();
            while(iterator.hasNext()) {
                pos = iterator.next();
                minY = Math.min(minY, pos.func_177956_o());
                maxY = Math.max(maxY, pos.func_177956_o());
            }
            if(order == EnumOrder.HIGH_TO_LOW) {
                curY = maxY;
            } else if(order == EnumOrder.LOW_TO_HIGH) {
                curY = minY;
            }
        }
    }

    /**
     * Returns whether the EntityAIBase should begin execution.
     */
    @Override
    public boolean func_75250_a(){
        if(aborted || maxActions >= 0 && totalActions >= maxActions) {
            return false;
        } else {
            if(!searching) {
                searching = true;
                searchIndex = 0;
                curPos = null;
                lastSuccessfulY = curY;
                if(sorter == null || sorter.isDone()) sorter = new ThreadedSorter(area, new ChunkPositionSorter(drone));
                return true;
            } else {
                return false;
            }
        }
    }

    private void updateY(){
        searchIndex = 0;
        if(order == ProgWidgetPlace.EnumOrder.LOW_TO_HIGH) {
            if(++curY > maxY) curY = minY;
        } else if(order == ProgWidgetPlace.EnumOrder.HIGH_TO_LOW) {
            if(--curY < minY) curY = maxY;
        }
    }

    private boolean isYValid(int y){
        return order == ProgWidgetPlace.EnumOrder.CLOSEST || y == curY;
    }

    public DroneAIBlockInteraction setMaxActions(int maxActions){
        this.maxActions = maxActions;
        return this;
    }

    protected abstract boolean isValidPosition(BlockPos pos);

    protected abstract boolean doBlockInteraction(BlockPos pos, double distToBlock);

    /**
     * Returns whether an in-progress EntityAIBase should continue executing
     */
    @Override
    public boolean func_75253_b(){
        if(aborted) return false;
        if(searching) {
            if(!sorter.isDone()) return true;//Wait until the area is sorted from closest to furtherest.
            boolean firstRun = true;
            int searchedBlocks = 0; //keeps track of the looked up blocks, and stops searching when we reach our quota.
            while(curPos == null && curY != lastSuccessfulY && order != ProgWidgetDigAndPlace.EnumOrder.CLOSEST || firstRun) {
                firstRun = false;
                while(!shouldAbort() && searchIndex < area.size()) {
                    BlockPos pos = area.get(searchIndex);
                    if(isYValid(pos.func_177956_o()) && !blacklist.contains(pos) && (!respectClaims() || !DroneClaimManager.getInstance(drone.getWorld()).isClaimed(pos))) {
                        indicateToListeningPlayers(pos);
                        if(isValidPosition(pos)) {
                            curPos = pos;
                            if(moveToPositions()) {
                                if(moveIntoBlock()) {
                                    if(drone.getPathNavigator().moveToXYZ(curPos.func_177958_n(), curPos.func_177956_o() + 0.5, curPos.func_177952_p())) {
                                        searching = false;
                                        totalActions++;
                                        if(respectClaims()) DroneClaimManager.getInstance(drone.getWorld()).claim(pos);
                                        blacklist.clear();//clear the list for next time (maybe the blocks/rights have changed by the time there will be dug again).
                                        return true;
                                    }
                                } else {
                                    for(EnumFacing dir : EnumFacing.field_82609_l) {
                                        if(drone.getPathNavigator().moveToXYZ(curPos.func_177958_n() + dir.func_82601_c(), curPos.func_177956_o() + dir.func_96559_d() + 0.5, curPos.func_177952_p() + dir.func_82599_e())) {
                                            searching = false;
                                            totalActions++;
                                            if(respectClaims()) DroneClaimManager.getInstance(drone.getWorld()).claim(pos);
                                            blacklist.clear();//clear the list for next time (maybe the blocks/rights have changed by the time there will be dug again).
                                            return true;
                                        }
                                    }
                                }
                                if(drone.getPathNavigator().isGoingToTeleport()) {
                                    searching = false;
                                    totalActions++;
                                    if(respectClaims()) DroneClaimManager.getInstance(drone.getWorld()).claim(pos);
                                    blacklist.clear();//clear the list for next time (maybe the blocks/rights have changed by the time there will be dug again).
                                    return true;
                                } else {
                                    drone.addDebugEntry("gui.progWidget.general.debug.cantNavigate", pos);
                                }
                            } else {
                                searching = false;
                                totalActions++;
                                return true;
                            }
                        }
                        searchedBlocks++;
                    }
                    searchIndex++;
                    if(searchedBlocks >= lookupsPerSearch()) return true;
                }
                if(curPos == null) updateY();
            }
            if(!shouldAbort()) addEndingDebugEntry();
            return false;
        } else {
            Vec3 dronePos = drone.getDronePos();
            double dist = curPos != null ? PneumaticCraftUtils.distBetween(curPos.func_177958_n() + 0.5, curPos.func_177956_o() + 0.5, curPos.func_177952_p() + 0.5, dronePos.field_72450_a, dronePos.field_72448_b, dronePos.field_72449_c) : 0;
            if(curPos != null) {
                if(!moveToPositions()) return doBlockInteraction(curPos, dist);
                if(respectClaims()) DroneClaimManager.getInstance(drone.getWorld()).claim(curPos);
                if(dist < (moveIntoBlock() ? 1 : 2)) {
                    return doBlockInteraction(curPos, dist);
                }
            }
            return !drone.getPathNavigator().hasNoPath();
        }
    }

    protected void addEndingDebugEntry(){
        drone.addDebugEntry("gui.progWidget.blockInteraction.debug.noBlocksValid");
    }

    protected int lookupsPerSearch(){
        return LOOKUPS_PER_SEARCH_TICK;
    }

    protected boolean respectClaims(){
        return false;
    }

    protected boolean moveIntoBlock(){
        return false;
    }

    protected boolean shouldAbort(){
        return aborted;
    }

    public void abort(){
        aborted = true;
    }

    protected boolean moveToPositions(){
        return true;
    }

    /**
     * Sends particle spawn packets to any close player that has a charged pneumatic helmet with entity tracker.
     * @param pos
     */
    protected void indicateToListeningPlayers(BlockPos pos){
        for(EntityPlayer player : drone.getWorld().field_73010_i) {
            if(player.func_82169_q(3) != null && player.func_82169_q(3).func_77973_b() == Itemss.pneumaticHelmet && ItemPneumaticArmor.getUpgrades(EnumUpgrade.ENTITY_TRACKER, player.func_82169_q(3)) > 0 && ((IPressurizable)Itemss.pneumaticHelmet).getPressure(player.func_82169_q(3)) > 0) {
                NetworkHandler.sendTo(new PacketSpawnParticle(EnumParticleTypes.REDSTONE, pos.func_177958_n() + 0.5, pos.func_177956_o() + 0.5, pos.func_177952_p() + 0.5, 0, 0, 0), (EntityPlayerMP)player);
            }
        }
    }

    protected void addToBlacklist(BlockPos coord){
        blacklist.add(coord);
        drone.sendWireframeToClient(coord);
    }

}
