package pneumaticCraft.common.ai;

import java.util.Arrays;
import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.IBlockAccess;
import pneumaticCraft.common.progwidgets.ProgWidgetAreaItemBase;
import pneumaticCraft.common.util.PneumaticCraftUtils;

public class DroneAIDig extends DroneAIBlockInteraction{

    /**
     * 
     * @param drone
     * @param speed
     * @param widget needs to implement IBlockOrdered.
     */
    public DroneAIDig(IDroneBase drone, ProgWidgetAreaItemBase widget){
        super(drone, widget);
    }

    @Override
    protected boolean isValidPosition(BlockPos pos){
        IBlockState blockState = worldCache.func_180495_p(pos);
        Block block = blockState.func_177230_c();
        if(!worldCache.func_175623_d(pos) && !ignoreBlock(block)) {
            List<ItemStack> droppedStacks;
            if(block.canSilkHarvest(drone.getWorld(), pos, blockState, drone.getFakePlayer())) {
                droppedStacks = Arrays.asList(new ItemStack[]{getSilkTouchBlock(block, blockState)});
            } else {
                droppedStacks = block.getDrops(drone.getWorld(), pos, blockState, 0);
            }
            for(ItemStack droppedStack : droppedStacks) {
                if(widget.isItemValidForFilters(droppedStack, blockState)) {
                    swapBestItemToFirstSlot(block, pos);
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    protected boolean respectClaims(){
        return true;
    }

    private void swapBestItemToFirstSlot(Block block, BlockPos pos){
        int bestSlot = 0;
        float bestSoftness = Float.MIN_VALUE;
        ItemStack oldCurrentStack = drone.getInv().func_70301_a(0);
        for(int i = 0; i < drone.getInv().func_70302_i_(); i++) {
            drone.getInv().func_70299_a(0, drone.getInv().func_70301_a(i));
            float softness = block.func_180647_a(drone.getFakePlayer(), drone.getWorld(), pos);
            if(softness > bestSoftness) {
                bestSlot = i;
                bestSoftness = softness;
            }
        }
        drone.getInv().func_70299_a(0, oldCurrentStack);
        if(bestSlot != 0) {
            ItemStack bestItem = drone.getInv().func_70301_a(bestSlot);
            drone.getInv().func_70299_a(bestSlot, drone.getInv().func_70301_a(0));
            drone.getInv().func_70299_a(0, bestItem);
        }
    }

    @Override
    protected boolean doBlockInteraction(BlockPos pos, double distToBlock){
        if(!((FakePlayerItemInWorldManager)drone.getFakePlayer().field_71134_c).isDigging() || !((FakePlayerItemInWorldManager)drone.getFakePlayer().field_71134_c).isAcknowledged()) {
            IBlockState blockState = worldCache.func_180495_p(pos);
            Block block = blockState.func_177230_c();
            if(!ignoreBlock(block) && isBlockValidForFilter(worldCache, drone, pos, widget)) {
                if(block.func_176195_g(drone.getWorld(), pos) < 0) {
                    addToBlacklist(pos);
                    drone.addDebugEntry("gui.progWidget.dig.debug.cantDigBlock", pos);
                    drone.setDugBlock(null);
                    return false;
                }
                FakePlayerItemInWorldManager manager = (FakePlayerItemInWorldManager)drone.getFakePlayer().field_71134_c;
                manager.func_180784_a(pos, EnumFacing.DOWN);
                if(!manager.isAccepted) {
                    addToBlacklist(pos);
                    drone.addDebugEntry("gui.progWidget.dig.debug.cantDigBlock", pos);
                    drone.setDugBlock(null);
                    return false;
                }
                drone.setDugBlock(pos);
                return true;
            }
            drone.setDugBlock(null);
            return false;
        } else {
            return true;
        }
    }

    public static boolean isBlockValidForFilter(IBlockAccess worldCache, IDroneBase drone, BlockPos pos, ProgWidgetAreaItemBase widget){
        IBlockState blockState = worldCache.func_180495_p(pos);
        Block block = blockState.func_177230_c();

        if(!block.isAir(worldCache, pos)) {
            List<ItemStack> droppedStacks;
            if(block.canSilkHarvest(drone.getWorld(), pos, blockState, drone.getFakePlayer())) {
                droppedStacks = Arrays.asList(new ItemStack[]{getSilkTouchBlock(block, blockState)});
            } else {
                droppedStacks = block.getDrops(drone.getWorld(), pos, blockState, 0);
            }
            for(ItemStack droppedStack : droppedStacks) {
                if(widget.isItemValidForFilters(droppedStack, blockState)) {
                    return true;
                }
            }
        }
        return false;
    }

    private static ItemStack getSilkTouchBlock(Block block, IBlockState state){
        return null;//TODO 1.8 return block.createStackedBlock(state);
        //  return SilkTouchBlockGetter.getSilkTouchStack(block, state);
    }

    private static boolean ignoreBlock(Block block){
        return PneumaticCraftUtils.isBlockLiquid(block);
    }

}
