package pneumaticCraft.common.ai;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.Vec3;
import pneumaticCraft.common.entity.living.EntityDrone;
import pneumaticCraft.common.progwidgets.ISidedWidget;
import pneumaticCraft.common.progwidgets.ProgWidgetAreaItemBase;
import pneumaticCraft.common.progwidgets.ProgWidgetPlace;
import pneumaticCraft.lib.PneumaticValues;

public class DroneAIPlace extends DroneAIBlockInteraction{

    /** 
     * @param drone
     * @param speed
     * @param widget needs to implement IBlockOrdered and IDirectionalWidget.
     */
    public DroneAIPlace(IDroneBase drone, ProgWidgetAreaItemBase widget){
        super(drone, widget);
    }

    @Override
    protected boolean respectClaims(){
        return true;
    }

    @Override
    protected boolean isValidPosition(BlockPos pos){
        if(drone.getWorld().func_175623_d(pos)) {
            boolean failedOnPlacement = false;
            for(int i = 0; i < drone.getInv().func_70302_i_(); i++) {
                ItemStack droneStack = drone.getInv().func_70301_a(i);
                if(droneStack != null && droneStack.func_77973_b() instanceof ItemBlock) {
                    if(widget.isItemValidForFilters(droneStack)) {
                        Block placingBlock = ((ItemBlock)droneStack.func_77973_b()).func_179223_d();
                        EnumFacing side = ProgWidgetPlace.getDirForSides(((ISidedWidget)widget).getSides());
                        if(drone.getWorld().func_175716_a(placingBlock, pos, false, side, drone instanceof EntityDrone ? (EntityDrone)drone : null, droneStack)) {
                            return true;
                        } else {
                            if(drone.getWorld().func_175716_a(placingBlock, pos, true, side, drone instanceof EntityDrone ? (EntityDrone)drone : null, droneStack)) {
                                drone.addDebugEntry("gui.progWidget.place.debug.cantPlaceBlock", pos);
                            } else {
                                drone.addDebugEntry("gui.progWidget.place.debug.entityInWay", pos);
                            }
                            failedOnPlacement = true;
                        }
                    }
                }
            }
            if(!failedOnPlacement) abort();
        }
        return false;
    }

    //TODO 1.8 test
    @Override
    protected boolean doBlockInteraction(BlockPos pos, double distToBlock){
        if(drone.getPathNavigator().hasNoPath()) {
            EnumFacing side = ProgWidgetPlace.getDirForSides(((ISidedWidget)widget).getSides());
            for(int i = 0; i < drone.getInv().func_70302_i_(); i++) {
                ItemStack droneStack = drone.getInv().func_70301_a(i);
                if(droneStack != null && droneStack.func_77973_b() instanceof ItemBlock && ((ItemBlock)droneStack.func_77973_b()).func_179223_d().func_176198_a(drone.getWorld(), pos, ProgWidgetPlace.getDirForSides(((ISidedWidget)widget).getSides()))) {
                    if(widget.isItemValidForFilters(droneStack)) {
                        ItemBlock itemBlock = (ItemBlock)droneStack.func_77973_b();
                        Block block = itemBlock.func_179223_d();
                        if(drone.getWorld().func_175716_a(block, pos, false, side, drone instanceof EntityDrone ? (EntityDrone)drone : null, droneStack)) {
                            int newMeta = itemBlock.func_77647_b(droneStack.func_77960_j());
                            setFakePlayerAccordingToDir();
                            IBlockState iblockstate1 = block.func_180642_a(drone.getWorld(), pos, side, side.func_82601_c(), side.func_96559_d(), side.func_82599_e(), newMeta, drone.getFakePlayer());
                            if(itemBlock.placeBlockAt(droneStack, drone.getFakePlayer(), drone.getWorld(), pos, side, side.func_82601_c(), side.func_96559_d(), side.func_82599_e(), iblockstate1)) {
                                drone.addAir(null, -PneumaticValues.DRONE_USAGE_PLACE);
                                drone.getWorld().func_72908_a(pos.func_177958_n() + 0.5F, pos.func_177956_o() + 0.5F, pos.func_177952_p() + 0.5F, block.field_149762_H.func_150496_b(), (block.field_149762_H.func_150497_c() + 1.0F) / 2.0F, block.field_149762_H.func_150494_d() * 0.8F);
                                if(--droneStack.field_77994_a <= 0) {
                                    drone.getInv().func_70299_a(i, null);
                                }
                            }
                            return false;
                        }
                    }
                }
            }
            return false;
        } else {
            return true;
        }
    }

    private void setFakePlayerAccordingToDir(){
        EntityPlayer fakePlayer = drone.getFakePlayer();
        Vec3 pos = drone.getDronePos();
        fakePlayer.field_70165_t = pos.field_72450_a;
        fakePlayer.field_70161_v = pos.field_72449_c;
        switch(ProgWidgetPlace.getDirForSides(((ISidedWidget)widget).getSides())){
            case UP:
                fakePlayer.field_70125_A = -90;
                fakePlayer.field_70163_u = pos.field_72448_b - 10;//do this for PistonBase.determineDirection()
                return;
            case DOWN:
                fakePlayer.field_70125_A = 90;
                fakePlayer.field_70163_u = pos.field_72448_b + 10;//do this for PistonBase.determineDirection()
                return;
            case NORTH:
                fakePlayer.field_70177_z = 180;
                fakePlayer.field_70163_u = pos.field_72448_b;//do this for PistonBase.determineDirection()
                break;
            case EAST:
                fakePlayer.field_70177_z = 270;
                fakePlayer.field_70163_u = pos.field_72448_b;//do this for PistonBase.determineDirection()
                break;
            case SOUTH:
                fakePlayer.field_70177_z = 0;
                fakePlayer.field_70163_u = pos.field_72448_b;//do this for PistonBase.determineDirection()
                break;
            case WEST:
                fakePlayer.field_70177_z = 90;
                fakePlayer.field_70163_u = pos.field_72448_b;//do this for PistonBase.determineDirection()
                break;
        }
    }

}
