package pneumaticCraft.common.ai;

import java.util.Collections;
import java.util.List;

import net.minecraft.entity.Entity;
import net.minecraft.entity.ai.EntityAIBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.Vec3;
import net.minecraftforge.event.entity.player.EntityItemPickupEvent;
import pneumaticCraft.common.EventHandlerPneumaticCraft;
import pneumaticCraft.common.progwidgets.ProgWidgetAreaItemBase;
import pneumaticCraft.common.util.PneumaticCraftUtils;

import com.google.common.base.Predicate;

public class DroneEntityAIPickupItems extends EntityAIBase{
    private final IDroneBase drone;
    private final ProgWidgetAreaItemBase itemPickupWidget;
    private EntityItem curPickingUpEntity;
    private final DistanceEntitySorter theNearestAttackableTargetSorter;

    public DroneEntityAIPickupItems(IDroneBase drone, ProgWidgetAreaItemBase progWidgetPickupItem){
        this.drone = drone;
        func_75248_a(63);//binary 111111, so it won't run along with other AI tasks.
        itemPickupWidget = progWidgetPickupItem;
        theNearestAttackableTargetSorter = new DistanceEntitySorter(drone);
    }

    /**
     * Returns whether the EntityAIBase should begin execution.
     */
    @Override
    public boolean func_75250_a(){
        List<Entity> pickableItems = itemPickupWidget.getEntitiesInArea(drone.getWorld(), new Predicate<Entity>(){
            @Override
            public boolean apply(Entity entity){
                return entity instanceof EntityItem && entity.func_70089_S();
            }
        });

        Collections.sort(pickableItems, theNearestAttackableTargetSorter);
        for(Entity ent : pickableItems) {
            ItemStack stack = ((EntityItem)ent).func_92059_d();
            if(itemPickupWidget.isItemValidForFilters(stack)) {
                for(int i = 0; i < drone.getInv().func_70302_i_(); i++) {
                    ItemStack droneStack = drone.getInv().func_70301_a(i);
                    if(droneStack == null || droneStack.func_77969_a(stack) && droneStack.field_77994_a < droneStack.func_77976_d()) {
                        if(drone.getPathNavigator().moveToEntity(ent)) {
                            curPickingUpEntity = (EntityItem)ent;
                            return true;
                        }
                    }
                }
            } else {
                drone.addDebugEntry("gui.progWidget.itemPickup.debug.itemNotValid");
            }
        }
        return false; // 

    }

    /**
     * Returns whether an in-progress EntityAIBase should continue executing
     */
    @Override
    public boolean func_75253_b(){
        if(curPickingUpEntity.field_70128_L) return false;
        if(new Vec3(curPickingUpEntity.field_70165_t, curPickingUpEntity.field_70163_u, curPickingUpEntity.field_70161_v).func_72438_d(drone.getDronePos()) < 1.5) {
            ItemStack stack = curPickingUpEntity.func_92059_d();
            if(itemPickupWidget.isItemValidForFilters(stack)) {
                new EventHandlerPneumaticCraft().onPlayerPickup(new EntityItemPickupEvent(drone.getFakePlayer(), curPickingUpEntity));//not posting the event globally, as I don't have a way of handling a canceled event.
                int stackSize = stack.field_77994_a;
                ItemStack remainder = PneumaticCraftUtils.exportStackToInventory(drone.getInv(), stack, EnumFacing.UP);//side doesn't matter, drones aren't ISided.
                if(remainder == null) {
                    drone.onItemPickupEvent(curPickingUpEntity, stackSize);
                    curPickingUpEntity.func_70106_y();
                }
            }
            return false;
        }
        return !drone.getPathNavigator().hasNoPath();
    }
}
