package pneumaticCraft.common.ai;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import net.minecraft.entity.ai.EntityAIBase;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.common.entity.living.EntityDrone;
import pneumaticCraft.common.tileentity.TileEntityChargingStation;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.PneumaticValues;

public class DroneGoToChargingStation extends EntityAIBase{
    private final EntityDrone drone;
    public boolean isExecuting;
    public TileEntityChargingStation curCharger;
    private int chargingTime;

    public DroneGoToChargingStation(EntityDrone drone){
        this.drone = drone;
        func_75248_a(63);//binary 111111, so it won't run along with other AI tasks.
    }

    /**
     * Returns whether the EntityAIBase should begin execution.
     */
    @Override
    public boolean func_75250_a(){
        List<TileEntityChargingStation> validChargingStations = new ArrayList<TileEntityChargingStation>();
        if(drone.getPressure(null) < PneumaticValues.DRONE_LOW_PRESSURE) {
            for(TileEntity te : drone.field_70170_p.field_147482_g) {
                if(te instanceof TileEntityChargingStation) {
                    TileEntityChargingStation station = (TileEntityChargingStation)te;
                    BlockPos pos = new BlockPos(station.func_174877_v().func_177958_n(), station.func_174877_v().func_177956_o(), station.func_174877_v().func_177952_p());
                    if(DroneClaimManager.getInstance(drone.field_70170_p).isClaimed(pos)) {
                        drone.addDebugEntry("gui.progWidget.chargingStation.debug.claimed", pos);
                    } else if(station.getPressure() <= PneumaticValues.DRONE_LOW_PRESSURE) {
                        drone.addDebugEntry("gui.progWidget.chargingStation.debug.notEnoughPressure", pos);
                    } else if(station.getUpgrades(EnumUpgrade.DISPENSER) == 0) {
                        drone.addDebugEntry("gui.progWidget.chargingStation.debug.noDispenserUpgrades", pos);
                    } else {
                        validChargingStations.add(station);
                    }
                }
            }
        }

        Collections.sort(validChargingStations, new Comparator(){
            @Override
            public int compare(Object arg1, Object arg2){
                TileEntity te1 = (TileEntity)arg1;
                TileEntity te2 = (TileEntity)arg2;
                return Double.compare(PneumaticCraftUtils.distBetween(te1.func_174877_v().func_177958_n(), te1.func_174877_v().func_177956_o(), te1.func_174877_v().func_177952_p(), drone.field_70165_t, drone.field_70163_u, drone.field_70161_v), PneumaticCraftUtils.distBetween(te2.func_174877_v().func_177958_n(), te2.func_174877_v().func_177956_o(), te2.func_174877_v().func_177952_p(), drone.field_70165_t, drone.field_70163_u, drone.field_70161_v));
            }
        });

        for(TileEntityChargingStation station : validChargingStations) {
            boolean protect = PneumaticCraftUtils.getProtectingSecurityStations(drone.field_70170_p, station.func_174877_v(), drone.getFakePlayer(), false, false) > 0;
            BlockPos pos = new BlockPos(station.func_174877_v().func_177958_n(), station.func_174877_v().func_177956_o(), station.func_174877_v().func_177952_p());
            if(protect) {
                drone.addDebugEntry("gui.progWidget.chargingStation.debug.protected", pos);
            } else if(drone.getPathNavigator().moveToXYZ(station.func_174877_v().func_177958_n(), station.func_174877_v().func_177956_o() + 1.5, station.func_174877_v().func_177952_p()) || drone.getPathNavigator().isGoingToTeleport()) {
                isExecuting = true;
                curCharger = station;
                DroneClaimManager.getInstance(drone.field_70170_p).claim(pos);
                return true;
            } else {
                drone.addDebugEntry("gui.progWidget.chargingStation.debug.cantNavigate", pos);
            }
        }
        isExecuting = false;
        return false;
    }

    /**
     * Returns whether an in-progress EntityAIBase should continue executing
     */
    @Override
    public boolean func_75253_b(){
        if(curCharger.getUpgrades(EnumUpgrade.DISPENSER) == 0 || curCharger.func_145837_r()) {//If our path was blocked.
            isExecuting = false;
            return false;
        } else if(!drone.getPathNavigator().isGoingToTeleport() && (drone.func_70661_as().func_75505_d() == null || drone.func_70661_as().func_75505_d().func_75879_b())) {
            isExecuting = drone.getPressure(null) < 9.9F && curCharger.getPressure() > drone.getPressure(null) + 0.1F;
            if(isExecuting) {
                chargingTime++;
                if(chargingTime > 20) {
                    drone.getPathNavigator().moveToXYZ(curCharger.func_174877_v().func_177958_n(), curCharger.func_174877_v().func_177956_o() + 1.5, curCharger.func_174877_v().func_177952_p());
                    if(drone.func_70661_as().func_75505_d() == null || drone.func_70661_as().func_75505_d().func_75879_b()) {
                        drone.setStandby(true);
                    } else {
                        chargingTime = 0;
                    }
                }
                DroneClaimManager.getInstance(drone.field_70170_p).claim(new BlockPos(curCharger.func_174877_v().func_177958_n(), curCharger.func_174877_v().func_177956_o(), curCharger.func_174877_v().func_177952_p()));
            }
            return isExecuting;
        } else {
            chargingTime = 0;
            DroneClaimManager.getInstance(drone.field_70170_p).claim(new BlockPos(curCharger.func_174877_v().func_177958_n(), curCharger.func_174877_v().func_177956_o(), curCharger.func_174877_v().func_177952_p()));
            return drone.isAccelerating();
        }
    }
}
