package pneumaticCraft.common.ai;

import java.util.Random;

import net.minecraft.entity.Entity;
import net.minecraft.pathfinding.PathEntity;
import net.minecraft.pathfinding.PathFinder;
import net.minecraft.pathfinding.PathNavigate;
import net.minecraft.pathfinding.PathPoint;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;
import pneumaticCraft.api.drone.IPathNavigator;
import pneumaticCraft.common.entity.living.EntityDrone;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketPlaySound;
import pneumaticCraft.common.network.PacketSpawnParticle;
import pneumaticCraft.lib.Sounds;

public class EntityPathNavigateDrone extends PathNavigate implements IPathNavigator{

    private final EntityDrone pathfindingEntity;
    public boolean pathThroughLiquid;
    private boolean forceTeleport;
    private int teleportCounter = -1;
    private BlockPos telPos;
    private static final int TELEPORT_TICKS = 120;

    public EntityPathNavigateDrone(EntityDrone pathfindingEntity, World par2World){
        super(pathfindingEntity, par2World);
        this.pathfindingEntity = pathfindingEntity;
    }

    @Override
    public boolean func_75497_a(Entity p_75497_1_, double p_75497_2_){
        return super.func_75497_a(p_75497_1_, p_75497_2_) || isGoingToTeleport();
    }

    /**
     * Returns the path to the given EntityLiving
     */
    @Override
    public PathEntity func_75494_a(Entity par1Entity){
        BlockPos pos = new BlockPos(par1Entity.field_70165_t, par1Entity.func_174813_aQ().field_72338_b, par1Entity.field_70161_v);
        return func_179680_a(pos); //TODO 1.8 test
    }

    public void setForceTeleport(boolean forceTeleport){
        this.forceTeleport = forceTeleport;
    }

    @Override
    public PathEntity func_179680_a(BlockPos pos){
        if(!pathfindingEntity.isBlockValidPathfindBlock(pos)) return null;
        PathEntity path = null;
        if(!forceTeleport || pos.equals(new BlockPos(pathfindingEntity))) {
            path = super.func_179680_a(pos.func_177977_b());
            if(path != null) {
                PathPoint finalPoint = path.func_75870_c();
                if(finalPoint == null || !pos.equals(new BlockPos(finalPoint.field_75839_a, finalPoint.field_75837_b, finalPoint.field_75838_c))) path = null;
            }
        }
        teleportCounter = path != null ? -1 : 0;
        telPos = pos;
        pathfindingEntity.setStandby(false);
        return path;
    }

    @Override
    public float func_111269_d(){
        return (float)pathfindingEntity.getRange();
    }

    @Override
    public boolean isGoingToTeleport(){
        return teleportCounter >= 0;
    }

    @Override
    public boolean func_75500_f(){
        return super.func_75500_f() && !isGoingToTeleport();
    }

    @Override
    public void func_75501_e(){
        if(isGoingToTeleport()) {
            if(teleportCounter == 0 || teleportCounter == 60) {
                NetworkHandler.sendToAllAround(new PacketPlaySound(Sounds.HUD_INIT, pathfindingEntity.field_70165_t, pathfindingEntity.field_70163_u, pathfindingEntity.field_70161_v, 0.1F, teleportCounter == 0 ? 0.7F : 1F, true), pathfindingEntity.field_70170_p);
            }

            if(teleportCounter < TELEPORT_TICKS - 40) {
                Random rand = pathfindingEntity.func_70681_au();
                float f = (rand.nextFloat() - 0.5F) * 0.02F * teleportCounter;
                float f1 = (rand.nextFloat() - 0.5F) * 0.02F * teleportCounter;
                float f2 = (rand.nextFloat() - 0.5F) * 0.02F * teleportCounter;
                NetworkHandler.sendToAllAround(new PacketSpawnParticle(EnumParticleTypes.PORTAL, pathfindingEntity.field_70165_t, pathfindingEntity.field_70163_u, pathfindingEntity.field_70161_v, f, f1, f2), pathfindingEntity.field_70170_p);
            }

            if(++teleportCounter > TELEPORT_TICKS) {
                if(pathfindingEntity.isBlockValidPathfindBlock(telPos)) {
                    teleport();
                }
                teleportCounter = -1;
                func_75484_a(null, 0);
                pathfindingEntity.func_70605_aq().func_75642_a(telPos.func_177958_n(), telPos.func_177956_o(), telPos.func_177952_p(), pathfindingEntity.getSpeed());
                pathfindingEntity.addAir(null, -10000);
            }
        } else {
            // super.onUpdateNavigation();
            if(!func_75500_f()) {
                func_75508_h();

                if(!func_75500_f()) {
                    Vec3 vec32 = field_75514_c.func_75878_a(field_75515_a);

                    if(vec32 != null) {
                        field_75515_a.func_70605_aq().func_75642_a(vec32.field_72450_a, vec32.field_72448_b, vec32.field_72449_c, field_75511_d);
                    }
                }
            }
        }
    }

    public void teleport(){

        Random rand = pathfindingEntity.func_70681_au();
        double width = pathfindingEntity.field_70130_N;
        double height = pathfindingEntity.field_70131_O;

        short short1 = 128;

        for(int l = 0; l < short1; ++l) {
            double d6 = l / (short1 - 1.0D);
            float f = (rand.nextFloat() - 0.5F) * 0.2F;
            float f1 = (rand.nextFloat() - 0.5F) * 0.2F;
            float f2 = (rand.nextFloat() - 0.5F) * 0.2F;
            double d7 = pathfindingEntity.field_70165_t + (telPos.func_177958_n() + 0.5 - pathfindingEntity.field_70165_t) * d6 + (rand.nextDouble() - 0.5D) * width * 2.0D;
            double d8 = pathfindingEntity.field_70163_u + (telPos.func_177956_o() - pathfindingEntity.field_70163_u) * d6 + rand.nextDouble() * height;
            double d9 = pathfindingEntity.field_70161_v + (telPos.func_177952_p() + 0.5 - pathfindingEntity.field_70161_v) * d6 + (rand.nextDouble() - 0.5D) * width * 2.0D;
            NetworkHandler.sendToAllAround(new PacketSpawnParticle(EnumParticleTypes.PORTAL, d7, d8, d9, f, f1, f2), pathfindingEntity.field_70170_p);
        }

        pathfindingEntity.field_70170_p.func_72908_a(pathfindingEntity.field_70165_t, pathfindingEntity.field_70163_u, pathfindingEntity.field_70161_v, "mob.endermen.portal", 1.0F, 1.0F);
        pathfindingEntity.func_85030_a("mob.endermen.portal", 1.0F, 1.0F);

        pathfindingEntity.func_70107_b(telPos.func_177958_n() + 0.5, telPos.func_177956_o() + 0.5, telPos.func_177952_p() + 0.5);
    }

    @Override
    public boolean moveToXYZ(double x, double y, double z){
        return func_75492_a(x, y, z, pathfindingEntity.getSpeed());
    }

    @Override
    public boolean moveToEntity(Entity entity){
        return func_75497_a(entity, pathfindingEntity.getSpeed());
    }

    @Override
    public boolean hasNoPath(){
        return func_75500_f();
    }

    @Override
    public boolean func_75493_a(Vec3 p_75493_1_, Vec3 p_75493_2_, int p_75493_3_, int p_75493_4_, int p_75493_5_){
        return false;
    }

    @Override
    protected PathFinder func_179679_a(){
        return new PathFinder(new NodeProcessorDrone());
    }

    @Override
    protected Vec3 func_75502_i(){
        return pathfindingEntity.getDronePos();//TODO 1.8 test if offset is necessary.
    }

    @Override
    protected boolean func_75485_k(){
        return true;
    }
}
