package pneumaticCraft.common.block.tubes;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.WorldRenderer;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.client.model.IFlexibleBakedModel;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.lwjgl.opengl.GL11;

import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.item.Itemss;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketOpenTubeModuleGui;
import pneumaticCraft.common.thirdparty.ModInteractionUtils;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.BBConstants;
import pneumaticCraft.proxy.CommonProxy.EnumGuiId;

public abstract class TubeModule implements ISidedPart{
    protected IPneumaticPosProvider pressureTube;
    protected EnumFacing dir = EnumFacing.UP;
    public AxisAlignedBB[] boundingBoxes = new AxisAlignedBB[6];
    protected boolean upgraded;
    public float lowerBound = 7.5F, higherBound = 0, maxValue = 30;
    private boolean fake;
    public boolean advancedConfig;
    public boolean shouldDrop;

    public TubeModule(){
        double width = getWidth() / 2;
        double height = getHeight();

        boundingBoxes[0] = new AxisAlignedBB(0.5 - width, BBConstants.PRESSURE_PIPE_MIN_POS - height, 0.5 - width, 0.5 + width, BBConstants.PRESSURE_PIPE_MIN_POS, 0.5 + width);
        boundingBoxes[1] = new AxisAlignedBB(0.5 - width, BBConstants.PRESSURE_PIPE_MAX_POS, 0.5 - width, 0.5 + width, BBConstants.PRESSURE_PIPE_MAX_POS + height, 0.5 + width);
        boundingBoxes[2] = new AxisAlignedBB(0.5 - width, 0.5 - width, BBConstants.PRESSURE_PIPE_MIN_POS - height, 0.5 + width, 0.5 + width, BBConstants.PRESSURE_PIPE_MIN_POS);
        boundingBoxes[3] = new AxisAlignedBB(0.5 - width, 0.5 - width, BBConstants.PRESSURE_PIPE_MAX_POS, 0.5 + width, 0.5 + width, BBConstants.PRESSURE_PIPE_MAX_POS + height);
        boundingBoxes[4] = new AxisAlignedBB(BBConstants.PRESSURE_PIPE_MIN_POS - height, 0.5 - width, 0.5 - width, BBConstants.PRESSURE_PIPE_MIN_POS, 0.5 + width, 0.5 + width);
        boundingBoxes[5] = new AxisAlignedBB(BBConstants.PRESSURE_PIPE_MAX_POS, 0.5 - width, 0.5 - width, BBConstants.PRESSURE_PIPE_MAX_POS + height, 0.5 + width, 0.5 + width);
    }

    public void markFake(){
        fake = true;
    }

    public boolean isFake(){
        return fake;
    }

    public void setTube(IPneumaticPosProvider pressureTube){
        this.pressureTube = pressureTube;
    }

    public IPneumaticPosProvider getTube(){
        return pressureTube;
    }

    public double getWidth(){
        return BBConstants.PRESSURE_PIPE_MAX_POS - BBConstants.PRESSURE_PIPE_MIN_POS;
    }

    protected double getHeight(){
        return BBConstants.PRESSURE_PIPE_MIN_POS;
    }

    public float getThreshold(int redstone){
        double slope = (higherBound - lowerBound) / 15;
        double threshold = lowerBound + slope * redstone;
        return (float)threshold;
    }

    /**
     * Returns the item that this part drops.
     * @return
     */
    public List<ItemStack> getDrops(){
        List<ItemStack> drops = new ArrayList<ItemStack>();
        if(shouldDrop) {
            drops.add(new ItemStack(ModuleRegistrator.getModuleItem(getType())));
            if(upgraded) drops.add(new ItemStack(Itemss.advancedPCB));
        }
        return drops;
    }

    @Override
    public void setDirection(EnumFacing dir){
        this.dir = dir;
    }

    public EnumFacing getDirection(){
        return dir;
    }

    public void readFromNBT(NBTTagCompound nbt){
        dir = EnumFacing.func_82600_a(nbt.func_74762_e("dir"));
        upgraded = nbt.func_74767_n("upgraded");
        lowerBound = nbt.func_74760_g("lowerBound");
        higherBound = nbt.func_74760_g("higherBound");
        advancedConfig = nbt.func_74764_b("advancedConfig") ? nbt.func_74767_n("advancedConfig") : true;
    }

    public void writeToNBT(NBTTagCompound nbt){
        nbt.func_74768_a("dir", dir.ordinal());
        nbt.func_74757_a("upgraded", upgraded);
        nbt.func_74776_a("lowerBound", lowerBound);
        nbt.func_74776_a("higherBound", higherBound);
        nbt.func_74757_a("advancedConfig", advancedConfig);
    }

    /* TODO FMP dep @Optional.Method(modid = ModIds.FMP)
     public void writeDesc(MCDataOutput data){
         data.writeInt(dir.ordinal());
     }

     @Optional.Method(modid = ModIds.FMP)
     public void readDesc(MCDataInput data){
         dir = EnumFacing.getFront(data.readInt());
     }*/

    public void update(){}

    public void onNeighborTileUpdate(){}

    public void onNeighborBlockUpdate(){}

    /**
     * Used by multiparts and/or by NBT saving.
     * @return
     */
    public abstract String getType();

    @SideOnly(Side.CLIENT)
    public void renderDynamic(double x, double y, double z, float partialTicks, int renderPass, boolean itemRender){
        if(renderPass == 0) {
            IFlexibleBakedModel model = ModuleRegistrator.models.get(getClass());
            if(model == null) return;

            GL11.glPushMatrix(); // start
            // GL11.glDisable(GL11.GL_TEXTURE_2D);
            // GL11.glEnable(GL11.GL_BLEND);
            // GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);

            // GL11.glColor4f(0.82F, 0.56F, 0.09F, 1.0F);
            GL11.glTranslatef((float)x + 0.5F, (float)y - 0.5F, (float)z + 0.5F); // size
            GL11.glRotatef(0, 0.0F, 1.0F, 0.0F);

            GL11.glScalef(1.0F, -1F, -1F); // to make your block have a normal
                                           // positioning. comment out to see what
                                           // happens

            PneumaticCraftUtils.rotateMatrixByMetadata(dir.ordinal());
            renderModule();
            WorldRenderer worldRenderer = Tessellator.func_178181_a().func_178180_c();
            worldRenderer.func_178969_c(-getTube().pos().func_177958_n(), -getTube().pos().func_177956_o(), -getTube().pos().func_177952_p());
            worldRenderer.func_181668_a(7, DefaultVertexFormats.field_176600_a);
            Minecraft.func_71410_x().func_175602_ab().func_175019_b().func_178258_b(getTube().world(), model, Blockss.pressureTube, getTube().pos(), worldRenderer, false);
            Tessellator.func_178181_a().func_78381_a();
            worldRenderer.func_178969_c(0, 0, 0);
            GL11.glPopMatrix();
        }
    }

    protected void renderModule(){}

    public abstract String getModelName();

    public int getRedstoneLevel(){
        return 0;
    }

    protected void updateNeighbors(){
        pressureTube.world().func_175685_c(pressureTube.pos(), pressureTube.world().func_180495_p(pressureTube.pos()).func_177230_c());
    }

    public boolean isInline(){
        return false;
    }

    public void sendDescriptionPacket(){
        ModInteractionUtils.getInstance().sendDescriptionPacket(pressureTube);
    }

    public void addInfo(List<String> curInfo){}

    public void addItemDescription(List<String> curInfo){}

    public boolean canUpgrade(){
        return true;
    }

    public void upgrade(){
        upgraded = true;
    }

    public boolean isUpgraded(){
        return upgraded;
    }

    public boolean onActivated(EntityPlayer player){
        if(!player.field_70170_p.field_72995_K && upgraded && getGuiId() != null) {
            NetworkHandler.sendTo(new PacketOpenTubeModuleGui(getGuiId().ordinal(), pressureTube.pos()), (EntityPlayerMP)player);
            return true;
        }
        return false;
    }

    protected abstract EnumGuiId getGuiId();

}
