package pneumaticCraft.common.heat;

import java.util.HashMap;
import java.util.Map;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidRegistry;
import pneumaticCraft.api.heat.HeatBehaviour;
import pneumaticCraft.api.heat.IHeatExchangerLogic;
import pneumaticCraft.api.tileentity.IHeatExchanger;
import pneumaticCraft.api.tileentity.IHeatRegistry;
import pneumaticCraft.common.heat.behaviour.HeatBehaviourManager;
import pneumaticCraft.lib.Log;

public class HeatExchangerManager implements IHeatRegistry{
    /**
     * Used for blocks like lava and ice.
     */
    private final Map<Block, IHeatExchanger> specialBlockExchangers = new HashMap<Block, IHeatExchanger>();
    private final IHeatExchangerLogic AIR_EXCHANGER = new HeatExchangerLogicConstant(295, 100);
    public static final double FLUID_RESISTANCE = 10;

    private static HeatExchangerManager INSTANCE = new HeatExchangerManager();

    public static HeatExchangerManager getInstance(){
        return INSTANCE;
    }

    public void init(){
        registerBlockExchanger(Blocks.field_150432_aD, 263, 500);
        registerBlockExchanger(Blocks.field_150403_cj, 263, 500);
        registerBlockExchanger(Blocks.field_150433_aE, 268, 1000);
        registerBlockExchanger(Blocks.field_150478_aa, 1700, 100000);
        registerBlockExchanger(Blocks.field_150480_ab, 1700, 1000);

        Map<String, Fluid> fluids = FluidRegistry.getRegisteredFluids();
        for(Fluid fluid : fluids.values()) {
            if(fluid.getBlock() != null) {
                registerBlockExchanger(fluid.getBlock(), fluid.getTemperature(), FLUID_RESISTANCE);
            }
        }
        registerBlockExchanger(Blocks.field_150358_i, FluidRegistry.WATER.getTemperature(), 500);
        registerBlockExchanger(Blocks.field_150356_k, FluidRegistry.LAVA.getTemperature(), 500);
    }

    public IHeatExchangerLogic getLogic(World world, BlockPos pos, EnumFacing side){
        if(!world.func_175667_e(pos)) return null;
        TileEntity te = world.func_175625_s(pos);
        if(te instanceof IHeatExchanger) {
            return ((IHeatExchanger)te).getHeatExchangerLogic(side);
        } else {
            if(world.func_175623_d(pos)) {
                return AIR_EXCHANGER;
            } else {
                Block block = world.func_180495_p(pos).func_177230_c();
                if(block instanceof IHeatExchanger) {
                    return ((IHeatExchanger)block).getHeatExchangerLogic(side);
                } else {
                    IHeatExchanger exchanger = specialBlockExchangers.get(block);
                    return exchanger == null ? null : exchanger.getHeatExchangerLogic(side);
                }
            }
        }
    }

    public void registerBlockExchanger(Block block, IHeatExchanger heatExchanger){
        if(block == null) throw new IllegalArgumentException("block is null when trying to register a heat exchanger!");
        if(block instanceof IHeatExchanger) Log.warning("The block " + block.func_149739_a() + " is implementing IHeatExchanger. Therefore you don't need to register it as such");
        if(specialBlockExchangers.containsKey(block)) {
            Log.error("The block " + block.func_149739_a() + " was registered as heat exchanger already! It won't be added!");
        } else {
            specialBlockExchangers.put(block, heatExchanger);
        }
    }

    public void registerBlockExchanger(Block block, IHeatExchangerLogic heatExchangerLogic){
        registerBlockExchanger(block, new SimpleHeatExchanger(heatExchangerLogic));
    }

    @Override
    public void registerBlockExchanger(Block block, double temperature, double thermalResistance){
        registerBlockExchanger(block, new HeatExchangerLogicConstant(temperature, thermalResistance));
    }

    @Override
    public void registerHeatBehaviour(Class<? extends HeatBehaviour> heatBehaviour){
        HeatBehaviourManager.getInstance().registerBehaviour(heatBehaviour);
    }

    @Override
    public IHeatExchangerLogic getHeatExchangerLogic(){
        return new HeatExchangerLogic();
    }

}
