package pneumaticCraft.common.heat.behaviour;

import java.util.HashSet;
import java.util.Set;
import java.util.Stack;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraftforge.fluids.Fluid;
import pneumaticCraft.common.heat.HeatExchangerManager;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketPlaySound;
import pneumaticCraft.common.network.PacketSpawnParticle;
import pneumaticCraft.common.util.FluidUtils;

public abstract class HeatBehaviourLiquidTransition extends HeatBehaviourLiquid{
    private double extractedHeat;
    private double maxExchangedHeat;
    private int fluidTemp = -1;

    @Override
    public boolean isApplicable(){
        Fluid fluid = getFluid();
        return fluid != null && fluid.getTemperature() >= getMinFluidTemp() && fluid.getTemperature() <= getMaxFluidTemp();
    }

    protected abstract int getMinFluidTemp();

    protected abstract int getMaxFluidTemp();

    protected abstract int getMaxExchangedHeat();

    protected abstract Block getTransitionedSourceBlock();

    protected abstract Block getTransitionedFlowingBlock();

    protected abstract boolean transitionOnTooMuchExtraction();

    @Override
    public void update(){
        if(fluidTemp == -1) {
            fluidTemp = getFluid().getTemperature();
            maxExchangedHeat = getMaxExchangedHeat() * (HeatExchangerManager.FLUID_RESISTANCE + getHeatExchanger().getThermalResistance());
        }
        extractedHeat += fluidTemp - getHeatExchanger().getTemperature();
        if(transitionOnTooMuchExtraction() ? extractedHeat > maxExchangedHeat : extractedHeat < -maxExchangedHeat) {
            transformSourceBlock(getTransitionedSourceBlock(), getTransitionedFlowingBlock());
            extractedHeat -= maxExchangedHeat;
        }
    }

    @Override
    public void writeToNBT(NBTTagCompound tag){
        super.writeToNBT(tag);
        tag.func_74780_a("extractedHeat", extractedHeat);
    }

    @Override
    public void readFromNBT(NBTTagCompound tag){
        super.readFromNBT(tag);
        extractedHeat = tag.func_74769_h("extractedHeat");
    }

    protected void transformSourceBlock(Block turningBlockSource, Block turningBlockFlowing){
        if(FluidUtils.isSourceBlock(getWorld(), getPos())) {
            getWorld().func_175656_a(getPos(), turningBlockSource.func_176223_P());
            onLiquidTransition(getPos());
        } else {
            Set<BlockPos> traversed = new HashSet<BlockPos>();
            Stack<BlockPos> pending = new Stack<BlockPos>();
            pending.push(getPos());
            while(!pending.isEmpty()) {
                BlockPos pos = pending.pop();
                for(EnumFacing d : EnumFacing.field_82609_l) {
                    BlockPos newPos = pos.func_177972_a(d);
                    Block checkingBlock = getWorld().func_180495_p(newPos).func_177230_c();
                    if((checkingBlock == getBlockState().func_177230_c() || getBlockState().func_177230_c() == Blocks.field_150358_i && checkingBlock == Blocks.field_150355_j || getBlockState().func_177230_c() == Blocks.field_150356_k && checkingBlock == Blocks.field_150353_l) && traversed.add(newPos)) {
                        if(FluidUtils.isSourceBlock(getWorld(), newPos)) {
                            getWorld().func_175656_a(newPos, turningBlockSource.func_176223_P());
                            onLiquidTransition(newPos);
                            return;
                        } else {
                            getWorld().func_175656_a(newPos, turningBlockFlowing.func_176223_P());
                            onLiquidTransition(newPos);
                            pending.push(newPos);
                        }
                    }
                }
            }
        }
    }

    protected void onLiquidTransition(BlockPos pos){
        NetworkHandler.sendToAllAround(new PacketPlaySound("random.fizz", pos.func_177958_n() + 0.5, pos.func_177956_o() + 0.5, pos.func_177952_p() + 0.5, 0.5F, 2.6F + (getWorld().field_73012_v.nextFloat() - getWorld().field_73012_v.nextFloat()) * 0.8F, true), getWorld());
        for(int i = 0; i < 8; i++) {
            double randX = pos.func_177958_n() + getWorld().field_73012_v.nextDouble();
            double randZ = pos.func_177952_p() + getWorld().field_73012_v.nextDouble();
            NetworkHandler.sendToAllAround(new PacketSpawnParticle(EnumParticleTypes.SMOKE_LARGE, randX, pos.func_177956_o() + 1, randZ, 0, 0, 0), getWorld());
        }
    }
}
