package pneumaticCraft.common.inventory;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.InventoryBasic;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockPos;
import net.minecraft.util.ChatComponentTranslation;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.IFluidHandler;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.api.item.IPressurizable;
import pneumaticCraft.common.DroneRegistry;
import pneumaticCraft.common.config.AmadronOfferSettings;
import pneumaticCraft.common.config.AmadronOfferStaticConfig;
import pneumaticCraft.common.entity.living.EntityDrone;
import pneumaticCraft.common.item.ItemAmadronTablet;
import pneumaticCraft.common.item.Itemss;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketAmadronTradeRemoved;
import pneumaticCraft.common.recipes.AmadronOffer;
import pneumaticCraft.common.recipes.AmadronOfferCustom;
import pneumaticCraft.common.recipes.AmadronOfferManager;
import pneumaticCraft.common.util.IOHelper;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.proxy.CommonProxy;

public class ContainerAmadron extends ContainerPneumaticBase{
    public static final int ROWS = 4;

    public List<AmadronOffer> offers = new ArrayList<AmadronOffer>(AmadronOfferManager.getInstance().getAllOffers());

    private final InventoryBasic inv = new InventoryBasic("amadron", true, ROWS * 4);
    @GuiSynced
    private final int[] shoppingItems = new int[8];
    @GuiSynced
    private final int[] shoppingAmounts = new int[8];
    @GuiSynced
    public boolean[] buyableOffers = new boolean[offers.size()];
    @GuiSynced
    public EnumProblemState problemState = EnumProblemState.NO_PROBLEMS;
    @GuiSynced
    public int maxOffers = 0;
    @GuiSynced
    public int currentOffers = 0;

    public static enum EnumProblemState{
        NO_PROBLEMS("noProblems"), NO_ITEM_PROVIDER("noItemProvider"), NO_FLUID_PROVIDER("noFluidProvider"), NOT_ENOUGH_ITEM_SPACE(
                "notEnoughItemSpace"), NOT_ENOUGH_FLUID_SPACE("notEnoughFluidSpace"), NOT_ENOUGH_ITEMS("notEnoughItems") /*not a ChickenBones reference*/, NOT_ENOUGH_FLUID(
                "notEnoughFluid"), OUT_OF_STOCK("outOfStock");
        private final String locKey;

        private EnumProblemState(String locKey){
            this.locKey = locKey;
        }

        public String getLocalizationKey(){
            return "gui.tab.problems.amadron." + locKey;
        }
    }

    public ContainerAmadron(EntityPlayer player){
        super(null);
        for(int y = 0; y < ROWS; y++) {
            for(int x = 0; x < 2; x++) {
                func_75146_a(new SlotUntouchable(inv, y * 4 + x * 2, x * 73 + 12, y * 35 + 70));
                func_75146_a(new SlotUntouchable(inv, y * 4 + x * 2 + 1, x * 73 + 57, y * 35 + 70));
            }
        }
        addSyncedFields(this);
        Arrays.fill(shoppingItems, -1);

        if(!player.field_70170_p.field_72995_K) {
            IInventory inv = ItemAmadronTablet.getItemProvider(player.func_71045_bC());
            IFluidHandler fluidHandler = ItemAmadronTablet.getLiquidProvider(player.func_71045_bC());
            for(int i = 0; i < offers.size(); i++) {
                int amount = capShoppingAmount(offers.get(i), 1, inv, fluidHandler, this);
                buyableOffers[i] = amount > 0;
            }
            problemState = EnumProblemState.NO_PROBLEMS;

            Map<AmadronOffer, Integer> shoppingCart = ItemAmadronTablet.getShoppingCart(player.func_71045_bC());
            for(Map.Entry<AmadronOffer, Integer> cartItem : shoppingCart.entrySet()) {
                int offerId = offers.indexOf(cartItem.getKey());
                if(offerId >= 0) {
                    int index = getCartSlot(offerId);
                    if(index >= 0) {
                        shoppingItems[index] = offerId;
                        shoppingAmounts[index] = cartItem.getValue();
                    }
                }
            }
            currentOffers = AmadronOfferManager.getInstance().countOffers(player.func_146103_bH().getId().toString());
            maxOffers = PneumaticCraftUtils.isPlayerOp(player) ? Integer.MAX_VALUE : AmadronOfferSettings.maxTradesPerPlayer;
        }
    }

    /**
     * args: slotID, itemStack to put in slot
     */
    @Override
    public void func_75141_a(int p_75141_1_, ItemStack p_75141_2_){

    }

    /**
     * places itemstacks in first x slots, x being aitemstack.lenght
     */
    @Override
    @SideOnly(Side.CLIENT)
    public void func_75131_a(ItemStack[] p_75131_1_){

    }

    @Override
    public boolean func_75145_c(EntityPlayer player){
        if(player.func_71045_bC() != null && player.func_71045_bC().func_77973_b() == Itemss.amadronTablet) {
            IPressurizable pressurizable = (IPressurizable)Itemss.amadronTablet;
            pressurizable.addAir(player.func_71045_bC(), -1);
            if(pressurizable.getPressure(player.func_71045_bC()) > 0) return true;
            else {
                player.func_145747_a(new ChatComponentTranslation("gui.tab.problems.notEnoughPressure"));
            }
        }
        return false;
    }

    public void clearStacks(){
        for(int i = 0; i < inv.func_70302_i_(); i++) {
            inv.func_70299_a(i, null);
        }
    }

    public void setStack(int index, ItemStack stack){
        inv.func_70299_a(index, stack);
    }

    @Override
    public ItemStack func_82846_b(EntityPlayer p_82846_1_, int p_82846_2_){
        return null;
    }

    public void clickOffer(int offerId, int mouseButton, boolean sneaking, EntityPlayer player){
        problemState = EnumProblemState.NO_PROBLEMS;
        int cartSlot = getCartSlot(offerId);
        if(cartSlot >= 0) {
            if(mouseButton == 2) {
                shoppingAmounts[cartSlot] = 0;
            } else if(sneaking) {
                if(mouseButton == 0) shoppingAmounts[cartSlot] /= 2;
                else {
                    AmadronOffer offer = offers.get(offerId);
                    if(offer instanceof AmadronOfferCustom) {
                        AmadronOfferCustom custom = (AmadronOfferCustom)offer;
                        if(custom.getPlayerId().equals(player.func_146103_bH().getId().toString())) {
                            if(AmadronOfferManager.getInstance().removeStaticOffer(custom)) {
                                if(AmadronOfferSettings.notifyOfTradeRemoval) NetworkHandler.sendToAll(new PacketAmadronTradeRemoved(custom));
                                custom.returnStock();
                                try {
                                    AmadronOfferStaticConfig.INSTANCE.writeToFile();
                                } catch(IOException e) {
                                    e.printStackTrace();
                                }
                                player.func_71053_j();
                            }
                        }
                    }
                    shoppingAmounts[cartSlot] *= 2;
                    if(shoppingAmounts[cartSlot] == 0) shoppingAmounts[cartSlot] = 1;
                }
            } else {
                if(mouseButton == 0) shoppingAmounts[cartSlot]--;
                else shoppingAmounts[cartSlot]++;
            }
            if(shoppingAmounts[cartSlot] <= 0) {
                shoppingAmounts[cartSlot] = 0;
                shoppingItems[cartSlot] = -1;
            } else {
                shoppingAmounts[cartSlot] = capShoppingAmount(offerId, shoppingAmounts[cartSlot], player);
                if(shoppingAmounts[cartSlot] > 0) shoppingItems[cartSlot] = offerId;
                else shoppingItems[cartSlot] = -1;
            }
        }
    }

    @Override
    public void handleGUIButtonPress(int guiID, EntityPlayer player){
        super.handleGUIButtonPress(guiID, player);
        if(guiID == 1) {
            for(int i = 0; i < shoppingItems.length; i++) {
                if(shoppingItems[i] >= 0) {
                    AmadronOffer offer = offers.get(shoppingItems[i]);
                    BlockPos itemPos = ItemAmadronTablet.getItemProvidingLocation(player.func_71045_bC());
                    World itemWorld = null;
                    if(itemPos == null) {
                        itemPos = new BlockPos((int)player.field_70165_t, (int)player.field_70163_u, (int)player.field_70161_v);
                        itemWorld = player.field_70170_p;
                    } else {
                        itemWorld = PneumaticCraftUtils.getWorldForDimension(ItemAmadronTablet.getItemProvidingDimension(player.func_71045_bC()));
                    }
                    BlockPos liquidPos = ItemAmadronTablet.getLiquidProvidingLocation(player.func_71045_bC());
                    World liquidWorld = null;
                    if(liquidPos != null) {
                        liquidWorld = PneumaticCraftUtils.getWorldForDimension(ItemAmadronTablet.getLiquidProvidingDimension(player.func_71045_bC()));
                    }
                    EntityDrone drone = retrieveOrderItems(offer, shoppingAmounts[i], itemWorld, itemPos, liquidWorld, liquidPos);
                    if(drone != null) drone.setHandlingOffer(offer, shoppingAmounts[i], player.func_71045_bC(), player.func_70005_c_());
                }
            }
            Arrays.fill(shoppingAmounts, 0);
            Arrays.fill(shoppingItems, -1);
        } else if(guiID == 2) {
            player.openGui(PneumaticCraft.instance, CommonProxy.EnumGuiId.AMADRON_ADD_TRADE.ordinal(), player.field_70170_p, 0, 0, 0);
        }
    }

    public static EntityDrone retrieveOrderItems(AmadronOffer offer, int times, World itemWorld, BlockPos itemPos, World liquidWorld, BlockPos liquidPos){
        if(offer.getInput() instanceof ItemStack) {
            if(itemWorld == null || itemPos == null) return null;
            ItemStack queryingItems = (ItemStack)offer.getInput();
            int amount = queryingItems.field_77994_a * times;
            List<ItemStack> stacks = new ArrayList<ItemStack>();
            while(amount > 0) {
                ItemStack stack = queryingItems.func_77946_l();
                stack.field_77994_a = Math.min(amount, stack.func_77976_d());
                stacks.add(stack);
                amount -= stack.field_77994_a;
            }
            return (EntityDrone)DroneRegistry.getInstance().retrieveItemsAmazonStyle(itemWorld, itemPos, stacks.toArray(new ItemStack[stacks.size()]));
        } else {
            if(liquidWorld == null || liquidPos == null) return null;
            FluidStack queryingFluid = ((FluidStack)offer.getInput()).copy();
            queryingFluid.amount *= times;
            return (EntityDrone)DroneRegistry.getInstance().retrieveFluidAmazonStyle(liquidWorld, liquidPos, queryingFluid);
        }
    }

    public int capShoppingAmount(int offerId, int wantedAmount, EntityPlayer player){
        IInventory inv = ItemAmadronTablet.getItemProvider(player.func_71045_bC());
        IFluidHandler fluidHandler = ItemAmadronTablet.getLiquidProvider(player.func_71045_bC());
        return capShoppingAmount(offers.get(offerId), wantedAmount, inv, fluidHandler, this);
    }

    public static int capShoppingAmount(AmadronOffer offer, int wantedAmount, IInventory inv, IFluidHandler fluidHandler, ContainerAmadron container){
        return capShoppingAmount(offer, wantedAmount, inv, inv, fluidHandler, fluidHandler, container);
    }

    public static int capShoppingAmount(AmadronOffer offer, int wantedAmount, IInventory inputInv, IInventory outputInv, IFluidHandler inputFluidHandler, IFluidHandler outputFluidHandler, ContainerAmadron container){
        if(container != null && offer.getStock() >= 0 && wantedAmount > offer.getStock()) {
            wantedAmount = offer.getStock();
            container.problemState = EnumProblemState.OUT_OF_STOCK;
        }
        if(offer.getInput() instanceof ItemStack) {
            if(inputInv != null) {
                ItemStack searchingItem = (ItemStack)offer.getInput();
                int count = 0;
                for(int i = 0; i < inputInv.func_70302_i_(); i++) {
                    if(inputInv.func_70301_a(i) != null && inputInv.func_70301_a(i).func_77969_a(searchingItem) && ItemStack.func_77970_a(inputInv.func_70301_a(i), searchingItem)) {
                        count += inputInv.func_70301_a(i).field_77994_a;
                    }
                }
                int maxAmount = count / ((ItemStack)offer.getInput()).field_77994_a;
                if(wantedAmount > maxAmount) {
                    if(container != null) container.problemState = EnumProblemState.NOT_ENOUGH_ITEMS;
                    wantedAmount = maxAmount;
                }
            } else if(outputInv == null) {
                wantedAmount = 0;
                if(container != null) container.problemState = EnumProblemState.NO_ITEM_PROVIDER;
            }
        } else {
            if(inputFluidHandler != null) {
                FluidStack searchingFluid = ((FluidStack)offer.getInput()).copy();
                searchingFluid.amount = Integer.MAX_VALUE;
                FluidStack extracted = inputFluidHandler.drain(EnumFacing.UP, searchingFluid, false);
                int maxAmount = 0;
                if(extracted != null) maxAmount = extracted.amount / ((FluidStack)offer.getInput()).amount;
                if(wantedAmount > maxAmount) {
                    if(container != null) container.problemState = EnumProblemState.NOT_ENOUGH_FLUID;
                    wantedAmount = maxAmount;
                }
            } else if(outputFluidHandler == null) {
                wantedAmount = 0;
                if(container != null) container.problemState = EnumProblemState.NO_FLUID_PROVIDER;
            }
        }
        if(offer.getOutput() instanceof ItemStack) {
            if(outputInv != null) {
                ItemStack providingItem = ((ItemStack)offer.getOutput()).func_77946_l();
                providingItem.field_77994_a *= wantedAmount;
                ItemStack remainder = IOHelper.insert(outputInv, providingItem.func_77946_l(), EnumFacing.UP, true);
                if(remainder != null) {
                    int maxAmount = (providingItem.field_77994_a - remainder.field_77994_a) / ((ItemStack)offer.getOutput()).field_77994_a;
                    if(wantedAmount > maxAmount) {
                        wantedAmount = maxAmount;
                        if(container != null) container.problemState = EnumProblemState.NOT_ENOUGH_ITEM_SPACE;
                    }
                }
            } else if(inputInv == null) {
                wantedAmount = 0;
                if(container != null) container.problemState = EnumProblemState.NO_ITEM_PROVIDER;
            }
        } else {
            if(outputFluidHandler != null) {
                FluidStack providingFluid = ((FluidStack)offer.getOutput()).copy();
                providingFluid.amount *= wantedAmount;
                int amountFilled = outputFluidHandler.fill(EnumFacing.UP, providingFluid, false);
                int maxAmount = amountFilled / ((FluidStack)offer.getOutput()).amount;
                if(wantedAmount > maxAmount) {
                    wantedAmount = maxAmount;
                    if(container != null) container.problemState = EnumProblemState.NOT_ENOUGH_FLUID_SPACE;
                }
            } else if(inputFluidHandler == null) {
                wantedAmount = 0;
                if(container != null) container.problemState = EnumProblemState.NO_FLUID_PROVIDER;
            }
        }
        return wantedAmount;
    }

    public int getCartSlot(int offerId){
        int freeSlot = -1;
        for(int i = 0; i < shoppingItems.length; i++) {
            if(shoppingItems[i] == offerId) {
                return i;
            } else if(freeSlot == -1 && shoppingItems[i] == -1) {
                freeSlot = i;
            }
        }
        return freeSlot;
    }

    public int getShoppingCartAmount(AmadronOffer offer){
        int offerId = offers.indexOf(offer);
        for(int i = 0; i < shoppingItems.length; i++) {
            if(shoppingItems[i] == offerId) {
                return shoppingAmounts[i];
            }
        }
        return 0;
    }

    @Override
    public void func_75134_a(EntityPlayer player){
        super.func_75134_a(player);
        if(!player.field_70170_p.field_72995_K && player.func_71045_bC() != null && player.func_71045_bC().func_77973_b() == Itemss.amadronTablet) {
            Map<AmadronOffer, Integer> shoppingCart = new HashMap<AmadronOffer, Integer>();
            for(int i = 0; i < shoppingItems.length; i++) {
                if(shoppingItems[i] >= 0) {
                    shoppingCart.put(offers.get(shoppingItems[i]), shoppingAmounts[i]);
                }
            }
            ItemAmadronTablet.setShoppingCart(player.func_71045_bC(), shoppingCart);
        }
    }
}
