package pneumaticCraft.common.inventory;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.ICrafting;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraftforge.fml.common.network.simpleimpl.IMessage;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.NetworkUtils;
import pneumaticCraft.common.network.PacketUpdateGui;
import pneumaticCraft.common.tileentity.IGUIButtonSensitive;
import pneumaticCraft.common.tileentity.TileEntityBase;

public class ContainerPneumaticBase<Tile extends TileEntityBase> extends Container implements IGUIButtonSensitive{

    public Tile te;
    private final List<SyncedField> syncedFields = new ArrayList<SyncedField>();
    private boolean firstTick = true;
    private int playerSlotsStart;

    public ContainerPneumaticBase(Tile te){
        this.te = te;
        if(te != null) addSyncedFields(te);
    }

    protected void addSyncedField(SyncedField field){
        syncedFields.add(field);
        field.setLazy(false);
    }

    protected void addSyncedFields(Object annotatedObject){
        List<SyncedField> fields = NetworkUtils.getSyncedFields(annotatedObject, GuiSynced.class);
        for(SyncedField field : fields)
            addSyncedField(field);
    }

    public void updateField(int index, Object value){
        syncedFields.get(index).setValue(value);
        if(te != null) te.onGuiUpdate();
    }

    @Override
    public boolean func_75145_c(EntityPlayer player){
        return te.isGuiUseableByPlayer(player);
    }

    @Override
    public void func_75142_b(){
        super.func_75142_b();
        for(int i = 0; i < syncedFields.size(); i++) {
            if(syncedFields.get(i).update() || firstTick) {
                sendToCrafters(new PacketUpdateGui(i, syncedFields.get(i)));
            }
        }
        firstTick = false;
    }

    protected void sendToCrafters(IMessage message){
        for(ICrafting crafter : field_75149_d) {
            if(crafter instanceof EntityPlayerMP) {
                NetworkHandler.sendTo(message, (EntityPlayerMP)crafter);
            }
        }
    }

    protected void addPlayerSlots(InventoryPlayer inventoryPlayer, int yOffset){
        playerSlotsStart = field_75151_b.size();

        // Add the player's inventory slots to the container
        for(int inventoryRowIndex = 0; inventoryRowIndex < 3; ++inventoryRowIndex) {
            for(int inventoryColumnIndex = 0; inventoryColumnIndex < 9; ++inventoryColumnIndex) {
                func_75146_a(new Slot(inventoryPlayer, inventoryColumnIndex + inventoryRowIndex * 9 + 9, 8 + inventoryColumnIndex * 18, yOffset + inventoryRowIndex * 18));
            }
        }

        // Add the player's action bar slots to the container
        for(int actionBarSlotIndex = 0; actionBarSlotIndex < 9; ++actionBarSlotIndex) {
            func_75146_a(new Slot(inventoryPlayer, actionBarSlotIndex, 8 + actionBarSlotIndex * 18, yOffset + 58));
        }
    }

    @Override
    public ItemStack func_82846_b(EntityPlayer par1EntityPlayer, int par2){

        ItemStack var3 = null;
        Slot var4 = field_75151_b.get(par2);

        if(var4 != null && var4.func_75216_d()) {
            ItemStack var5 = var4.func_75211_c();
            var3 = var5.func_77946_l();

            if(par2 < playerSlotsStart) {
                if(!func_75135_a(var5, playerSlotsStart, playerSlotsStart + 36, false)) return null;

                var4.func_75220_a(var5, var3);
            } else {
                /* int validStartSlot = -1; Unncessary, as Forge now adds respect to slot.isItemValid to mergeItemStack
                 for(int i = 0; i < playerSlotsStart; i++) {
                     boolean slotValid = inventorySlots.get(i).isItemValid(var5);
                     if(slotValid && validStartSlot == -1) {
                         validStartSlot = i;
                     } else if(!slotValid && validStartSlot >= 0) {
                         if(!mergeItemStack(var5, validStartSlot, i, false)) return null;
                         validStartSlot = -1;
                     }
                 }
                 if(validStartSlot >= 0) {
                     if(!mergeItemStack(var5, validStartSlot, playerSlotsStart, false)) return null;
                 }*/
                if(!func_75135_a(var5, 0, playerSlotsStart, false)) return null;
                var4.func_75220_a(var5, var3);
            }

            if(var5.field_77994_a == 0) {
                var4.func_75215_d((ItemStack)null);
            } else {
                var4.func_75218_e();
            }

            if(var5.field_77994_a == var3.field_77994_a) return null;

            var4.func_82870_a(par1EntityPlayer, var5);
        }

        return var3;
    }

    /**
     * Source: Buildcraft
     */
    @Override
    public ItemStack func_75144_a(int slotNum, int modifier, int mouseButton, EntityPlayer player){
        Slot slot = slotNum < 0 ? null : (Slot)field_75151_b.get(slotNum);
        if(slot instanceof IPhantomSlot) {
            return slotClickPhantom(slot, modifier, mouseButton, player);
        }
        return super.func_75144_a(slotNum, modifier, mouseButton, player);
    }

    private ItemStack slotClickPhantom(Slot slot, int mouseButton, int modifier, EntityPlayer player){
        ItemStack stack = null;

        if(mouseButton == 2) {
            if(((IPhantomSlot)slot).canAdjust()) {
                slot.func_75215_d(null);
            }
        } else if(mouseButton == 0 || mouseButton == 1) {
            InventoryPlayer playerInv = player.field_71071_by;
            slot.func_75218_e();
            ItemStack stackSlot = slot.func_75211_c();
            ItemStack stackHeld = playerInv.func_70445_o();

            if(stackSlot != null) {
                stack = stackSlot.func_77946_l();
            }

            if(stackSlot == null) {
                if(stackHeld != null && slot.func_75214_a(stackHeld)) {
                    fillPhantomSlot(slot, stackHeld, mouseButton, modifier);
                }
            } else if(stackHeld == null) {
                adjustPhantomSlot(slot, mouseButton, modifier);
                slot.func_82870_a(player, playerInv.func_70445_o());
            } else if(slot.func_75214_a(stackHeld)) {
                if(canStacksMerge(stackSlot, stackHeld)) {
                    adjustPhantomSlot(slot, mouseButton, modifier);
                } else {
                    fillPhantomSlot(slot, stackHeld, mouseButton, modifier);
                }
            }
        }
        return stack;
    }

    public boolean canStacksMerge(ItemStack stack1, ItemStack stack2){
        if(stack1 == null || stack2 == null) return false;
        if(!stack1.func_77969_a(stack2)) return false;
        if(!ItemStack.func_77970_a(stack1, stack2)) return false;
        return true;

    }

    protected void adjustPhantomSlot(Slot slot, int mouseButton, int modifier){
        if(!((IPhantomSlot)slot).canAdjust()) {
            return;
        }
        ItemStack stackSlot = slot.func_75211_c();
        int stackSize;
        if(modifier == 1) {
            stackSize = mouseButton == 0 ? (stackSlot.field_77994_a + 1) / 2 : stackSlot.field_77994_a * 2;
        } else {
            stackSize = mouseButton == 0 ? stackSlot.field_77994_a - 1 : stackSlot.field_77994_a + 1;
        }

        if(stackSize > slot.func_75219_a()) {
            stackSize = slot.func_75219_a();
        }

        stackSlot.field_77994_a = stackSize;

        if(stackSlot.field_77994_a <= 0) {
            slot.func_75215_d((ItemStack)null);
        }
    }

    protected void fillPhantomSlot(Slot slot, ItemStack stackHeld, int mouseButton, int modifier){
        if(!((IPhantomSlot)slot).canAdjust()) {
            return;
        }
        int stackSize = mouseButton == 0 ? stackHeld.field_77994_a : 1;
        if(stackSize > slot.func_75219_a()) {
            stackSize = slot.func_75219_a();
        }
        ItemStack phantomStack = stackHeld.func_77946_l();
        phantomStack.field_77994_a = stackSize;

        slot.func_75215_d(phantomStack);
    }

    @Override
    public void handleGUIButtonPress(int guiID, EntityPlayer player){
        if(te instanceof IGUIButtonSensitive) {
            ((IGUIButtonSensitive)te).handleGUIButtonPress(guiID, player);
        }
    }
}
