// Code used from the Backpack Mod,
// https://github.com/Eydamos/Minecraft-Backpack-Mod

package pneumaticCraft.common.inventory;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.InventoryBasic;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import pneumaticCraft.common.NBTUtil;
import pneumaticCraft.common.tileentity.TileEntityChargingStation;
import pneumaticCraft.lib.Log;

public class InventoryPneumaticInventoryItem extends InventoryBasic{
    // the title of the backpack
    protected String inventoryTitle;
    // the original ItemStack to compare with the player inventory
    public ItemStack armorStack;
    private final TileEntityChargingStation te;

    // if class is reading from NBT tag
    protected boolean reading = false;

    public InventoryPneumaticInventoryItem(TileEntityChargingStation te){
        super("", false, getInventorySize(te.func_70301_a(TileEntityChargingStation.CHARGE_INVENTORY_INDEX)));
        this.te = te;
        armorStack = te.func_70301_a(TileEntityChargingStation.CHARGE_INVENTORY_INDEX);

        // check if inventory exists if not create one
        if(!hasInventory()) {
            createInventory();
        }
        loadInventory();
    }

    /**
     * Is called whenever something is changed in the inventory.
     */
    @Override
    public void func_70296_d(){
        super.func_70296_d();
        // if reading from NBT don't write
        if(!reading) {
            saveInventory();
        }
    }

    /**
     * This method is called when the chest opens the inventory. It loads the
     * content of the inventory and its title.
     */
    @Override
    public void func_174889_b(EntityPlayer player){
        loadInventory();
    }

    /**
     * This method is called when the chest closes the inventory. It then throws
     * out every backpack which is inside the backpack and saves the inventory.
     */
    @Override
    public void func_174886_c(EntityPlayer player){
        saveInventory();
    }

    /**
     * Returns the name of the inventory.
     */
    @Override
    public String func_70005_c_(){
        return "Pneumatic Helmet";
    }

    protected static int getInventorySize(ItemStack is){
        return 9;
    }

    /**
     * Returns if an Inventory is saved in the NBT.
     * 
     * @return True when the NBT is not null and the NBT has key "UpgradeInventory"
     *         otherwise false.
     */
    protected boolean hasInventory(){
        return NBTUtil.hasTag(armorStack, "UpgradeInventory");
    }

    /**
     * Creates the Inventory Tag in the NBT with an empty inventory.
     */
    protected void createInventory(){
        writeToNBT();
    }

    /**
     * If there is no inventory create one. Then load the content and title of
     * the inventory from the NBT
     */
    public void loadInventory(){
        readFromNBT();
    }

    /**
     * Saves the actual content of the inventory to the NBT.
     */
    public void saveInventory(){
        writeToNBT();
        //   updateToChargingStation();
    }

    private void updateToChargingStation(){
        te.func_70299_a(TileEntityChargingStation.CHARGE_INVENTORY_INDEX, armorStack);
    }

    /**
     * Writes a NBT Node with inventory.
     * 
     * @param outerTag
     *            The NBT Node to write to.
     * @return The written NBT Node.
     */
    public void writeToNBT(){
        NBTTagList itemList = new NBTTagList();
        for(int i = 0; i < func_70302_i_(); i++) {
            if(func_70301_a(i) != null) {
                NBTTagCompound slotEntry = new NBTTagCompound();
                slotEntry.func_74774_a("Slot", (byte)i);
                func_70301_a(i).func_77955_b(slotEntry);
                itemList.func_74742_a(slotEntry);
            }
        }
        // save content in Inventory->Items
        NBTTagCompound inventory = new NBTTagCompound();
        inventory.func_74782_a("Items", itemList);
        NBTUtil.setCompoundTag(armorStack, "UpgradeInventory", inventory);
        NBTUtil.removeTag(armorStack, "Inventory");
        // return outerTag;
    }

    /**
     * Reads the inventory from a NBT Node.
     * 
     * @param outerTag
     *            The NBT Node to read from.
     */
    protected void readFromNBT(){
        reading = true;
        if(NBTUtil.hasTag(armorStack, "Inventory") && armorStack.func_77978_p().func_74781_a("Inventory") instanceof NBTTagCompound) {
            Log.info("Converting 'Inventory' tag to 'UpgradeInventory' in Pneumatic items");
            armorStack.func_77978_p().func_74782_a("UpgradeInventory", armorStack.func_77978_p().func_74781_a("Inventory"));
            armorStack.func_77978_p().func_82580_o("Inventory");
        }
        NBTTagList itemList = NBTUtil.getCompoundTag(armorStack, "UpgradeInventory").func_150295_c("Items", 10);
        for(int i = 0; i < itemList.func_74745_c(); i++) {
            NBTTagCompound slotEntry = itemList.func_150305_b(i);
            int j = slotEntry.func_74771_c("Slot");

            if(j >= 0 && j < func_70302_i_()) {
                func_70299_a(j, ItemStack.func_77949_a(slotEntry));
            }
        }
        reading = false;
    }

}
