package pneumaticCraft.common.item;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.minecraft.client.resources.I18n;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.ChatComponentTranslation;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;
import net.minecraftforge.fluids.IFluidHandler;
import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.common.NBTUtil;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketSyncAmadronOffers;
import pneumaticCraft.common.recipes.AmadronOffer;
import pneumaticCraft.common.recipes.AmadronOfferCustom;
import pneumaticCraft.common.recipes.AmadronOfferManager;
import pneumaticCraft.common.util.IOHelper;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.proxy.CommonProxy.EnumGuiId;

public class ItemAmadronTablet extends ItemPressurizable implements IAmadronInterface{

    public ItemAmadronTablet(String textureLocation, int maxAir, int volume){
        super(textureLocation, maxAir, volume);
        func_77625_d(1);
    }

    /**
     * Called whenever this item is equipped and the right mouse button is pressed. Args: itemStack, world, entityPlayer
     */
    @Override
    public ItemStack func_77659_a(ItemStack stack, World world, EntityPlayer player){
        if(!world.field_72995_K) {
            NetworkHandler.sendTo(new PacketSyncAmadronOffers(AmadronOfferManager.getInstance().getAllOffers()), (EntityPlayerMP)player);
            player.openGui(PneumaticCraft.instance, EnumGuiId.AMADRON.ordinal(), player.field_70170_p, (int)player.field_70165_t, (int)player.field_70163_u, (int)player.field_70161_v);
        }
        return stack;
    }

    @Override
    public boolean func_180614_a(ItemStack tablet, EntityPlayer player, World world, BlockPos pos, EnumFacing side, float par8, float par9, float par10){

        TileEntity te = world.func_175625_s(pos);
        if(te instanceof IFluidHandler) {
            if(!world.field_72995_K) {
                setLiquidProvidingLocation(tablet, pos, world.field_73011_w.func_177502_q());
                player.func_146105_b(new ChatComponentTranslation("message.amadronTable.setLiquidProvidingLocation", pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p(), world.field_73011_w.func_177502_q(), world.field_73011_w.func_80007_l()));
            }
        } else if(te instanceof IInventory) {
            if(!world.field_72995_K) {
                setItemProvidingLocation(tablet, pos, world.field_73011_w.func_177502_q());
                player.func_146105_b(new ChatComponentTranslation("message.amadronTable.setItemProvidingLocation", pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p(), world.field_73011_w.func_177502_q(), world.field_73011_w.func_80007_l()));
            }
        } else {
            return false;
        }
        return true;

    }

    @Override
    public void func_77624_a(ItemStack stack, EntityPlayer player, List infoList, boolean par4){
        super.func_77624_a(stack, player, infoList, par4);
        BlockPos pos = getItemProvidingLocation(stack);
        if(pos != null) {
            int dim = getItemProvidingDimension(stack);
            infoList.add(I18n.func_135052_a("gui.tooltip.amadronTablet.itemLocation", pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p(), dim));
        } else {
            infoList.add(I18n.func_135052_a("gui.tooltip.amadronTablet.selectItemLocation"));
        }

        pos = getLiquidProvidingLocation(stack);
        if(pos != null) {
            int dim = getLiquidProvidingDimension(stack);
            infoList.add(I18n.func_135052_a("gui.tooltip.amadronTablet.fluidLocation", pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p(), dim));
        } else {
            infoList.add(I18n.func_135052_a("gui.tooltip.amadronTablet.selectFluidLocation"));
        }
    }

    public static IInventory getItemProvider(ItemStack tablet){
        BlockPos pos = getItemProvidingLocation(tablet);
        if(pos != null) {
            int dimension = getItemProvidingDimension(tablet);
            TileEntity te = PneumaticCraftUtils.getTileEntity(pos, dimension);
            return IOHelper.getInventoryForTE(te);
        }
        return null;
    }

    public static BlockPos getItemProvidingLocation(ItemStack tablet){
        NBTTagCompound compound = tablet.func_77978_p();
        if(compound != null) {
            int x = compound.func_74762_e("itemX");
            int y = compound.func_74762_e("itemY");
            int z = compound.func_74762_e("itemZ");
            if(x != 0 || y != 0 || z != 0) {
                return new BlockPos(x, y, z);
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    public static int getItemProvidingDimension(ItemStack tablet){
        return tablet.func_77942_o() ? tablet.func_77978_p().func_74762_e("itemDim") : 0;
    }

    public static void setItemProvidingLocation(ItemStack tablet, BlockPos pos, int dimensionId){
        NBTUtil.setInteger(tablet, "itemX", pos.func_177958_n());
        NBTUtil.setInteger(tablet, "itemY", pos.func_177956_o());
        NBTUtil.setInteger(tablet, "itemZ", pos.func_177952_p());
        NBTUtil.setInteger(tablet, "itemDim", dimensionId);
    }

    public static IFluidHandler getLiquidProvider(ItemStack tablet){
        BlockPos pos = getLiquidProvidingLocation(tablet);
        if(pos != null) {
            int dimension = getLiquidProvidingDimension(tablet);
            TileEntity te = PneumaticCraftUtils.getTileEntity(pos, dimension);
            if(te instanceof IFluidHandler) return (IFluidHandler)te;
        }
        return null;
    }

    public static BlockPos getLiquidProvidingLocation(ItemStack tablet){
        NBTTagCompound compound = tablet.func_77978_p();
        if(compound != null) {
            int x = compound.func_74762_e("liquidX");
            int y = compound.func_74762_e("liquidY");
            int z = compound.func_74762_e("liquidZ");
            if(x != 0 || y != 0 || z != 0) {
                return new BlockPos(x, y, z);
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    public static int getLiquidProvidingDimension(ItemStack tablet){
        return tablet.func_77942_o() ? tablet.func_77978_p().func_74762_e("liquidDim") : 0;
    }

    public static void setLiquidProvidingLocation(ItemStack tablet, BlockPos pos, int dimensionId){
        NBTUtil.setInteger(tablet, "liquidX", pos.func_177958_n());
        NBTUtil.setInteger(tablet, "liquidY", pos.func_177956_o());
        NBTUtil.setInteger(tablet, "liquidZ", pos.func_177952_p());
        NBTUtil.setInteger(tablet, "liquidDim", dimensionId);
    }

    public static Map<AmadronOffer, Integer> getShoppingCart(ItemStack tablet){
        Map<AmadronOffer, Integer> offers = new HashMap<AmadronOffer, Integer>();

        if(tablet.func_77942_o() && tablet.func_77978_p().func_74764_b("shoppingCart")) {
            NBTTagList list = tablet.func_77978_p().func_150295_c("shoppingCart", 10);
            for(int i = 0; i < list.func_74745_c(); i++) {
                NBTTagCompound tag = list.func_150305_b(i);
                offers.put(tag.func_74764_b("inStock") ? AmadronOfferCustom.loadFromNBT(tag) : AmadronOffer.loadFromNBT(tag), tag.func_74762_e("amount"));
            }
        }
        return offers;
    }

    public static void setShoppingCart(ItemStack tablet, Map<AmadronOffer, Integer> cart){
        NBTTagList list = new NBTTagList();
        for(Map.Entry<AmadronOffer, Integer> entry : cart.entrySet()) {
            NBTTagCompound tag = new NBTTagCompound();
            entry.getKey().writeToNBT(tag);
            tag.func_74768_a("amount", entry.getValue());
            list.func_74742_a(tag);
        }
        NBTUtil.setCompoundTag(tablet, "shoppingCart", list);
    }
}
