package pneumaticCraft.common.item;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.entity.IEntityLivingData;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.item.IPressurizable;
import pneumaticCraft.api.item.IProgrammable;
import pneumaticCraft.api.item.IUpgradeAcceptor;
import pneumaticCraft.common.NBTUtil;
import pneumaticCraft.common.entity.living.EntityDrone;
import pneumaticCraft.common.progwidgets.IProgWidget;
import pneumaticCraft.common.tileentity.TileEntityProgrammer;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.Log;
import pneumaticCraft.lib.PneumaticValues;
import pneumaticCraft.proxy.CommonProxy.EnumGuiId;

public class ItemDrone extends ItemPneumatic implements IPressurizable, IChargingStationGUIHolderItem, IProgrammable,
        IUpgradeAcceptor{

    public ItemDrone(){
        func_77625_d(1);
    }

    @Override
    public boolean func_180614_a(ItemStack iStack, EntityPlayer player, World world, BlockPos pos, EnumFacing side, float vecX, float vecY, float vecZ){
        if(!world.field_72995_K) {
            EntityDrone drone = new EntityDrone(world, player);

            BlockPos placePos = pos.func_177972_a(side);
            drone.func_70107_b(placePos.func_177958_n() + 0.5, placePos.func_177956_o() + 0.5, placePos.func_177952_p() + 0.5);
            world.func_72838_d(drone);

            NBTTagCompound stackTag = iStack.func_77978_p();
            NBTTagCompound entityTag = new NBTTagCompound();
            drone.func_70014_b(entityTag);
            if(stackTag != null) {
                entityTag.func_74782_a("widgets", stackTag.func_150295_c("widgets", 10).func_74737_b());
                entityTag.func_74776_a("currentAir", stackTag.func_74760_g("currentAir"));
                entityTag.func_74768_a("color", stackTag.func_74762_e("color"));
                NBTTagCompound invTag = stackTag.func_74775_l("UpgradeInventory");
                if(invTag != null) entityTag.func_74782_a("Inventory", invTag.func_74737_b());
            }
            drone.func_70037_a(entityTag);
            if(iStack.func_82837_s()) drone.func_96094_a(iStack.func_82833_r());

            drone.naturallySpawned = false;
            //TODO 1.8 check if valid replacement drone.onSpawnWithEgg(null);
            drone.func_180482_a(world.func_175649_E(placePos), (IEntityLivingData)null);
            iStack.field_77994_a--;
        }
        return true;
    }

    public static void setProgWidgets(List<IProgWidget> widgets, ItemStack iStack){
        NBTUtil.initNBTTagCompound(iStack);
        TileEntityProgrammer.setWidgetsToNBT(widgets, iStack.func_77978_p());
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void func_150895_a(Item par1, CreativeTabs tab, List subItems){
        subItems.add(new ItemStack(this));
        ItemStack chargedStack = new ItemStack(this);
        addAir(chargedStack, (int)(PneumaticValues.DRONE_VOLUME * PneumaticValues.DRONE_MAX_PRESSURE));
        subItems.add(chargedStack);
    }

    @Override
    @SideOnly(Side.CLIENT)
    /**
     * allows items to add custom lines of information to the mouseover description
     */
    public void func_77624_a(ItemStack stack, EntityPlayer player, List list, boolean par4){
        list.add("Pressure: " + PneumaticCraftUtils.roundNumberTo(getPressure(stack), 1) + " bar");
        super.func_77624_a(stack, player, list, par4);
    }

    @Override
    public float getPressure(ItemStack iStack){
        float volume = ItemPneumaticArmor.getUpgrades(EnumUpgrade.VOLUME, iStack) * PneumaticValues.VOLUME_VOLUME_UPGRADE + PneumaticValues.DRONE_VOLUME;
        float oldVolume = NBTUtil.getFloat(iStack, "volume");
        if(volume < oldVolume) {
            float currentAir = NBTUtil.getFloat(iStack, "currentAir");
            currentAir *= volume / oldVolume;
            NBTUtil.setFloat(iStack, "currentAir", currentAir);
        }
        NBTUtil.setFloat(iStack, "volume", volume);
        return NBTUtil.getFloat(iStack, "currentAir") / volume;
    }

    @Override
    public void addAir(ItemStack iStack, int amount){
        NBTUtil.setFloat(iStack, "currentAir", NBTUtil.getFloat(iStack, "currentAir") + amount);
    }

    @Override
    public float maxPressure(ItemStack iStack){
        return PneumaticValues.DRONE_MAX_PRESSURE;
    }

    @Override
    public EnumGuiId getGuiID(){
        return EnumGuiId.DRONE;
    }

    @Override
    public boolean canProgram(ItemStack stack){
        return true;
    }

    @Override
    public boolean usesPieces(ItemStack stack){
        return true;
    }

    @Override
    public boolean showProgramTooltip(){
        return true;
    }

    /**
     * Called each tick as long the item is on a player inventory. Uses by maps to check if is on a player hand and
     * update it's contents.
     */
    @Override
    public void func_77663_a(ItemStack iStack, World world, Entity p_77663_3_, int p_77663_4_, boolean p_77663_5_){
        if(!world.field_72995_K && NBTUtil.hasTag(iStack, "Inventory") && iStack.func_77978_p().func_74781_a("Inventory") instanceof NBTTagCompound) {
            Log.info("Converting 'Inventory' tag to 'UpgradeInventory' in Pneumatic items");
            iStack.func_77978_p().func_74782_a("UpgradeInventory", iStack.func_77978_p().func_74781_a("Inventory"));
            iStack.func_77978_p().func_82580_o("Inventory");
        }
    }

    @Override
    public Set<Item> getApplicableUpgrades(){
        Set<Item> set = new HashSet<Item>();
        set.add(Itemss.upgrades.get(EnumUpgrade.VOLUME));
        set.add(Itemss.upgrades.get(EnumUpgrade.DISPENSER));
        set.add(Itemss.upgrades.get(EnumUpgrade.ITEM_LIFE));
        set.add(Itemss.upgrades.get(EnumUpgrade.SECURITY));
        set.add(Itemss.upgrades.get(EnumUpgrade.SPEED));
        set.add(Itemss.upgrades.get(EnumUpgrade.ENTITY_TRACKER));
        return set;
    }

    @Override
    public String getName(){
        return func_77658_a() + ".name";
    }

}
