package pneumaticCraft.common.item;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.minecraft.client.model.ModelBiped;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemArmor;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.EnumChatFormatting;
import net.minecraftforge.fml.client.FMLClientHandler;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.api.client.pneumaticHelmet.IUpgradeRenderHandler;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.item.IPressurizable;
import pneumaticCraft.api.item.IUpgradeAcceptor;
import pneumaticCraft.client.render.pneumaticArmor.RenderCoordWireframe;
import pneumaticCraft.client.render.pneumaticArmor.UpgradeRenderHandlerList;
import pneumaticCraft.common.DateEventHandler;
import pneumaticCraft.common.NBTUtil;
import pneumaticCraft.common.config.Config;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.PneumaticValues;
import pneumaticCraft.lib.Textures;
import pneumaticCraft.proxy.CommonProxy.EnumGuiId;

//TODO 1.8 Thaumcraft dep @Optional.InterfaceList({@Interface(iface = "thaumcraft.api.IRepairable", modid = ModIds.THAUMCRAFT), @Interface(iface = "thaumcraft.api.IGoggles", modid = ModIds.THAUMCRAFT), @Interface(iface = "thaumcraft.api.IVisDiscountGear", modid = ModIds.THAUMCRAFT), @Interface(iface = "thaumcraft.api.nodes.IRevealer", modid = ModIds.THAUMCRAFT)})
public class ItemPneumaticArmor extends ItemArmor implements IPressurizable, IChargingStationGUIHolderItem,
        IUpgradeAcceptor/*,
IRepairable, IRevealer, IGoggles, IVisDiscountGear*/{

    public ItemPneumaticArmor(ItemArmor.ArmorMaterial par2EnumArmorMaterial, int renderIndex, int armorType, int maxAir){
        super(par2EnumArmorMaterial, renderIndex, armorType);
        func_77656_e(maxAir);
        func_77637_a(PneumaticCraft.tabPneumaticCraft);
    }

    @Override
    public String getArmorTexture(ItemStack stack, Entity entity, int slot, String type){
        return Textures.ARMOR_PNEUMATIC + "_1.png";
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void func_150895_a(Item par1, CreativeTabs tab, List subItems){
        subItems.add(new ItemStack(this));
        ItemStack chargedStack = new ItemStack(this);
        addAir(chargedStack, PneumaticValues.PNEUMATIC_HELMET_VOLUME * 10);
        subItems.add(chargedStack);
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void func_77624_a(ItemStack iStack, EntityPlayer player, List textList, boolean par4){
        float pressure = getPressure(iStack);
        textList.add((pressure < 0.5F ? EnumChatFormatting.RED : EnumChatFormatting.DARK_GREEN) + "Pressure: " + Math.round(pressure * 10D) / 10D + " bar");
        ItemStack[] inventoryStacks = getUpgradeStacks(iStack);
        boolean isArmorEmpty = true;
        for(ItemStack stack : inventoryStacks) {
            if(stack != null) {
                isArmorEmpty = false;
                break;
            }
        }
        if(isArmorEmpty) {
            textList.add("Insert in Charging Station to install upgrades");
        } else {
            textList.add("Upgrades installed:");
            PneumaticCraftUtils.sortCombineItemStacksAndToString(textList, inventoryStacks);
            ItemStack searchedStack = getSearchedStack(iStack);
            if(searchedStack != null) {
                for(int i = 0; i < textList.size(); i++) {
                    if(((String)textList.get(i)).contains("Item Search")) {
                        textList.set(i, textList.get(i) + " (searching " + searchedStack.func_82833_r() + ")");
                        break;
                    }
                }
            }
            RenderCoordWireframe coordHandler = getCoordTrackLocation(iStack);
            if(coordHandler != null) {
                for(int i = 0; i < textList.size(); i++) {
                    if(((String)textList.get(i)).contains("Coordinate Tracker")) {
                        textList.set(i, textList.get(i) + " (tracking " + coordHandler.pos.func_177958_n() + ", " + coordHandler.pos.func_177956_o() + ", " + coordHandler.pos.func_177952_p() + " in " + coordHandler.worldObj.field_73011_w.func_80007_l() + ")");
                        break;
                    }
                }
            }
        }
        ItemPneumatic.addTooltip(iStack, player, textList);
    }

    /**
     * Retrieves the upgrades currently installed on the given armor stack.
     */
    public static ItemStack[] getUpgradeStacks(ItemStack iStack){
        NBTTagCompound tag = NBTUtil.getCompoundTag(iStack, "UpgradeInventory");
        ItemStack[] inventoryStacks = new ItemStack[9];
        if(tag != null) {
            NBTTagList itemList = tag.func_150295_c("Items", 10);
            if(itemList != null) {
                for(int i = 0; i < itemList.func_74745_c(); i++) {
                    NBTTagCompound slotEntry = itemList.func_150305_b(i);
                    int j = slotEntry.func_74771_c("Slot");
                    if(j >= 0 && j < 9) {
                        inventoryStacks[j] = ItemStack.func_77949_a(slotEntry);
                    }
                }
            }
        }
        return inventoryStacks;
    }

    public static int getUpgrades(EnumUpgrade upgrade, ItemStack stack){
        return getUpgrades(Itemss.upgrades.get(upgrade), stack);
    }

    public static int getUpgrades(Item upgrade, ItemStack iStack){
        int upgrades = 0;
        ItemStack[] stacks = getUpgradeStacks(iStack);
        for(ItemStack stack : stacks) {
            if(stack != null && stack.func_77973_b() == upgrade) {
                upgrades += stack.field_77994_a;
            }
        }
        return upgrades;
    }

    @SideOnly(Side.CLIENT)
    public static ItemStack getSearchedStack(){
        return getSearchedStack(PneumaticCraft.proxy.getPlayer().func_82169_q(3));
    }

    public static ItemStack getSearchedStack(ItemStack helmetStack){
        if(helmetStack == null || !NBTUtil.hasTag(helmetStack, "SearchStack")) return null;
        NBTTagCompound tag = NBTUtil.getCompoundTag(helmetStack, "SearchStack");
        if(tag.func_74762_e("itemID") == -1) return null;
        return new ItemStack(Item.func_150899_d(tag.func_74762_e("itemID")), 1, tag.func_74762_e("itemDamage"));
    }

    @SideOnly(Side.CLIENT)
    public static RenderCoordWireframe getCoordTrackLocation(ItemStack helmetStack){
        if(helmetStack == null || !NBTUtil.hasTag(helmetStack, "CoordTracker")) return null;
        NBTTagCompound tag = NBTUtil.getCompoundTag(helmetStack, "CoordTracker");
        if(tag.func_74762_e("y") == -1 || FMLClientHandler.instance().getClient().field_71441_e.field_73011_w.func_177502_q() != tag.func_74762_e("dimID")) return null;
        return new RenderCoordWireframe(FMLClientHandler.instance().getClient().field_71441_e, NBTUtil.getPos(tag));
    }

    @SideOnly(Side.CLIENT)
    public static String getEntityFilter(ItemStack helmetStack){
        if(helmetStack == null || !NBTUtil.hasTag(helmetStack, "entityFilter")) return "";
        return NBTUtil.getString(helmetStack, "entityFilter");
    }

    public static void setEntityFilter(ItemStack helmetStack, String filter){
        if(helmetStack != null) {
            NBTUtil.setString(helmetStack, "entityFilter", filter);
        }
    }

    @Override
    public boolean func_82789_a(ItemStack par1ItemStack, ItemStack par2ItemStack){
        return false;
    }

    @Override
    public float getPressure(ItemStack iStack){
        int volume = ItemPneumaticArmor.getUpgrades(EnumUpgrade.VOLUME, iStack) * PneumaticValues.VOLUME_VOLUME_UPGRADE + PneumaticValues.PNEUMATIC_HELMET_VOLUME;
        int oldVolume = NBTUtil.getInteger(iStack, "volume");
        if(volume < oldVolume) {
            int currentAir = NBTUtil.getInteger(iStack, "air");
            currentAir = currentAir * volume / oldVolume;
            NBTUtil.setInteger(iStack, "air", currentAir);
        }
        NBTUtil.setInteger(iStack, "volume", volume);
        return (float)NBTUtil.getInteger(iStack, "air") / volume;
    }

    public boolean hasSufficientPressure(ItemStack iStack){
        return getPressure(iStack) > 0F;
    }

    @Override
    public float maxPressure(ItemStack iStack){
        return 10F;
    }

    @Override
    public void addAir(ItemStack iStack, int amount){
        int oldAir = NBTUtil.getInteger(iStack, "air");
        NBTUtil.setInteger(iStack, "air", Math.max(oldAir + amount, 0));
    }

    @Override
    public EnumGuiId getGuiID(){
        return EnumGuiId.PNEUMATIC_HELMET;
    }

    /**
     * Override this method to have an item handle its own armor rendering.
     * 
     * @param  entityLiving  The entity wearing the armor 
     * @param  itemStack  The itemStack to render the model of 
     * @param  armorSlot  0=head, 1=torso, 2=legs, 3=feet
     * 
     * @return  A ModelBiped to render instead of the default
     */
    @Override
    @SideOnly(Side.CLIENT)
    public ModelBiped getArmorModel(EntityLivingBase entityLiving, ItemStack itemStack, int armorSlot){
        if(armorSlot == 0 && (Config.useHelmetModel || DateEventHandler.isIronManEvent())) {
            /*RenderItemPneumaticHelmet.INSTANCE.render(entityLiving);

            RenderPlayer render = (RenderPlayer)Minecraft.getMinecraft().getRenderManager().entityRenderMap.get(EntityPlayer.class);
            ModelBiped model = armorSlot == 2 ? render.modelArmor : render.modelArmorChestplate;
            model.bipedHead.showModel = false;
            return model;*///TODO 1.8 fix
        }
        return null;
    }

    @Override
    public Set<Item> getApplicableUpgrades(){
        Set<Item> items = new HashSet<Item>();
        for(IUpgradeRenderHandler handler : UpgradeRenderHandlerList.instance().upgradeRenderers) {
            for(Item requiredItem : handler.getRequiredUpgrades()) {
                items.add(requiredItem);
            }
        }
        return items;
    }

    @Override
    public String getName(){
        return func_77658_a() + ".name";
    }

    /*private boolean hasThaumcraftUpgradeAndPressure(ItemStack stack){
        return hasSufficientPressure(stack) && getUpgrades(ItemMachineUpgrade.UPGRADE_THAUMCRAFT, stack) > 0;
    }

      @Override
      @Optional.Method(modid = ModIds.THAUMCRAFT)
      public int getVisDiscount(ItemStack stack, EntityPlayer player, Aspect aspect){
          return hasThaumcraftUpgradeAndPressure(stack) ? 5 : 0;
      }

      @Override
      public boolean showIngamePopups(ItemStack itemstack, EntityLivingBase player){
          return hasThaumcraftUpgradeAndPressure(itemstack);
      }

      @Override
      public boolean showNodes(ItemStack itemstack, EntityLivingBase player){
          return hasThaumcraftUpgradeAndPressure(itemstack);
      }*/

}
