package pneumaticCraft.common.item;

import java.util.List;

import net.minecraft.client.resources.I18n;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.server.MinecraftServer;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.ChatComponentTranslation;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;
import net.minecraft.world.WorldServer;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.apache.commons.lang3.text.WordUtils;

import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketNotifyVariablesRemote;
import pneumaticCraft.common.remote.GlobalVariableManager;
import pneumaticCraft.common.tileentity.TileEntitySecurityStation;
import pneumaticCraft.proxy.CommonProxy.EnumGuiId;

public class ItemRemote extends ItemPneumatic{
    public ItemRemote(String texture){
        super(texture);
        func_77625_d(1);
    }

    /**
     * Called whenever this item is equipped and the right mouse button is pressed. Args: itemStack, world, entityPlayer
     */
    @Override
    public ItemStack func_77659_a(ItemStack stack, World world, EntityPlayer player){
        if(!world.field_72995_K) {
            openGui(player, stack);
        }
        return stack;
    }

    /**
     * Callback for item usage. If the item does something special on right clicking, he will have one of those. Return
     * True if something happen and false if it don't. This is for ITEMS, not BLOCKS
     */
    @Override
    public boolean onItemUseFirst(ItemStack remote, EntityPlayer player, World world, BlockPos pos, EnumFacing side, float hitX, float hitY, float hitZ){
        if(!world.field_72995_K && !player.func_70093_af() && isAllowedToEdit(player, remote)) {
            TileEntity te = world.func_175625_s(pos);
            if(te instanceof TileEntitySecurityStation) {
                if(((TileEntitySecurityStation)te).doesAllowPlayer(player)) {
                    NBTTagCompound tag = remote.func_77978_p();
                    if(tag == null) {
                        tag = new NBTTagCompound();
                        remote.func_77982_d(tag);
                    }
                    tag.func_74768_a("securityX", pos.func_177958_n());
                    tag.func_74768_a("securityY", pos.func_177956_o());
                    tag.func_74768_a("securityZ", pos.func_177952_p());
                    tag.func_74768_a("securityDimension", world.field_73011_w.func_177502_q());
                    player.func_146105_b(new ChatComponentTranslation("gui.remote.boundSecurityStation", pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p()));
                    return true;
                } else {
                    player.func_146105_b(new ChatComponentTranslation("gui.remote.cantBindSecurityStation"));
                }
            }
        }
        return false;
    }

    /**
     * allows items to add custom lines of information to the mouseover description
     */
    @Override
    @SideOnly(Side.CLIENT)
    public void func_77624_a(ItemStack remote, EntityPlayer player, List curInfo, boolean moreInfo){
        super.func_77624_a(remote, player, curInfo, moreInfo);
        curInfo.add(I18n.func_135052_a("gui.remote.tooltip.sneakRightClickToEdit"));
        NBTTagCompound tag = remote.func_77978_p();
        if(tag != null && tag.func_74764_b("securityX")) {
            int x = tag.func_74762_e("securityX");
            int y = tag.func_74762_e("securityY");
            int z = tag.func_74762_e("securityZ");
            int dimensionId = tag.func_74762_e("securityDimension");
            for(String s : WordUtils.wrap(I18n.func_135052_a("gui.remote.tooltip.boundToSecurityStation", dimensionId, x, y, z), 40).split(System.getProperty("line.separator"))) {
                curInfo.add(s);
            }
        } else {
            for(String s : WordUtils.wrap(I18n.func_135052_a("gui.remote.tooltip.rightClickToBind"), 40).split(System.getProperty("line.separator"))) {
                curInfo.add(s);
            }
        }
    }

    private void openGui(EntityPlayer player, ItemStack remote){
        if(player.func_70093_af()) {
            if(isAllowedToEdit(player, remote)) {
                player.openGui(PneumaticCraft.instance, EnumGuiId.REMOTE_EDITOR.ordinal(), player.field_70170_p, (int)player.field_70165_t, (int)player.field_70163_u, (int)player.field_70161_v);
                NetworkHandler.sendTo(new PacketNotifyVariablesRemote(GlobalVariableManager.getInstance().getAllActiveVariableNames()), (EntityPlayerMP)player);
            }
        } else {
            player.openGui(PneumaticCraft.instance, EnumGuiId.REMOTE.ordinal(), player.field_70170_p, (int)player.field_70165_t, (int)player.field_70163_u, (int)player.field_70161_v);
        }
    }

    public static boolean hasSameSecuritySettings(ItemStack remote1, ItemStack remote2){
        NBTTagCompound tag1 = remote1.func_77978_p();
        NBTTagCompound tag2 = remote2.func_77978_p();
        if(tag1 == null && tag2 == null) return true;
        if(tag1 == null || tag2 == null) return false;
        int x1 = tag1.func_74762_e("securityX");
        int y1 = tag1.func_74762_e("securityY");
        int z1 = tag1.func_74762_e("securityZ");
        int dimension1 = tag1.func_74762_e("securityDimension");
        int x2 = tag2.func_74762_e("securityX");
        int y2 = tag2.func_74762_e("securityY");
        int z2 = tag2.func_74762_e("securityZ");
        int dimension2 = tag2.func_74762_e("securityDimension");
        return x1 == x2 && y1 == y2 && z1 == z2 && dimension1 == dimension2;
    }

    private boolean isAllowedToEdit(EntityPlayer player, ItemStack remote){
        NBTTagCompound tag = remote.func_77978_p();
        if(tag != null) {
            if(tag.func_74764_b("securityX")) {
                int x = tag.func_74762_e("securityX");
                int y = tag.func_74762_e("securityY");
                int z = tag.func_74762_e("securityZ");
                int dimensionId = tag.func_74762_e("securityDimension");
                WorldServer world = null;
                for(WorldServer w : MinecraftServer.func_71276_C().field_71305_c) {
                    if(w.field_73011_w.func_177502_q() == dimensionId) {
                        world = w;
                        break;
                    }
                }
                if(world != null) {
                    TileEntity te = world.func_175625_s(new BlockPos(x, y, z));
                    if(te instanceof TileEntitySecurityStation) {
                        boolean canAccess = ((TileEntitySecurityStation)te).doesAllowPlayer(player);
                        if(!canAccess) {
                            player.func_146105_b(new ChatComponentTranslation("gui.remote.noEditRights", x, y, z));
                        }
                        return canAccess;
                    }
                }
            }
        }
        return true;
    }

    @Override
    public void func_77663_a(ItemStack remote, World worl, Entity entity, int slot, boolean holdingItem){
        if(!worl.field_72995_K) {
            NBTTagCompound tag = remote.func_77978_p();
            if(tag != null) {
                if(tag.func_74764_b("securityX")) {
                    int x = tag.func_74762_e("securityX");
                    int y = tag.func_74762_e("securityY");
                    int z = tag.func_74762_e("securityZ");
                    int dimensionId = tag.func_74762_e("securityDimension");
                    WorldServer world = null;
                    for(WorldServer w : MinecraftServer.func_71276_C().field_71305_c) {
                        if(w.field_73011_w.func_177502_q() == dimensionId) {
                            world = w;
                            break;
                        }
                    }
                    if(world != null) {
                        TileEntity te = world.func_175625_s(new BlockPos(x, y, z));
                        if(!(te instanceof TileEntitySecurityStation)) {
                            tag.func_82580_o("securityX");
                            tag.func_82580_o("securityY");
                            tag.func_82580_o("securityZ");
                            tag.func_82580_o("securityDimension");
                        }
                    }
                }
            }
        }
    }
}
