package pneumaticCraft.common.progwidgets;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import net.minecraft.client.gui.GuiScreen;
import net.minecraft.entity.Entity;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.client.gui.GuiProgrammer;
import pneumaticCraft.client.gui.programmer.GuiProgWidgetArea;
import pneumaticCraft.common.ai.DroneAIManager;
import pneumaticCraft.common.item.ItemPlastic;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.Textures;

import com.google.common.base.Predicate;

public class ProgWidgetArea extends ProgWidget implements IAreaProvider, IVariableWidget{
    public int x1, y1, z1, x2, y2, z2;
    private String coord1Variable = "", coord2Variable = "";
    private DroneAIManager aiManager;
    public EnumAreaType type = EnumAreaType.FILL;
    public int typeInfo;//For the grid type, it's the grid interval, for Random it's the amount of selected blocks.

    public enum EnumAreaType{
        FILL("Filled"), FRAME("Frame"), WALL("Walls"), SPHERE("Sphere"), LINE("Line"), X_WALL("X-Wall"), Y_WALL(
                "Y-Wall"), Z_WALL("Z-Wall"), X_CYLINDER("X-Cylinder"), Y_CYLINDER("Y-Cylinder"), Z_CYLINDER(
                "Z-Cylinder"), X_PYRAMID("X-Pyramid"), Y_PYRAMID("Y-Pyramid"), Z_PYRAMID("Z-Pyramid"), GRID("Grid",
                true), RANDOM("Random", true);

        private final String name;
        public final boolean utilizesTypeInfo;

        private EnumAreaType(String name){
            this(name, false);
        }

        private EnumAreaType(String name, boolean utilizesTypeInfo){
            this.name = name;
            this.utilizesTypeInfo = utilizesTypeInfo;
        }

        @Override
        public String toString(){
            return name;
        }
    }

    @Override
    public void getTooltip(List<String> curTooltip){
        super.getTooltip(curTooltip);

        String c1;
        if(coord1Variable.equals("")) {
            c1 = x1 != 0 || y1 != 0 || z1 != 0 ? "X%s: " + x1 + ", Y%s: " + y1 + ", Z%s: " + z1 : null;
        } else {
            c1 = "XYZ%s: \"" + coord1Variable + "\"";
        }
        String c2;
        if(coord2Variable.equals("")) {
            c2 = x2 != 0 || y2 != 0 || z2 != 0 ? "X%s: " + x2 + ", Y%s: " + y2 + ", Z%s: " + z2 : null;
        } else {
            c2 = "XYZ%s: \"" + coord2Variable + "\"";
        }
        if(c1 == null) {
            c1 = c2;
            c2 = null;
        }

        if(c1 != null) {
            if(c2 != null) {
                curTooltip.add("Contains the points:");
                curTooltip.add(c1.replace("%s", "1"));
                curTooltip.add(c2.replace("%s", "2"));
            } else {
                curTooltip.add("Contains the point:");
                curTooltip.add(c1.replace("%s", "1"));
            }
        }

        curTooltip.add("Area type: " + type);
    }

    @Override
    public void addErrors(List<String> curInfo, List<IProgWidget> widgets){
        super.addErrors(curInfo, widgets);
        if(coord1Variable.equals("") && coord2Variable.equals("") && x1 == 0 && y1 == 0 && z1 == 0 && x2 == 0 && y2 == 0 && z2 == 0) {
            curInfo.add("gui.progWidget.area.error.noArea");
        }
    }

    private BlockPos[] getAreaPoints(){
        BlockPos c1;
        if(coord1Variable.equals("")) {
            c1 = x1 != 0 || y1 != 0 || z1 != 0 ? new BlockPos(x1, y1, z1) : null;
        } else {
            c1 = aiManager != null ? aiManager.getCoordinate(coord1Variable) : null;
        }
        BlockPos c2;
        if(coord2Variable.equals("")) {
            c2 = x2 != 0 || y2 != 0 || z2 != 0 ? new BlockPos(x2, y2, z2) : null;
        } else {
            c2 = aiManager != null ? aiManager.getCoordinate(coord2Variable) : null;
        }
        if(c1 == null && c2 == null) {
            return new BlockPos[]{null, null};
        } else if(c1 == null) {
            return new BlockPos[]{c2, null};
        } else if(c2 == null) {
            return new BlockPos[]{c1, null};
        } else {
            return new BlockPos[]{c1, c2};
        }
    }

    @Override
    public boolean hasStepInput(){
        return false;
    }

    @Override
    public Class<? extends IProgWidget> returnType(){
        return ProgWidgetArea.class;
    }

    @Override
    public Class<? extends IProgWidget>[] getParameters(){
        return new Class[]{ProgWidgetArea.class};
    }

    @Override
    public String getWidgetString(){
        return "area";
    }

    @Override
    public ResourceLocation getTexture(){
        return Textures.PROG_WIDGET_AREA;
    }

    @Override
    public void getArea(Set<BlockPos> area){
        BlockPos[] areaPoints = getAreaPoints();
        if(areaPoints[0] == null && areaPoints[1] == null) return;

        int minX;
        int minY;
        int minZ;
        int maxX;
        int maxY;
        int maxZ;
        if(areaPoints[1] != null) {
            minX = Math.min(areaPoints[0].func_177958_n(), areaPoints[1].func_177958_n());
            minY = Math.min(areaPoints[0].func_177956_o(), areaPoints[1].func_177956_o());
            minZ = Math.min(areaPoints[0].func_177952_p(), areaPoints[1].func_177952_p());
            maxX = Math.max(areaPoints[0].func_177958_n(), areaPoints[1].func_177958_n());
            maxY = Math.max(areaPoints[0].func_177956_o(), areaPoints[1].func_177956_o());
            maxZ = Math.max(areaPoints[0].func_177952_p(), areaPoints[1].func_177952_p());
        } else {
            minX = maxX = areaPoints[0].func_177958_n();
            minY = maxY = areaPoints[0].func_177956_o();
            minZ = maxZ = areaPoints[0].func_177952_p();
        }
        int size = (maxX - minX) * (maxY - minY) * (maxZ - minZ);
        if(size > 100000) { //Prevent memory problems when getting to ridiculous areas.
            if(aiManager != null) aiManager.getDrone().overload();
            return;
        }

        switch(type){
            case FILL:
                for(int x = minX; x <= maxX; x++) {
                    for(int y = Math.min(255, maxY); y >= minY && y >= 0; y--) {
                        for(int z = minZ; z <= maxZ; z++) {
                            area.add(new BlockPos(x, y, z));
                        }
                    }
                }
                break;
            case FRAME:
                for(int x = minX; x <= maxX; x++) {
                    for(int y = Math.max(0, minY); y <= maxY && y < 256; y++) {
                        for(int z = minZ; z <= maxZ; z++) {
                            int axisRight = 0;
                            if(x == minX || x == maxX) axisRight++;
                            if(y == minY || y == maxY) axisRight++;
                            if(z == minZ || z == maxZ) axisRight++;
                            if(axisRight > 1) {
                                area.add(new BlockPos(x, y, z));
                            }
                        }
                    }
                }
                break;
            case WALL:
                for(int x = minX; x <= maxX; x++) {
                    for(int y = Math.max(0, minY); y <= maxY && y < 256; y++) {
                        for(int z = minZ; z <= maxZ; z++) {
                            if(x == minX || x == maxX || y == minY || y == maxY || z == minZ || z == maxZ) {
                                area.add(new BlockPos(x, y, z));
                            }
                        }
                    }
                }
                break;
            case SPHERE:
                double radius = areaPoints[1] != null ? PneumaticCraftUtils.distBetween(areaPoints[0], areaPoints[1]) : 0;
                minX = (int)(areaPoints[0].func_177958_n() - radius - 1);
                minY = (int)(areaPoints[0].func_177956_o() - radius - 1);
                minZ = (int)(areaPoints[0].func_177952_p() - radius - 1);
                maxX = (int)(areaPoints[0].func_177958_n() + radius + 1);
                maxY = (int)(areaPoints[0].func_177956_o() + radius + 1);
                maxZ = (int)(areaPoints[0].func_177952_p() + radius + 1);
                for(int x = minX; x <= maxX; x++) {
                    for(int y = Math.max(0, minY); y <= maxY && y < 256; y++) {
                        for(int z = minZ; z <= maxZ; z++) {
                            if(PneumaticCraftUtils.distBetween(areaPoints[0], x + 0.5, y + 0.5, z + 0.5) <= radius) {
                                area.add(new BlockPos(x, y, z));
                            }
                        }
                    }
                }
                break;
            case LINE:
                if(areaPoints[1] != null) {
                    Vec3 lineVec = new Vec3(areaPoints[1].func_177958_n() - areaPoints[0].func_177958_n(), areaPoints[1].func_177956_o() - areaPoints[0].func_177956_o(), areaPoints[1].func_177952_p() - areaPoints[0].func_177952_p()).func_72432_b();
                    lineVec = new Vec3(lineVec.field_72450_a / 10, lineVec.field_72448_b / 10, lineVec.field_72449_c / 10);
                    double curX = areaPoints[0].func_177958_n() + 0.5;
                    double curY = areaPoints[0].func_177956_o() + 0.5;
                    double curZ = areaPoints[0].func_177952_p() + 0.5;
                    double totalDistance = 0;
                    double maxDistance = PneumaticCraftUtils.distBetween(areaPoints[0], areaPoints[1]);
                    while(totalDistance <= maxDistance) {
                        totalDistance += 0.1;
                        curX += lineVec.field_72450_a;
                        curY += lineVec.field_72448_b;
                        curZ += lineVec.field_72449_c;
                        if(curY >= 0 && curY < 256) {
                            BlockPos pos = new BlockPos((int)curX, (int)curY, (int)curZ);
                            if(!area.contains(pos)) area.add(pos);
                        }
                    }
                }
                break;
            case X_WALL:
                if(areaPoints[1] != null) {
                    Vec3 lineVec = new Vec3(0, areaPoints[1].func_177956_o() - areaPoints[0].func_177956_o(), areaPoints[1].func_177952_p() - areaPoints[0].func_177952_p()).func_72432_b();
                    lineVec = new Vec3(lineVec.field_72450_a, lineVec.field_72448_b / 10, lineVec.field_72449_c / 10);
                    double curY = areaPoints[0].func_177956_o() + 0.5;
                    double curZ = areaPoints[0].func_177952_p() + 0.5;
                    double totalDistance = 0;
                    double maxDistance = Math.sqrt(Math.pow(areaPoints[0].func_177956_o() - areaPoints[1].func_177956_o(), 2) + Math.pow(areaPoints[0].func_177952_p() - areaPoints[1].func_177952_p(), 2));
                    while(totalDistance <= maxDistance) {
                        totalDistance += 0.1;
                        curY += lineVec.field_72448_b;
                        curZ += lineVec.field_72449_c;
                        for(int i = minX; i <= maxX; i++) {
                            if(curY >= 0 && curY < 256) {
                                BlockPos pos = new BlockPos(i, (int)curY, (int)curZ);
                                if(!area.contains(pos)) area.add(pos);
                            }
                        }
                    }
                }
                break;
            case Y_WALL:
                if(areaPoints[1] != null) {
                    Vec3 lineVec = new Vec3(areaPoints[1].func_177958_n() - areaPoints[0].func_177958_n(), 0, areaPoints[1].func_177952_p() - areaPoints[0].func_177952_p()).func_72432_b();
                    lineVec = new Vec3(lineVec.field_72450_a, lineVec.field_72448_b / 10, lineVec.field_72449_c / 10);
                    double curX = areaPoints[0].func_177958_n() + 0.5;
                    double curZ = areaPoints[0].func_177952_p() + 0.5;
                    double totalDistance = 0;
                    double maxDistance = Math.sqrt(Math.pow(areaPoints[0].func_177958_n() - areaPoints[1].func_177958_n(), 2) + Math.pow(areaPoints[0].func_177952_p() - areaPoints[1].func_177952_p(), 2));
                    while(totalDistance <= maxDistance) {
                        totalDistance += 0.1;
                        curX += lineVec.field_72450_a;
                        curZ += lineVec.field_72449_c;
                        for(int i = Math.max(0, minY); i <= Math.min(maxY, 255); i++) {
                            BlockPos pos = new BlockPos((int)curX, i, (int)curZ);
                            if(!area.contains(pos)) area.add(pos);
                        }
                    }
                }
                break;
            case Z_WALL:
                if(areaPoints[1] != null) {
                    Vec3 lineVec = new Vec3(areaPoints[1].func_177958_n() - areaPoints[0].func_177958_n(), areaPoints[1].func_177956_o() - areaPoints[0].func_177956_o(), 0).func_72432_b();
                    lineVec = new Vec3(lineVec.field_72450_a / 10, lineVec.field_72448_b / 10, lineVec.field_72449_c);
                    double curX = areaPoints[0].func_177958_n() + 0.5;
                    double curY = areaPoints[0].func_177956_o() + 0.5;
                    double totalDistance = 0;
                    double maxDistance = Math.sqrt(Math.pow(areaPoints[0].func_177958_n() - areaPoints[1].func_177958_n(), 2) + Math.pow(areaPoints[0].func_177956_o() - areaPoints[1].func_177956_o(), 2));
                    while(totalDistance <= maxDistance) {
                        totalDistance += 0.1;
                        curX += lineVec.field_72450_a;
                        curY += lineVec.field_72448_b;
                        for(int i = minZ; i <= maxZ; i++) {
                            if(curY >= 0 && curY < 256) {
                                BlockPos pos = new BlockPos((int)curX, (int)curY, i);
                                if(!area.contains(pos)) area.add(pos);
                            }
                        }
                    }
                }
                break;
            case X_CYLINDER:
                if(areaPoints[1] != null) {
                    double rad = areaPoints[1] != null ? PneumaticCraftUtils.distBetween(areaPoints[0].func_177956_o(), areaPoints[0].func_177952_p(), areaPoints[1].func_177956_o(), areaPoints[1].func_177952_p()) : 0;
                    minY = (int)(areaPoints[0].func_177956_o() - rad - 1);
                    minZ = (int)(areaPoints[0].func_177952_p() - rad - 1);
                    maxY = (int)(areaPoints[0].func_177956_o() + rad + 1);
                    maxZ = (int)(areaPoints[0].func_177952_p() + rad + 1);
                    for(int y = Math.max(0, minY); y <= maxY && y < 256; y++) {
                        for(int z = minZ; z <= maxZ; z++) {
                            if(PneumaticCraftUtils.distBetween(areaPoints[0].func_177956_o(), areaPoints[0].func_177952_p(), y, z) <= rad) {
                                for(int x = minX; x <= maxX; x++) {
                                    area.add(new BlockPos(x, y, z));
                                }
                            }
                        }
                    }
                }
                break;
            case Y_CYLINDER:
                if(areaPoints[1] != null) {
                    double rad = areaPoints[1] != null ? PneumaticCraftUtils.distBetween(areaPoints[0].func_177958_n(), areaPoints[0].func_177952_p(), areaPoints[1].func_177958_n(), areaPoints[1].func_177952_p()) : 0;
                    minX = (int)(areaPoints[0].func_177958_n() - rad - 1);
                    minZ = (int)(areaPoints[0].func_177952_p() - rad - 1);
                    maxX = (int)(areaPoints[0].func_177958_n() + rad + 1);
                    maxZ = (int)(areaPoints[0].func_177952_p() + rad + 1);
                    for(int x = minX; x <= maxX; x++) {
                        for(int z = minZ; z <= maxZ; z++) {
                            if(PneumaticCraftUtils.distBetween(areaPoints[0].func_177958_n(), areaPoints[0].func_177952_p(), x, z) <= rad) {
                                for(int y = Math.max(0, minY); y <= maxY && y < 256; y++) {
                                    area.add(new BlockPos(x, y, z));
                                }
                            }
                        }
                    }
                }
                break;
            case Z_CYLINDER:
                if(areaPoints[1] != null) {
                    double rad = areaPoints[1] != null ? PneumaticCraftUtils.distBetween(areaPoints[0].func_177958_n(), areaPoints[0].func_177956_o(), areaPoints[1].func_177958_n(), areaPoints[1].func_177956_o()) : 0;
                    minX = (int)(areaPoints[0].func_177958_n() - rad - 1);
                    minY = (int)(areaPoints[0].func_177956_o() - rad - 1);
                    maxX = (int)(areaPoints[0].func_177958_n() + rad + 1);
                    maxY = (int)(areaPoints[0].func_177956_o() + rad + 1);
                    for(int x = minX; x <= maxX; x++) {
                        for(int y = Math.max(0, minY); y <= maxY && y < 256; y++) {
                            if(PneumaticCraftUtils.distBetween(areaPoints[0].func_177958_n(), areaPoints[0].func_177956_o(), x, y) <= rad) {
                                for(int z = minZ; z <= maxZ; z++) {
                                    area.add(new BlockPos(x, y, z));
                                }
                            }
                        }
                    }
                }
                break;
            case X_PYRAMID:
                if(areaPoints[1] != null && areaPoints[1].func_177958_n() != areaPoints[0].func_177958_n()) {
                    Vec3 lineVec = new Vec3(areaPoints[1].func_177958_n() - areaPoints[0].func_177958_n(), areaPoints[1].func_177956_o() - areaPoints[0].func_177956_o(), areaPoints[1].func_177952_p() - areaPoints[0].func_177952_p()).func_72432_b();
                    lineVec = new Vec3(lineVec.field_72450_a, lineVec.field_72448_b / lineVec.field_72450_a, lineVec.field_72449_c / lineVec.field_72450_a);
                    double curY = areaPoints[0].func_177956_o() - lineVec.field_72448_b;
                    int x = areaPoints[0].func_177958_n() + (areaPoints[1].func_177958_n() > areaPoints[0].func_177958_n() ? -1 : 1);
                    double curZ = areaPoints[0].func_177952_p() - lineVec.field_72449_c;
                    while(x != areaPoints[1].func_177958_n()) {

                        x += areaPoints[1].func_177958_n() > areaPoints[0].func_177958_n() ? 1 : -1;
                        curY += lineVec.field_72448_b;
                        curZ += lineVec.field_72449_c;

                        int dY = Math.abs((int)(curY - areaPoints[0].func_177956_o()));
                        int dZ = Math.abs((int)(curZ - areaPoints[0].func_177952_p()));
                        for(int y = areaPoints[0].func_177956_o() - dY; y <= areaPoints[0].func_177956_o() + dY; y++) {
                            for(int z = areaPoints[0].func_177952_p() - dZ; z <= areaPoints[0].func_177952_p() + dZ; z++) {
                                if(y > 0 && y < 256) area.add(new BlockPos(x, y, z));
                            }
                        }
                    }
                }
                break;
            case Y_PYRAMID:
                if(areaPoints[1] != null && areaPoints[1].func_177956_o() != areaPoints[0].func_177956_o()) {
                    Vec3 lineVec = new Vec3(areaPoints[1].func_177958_n() - areaPoints[0].func_177958_n(), areaPoints[1].func_177956_o() - areaPoints[0].func_177956_o(), areaPoints[1].func_177952_p() - areaPoints[0].func_177952_p()).func_72432_b();
                    lineVec = new Vec3(lineVec.field_72450_a / lineVec.field_72448_b, lineVec.field_72448_b, lineVec.field_72449_c / lineVec.field_72448_b);
                    double curX = areaPoints[0].func_177958_n() - lineVec.field_72450_a;
                    int y = areaPoints[0].func_177956_o() + (areaPoints[1].func_177956_o() > areaPoints[0].func_177956_o() ? -1 : 1);
                    double curZ = areaPoints[0].func_177952_p() - lineVec.field_72449_c;
                    while(y != areaPoints[1].func_177956_o()) {

                        y += areaPoints[1].func_177956_o() > areaPoints[0].func_177956_o() ? 1 : -1;
                        curX += lineVec.field_72450_a;
                        curZ += lineVec.field_72449_c;

                        int dX = Math.abs((int)(curX - areaPoints[0].func_177958_n()));
                        int dZ = Math.abs((int)(curZ - areaPoints[0].func_177952_p()));
                        for(int x = areaPoints[0].func_177958_n() - dX; x <= areaPoints[0].func_177958_n() + dX; x++) {
                            for(int z = areaPoints[0].func_177952_p() - dZ; z <= areaPoints[0].func_177952_p() + dZ; z++) {
                                if(y > 0 && y < 256) area.add(new BlockPos(x, y, z));
                            }
                        }
                    }
                }
                break;
            case Z_PYRAMID:
                if(areaPoints[1] != null && areaPoints[1].func_177952_p() != areaPoints[0].func_177952_p()) {
                    Vec3 lineVec = new Vec3(areaPoints[1].func_177958_n() - areaPoints[0].func_177958_n(), areaPoints[1].func_177956_o() - areaPoints[0].func_177956_o(), areaPoints[1].func_177952_p() - areaPoints[0].func_177952_p()).func_72432_b();
                    lineVec = new Vec3(lineVec.field_72450_a / lineVec.field_72449_c, lineVec.field_72448_b / lineVec.field_72449_c, lineVec.field_72449_c);
                    double curX = areaPoints[0].func_177958_n() - lineVec.field_72450_a;
                    int z = areaPoints[0].func_177952_p() + (areaPoints[1].func_177952_p() > areaPoints[0].func_177952_p() ? -1 : 1);
                    double curY = areaPoints[0].func_177956_o() - lineVec.field_72448_b;
                    while(z != areaPoints[1].func_177952_p()) {

                        z += areaPoints[1].func_177952_p() > areaPoints[0].func_177952_p() ? 1 : -1;
                        curX += lineVec.field_72450_a;
                        curY += lineVec.field_72448_b;

                        int dX = Math.abs((int)(curX - areaPoints[0].func_177958_n()));
                        int dY = Math.abs((int)(curY - areaPoints[0].func_177956_o()));
                        for(int x = areaPoints[0].func_177958_n() - dX; x <= areaPoints[0].func_177958_n() + dX; x++) {
                            for(int y = areaPoints[0].func_177956_o() - dY; y <= areaPoints[0].func_177956_o() + dY; y++) {
                                if(y > 0 && y < 256) area.add(new BlockPos(x, y, z));
                            }
                        }
                    }
                }
                break;
            case GRID:
                if(areaPoints[1] == null || areaPoints[0].equals(areaPoints[1]) || typeInfo <= 0) {
                    area.add(areaPoints[0]);
                } else {
                    int interval = typeInfo;
                    for(int x = areaPoints[0].func_177958_n(); areaPoints[0].func_177958_n() < areaPoints[1].func_177958_n() ? x <= areaPoints[1].func_177958_n() : x >= areaPoints[1].func_177958_n(); x += (areaPoints[0].func_177958_n() < areaPoints[1].func_177958_n() ? 1 : -1) * interval) {
                        for(int y = areaPoints[0].func_177956_o(); areaPoints[0].func_177956_o() < areaPoints[1].func_177956_o() ? y <= areaPoints[1].func_177956_o() : y >= areaPoints[1].func_177956_o(); y += (areaPoints[0].func_177956_o() < areaPoints[1].func_177956_o() ? 1 : -1) * interval) {
                            for(int z = areaPoints[0].func_177952_p(); areaPoints[0].func_177952_p() < areaPoints[1].func_177952_p() ? z <= areaPoints[1].func_177952_p() : z >= areaPoints[1].func_177952_p(); z += (areaPoints[0].func_177952_p() < areaPoints[1].func_177952_p() ? 1 : -1) * interval) {
                                if(y > 0 && y < 256) area.add(new BlockPos(x, y, z));
                            }
                        }
                    }
                }
                break;
            case RANDOM:
                type = EnumAreaType.FILL;

                Set<BlockPos> filledArea = new HashSet<BlockPos>();
                getArea(filledArea);
                type = EnumAreaType.RANDOM;
                if(typeInfo >= filledArea.size()) {
                    area.addAll(filledArea);
                    return;
                }
                Random rand = new Random();
                Set<Integer> randomIndexes = new HashSet<Integer>();
                while(randomIndexes.size() < typeInfo) {
                    randomIndexes.add(rand.nextInt(filledArea.size()));
                }
                int curIndex = 0;
                for(BlockPos pos : filledArea) {
                    if(randomIndexes.contains(curIndex)) area.add(pos);
                    curIndex++;
                }
                break;
        }
    }

    private AxisAlignedBB getAABB(){
        BlockPos[] areaPoints = getAreaPoints();
        if(areaPoints[0] == null) return null;
        int minX;
        int minY;
        int minZ;
        int maxX;
        int maxY;
        int maxZ;
        if(areaPoints[1] != null) {
            minX = Math.min(areaPoints[0].func_177958_n(), areaPoints[1].func_177958_n());
            minY = Math.min(areaPoints[0].func_177956_o(), areaPoints[1].func_177956_o());
            minZ = Math.min(areaPoints[0].func_177952_p(), areaPoints[1].func_177952_p());
            maxX = Math.max(areaPoints[0].func_177958_n(), areaPoints[1].func_177958_n());
            maxY = Math.max(areaPoints[0].func_177956_o(), areaPoints[1].func_177956_o());
            maxZ = Math.max(areaPoints[0].func_177952_p(), areaPoints[1].func_177952_p());
        } else {
            minX = maxX = areaPoints[0].func_177958_n();
            minY = maxY = areaPoints[0].func_177956_o();
            minZ = maxZ = areaPoints[0].func_177952_p();
        }
        return new AxisAlignedBB(minX, minY, minZ, maxX + 1, maxY + 1, maxZ + 1);
    }

    public List<Entity> getEntitiesWithinArea(World world, Predicate<? super Entity> predicate){
        AxisAlignedBB aabb = getAABB();
        return aabb != null ? world.func_175674_a(null, aabb, predicate) : new ArrayList<Entity>();
    }

    @Override
    public void writeToNBT(NBTTagCompound tag){
        super.writeToNBT(tag);
        tag.func_74768_a("x1", x1);
        tag.func_74768_a("y1", y1);
        tag.func_74768_a("z1", z1);
        tag.func_74768_a("x2", x2);
        tag.func_74768_a("y2", y2);
        tag.func_74768_a("z2", z2);
        if(type != null) tag.func_74768_a("type", type.ordinal());
        tag.func_74768_a("typeInfo", typeInfo);
        tag.func_74778_a("coord1Variable", coord1Variable);
        tag.func_74778_a("coord2Variable", coord2Variable);
    }

    @Override
    public void readFromNBT(NBTTagCompound tag){
        super.readFromNBT(tag);
        x1 = tag.func_74762_e("x1");
        y1 = tag.func_74762_e("y1");
        z1 = tag.func_74762_e("z1");
        x2 = tag.func_74762_e("x2");
        y2 = tag.func_74762_e("y2");
        z2 = tag.func_74762_e("z2");
        type = EnumAreaType.values()[tag.func_74762_e("type")];
        typeInfo = tag.func_74762_e("typeInfo");
        coord1Variable = tag.func_74779_i("coord1Variable");
        coord2Variable = tag.func_74779_i("coord2Variable");
    }

    @Override
    @SideOnly(Side.CLIENT)
    public GuiScreen getOptionWindow(GuiProgrammer guiProgrammer){
        return new GuiProgWidgetArea(this, guiProgrammer);
    }

    @Override
    public WidgetDifficulty getDifficulty(){
        return WidgetDifficulty.EASY;
    }

    @Override
    public int getCraftingColorIndex(){
        return ItemPlastic.CREEPER_PLANT_DAMAGE;
    }

    public String getCoord1Variable(){
        return coord1Variable;
    }

    public void setCoord1Variable(String coord1Variable){
        this.coord1Variable = coord1Variable;
    }

    public String getCoord2Variable(){
        return coord2Variable;
    }

    public void setCoord2Variable(String coord2Variable){
        this.coord2Variable = coord2Variable;
    }

    @Override
    public void setAIManager(DroneAIManager aiManager){
        this.aiManager = aiManager;
    }

    @Override
    public void addVariables(Set<String> variables){
        variables.add(coord1Variable);
        variables.add(coord2Variable);
    }

}
