package pneumaticCraft.common.progwidgets;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.client.gui.GuiScreen;
import net.minecraft.entity.ai.EntityAIBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.entity.player.PlayerDestroyItemEvent;
import net.minecraftforge.fml.common.FMLCommonHandler;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.client.gui.GuiProgrammer;
import pneumaticCraft.client.gui.programmer.GuiProgWidgetImportExport;
import pneumaticCraft.common.ai.IDroneBase;
import pneumaticCraft.common.item.ItemPlastic;
import pneumaticCraft.common.util.IOHelper;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.Textures;

public class ProgWidgetCrafting extends ProgWidget implements ICraftingWidget, ICountWidget{
    private boolean useCount;
    private int count;

    @Override
    public void addErrors(List<String> curInfo, List<IProgWidget> widgets){
        super.addErrors(curInfo, widgets);
        boolean usingVariables = false;
        for(int y = 0; y < 3; y++) {
            ProgWidgetItemFilter itemFilter = (ProgWidgetItemFilter)getConnectedParameters()[y];
            for(int x = 0; x < 3 && itemFilter != null; x++) {
                if(!itemFilter.getVariable().equals("")) usingVariables = true;
                itemFilter = (ProgWidgetItemFilter)itemFilter.getConnectedParameters()[0];
            }
        }
        if(!usingVariables && getRecipeResult(PneumaticCraft.proxy.getClientWorld()) == null) {
            curInfo.add("gui.progWidget.crafting.error.noCraftingRecipe");
        }
    }

    @Override
    public boolean hasStepInput(){
        return true;
    }

    @Override
    public Class<? extends IProgWidget> returnType(){
        return null;
    }

    @Override
    public Class<? extends IProgWidget>[] getParameters(){
        return new Class[]{ProgWidgetItemFilter.class, ProgWidgetItemFilter.class, ProgWidgetItemFilter.class};
    }

    @Override
    public String getWidgetString(){
        return "crafting";
    }

    @Override
    public int getCraftingColorIndex(){
        return ItemPlastic.ENDER_PLANT_DAMAGE;
    }

    @Override
    public WidgetDifficulty getDifficulty(){
        return WidgetDifficulty.EASY;
    }

    @Override
    public ResourceLocation getTexture(){
        return Textures.PROG_WIDGET_CRAFTING;
    }

    @Override
    protected boolean hasBlacklist(){
        return false;
    }

    @Override
    public InventoryCrafting getCraftingGrid(){
        InventoryCrafting invCrafting = new InventoryCrafting(new Container(){
            @Override
            public boolean func_75145_c(EntityPlayer p_75145_1_){
                return false;
            }

        }, 3, 3);
        for(int y = 0; y < 3; y++) {
            ProgWidgetItemFilter itemFilter = (ProgWidgetItemFilter)getConnectedParameters()[y];
            for(int x = 0; x < 3 && itemFilter != null; x++) {
                invCrafting.func_70299_a(y * 3 + x, itemFilter.getFilter());
                itemFilter = (ProgWidgetItemFilter)itemFilter.getConnectedParameters()[0];
            }
        }
        return invCrafting;
    }

    private ItemStack getRecipeResult(World world){
        return CraftingManager.func_77594_a().func_82787_a(getCraftingGrid(), world);
    }

    public static IRecipe getRecipe(World world, ICraftingWidget widget){
        InventoryCrafting craftingGrid = widget.getCraftingGrid();
        for(IRecipe recipe : CraftingManager.func_77594_a().func_77592_b()) {
            if(recipe.func_77569_a(craftingGrid, world)) {
                return recipe;
            }
        }
        return null;
    }

    @Override
    public void renderExtraInfo(){
        ItemStack recipe = getRecipeResult(PneumaticCraft.proxy.getClientWorld());
        if(recipe != null) {
            ProgWidgetItemFilter.drawItemStack(recipe, 8, getHeight() / 2 - 8, recipe.field_77994_a + "");
        }
    }

    @Override
    public EntityAIBase getWidgetAI(IDroneBase drone, IProgWidget widget){
        return new DroneAICrafting(drone, (ICraftingWidget)widget);
    }

    public static class DroneAICrafting extends EntityAIBase{

        private final ICraftingWidget widget;
        private final IDroneBase drone;

        public DroneAICrafting(IDroneBase drone, ICraftingWidget widget){
            this.drone = drone;
            this.widget = widget;
        }

        @Override
        public boolean func_75250_a(){
            IRecipe recipe = ProgWidgetCrafting.getRecipe(drone.getWorld(), widget);
            if(recipe == null) return false;
            InventoryCrafting craftingGrid = widget.getCraftingGrid();
            for(int crafted = 0; !((ICountWidget)widget).useCount() || crafted < ((ICountWidget)widget).getCount(); crafted++) {
                List<ItemStack>[] equivalentsList = new List[9];
                for(int i = 0; i < equivalentsList.length; i++) {
                    ItemStack originalStack = craftingGrid.func_70301_a(i);
                    if(originalStack != null) {
                        List<ItemStack> equivalents = new ArrayList<ItemStack>();
                        for(int j = 0; j < drone.getInv().func_70302_i_(); j++) {
                            ItemStack droneStack = drone.getInv().func_70301_a(j);
                            if(droneStack != null && (droneStack.func_77973_b() == originalStack.func_77973_b() || PneumaticCraftUtils.isSameOreDictStack(droneStack, originalStack))) {
                                equivalents.add(droneStack);
                            }
                        }
                        if(equivalents.isEmpty()) return false;
                        equivalentsList[i] = equivalents;
                    }
                }

                int[] curIndexes = new int[9];
                boolean first = true;
                boolean hasCrafted = false;
                while(first || count(curIndexes, equivalentsList)) {
                    first = false;
                    InventoryCrafting craftMatrix = new InventoryCrafting(new Container(){
                        @Override
                        public boolean func_75145_c(EntityPlayer p_75145_1_){
                            return false;
                        }

                    }, 3, 3);
                    for(int i = 0; i < 9; i++) {
                        ItemStack stack = equivalentsList[i] == null ? null : equivalentsList[i].get(curIndexes[i]);
                        craftMatrix.func_70299_a(i, stack);
                    }
                    if(recipe.func_77569_a(craftMatrix, drone.getWorld())) {
                        if(craft(recipe.func_77572_b(craftMatrix), craftMatrix)) {
                            hasCrafted = true;
                            break;
                        }
                    }
                }
                if(!hasCrafted) return false;
            }
            return false;
        }

        private boolean count(int[] curIndexes, List<ItemStack>[] equivalentsList){
            for(int i = 0; i < equivalentsList.length; i++) {
                List<ItemStack> list = equivalentsList[i];
                curIndexes[i]++;
                if(list == null || curIndexes[i] >= list.size()) {
                    curIndexes[i] = 0;
                } else {
                    return true;
                }
            }
            return false;
        }

        public boolean craft(ItemStack craftedStack, InventoryCrafting craftMatrix){
            for(int i = 0; i < craftMatrix.func_70302_i_(); i++) {
                int requiredCount = 0;
                ItemStack stack = craftMatrix.func_70301_a(i);
                if(stack != null) {
                    for(int j = 0; j < craftMatrix.func_70302_i_(); j++) {
                        if(stack == craftMatrix.func_70301_a(j)) {
                            requiredCount++;
                        }
                    }
                    if(requiredCount > stack.field_77994_a) return false;
                }
            }

            FMLCommonHandler.instance().firePlayerCraftingEvent(drone.getFakePlayer(), craftedStack, craftMatrix);

            for(int i = 0; i < craftMatrix.func_70302_i_(); ++i) {
                ItemStack itemstack1 = craftMatrix.func_70301_a(i);

                if(itemstack1 != null) {
                    if(itemstack1.func_77973_b().hasContainerItem(itemstack1)) {
                        ItemStack itemstack2 = itemstack1.func_77973_b().getContainerItem(itemstack1);

                        if(itemstack2 != null && itemstack2.func_77984_f() && itemstack2.func_77952_i() > itemstack2.func_77958_k()) {
                            MinecraftForge.EVENT_BUS.post(new PlayerDestroyItemEvent(drone.getFakePlayer(), itemstack2));
                            continue;
                        }

                        ItemStack remainder = IOHelper.insert(drone.getInv(), itemstack2.func_77946_l(), EnumFacing.UP, false);
                        if(remainder != null) {
                            Vec3 pos = drone.getDronePos();
                            EntityItem item = new EntityItem(drone.getWorld(), pos.field_72450_a, pos.field_72448_b, pos.field_72449_c, remainder);
                            drone.getWorld().func_72838_d(item);
                        }
                    }
                    itemstack1.field_77994_a--;//As this stack references to the Drones stacks in its inventory, we can do this.
                }
            }

            for(int i = 0; i < drone.getInv().func_70302_i_(); i++) {
                ItemStack stack = drone.getInv().func_70301_a(i);
                if(stack != null && stack.field_77994_a <= 0) {
                    drone.getInv().func_70299_a(i, null);
                }
            }

            ItemStack remainder = IOHelper.insert(drone.getInv(), craftedStack, EnumFacing.UP, false);
            if(remainder != null) {
                Vec3 pos = drone.getDronePos();
                EntityItem item = new EntityItem(drone.getWorld(), pos.field_72450_a, pos.field_72448_b, pos.field_72449_c, remainder);
                drone.getWorld().func_72838_d(item);
            }
            return true;
        }
    }

    @Override
    public boolean useCount(){
        return useCount;
    }

    @Override
    public void setUseCount(boolean useCount){
        this.useCount = useCount;
    }

    @Override
    public int getCount(){
        return count;
    }

    @Override
    public void setCount(int count){
        this.count = count;
    }

    @Override
    public void writeToNBT(NBTTagCompound tag){
        super.writeToNBT(tag);
        tag.func_74757_a("useCount", useCount);
        tag.func_74768_a("count", count);
    }

    @Override
    public void readFromNBT(NBTTagCompound tag){
        super.readFromNBT(tag);
        useCount = tag.func_74767_n("useCount");
        count = tag.func_74762_e("count");
    }

    @Override
    @SideOnly(Side.CLIENT)
    public GuiScreen getOptionWindow(GuiProgrammer guiProgrammer){
        return new GuiProgWidgetImportExport(this, guiProgrammer){
            @Override
            protected boolean showSides(){
                return false;
            }
        };
    }
}
