package pneumaticCraft.common.progwidgets;

import java.util.HashSet;
import java.util.Set;

import net.minecraft.client.gui.GuiScreen;
import net.minecraft.entity.ai.EntityAIBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.BlockPos;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.client.gui.GuiProgrammer;
import pneumaticCraft.client.gui.programmer.GuiProgWidgetDropItem;
import pneumaticCraft.common.ai.DroneAIImExBase;
import pneumaticCraft.common.ai.IDroneBase;
import pneumaticCraft.common.item.ItemPlastic;
import pneumaticCraft.lib.Textures;

public class ProgWidgetDropItem extends ProgWidgetInventoryBase implements IItemDropper{
    private boolean dropStraight;

    @Override
    public String getWidgetString(){
        return "dropItem";
    }

    @Override
    public ResourceLocation getTexture(){
        return Textures.PROG_WIDGET_DROP_ITEM;
    }

    @Override
    public boolean dropStraight(){
        return dropStraight;
    }

    @Override
    public void setDropStraight(boolean dropStraight){
        this.dropStraight = dropStraight;
    }

    @Override
    public void writeToNBT(NBTTagCompound tag){
        super.writeToNBT(tag);
        tag.func_74757_a("dropStraight", dropStraight);
    }

    @Override
    public void readFromNBT(NBTTagCompound tag){
        super.readFromNBT(tag);
        dropStraight = tag.func_74767_n("dropStraight");
    }

    @Override
    @SideOnly(Side.CLIENT)
    public GuiScreen getOptionWindow(GuiProgrammer guiProgrammer){
        return new GuiProgWidgetDropItem(this, guiProgrammer);
    }

    @Override
    public EntityAIBase getWidgetAI(IDroneBase drone, IProgWidget widget){
        return new DroneAIImExBase(drone, (ProgWidgetAreaItemBase)widget){

            private final Set<BlockPos> visitedPositions = new HashSet<BlockPos>();

            @Override
            public boolean func_75250_a(){
                boolean shouldExecute = false;
                for(int i = 0; i < drone.getInv().func_70302_i_(); i++) {
                    ItemStack stack = drone.getInv().func_70301_a(i);
                    if(stack != null && widget.isItemValidForFilters(stack)) {
                        shouldExecute = super.func_75250_a();
                        break;
                    }
                }
                return shouldExecute;
            }

            @Override
            protected boolean moveIntoBlock(){
                return true;
            }

            @Override
            protected boolean isValidPosition(BlockPos pos){
                return !visitedPositions.contains(pos);//another requirement is that the drone can navigate to this exact block, but that's handled by the pathfinder.
            }

            @Override
            protected boolean doBlockInteraction(BlockPos pos, double distToBlock){
                visitedPositions.add(pos);
                for(int i = 0; i < drone.getInv().func_70302_i_(); i++) {
                    ItemStack stack = drone.getInv().func_70301_a(i);
                    if(stack != null && widget.isItemValidForFilters(stack)) {

                        if(useCount() && getRemainingCount() < stack.field_77994_a) {
                            stack = stack.func_77979_a(getRemainingCount());
                            decreaseCount(getRemainingCount());
                        } else {
                            decreaseCount(stack.field_77994_a);
                            drone.getInv().func_70299_a(i, null);
                        }
                        EntityItem item = new EntityItem(drone.getWorld(), pos.func_177958_n() + 0.5, pos.func_177956_o() + 0.5, pos.func_177952_p() + 0.5, stack);
                        if(((IItemDropper)widget).dropStraight()) {
                            item.field_70159_w = 0;
                            item.field_70181_x = 0;
                            item.field_70179_y = 0;
                        }
                        drone.getWorld().func_72838_d(item);
                        if(useCount() && getRemainingCount() == 0) break;
                    }
                }
                return false;
            }

        };
    }

    @Override
    public int getCraftingColorIndex(){
        return ItemPlastic.POTION_PLANT_DAMAGE;
    }

    @Override
    protected boolean isUsingSides(){
        return false;
    }

    @Override
    public String getExtraStringInfo(){
        return dropStraight() ? "Straight" : "Random";
    }
}
