package pneumaticCraft.common.progwidgets;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.minecraft.client.gui.GuiScreen;
import net.minecraft.entity.ai.EntityAIBase;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.BlockPos;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.client.gui.GuiProgrammer;
import pneumaticCraft.client.gui.programmer.GuiProgWidgetForEach;
import pneumaticCraft.common.ai.DroneAIForEachCoordinate;
import pneumaticCraft.common.ai.IDroneBase;
import pneumaticCraft.common.item.ItemPlastic;
import pneumaticCraft.lib.Textures;

public class ProgWidgetForEachCoordinate extends ProgWidgetAreaItemBase implements IJumpBackWidget, IJump,
        IVariableSetWidget{
    private String elementVariable = "";
    private final Set<BlockPos> traversedPositions = new HashSet<BlockPos>();
    private DroneAIForEachCoordinate ai;

    @Override
    public String getWidgetString(){
        return "forEachCoordinate";
    }

    @Override
    public int getCraftingColorIndex(){
        return ItemPlastic.HELIUM_PLANT_DAMAGE;
    }

    @Override
    public ResourceLocation getTexture(){
        return Textures.PROG_WIDGET_FOR_EACH_COORDINATE;
    }

    @Override
    public Class<? extends IProgWidget>[] getParameters(){
        return new Class[]{ProgWidgetArea.class, ProgWidgetString.class};
    }

    @Override
    public void addVariables(Set<String> variables){
        super.addVariables(variables);
        variables.add(elementVariable);
    }

    @Override
    public String getVariable(){
        return elementVariable;
    }

    @Override
    public void setVariable(String variable){
        elementVariable = variable;
    }

    @Override
    public void writeToNBT(NBTTagCompound tag){
        tag.func_74778_a("variable", elementVariable);
        super.writeToNBT(tag);
    }

    @Override
    public void readFromNBT(NBTTagCompound tag){
        elementVariable = tag.func_74779_i("variable");
        super.readFromNBT(tag);
    }

    @Override
    public WidgetDifficulty getDifficulty(){
        return WidgetDifficulty.ADVANCED;
    }

    @Override
    public IProgWidget getOutputWidget(IDroneBase drone, List<IProgWidget> allWidgets){
        List<String> locations = getPossibleJumpLocations();
        if(locations.size() > 0 && ai != null && (traversedPositions.size() == 1 || !aiManager.getCoordinate(elementVariable).equals(new BlockPos(0, 0, 0)))) {
            BlockPos pos = ai.getCurCoord();
            if(pos != null) {
                aiManager.setCoordinate(elementVariable, pos);
                return ProgWidgetJump.jumpToLabel(drone, allWidgets, locations.get(0));
            }
        }
        traversedPositions.clear();
        return super.getOutputWidget(drone, allWidgets);
    }

    @Override
    public List<String> getPossibleJumpLocations(){
        IProgWidget widget = getConnectedParameters()[getParameters().length - 1];
        ProgWidgetString textWidget = widget != null ? (ProgWidgetString)widget : null;
        List<String> locations = new ArrayList<String>();
        if(textWidget != null) locations.add(textWidget.string);
        return locations;
    }

    @Override
    public EntityAIBase getWidgetAI(IDroneBase drone, IProgWidget widget){
        return ai = new DroneAIForEachCoordinate(drone, (ProgWidgetForEachCoordinate)widget);
    }

    public boolean isValidPosition(BlockPos pos){
        return traversedPositions.add(pos);
    }

    @Override
    public boolean canBeRunByComputers(IDroneBase drone, IProgWidget widget){
        return false;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public GuiScreen getOptionWindow(GuiProgrammer guiProgrammer){
        return new GuiProgWidgetForEach(this, guiProgrammer);
    }

    @Override
    public String getExtraStringInfo(){
        return "\"" + elementVariable + "\"";
    }

    @Override
    public boolean canSetParameter(int index){
        return index != 2;//Don't use the blacklist side of the jump parameter.
    }
}
