package pneumaticCraft.common.progwidgets;

import java.util.List;
import java.util.Set;

import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.gui.GuiScreen;
import net.minecraft.client.renderer.RenderHelper;
import net.minecraft.client.renderer.entity.RenderItem;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.lwjgl.opengl.GL11;

import pneumaticCraft.client.gui.GuiProgrammer;
import pneumaticCraft.client.gui.programmer.GuiProgWidgetItemFilter;
import pneumaticCraft.common.ai.DroneAIManager;
import pneumaticCraft.common.item.ItemPlastic;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.Textures;

public class ProgWidgetItemFilter extends ProgWidget implements IVariableWidget{
    private ItemStack filter;
    public boolean useMetadata = true, useNBT, useOreDict, useModSimilarity;
    public int specificMeta;
    private DroneAIManager aiManager;
    private String variable = "";

    @SideOnly(Side.CLIENT)
    private static RenderItem itemRender;

    @Override
    public void addErrors(List<String> curInfo, List<IProgWidget> widgets){
        super.addErrors(curInfo, widgets);
        if(variable.equals("") && filter == null) {
            curInfo.add("gui.progWidget.itemFilter.error.noFilter");
        }
    }

    @Override
    public void renderExtraInfo(){
        if(variable.equals("")) {
            if(filter != null) {
                drawItemStack(filter, 10, 2, "");
            }
        } else {
            super.renderExtraInfo();
        }
    }

    @Override
    public String getExtraStringInfo(){
        return "\"" + variable + "\"";
    }

    public ItemStack getFilter(){
        return variable.equals("") ? filter : aiManager != null ? aiManager.getStack(variable) : null;
    }

    public void setFilter(ItemStack filter){
        this.filter = filter;
    }

    public static void drawItemStack(ItemStack p_146982_1_, int p_146982_2_, int p_146982_3_, String p_146982_4_){
        RenderHelper.func_74518_a();
        GL11.glPushMatrix();
        Minecraft mc = Minecraft.func_71410_x();
        GL11.glTranslatef(0.0F, 0.0F, 32.0F);
        //  zLevel = 200.0F;
        if(itemRender == null) itemRender = Minecraft.func_71410_x().func_175599_af();
        itemRender.field_77023_b = 200.0F;
        FontRenderer font = null;
        if(p_146982_1_ != null) font = p_146982_1_.func_77973_b().getFontRenderer(p_146982_1_);
        if(font == null) font = mc.field_71466_p;
        itemRender.func_180450_b(p_146982_1_, p_146982_2_, p_146982_3_);
        itemRender.func_180453_a(font, p_146982_1_, p_146982_2_, p_146982_3_, p_146982_4_);
        GL11.glPopMatrix();

        //GL11.glEnable(GL11.GL_LIGHTING);
        RenderHelper.func_74520_c();
        // zLevel = 0.0F;
        //  itemRender.zLevel = 0.0F;
        GL11.glDisable(GL11.GL_LIGHTING);
    }

    @Override
    public void getTooltip(List<String> curTooltip){
        super.getTooltip(curTooltip);
        if(filter != null) {
            curTooltip.add("Current filter:");
            curTooltip.add(filter.func_82833_r());
            if(useOreDict) {
                curTooltip.add("Using Ore Dictionary");
            } else if(useModSimilarity) {
                curTooltip.add("Using Mod similarity");
            } else {
                curTooltip.add((useMetadata ? "Using" : "Ignoring") + " metadata / damage values");
                curTooltip.add((useNBT ? "Using" : "Ignoring") + " NBT tags");
            }

        }
    }

    @Override
    public boolean hasStepInput(){
        return false;
    }

    @Override
    public Class<? extends IProgWidget> returnType(){
        return ProgWidgetItemFilter.class;
    }

    @Override
    public Class<? extends IProgWidget>[] getParameters(){
        return new Class[]{ProgWidgetItemFilter.class};
    }

    @Override
    public String getWidgetString(){
        return "itemFilter";
    }

    @Override
    public ResourceLocation getTexture(){
        return Textures.PROG_WIDGET_ITEM_FILTER;
    }

    @Override
    public void writeToNBT(NBTTagCompound tag){
        super.writeToNBT(tag);
        if(filter != null) {
            filter.func_77955_b(tag);
        }
        tag.func_74757_a("useMetadata", useMetadata);
        tag.func_74757_a("useNBT", useNBT);
        tag.func_74757_a("useOreDict", useOreDict);
        tag.func_74757_a("useModSimilarity", useModSimilarity);
        tag.func_74768_a("specificMeta", specificMeta);
        tag.func_74778_a("variable", variable);
    }

    @Override
    public void readFromNBT(NBTTagCompound tag){
        super.readFromNBT(tag);
        filter = ItemStack.func_77949_a(tag);
        useMetadata = tag.func_74767_n("useMetadata");
        useNBT = tag.func_74767_n("useNBT");
        useOreDict = tag.func_74767_n("useOreDict");
        useModSimilarity = tag.func_74767_n("useModSimilarity");
        specificMeta = tag.func_74762_e("specificMeta");
        variable = tag.func_74779_i("variable");
    }

    @Override
    @SideOnly(Side.CLIENT)
    public GuiScreen getOptionWindow(GuiProgrammer guiProgrammer){
        return new GuiProgWidgetItemFilter(this, guiProgrammer);
    }

    public static boolean isItemValidForFilters(ItemStack item, List<ProgWidgetItemFilter> whitelist, List<ProgWidgetItemFilter> blacklist, IBlockState blockState){
        if(blacklist != null) {
            for(ProgWidgetItemFilter black : blacklist) {
                if(PneumaticCraftUtils.areStacksEqual(black.getFilter(), item, black.useMetadata && blockState == null, black.useNBT, black.useOreDict, black.useModSimilarity)) {
                    if(blockState == null || !black.useMetadata || black.specificMeta == blockState.func_177230_c().func_176201_c(blockState)) {
                        return false;
                    }
                }
            }
        }
        if(whitelist == null || whitelist.size() == 0) {
            return true;
        } else {
            for(ProgWidgetItemFilter white : whitelist) {
                if(PneumaticCraftUtils.areStacksEqual(white.getFilter(), item, white.useMetadata && blockState == null, white.useNBT, white.useOreDict, white.useModSimilarity)) {
                    if(blockState == null || !white.useMetadata || white.specificMeta == blockState.func_177230_c().func_176201_c(blockState)) {
                        return true;
                    }
                }
            }
            return false;
        }
    }

    @Override
    public WidgetDifficulty getDifficulty(){
        return WidgetDifficulty.EASY;
    }

    @Override
    public int getCraftingColorIndex(){
        return ItemPlastic.BURST_PLANT_DAMAGE;
    }

    @Override
    public void setAIManager(DroneAIManager aiManager){
        this.aiManager = aiManager;
    }

    public void setVariable(String variable){
        this.variable = variable;
    }

    public String getVariable(){
        return variable;
    }

    @Override
    public void addVariables(Set<String> variables){
        variables.add(variable);
    }
}
