package pneumaticCraft.common.recipes;

import io.netty.buffer.ByteBuf;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.IFluidHandler;
import net.minecraftforge.fml.common.network.ByteBufUtils;
import pneumaticCraft.common.DroneRegistry;
import pneumaticCraft.common.config.AmadronOfferSettings;
import pneumaticCraft.common.inventory.ContainerAmadron;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketAmadronTradeNotifyDeal;
import pneumaticCraft.common.network.PacketSyncAmadronOffers;
import pneumaticCraft.common.util.PneumaticCraftUtils;

import com.google.gson.JsonObject;

public class AmadronOfferCustom extends AmadronOffer{
    private final String offeringPlayerName;
    private String offeringPlayerId;
    private int providingDimensionId, returningDimensionId;
    private BlockPos providingPosition, returningPosition;
    private int inStock;
    private int maxTrades = -1;
    private int pendingPayments;
    private TileEntity cachedInput, cachedOutput;

    public AmadronOfferCustom(Object input, Object output, EntityPlayer offeringPlayer){
        this(input, output, offeringPlayer.func_146103_bH().getName(), offeringPlayer.func_146103_bH().getId().toString());
    }

    public AmadronOfferCustom(Object input, Object output, String playerName, String playerId){
        super(input, output);
        offeringPlayerName = playerName;
        offeringPlayerId = playerId;
    }

    public AmadronOfferCustom setProvidingPosition(BlockPos pos, int dimensionId){
        providingPosition = pos;
        providingDimensionId = dimensionId;
        cachedInput = null;
        return this;
    }

    public AmadronOfferCustom setReturningPosition(BlockPos pos, int dimensionId){
        returningPosition = pos;
        returningDimensionId = dimensionId;
        cachedOutput = null;
        return this;
    }

    public AmadronOfferCustom invert(){
        Object temp = input;
        input = output;
        output = temp;
        return this;
    }

    public AmadronOfferCustom copy(){
        NBTTagCompound tag = new NBTTagCompound();
        writeToNBT(tag);
        return loadFromNBT(tag);
    }

    public void updatePlayerId(){
        EntityPlayer player = PneumaticCraftUtils.getPlayerFromName(offeringPlayerName);
        if(player != null) offeringPlayerId = player.func_146103_bH().getId().toString();
    }

    public void addStock(int stock){
        inStock += stock;
    }

    @Override
    public int getStock(){
        return inStock;
    }

    public void addPayment(int payment){
        pendingPayments += payment;
    }

    public void setMaxTrades(int maxTrades){
        this.maxTrades = maxTrades;
    }

    @Override
    public String getVendor(){
        return offeringPlayerName;
    }

    public String getPlayerId(){
        return offeringPlayerId;
    }

    @Override
    public void onTrade(int tradingAmount, String buyingPlayer){
        EntityPlayer player = PneumaticCraftUtils.getPlayerFromId(offeringPlayerId);
        if(player != null && AmadronOfferSettings.notifyOfDealMade) {
            NetworkHandler.sendTo(new PacketAmadronTradeNotifyDeal(this, tradingAmount, buyingPlayer), (EntityPlayerMP)player);
        }
    }

    public void payout(){
        TileEntity returning = getReturningTileEntity();
        TileEntity provider = getProvidingTileEntity();
        if(pendingPayments > 0) {
            int paying = Math.min(pendingPayments, 50);
            paying = ContainerAmadron.capShoppingAmount(this, paying, provider instanceof IInventory ? (IInventory)provider : null, returning instanceof IInventory ? (IInventory)returning : null, provider instanceof IFluidHandler ? (IFluidHandler)provider : null, returning instanceof IFluidHandler ? (IFluidHandler)returning : null, null);
            if(paying > 0) {
                pendingPayments -= paying;
                if(getInput() instanceof ItemStack) {
                    ItemStack deliveringItems = (ItemStack)getInput();
                    int amount = deliveringItems.field_77994_a * paying;
                    List<ItemStack> stacks = new ArrayList<ItemStack>();
                    while(amount > 0) {
                        ItemStack stack = deliveringItems.func_77946_l();
                        stack.field_77994_a = Math.min(amount, stack.func_77976_d());
                        stacks.add(stack);
                        amount -= stack.field_77994_a;
                    }
                    DroneRegistry.getInstance().deliverItemsAmazonStyle(returning.func_145831_w(), returning.func_174877_v(), stacks.toArray(new ItemStack[stacks.size()]));
                } else {
                    FluidStack deliveringFluid = ((FluidStack)getInput()).copy();
                    deliveringFluid.amount *= paying;
                    DroneRegistry.getInstance().deliverFluidAmazonStyle(returning.func_145831_w(), returning.func_174877_v(), deliveringFluid);
                }
            }
        }
    }

    public void returnStock(){
        TileEntity provider = getProvidingTileEntity();
        TileEntity returning = getReturningTileEntity();
        invert();
        while(inStock > 0) {
            int stock = Math.min(inStock, 50);
            stock = ContainerAmadron.capShoppingAmount(this, stock, returning instanceof IInventory ? (IInventory)returning : null, provider instanceof IInventory ? (IInventory)provider : null, returning instanceof IFluidHandler ? (IFluidHandler)returning : null, provider instanceof IFluidHandler ? (IFluidHandler)provider : null, null);
            if(stock > 0) {
                inStock -= stock;
                if(getInput() instanceof ItemStack) {
                    ItemStack deliveringItems = (ItemStack)getInput();
                    int amount = deliveringItems.field_77994_a * stock;
                    List<ItemStack> stacks = new ArrayList<ItemStack>();
                    while(amount > 0) {
                        ItemStack stack = deliveringItems.func_77946_l();
                        stack.field_77994_a = Math.min(amount, stack.func_77976_d());
                        stacks.add(stack);
                        amount -= stack.field_77994_a;
                    }
                    DroneRegistry.getInstance().deliverItemsAmazonStyle(provider.func_145831_w(), provider.func_174877_v(), stacks.toArray(new ItemStack[stacks.size()]));
                } else {
                    FluidStack deliveringFluid = ((FluidStack)getInput()).copy();
                    deliveringFluid.amount *= stock;
                    DroneRegistry.getInstance().deliverFluidAmazonStyle(provider.func_145831_w(), provider.func_174877_v(), deliveringFluid);
                }
            } else {
                break;
            }
        }
    }

    public TileEntity getProvidingTileEntity(){
        if(cachedInput == null || cachedInput.func_145837_r()) {
            if(providingPosition != null) {
                cachedInput = PneumaticCraftUtils.getTileEntity(providingPosition, providingDimensionId);
            }
        }
        return cachedInput;
    }

    public TileEntity getReturningTileEntity(){
        if(cachedOutput == null || cachedOutput.func_145837_r()) {
            if(returningPosition != null) {
                cachedOutput = PneumaticCraftUtils.getTileEntity(returningPosition, returningDimensionId);
            }
        }
        return cachedOutput;
    }

    @Override
    public void writeToNBT(NBTTagCompound tag){
        super.writeToNBT(tag);
        tag.func_74778_a("offeringPlayerId", offeringPlayerId);
        tag.func_74778_a("offeringPlayerName", offeringPlayerName);
        tag.func_74768_a("inStock", inStock);
        tag.func_74768_a("maxTrades", maxTrades);
        tag.func_74768_a("pendingPayments", pendingPayments);
        if(providingPosition != null) {
            tag.func_74768_a("providingDimensionId", providingDimensionId);
            tag.func_74768_a("providingX", providingPosition.func_177958_n());
            tag.func_74768_a("providingY", providingPosition.func_177956_o());
            tag.func_74768_a("providingZ", providingPosition.func_177952_p());
        }
        if(returningPosition != null) {
            tag.func_74768_a("returningDimensionId", returningDimensionId);
            tag.func_74768_a("returningX", returningPosition.func_177958_n());
            tag.func_74768_a("returningY", returningPosition.func_177956_o());
            tag.func_74768_a("returningZ", returningPosition.func_177952_p());
        }
    }

    public static AmadronOfferCustom loadFromNBT(NBTTagCompound tag){
        AmadronOffer offer = AmadronOffer.loadFromNBT(tag);
        AmadronOfferCustom custom = new AmadronOfferCustom(offer.getInput(), offer.getOutput(), tag.func_74779_i("offeringPlayerName"), tag.func_74779_i("offeringPlayerId"));
        custom.inStock = tag.func_74762_e("inStock");
        custom.maxTrades = tag.func_74762_e("maxTrades");
        custom.pendingPayments = tag.func_74762_e("pendingPayments");
        if(tag.func_74764_b("providingDimensionId")) {
            custom.setProvidingPosition(new BlockPos(tag.func_74762_e("providingX"), tag.func_74762_e("providingY"), tag.func_74762_e("providingZ")), tag.func_74762_e("providingDimensionId"));
        }
        if(tag.func_74764_b("returningDimensionId")) {
            custom.setReturningPosition(new BlockPos(tag.func_74762_e("returningX"), tag.func_74762_e("returningY"), tag.func_74762_e("returningZ")), tag.func_74762_e("returningDimensionId"));
        }
        return custom;
    }

    public void writeToBuf(ByteBuf buf){
        ByteBufUtils.writeUTF8String(buf, offeringPlayerName);
        ByteBufUtils.writeUTF8String(buf, offeringPlayerId);
        if(providingPosition != null) {
            buf.writeBoolean(true);
            buf.writeInt(providingPosition.func_177958_n());
            buf.writeInt(providingPosition.func_177956_o());
            buf.writeInt(providingPosition.func_177952_p());
            buf.writeInt(providingDimensionId);
        } else {
            buf.writeBoolean(false);
        }
        if(returningPosition != null) {
            buf.writeBoolean(true);
            buf.writeInt(returningPosition.func_177958_n());
            buf.writeInt(returningPosition.func_177956_o());
            buf.writeInt(returningPosition.func_177952_p());
            buf.writeInt(returningDimensionId);
        } else {
            buf.writeBoolean(false);
        }
        buf.writeInt(inStock);
        buf.writeInt(maxTrades);
        buf.writeInt(pendingPayments);
    }

    public static AmadronOfferCustom loadFromBuf(ByteBuf buf){
        AmadronOfferCustom offer = new AmadronOfferCustom(PacketSyncAmadronOffers.getFluidOrItemStack(buf), PacketSyncAmadronOffers.getFluidOrItemStack(buf), ByteBufUtils.readUTF8String(buf), ByteBufUtils.readUTF8String(buf));
        if(buf.readBoolean()) {
            offer.setProvidingPosition(new BlockPos(buf.readInt(), buf.readInt(), buf.readInt()), buf.readInt());
        }
        if(buf.readBoolean()) {
            offer.setReturningPosition(new BlockPos(buf.readInt(), buf.readInt(), buf.readInt()), buf.readInt());
        }
        offer.inStock = buf.readInt();
        offer.maxTrades = buf.readInt();
        offer.pendingPayments = buf.readInt();
        return offer;
    }

    @Override
    public JsonObject toJson(){
        JsonObject json = super.toJson();
        json.addProperty("offeringPlayerName", offeringPlayerName);
        json.addProperty("offeringPlayerId", offeringPlayerId);
        json.addProperty("inStock", inStock);
        json.addProperty("maxTrades", maxTrades);
        json.addProperty("pendingPayments", pendingPayments);
        if(providingPosition != null) {
            json.addProperty("providingDimensionId", providingDimensionId);
            json.addProperty("providingX", providingPosition.func_177958_n());
            json.addProperty("providingY", providingPosition.func_177956_o());
            json.addProperty("providingZ", providingPosition.func_177952_p());
        }
        if(returningPosition != null) {
            json.addProperty("returningDimensionId", returningDimensionId);
            json.addProperty("returningX", returningPosition.func_177958_n());
            json.addProperty("returningY", returningPosition.func_177956_o());
            json.addProperty("returningZ", returningPosition.func_177952_p());
        }
        return json;
    }

    public static AmadronOfferCustom fromJson(JsonObject json){
        AmadronOffer offer = AmadronOffer.fromJson(json);
        if(offer != null) {
            AmadronOfferCustom custom = new AmadronOfferCustom(offer.input, offer.output, json.get("offeringPlayerName").getAsString(), json.get("offeringPlayerId").getAsString());
            custom.inStock = json.get("inStock").getAsInt();
            custom.maxTrades = json.get("maxTrades").getAsInt();
            custom.pendingPayments = json.get("pendingPayments").getAsInt();
            if(json.has("providingDimensionId")) {
                custom.providingDimensionId = json.get("providingDimensionId").getAsInt();
                custom.providingPosition = new BlockPos(json.get("providingX").getAsInt(), json.get("providingY").getAsInt(), json.get("providingZ").getAsInt());
            }
            if(json.has("returningDimensionId")) {
                custom.returningDimensionId = json.get("returningDimensionId").getAsInt();
                custom.returningPosition = new BlockPos(json.get("returningX").getAsInt(), json.get("returningY").getAsInt(), json.get("returningZ").getAsInt());
            }
            return custom;
        } else {
            return null;
        }
    }

    @Override
    public boolean equals(Object o){
        if(o instanceof AmadronOfferCustom) {
            AmadronOfferCustom offer = (AmadronOfferCustom)o;
            return super.equals(o) && offer.offeringPlayerId.equals(offeringPlayerId);
        } else {
            return false;
        }
    }

    @Override
    public int hashCode(){
        return super.hashCode() * 31 + offeringPlayerId.hashCode();
    }
}
