package pneumaticCraft.common.recipes;

import java.util.Random;

import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import pneumaticCraft.api.recipe.IPressureChamberRecipe;

public class PressureChamberVacuumEnchantHandler implements IPressureChamberRecipe{

    /**
     * Returns the threshold which is minimal to craft the recipe. Negative pressures also work.
     * @return threshold pressure
     */
    @Override
    public float getCraftingPressure(){
        return -0.75F;
    }

    /**
     * This method should return the used items in the recipe when the right items are provided to craft this recipe.
     * @param inputStacks
     * @return usedStacks, return null when the inputStacks aren't valid for this recipe.
     */
    @Override
    public ItemStack[] isValidRecipe(ItemStack[] inputStacks){
        int enchantmentCount = 0;
        for(ItemStack stack : inputStacks) {
            if(stack.func_77973_b() == Items.field_151134_bR) continue;
            enchantmentCount = EnchantmentHelper.func_82781_a(stack).size();
            if(enchantmentCount > 0) break;
        }
        if(enchantmentCount == 0) return null;
        for(ItemStack stack : inputStacks) {
            if(stack.func_77973_b() == Items.field_151122_aG) {
                return new ItemStack[]{new ItemStack(Items.field_151122_aG)};
            }
        }
        return null;

    }

    /**
     * This method will be called when the recipe should output its items. te stacks the recipe outputs, maybe dependent on the input stacks.
     * @param inputStacks. These stacks can be modified (like adding/removing NBT data eg.)
     * @return outputStacks. Stacks that will pop 'out of the chamber'
     */
    @Override
    public ItemStack[] craftRecipe(ItemStack[] inputStacks, ItemStack[] removedStacks){
        ItemStack enchantedStack = null;
        int enchantmentCount = 0;
        for(ItemStack stack : inputStacks) {
            if(stack.func_77973_b() == Items.field_151134_bR) continue;
            enchantmentCount = EnchantmentHelper.func_82781_a(stack).size();
            if(enchantmentCount > 0) {
                enchantedStack = stack;
                break;
            }
        }
        if(enchantedStack == null) {
            System.err.println("[Pressure Chamber Vacuum Enchantment Handler] No enchanted stack found! Report to MineMaarten!");
            return null;
        }

        // take a random enchantment of the enchanted stack.
        int enchIndex = new Random().nextInt(((NBTTagList)enchantedStack.func_77978_p().func_74781_a("ench")).func_74745_c());
        NBTTagCompound enchTag = ((NBTTagList)enchantedStack.func_77978_p().func_74781_a("ench")).func_150305_b(enchIndex);

        ((NBTTagList)enchantedStack.func_77978_p().func_74781_a("ench")).func_74744_a(enchIndex);
        if(((NBTTagList)enchantedStack.func_77978_p().func_74781_a("ench")).func_74745_c() == 0) {
            enchantedStack.func_77978_p().func_82580_o("ench");
            if(enchantedStack.func_77978_p().func_82582_d()) {
                enchantedStack.func_77982_d(null);
            }
        }

        //and create an enchanted book.
        ItemStack enchantedBook = new ItemStack(Items.field_151134_bR);
        NBTTagCompound baseTag = new NBTTagCompound();
        NBTTagList enchList = new NBTTagList();
        enchList.func_74742_a(enchTag);
        baseTag.func_74782_a("StoredEnchantments", enchList);
        enchantedBook.func_77982_d(baseTag);
        return new ItemStack[]{enchantedBook};
    }

    @Override
    public boolean shouldRemoveExactStacks(){
        return false;
    }

}
