package pneumaticCraft.common.remote;

import net.minecraft.client.gui.GuiScreen;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.BlockPos;
import pneumaticCraft.client.gui.GuiRemoteEditor;
import pneumaticCraft.client.gui.widget.IGuiWidget;
import pneumaticCraft.lib.Log;

public abstract class ActionWidget<Widget extends IGuiWidget> {
    protected Widget widget;
    private String enableVariable = "";
    private BlockPos enablingValue = new BlockPos(0, 0, 0);;

    public ActionWidget(Widget widget){
        this.widget = widget;
    }

    public ActionWidget(){}

    public void readFromNBT(NBTTagCompound tag, int guiLeft, int guiTop){
        enableVariable = tag.func_74779_i("enableVariable");
        enablingValue = tag.func_74764_b("enablingX") ? new BlockPos(tag.func_74762_e("enablingX"), tag.func_74762_e("enablingY"), tag.func_74762_e("enablingZ")) : new BlockPos(1, 0, 0);
    }

    public NBTTagCompound toNBT(int guiLeft, int guitTop){
        NBTTagCompound tag = new NBTTagCompound();
        tag.func_74778_a("id", getId());
        tag.func_74778_a("enableVariable", enableVariable);
        tag.func_74768_a("enablingX", enablingValue.func_177958_n());
        tag.func_74768_a("enablingY", enablingValue.func_177956_o());
        tag.func_74768_a("enablingZ", enablingValue.func_177952_p());
        return tag;
    }

    public ActionWidget copy(){
        try {
            ActionWidget widget = this.getClass().newInstance();
            widget.readFromNBT(this.toNBT(0, 0), 0, 0);
            return widget;
        } catch(Exception e) {
            Log.error("Error occured when trying to copy an " + getId() + " action widget.");
            e.printStackTrace();
            return null;
        }
    }

    public Widget getWidget(){
        return widget;
    }

    public abstract void setWidgetPos(int x, int y);

    public abstract String getId();

    public GuiScreen getGui(GuiRemoteEditor guiRemote){
        return null;
    }

    public void setEnableVariable(String varName){
        this.enableVariable = varName;
    }

    public String getEnableVariable(){
        return enableVariable;
    }

    public boolean isEnabled(){
        return enableVariable.equals("") || GlobalVariableManager.getInstance().getPos(enableVariable).equals(enablingValue);
    }

    public void setEnablingValue(int x, int y, int z){
        enablingValue = new BlockPos(x, y, z);
    }

    public BlockPos getEnablingValue(){
        return enablingValue;
    }
}