package pneumaticCraft.common.remote;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.GuiScreen;
import net.minecraft.client.resources.I18n;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.MathHelper;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.apache.commons.lang3.text.WordUtils;

import pneumaticCraft.client.gui.GuiRemoteEditor;
import pneumaticCraft.client.gui.remote.GuiRemoteDropdown;
import pneumaticCraft.client.gui.widget.WidgetComboBox;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketSetGlobalVariable;

public class ActionWidgetDropdown extends ActionWidgetVariable<WidgetComboBox>{

    private int x, y, width, height;
    private String dropDownElements = "";
    private String selectedElement = "";

    public ActionWidgetDropdown(){
        super();
    }

    public ActionWidgetDropdown(WidgetComboBox widget){
        super(widget);
        width = widget.field_146218_h;
        height = widget.field_146219_i;
        widget.func_146180_a(I18n.func_135052_a("remote.dropdown.name"));
        widget.setTooltip(WordUtils.wrap(I18n.func_135052_a("remote.dropdown.tooltip"), 50).split(System.getProperty("line.separator")));
    }

    @Override
    public void readFromNBT(NBTTagCompound tag, int guiLeft, int guiTop){
        super.readFromNBT(tag, guiLeft, guiTop);
        x = tag.func_74762_e("x") + guiLeft;
        y = tag.func_74762_e("y") + guiTop;
        width = tag.func_74762_e("width");
        height = tag.func_74762_e("height");
        dropDownElements = tag.func_74779_i("dropDownElements");
        updateWidget();
    }

    @Override
    public NBTTagCompound toNBT(int guiLeft, int guiTop){
        NBTTagCompound tag = super.toNBT(guiLeft, guiTop);
        tag.func_74768_a("x", x - guiLeft);
        tag.func_74768_a("y", y - guiTop);
        tag.func_74768_a("width", width);
        tag.func_74768_a("height", height);
        tag.func_74778_a("dropDownElements", dropDownElements);

        return tag;
    }

    @Override
    public String getId(){
        return "dropdown";
    }

    @Override
    public void onKeyTyped(){
        String[] elements = getDropdownElements();
        selectedElement = getWidget().func_146179_b();
        for(int i = 0; i < elements.length; i++) {
            if(elements[i].equals(selectedElement)) {
                NetworkHandler.sendToServer(new PacketSetGlobalVariable(getVariableName(), i));
                break;
            }
        }
    }

    @Override
    public void onVariableChange(){
        updateWidget();
    }

    @Override
    public void setWidgetPos(int x, int y){
        this.x = x;
        this.y = y;
        updateWidget();
    }

    @Override
    public WidgetComboBox getWidget(){
        if(widget == null) {
            widget = new WidgetComboBox(Minecraft.func_71410_x().field_71466_p, x, y, width, height);
            widget.setElements(getDropdownElements());
            widget.setFixedOptions();
            updateWidget();
        }
        return widget;
    }

    private String[] getDropdownElements(){
        return dropDownElements.split(",");
    }

    private void updateWidget(){
        String[] elements = getDropdownElements();
        selectedElement = elements[MathHelper.func_76125_a(GlobalVariableManager.getInstance().getInteger(getVariableName()), 0, elements.length - 1)];

        if(widget != null) {
            widget.field_146209_f = x;
            widget.field_146210_g = y;
            widget.field_146218_h = width;
            widget.field_146219_i = height;
            widget.setElements(getDropdownElements());
            widget.func_146180_a(selectedElement);
        }
    }

    @Override
    public void onActionPerformed(){}

    public void setDropDownElements(String dropDownElements){
        this.dropDownElements = dropDownElements;
        updateWidget();
    }

    public String getDropDownElements(){
        return dropDownElements;
    }

    public void setWidth(int width){
        this.width = width;
        updateWidget();
    }

    public int getWidth(){
        return width;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public GuiScreen getGui(GuiRemoteEditor guiRemote){
        return new GuiRemoteDropdown(this, guiRemote);
    }
}
