package pneumaticCraft.common.remote;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.WorldSavedData;
import net.minecraftforge.fml.common.FMLCommonHandler;
import net.minecraftforge.fml.relauncher.Side;

/**
 * Manages global variables. These are prefixed with '#'.
 */
public class GlobalVariableManager extends WorldSavedData{

    public static final String DATA_KEY = "PneumaticCraftGlobalVariables";
    private static GlobalVariableManager CLIENT_INSTANCE = new GlobalVariableManager(DATA_KEY);
    private final Map<String, BlockPos> globalVars = new HashMap<String, BlockPos>();
    private final Map<String, ItemStack> globalItemVars = new HashMap<String, ItemStack>();
    public static World overworld;

    public static GlobalVariableManager getInstance(){
        if(FMLCommonHandler.instance().getEffectiveSide() == Side.CLIENT) {
            return CLIENT_INSTANCE;
        } else {
            if(overworld != null) {
                GlobalVariableManager manager = (GlobalVariableManager)overworld.func_72943_a(GlobalVariableManager.class, DATA_KEY);
                if(manager == null) {
                    manager = new GlobalVariableManager(DATA_KEY);
                    overworld.func_72823_a(DATA_KEY, manager);
                }
                return manager;
            } else {
                throw new IllegalStateException("Overworld not initialized");
            }
        }
    }

    public void set(String varName, boolean value){
        set(varName, value ? 1 : 0);
    }

    public void set(String varName, int value){
        set(varName, value, 0, 0);
    }

    public void set(String varName, int x, int y, int z){
        set(varName, new BlockPos(x, y, z));
    }

    public void set(String varName, BlockPos pos){
        globalVars.put(varName, pos);
        save();
    }

    public void set(String varName, ItemStack item){
        globalItemVars.put(varName, item);
        save();
    }

    private void save(){
        func_76185_a();
    }

    public boolean getBoolean(String varName){
        return getInteger(varName) != 0;
    }

    public int getInteger(String varName){
        return getPos(varName).func_177958_n();
    }

    public BlockPos getPos(String varName){
        BlockPos pos = globalVars.get(varName);
        //if(pos != null) Log.info("getting var: " + varName + " set to " + pos.chunkPosX + ", " + pos.chunkPosY + ", " + pos.chunkPosZ);
        return pos != null ? pos : new BlockPos(0, 0, 0);
    }

    public ItemStack getItem(String varName){
        return globalItemVars.get(varName);
    }

    public GlobalVariableManager(String dataKey){
        super(dataKey);
    }

    @Override
    public void func_76184_a(NBTTagCompound tag){
        globalVars.clear();
        NBTTagList list = tag.func_150295_c("globalVars", 10);
        for(int i = 0; i < list.func_74745_c(); i++) {
            NBTTagCompound t = list.func_150305_b(i);
            globalVars.put(t.func_74779_i("varName"), new BlockPos(t.func_74762_e("x"), t.func_74762_e("y"), t.func_74762_e("z")));
        }

        readItemVars(tag, globalItemVars);
    }

    public static void readItemVars(NBTTagCompound tag, Map<String, ItemStack> map){
        map.clear();
        NBTTagList list = tag.func_150295_c("globalItemVars", 10);
        for(int i = 0; i < list.func_74745_c(); i++) {
            NBTTagCompound t = list.func_150305_b(i);
            map.put(t.func_74779_i("varName"), ItemStack.func_77949_a(t.func_74775_l("item")));
        }
    }

    @Override
    public void func_76187_b(NBTTagCompound tag){
        NBTTagList list = new NBTTagList();
        for(Map.Entry<String, BlockPos> entry : globalVars.entrySet()) {
            NBTTagCompound t = new NBTTagCompound();
            t.func_74778_a("varName", entry.getKey());
            BlockPos pos = entry.getValue();
            t.func_74768_a("x", pos.func_177958_n());
            t.func_74768_a("y", pos.func_177956_o());
            t.func_74768_a("z", pos.func_177952_p());
            list.func_74742_a(t);
        }
        tag.func_74782_a("globalVars", list);

        writeItemVars(tag, globalItemVars);
    }

    public void writeItemVars(NBTTagCompound tag, Map<String, ItemStack> map){
        NBTTagList list = new NBTTagList();
        for(Map.Entry<String, ItemStack> entry : globalItemVars.entrySet()) {
            NBTTagCompound t = new NBTTagCompound();
            t.func_74778_a("varName", entry.getKey());
            NBTTagCompound itemTag = new NBTTagCompound();
            entry.getValue().func_77955_b(itemTag);
            t.func_74782_a("item", itemTag);
            list.func_74742_a(t);
        }
        tag.func_74782_a("globalItemVars", list);
    }

    public String[] getAllActiveVariableNames(){
        Set<String> varNames = new HashSet<String>();
        varNames.addAll(globalVars.keySet());
        varNames.addAll(globalItemVars.keySet());
        return varNames.toArray(new String[varNames.size()]);
    }

}
