package pneumaticCraft.common.semiblock;

import java.util.List;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.FurnaceRecipes;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.StatCollector;

import org.apache.commons.lang3.tuple.Pair;

import pneumaticCraft.api.PneumaticRegistry;
import pneumaticCraft.api.heat.IHeatExchangerLogic;
import pneumaticCraft.api.tileentity.IHeatExchanger;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.recipes.PneumaticRecipeRegistry;
import pneumaticCraft.common.tileentity.TileEntityCompressedIronBlock;
import pneumaticCraft.common.util.IOHelper;

public class SemiBlockHeatFrame extends SemiBlockBasic implements IHeatExchanger{
    private final IHeatExchangerLogic logic = PneumaticRegistry.getInstance().getHeatRegistry().getHeatExchangerLogic();
    private int lastValidSlot;//Performance increaser
    private int cookingProgress;
    private int coolingProgress;
    @DescSynced
    private int heatLevel = 10;

    @Override
    public boolean canPlace(){
        return getTileEntity() instanceof IInventory;
    }

    public int getHeatLevel(){
        return heatLevel;
    }

    @Override
    public void update(){
        super.update();
        if(!getWorld().field_72995_K) {
            heatLevel = TileEntityCompressedIronBlock.getHeatLevelForTemperature(logic.getTemperature());
            if(logic.getTemperature() > 374) {
                if(cookingProgress < 100) {
                    int progress = Math.max(0, ((int)logic.getTemperature() - 343) / 30);
                    progress = Math.min(5, progress);
                    logic.addHeat(-progress * 1);
                    cookingProgress += progress;
                }
                if(cookingProgress >= 100) {
                    IInventory inv = IOHelper.getInventoryForTE(getTileEntity());
                    if(inv != null) {
                        if(!tryCookSlot(inv, lastValidSlot)) {
                            for(int i = 0; i < inv.func_70302_i_(); i++) {
                                if(tryCookSlot(inv, i)) {
                                    cookingProgress -= 100;
                                    break;
                                }
                            }
                        } else {
                            cookingProgress -= 100;
                        }
                    }
                }
            } else if(logic.getTemperature() < 273) {
                if(coolingProgress < 100) {
                    int progress = Math.max(0, ((int)logic.getTemperature() - 243) / 30);
                    progress = 6 - Math.min(5, progress);
                    logic.addHeat(progress * 1);
                    coolingProgress += progress;
                }
                if(coolingProgress >= 100) {
                    IInventory inv = IOHelper.getInventoryForTE(getTileEntity());
                    if(inv != null) {
                        if(!tryCoolSlot(inv, lastValidSlot)) {
                            for(int i = 0; i < inv.func_70302_i_(); i++) {
                                if(tryCoolSlot(inv, i)) {
                                    coolingProgress -= 100;
                                    break;
                                }
                            }
                        } else {
                            coolingProgress -= 100;
                        }
                    }
                }
            }
        }
    }

    private boolean tryCookSlot(IInventory inv, int slot){
        ItemStack stack = inv.func_70301_a(slot);
        if(stack != null) {
            ItemStack result = FurnaceRecipes.func_77602_a().func_151395_a(stack);
            if(result != null) {
                ItemStack remainder = IOHelper.insert(getTileEntity(), result, true);
                if(remainder == null) {
                    IOHelper.insert(getTileEntity(), result, false);
                    inv.func_70298_a(slot, 1);
                    lastValidSlot = slot;
                    return true;
                }
            }
        }
        return false;
    }

    private boolean tryCoolSlot(IInventory inv, int slot){
        ItemStack stack = inv.func_70301_a(slot);
        if(stack != null) {
            for(Pair<Object, ItemStack> recipe : PneumaticRecipeRegistry.getInstance().heatFrameCoolingRecipes) {
                if(PneumaticRecipeRegistry.isItemEqual(recipe.getKey(), stack)) {
                    int amount = PneumaticRecipeRegistry.getItemAmount(recipe.getKey());
                    if(stack.field_77994_a >= amount) {
                        ItemStack containerItem = stack.func_77973_b().getContainerItem(stack);
                        boolean canStoreContainerItem = false;
                        boolean canStoreOutput = false;
                        for(int i = 0; i < inv.func_70302_i_(); i++) {
                            ItemStack s = inv.func_70301_a(i);
                            if(s == null) {
                                if(canStoreOutput) {
                                    canStoreContainerItem = true;
                                } else {
                                    canStoreOutput = true;
                                }
                            } else {
                                if(s.func_77969_a(recipe.getRight()) && ItemStack.func_77970_a(s, recipe.getRight()) && s.func_77976_d() >= s.field_77994_a + recipe.getRight().field_77994_a) {
                                    canStoreOutput = true;
                                }
                                if(containerItem != null && s.func_77969_a(containerItem) && ItemStack.func_77970_a(s, containerItem) && s.func_77976_d() >= s.field_77994_a + containerItem.field_77994_a) {
                                    canStoreContainerItem = true;
                                }
                            }
                        }
                        if(canStoreOutput && (containerItem == null || canStoreContainerItem)) {
                            inv.func_70298_a(slot, amount);
                            IOHelper.insert(getTileEntity(), recipe.getValue().func_77946_l(), false);
                            if(containerItem != null) IOHelper.insert(getTileEntity(), containerItem.func_77946_l(), false);
                            return true;
                        }
                    }
                }
            }
        }
        return false;
    }

    @Override
    public void writeToNBT(NBTTagCompound tag){
        super.writeToNBT(tag);
        logic.writeToNBT(tag);
    }

    @Override
    public void readFromNBT(NBTTagCompound tag){
        super.readFromNBT(tag);
        logic.readFromNBT(tag);
    }

    @Override
    public void onPlaced(EntityPlayer player, ItemStack stack){
        super.onPlaced(player, stack);
        getWorld().func_175685_c(getPos(), getBlockState().func_177230_c());
    }

    @Override
    public IHeatExchangerLogic getHeatExchangerLogic(EnumFacing side){
        return logic;
    }

    @Override
    public void invalidate(){
        super.invalidate();
        getWorld().func_175685_c(getPos(), getBlockState().func_177230_c());
    }

    @Override
    public void addWailaInfoToTag(NBTTagCompound tag){
        super.addWailaInfoToTag(tag);
        tag.func_74768_a("temp", (int)logic.getTemperature());
    }

    @Override
    public void addWailaTooltip(List<String> curInfo, NBTTagCompound tag){
        super.addWailaTooltip(curInfo, tag);
        curInfo.add(StatCollector.func_74837_a("waila.temperature", tag.func_74762_e("temp") - 273));
    }
}
