package pneumaticCraft.common.semiblock;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.minecraft.block.Block;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.chunk.Chunk;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.event.world.ChunkDataEvent;
import net.minecraftforge.event.world.ChunkEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.TickEvent;
import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.common.NBTUtil;
import pneumaticCraft.common.item.Itemss;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketDescription;
import pneumaticCraft.common.network.PacketSetSemiBlock;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.Log;

import com.google.common.collect.HashBiMap;

public class SemiBlockManager{
    private final Map<Chunk, Map<BlockPos, ISemiBlock>> semiBlocks = new HashMap<Chunk, Map<BlockPos, ISemiBlock>>();
    private final List<ISemiBlock> addingBlocks = new ArrayList<ISemiBlock>();
    private final Map<Chunk, Set<EntityPlayer>> syncList = new HashMap<Chunk, Set<EntityPlayer>>();
    private final Set<Chunk> chunksMarkedForRemoval = new HashSet<Chunk>();
    public static final int SYNC_DISTANCE = 64;
    private static final HashBiMap<String, Class<? extends ISemiBlock>> registeredTypes = HashBiMap.create();
    private static final HashBiMap<Class<? extends ISemiBlock>, Item> semiBlockToItems = HashBiMap.create();
    private static final SemiBlockManager INSTANCE = new SemiBlockManager();
    private static final SemiBlockManager CLIENT_INSTANCE = new SemiBlockManager();

    public static SemiBlockManager getServerInstance(){
        return INSTANCE;
    }

    public static SemiBlockManager getClientOldInstance(){
        return CLIENT_INSTANCE;
    }

    public static SemiBlockManager getInstance(World world){
        return world.field_72995_K ? CLIENT_INSTANCE : INSTANCE;
    }

    public static Item registerSemiBlock(String key, Class<? extends ISemiBlock> semiBlock, boolean addItem){
        if(registeredTypes.containsKey(key)) throw new IllegalArgumentException("Duplicate registration key: " + key);
        registeredTypes.put(key, semiBlock);

        if(addItem) {
            ItemSemiBlockBase item = new ItemSemiBlockBase(key);
            Itemss.registerItem(item, key);
            PneumaticCraft.proxy.registerSemiBlockRenderer(item);
            registerSemiBlockToItemMapping(semiBlock, item);
            return item;
        } else {
            return null;
        }
    }

    public static void registerSemiBlockToItemMapping(Class<? extends ISemiBlock> semiBlock, Item item){
        semiBlockToItems.put(semiBlock, item);
    }

    public static Item getItemForSemiBlock(ISemiBlock semiBlock){
        return getItemForSemiBlock(semiBlock.getClass());
    }

    public static Item getItemForSemiBlock(Class<? extends ISemiBlock> semiBlock){
        return semiBlockToItems.get(semiBlock);
    }

    public static Class<? extends ISemiBlock> getSemiBlockForItem(Item item){
        return semiBlockToItems.inverse().get(item);
    }

    public static String getKeyForSemiBlock(ISemiBlock semiBlock){
        return getKeyForSemiBlock(semiBlock.getClass());
    }

    public static String getKeyForSemiBlock(Class<? extends ISemiBlock> semiBlock){
        return registeredTypes.inverse().get(semiBlock);
    }

    public static ISemiBlock getSemiBlockForKey(String key){
        try {
            Class<? extends ISemiBlock> clazz = registeredTypes.get(key);
            if(clazz != null) {
                return clazz.newInstance();
            } else {
                Log.warning("Semi Block with id \"" + key + "\" isn't registered!");
                return null;
            }
        } catch(Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    @SubscribeEvent
    public void onChunkUnLoad(ChunkEvent.Unload event){
        if(!event.world.field_72995_K) {
            chunksMarkedForRemoval.add(event.getChunk());
        }
    }

    @SubscribeEvent
    public void onChunkSave(ChunkDataEvent.Save event){
        Map<BlockPos, ISemiBlock> map = semiBlocks.get(event.getChunk());
        if(map != null && map.size() > 0) {
            NBTTagList tagList = new NBTTagList();
            for(Map.Entry<BlockPos, ISemiBlock> entry : map.entrySet()) {
                NBTTagCompound t = new NBTTagCompound();
                entry.getValue().writeToNBT(t);
                NBTUtil.setPos(t, entry.getKey());
                t.func_74778_a("type", getKeyForSemiBlock(entry.getValue()));
                tagList.func_74742_a(t);
            }
            event.getData().func_74782_a("SemiBlocks", tagList);
        }
    }

    @SubscribeEvent
    public void onChunkLoad(ChunkDataEvent.Load event){
        try {
            if(!event.world.field_72995_K) {
                if(event.getData().func_74764_b("SemiBlocks")) {
                    Map<BlockPos, ISemiBlock> map = getOrCreateMap(event.getChunk());
                    map.clear();
                    NBTTagList tagList = event.getData().func_150295_c("SemiBlocks", 10);
                    for(int i = 0; i < tagList.func_74745_c(); i++) {
                        NBTTagCompound t = tagList.func_150305_b(i);
                        ISemiBlock semiBlock = getSemiBlockForKey(t.func_74779_i("type"));
                        if(semiBlock != null) {
                            semiBlock.readFromNBT(t);
                            setSemiBlock(event.world, NBTUtil.getPos(t), semiBlock, event.getChunk());
                        }
                    }
                }
            }
        } catch(Throwable e) {
            e.printStackTrace();
        }
    }

    @SubscribeEvent
    public void onServerTick(TickEvent.ServerTickEvent event){
        for(ISemiBlock semiBlock : addingBlocks) {
            Chunk chunk = semiBlock.getWorld().func_175726_f(semiBlock.getPos());
            getOrCreateMap(chunk).put(semiBlock.getPos(), semiBlock);
            chunk.func_76630_e();

            for(EntityPlayer player : syncList.get(chunk)) {
                NetworkHandler.sendTo(new PacketSetSemiBlock(semiBlock), (EntityPlayerMP)player);
                PacketDescription descPacket = semiBlock.getDescriptionPacket();
                if(descPacket != null) NetworkHandler.sendTo(descPacket, (EntityPlayerMP)player);
            }
        }
        addingBlocks.clear();

        for(Chunk removingChunk : chunksMarkedForRemoval) {
            if(!removingChunk.func_177410_o()) {
                semiBlocks.remove(removingChunk);
                syncList.remove(removingChunk);
            }
        }
        chunksMarkedForRemoval.clear();

        for(Map<BlockPos, ISemiBlock> map : semiBlocks.values()) {
            for(ISemiBlock semiBlock : map.values()) {
                if(!semiBlock.isInvalid()) semiBlock.update();
            }
            Iterator<ISemiBlock> iterator = map.values().iterator();
            while(iterator.hasNext()) {
                if(iterator.next().isInvalid()) {
                    iterator.remove();
                }
            }
        }
    }

    @SubscribeEvent
    public void onClientTick(TickEvent.ClientTickEvent event){
        if(this == getServerInstance()) getClientOldInstance().onClientTick(event);
        else {
            EntityPlayer player = PneumaticCraft.proxy.getPlayer();
            if(player != null) {
                for(ISemiBlock semiBlock : addingBlocks) {
                    Chunk chunk = semiBlock.getWorld().func_175726_f(semiBlock.getPos());
                    getOrCreateMap(chunk).put(semiBlock.getPos(), semiBlock);
                }
                addingBlocks.clear();

                Iterator<Map.Entry<Chunk, Map<BlockPos, ISemiBlock>>> iterator = semiBlocks.entrySet().iterator();
                while(iterator.hasNext()) {
                    Map.Entry<Chunk, Map<BlockPos, ISemiBlock>> entry = iterator.next();
                    if(PneumaticCraftUtils.distBetween(player.field_70165_t, 0, player.field_70161_v, entry.getKey().field_76635_g * 16 - 8, 0, entry.getKey().field_76647_h * 16 - 8) > SYNC_DISTANCE + 10) {
                        iterator.remove();
                    } else {
                        for(ISemiBlock semiBlock : entry.getValue().values()) {
                            if(!semiBlock.isInvalid()) semiBlock.update();
                        }
                        Iterator<ISemiBlock> it = entry.getValue().values().iterator();
                        while(it.hasNext()) {
                            if(it.next().isInvalid()) {
                                it.remove();
                            }
                        }
                    }
                }
            } else {
                semiBlocks.clear();
            }
        }
    }

    @SubscribeEvent
    public void onWorldTick(TickEvent.WorldTickEvent event){
        if(!event.world.field_72995_K) {
            syncWithPlayers(event.world);
        }
    }

    private void syncWithPlayers(World world){
        List<EntityPlayer> players = world.field_73010_i;
        for(Map.Entry<Chunk, Set<EntityPlayer>> entry : syncList.entrySet()) {
            Chunk chunk = entry.getKey();
            Set<EntityPlayer> syncedPlayers = entry.getValue();
            int chunkX = chunk.field_76635_g * 16 - 8;
            int chunkZ = chunk.field_76647_h * 16 - 8;
            for(EntityPlayer player : players) {
                if(chunk.func_177412_p() == world) {
                    double dist = PneumaticCraftUtils.distBetween(player.field_70165_t, 0, player.field_70161_v, chunkX, 0, chunkZ);
                    if(dist < SYNC_DISTANCE) {
                        if(syncedPlayers.add(player)) {
                            for(ISemiBlock semiBlock : semiBlocks.get(chunk).values()) {
                                if(!semiBlock.isInvalid()) {
                                    NetworkHandler.sendTo(new PacketSetSemiBlock(semiBlock), (EntityPlayerMP)player);
                                    PacketDescription descPacket = semiBlock.getDescriptionPacket();
                                    if(descPacket != null) NetworkHandler.sendTo(descPacket, (EntityPlayerMP)player);
                                }
                            }
                        }
                    } else if(dist > SYNC_DISTANCE + 5) {
                        syncedPlayers.remove(player);
                    }
                } else {
                    syncedPlayers.remove(player);
                }
            }
        }
    }

    @SubscribeEvent
    public void onInteraction(PlayerInteractEvent event){
        if(!event.world.field_72995_K && event.action == PlayerInteractEvent.Action.RIGHT_CLICK_BLOCK) {
            ItemStack curItem = event.entityPlayer.func_71045_bC();
            if(curItem != null && curItem.func_77973_b() instanceof ISemiBlockItem) {
                if(getSemiBlock(event.world, event.pos) != null) {
                    if(event.entityPlayer.field_71075_bZ.field_75098_d) {
                        setSemiBlock(event.world, event.pos, null);
                    } else {
                        breakSemiBlock(event.world, event.pos, event.entityPlayer);
                    }
                    event.setCanceled(true);
                } else {
                    ISemiBlock newBlock = ((ISemiBlockItem)curItem.func_77973_b()).getSemiBlock(event.world, event.pos, curItem);
                    newBlock.initialize(event.world, event.pos);
                    if(newBlock.canPlace()) {
                        setSemiBlock(event.world, event.pos, newBlock);
                        newBlock.onPlaced(event.entityPlayer, curItem);
                        event.world.func_72908_a(event.pos.func_177958_n() + 0.5, event.pos.func_177956_o() + 0.5, event.pos.func_177952_p() + 0.5, Block.field_149778_k.func_150496_b(), (Block.field_149778_k.func_150497_c() + 1.0F) / 2.0F, Block.field_149778_k.func_150494_d() * 0.8F);
                        if(!event.entityPlayer.field_71075_bZ.field_75098_d) {
                            curItem.field_77994_a--;
                            if(curItem.field_77994_a <= 0) event.entityPlayer.func_70062_b(0, null);
                        }
                        event.setCanceled(true);
                    }
                }
            }
        }
    }

    private Map<BlockPos, ISemiBlock> getOrCreateMap(Chunk chunk){
        Map<BlockPos, ISemiBlock> map = semiBlocks.get(chunk);
        if(map == null) {
            map = new HashMap<BlockPos, ISemiBlock>();
            semiBlocks.put(chunk, map);
            syncList.put(chunk, new HashSet<EntityPlayer>());
        }
        return map;
    }

    public void breakSemiBlock(World world, BlockPos pos){
        breakSemiBlock(world, pos, null);
    }

    public void breakSemiBlock(World world, BlockPos pos, EntityPlayer player){
        ISemiBlock semiBlock = getSemiBlock(world, pos);
        if(semiBlock != null) {
            List<ItemStack> drops = new ArrayList<ItemStack>();
            semiBlock.addDrops(drops);
            for(ItemStack stack : drops) {
                EntityItem item = new EntityItem(world, pos.func_177958_n() + 0.5, pos.func_177956_o() + 0.5, pos.func_177952_p() + 0.5, stack);
                world.func_72838_d(item);
                if(player != null) item.func_70100_b_(player);
            }
            setSemiBlock(world, pos, null);
        }
    }

    public void setSemiBlock(World world, BlockPos pos, ISemiBlock semiBlock){
        setSemiBlock(world, pos, semiBlock, world.func_175726_f(pos));
    }

    private void setSemiBlock(World world, BlockPos pos, ISemiBlock semiBlock, Chunk chunk){
        if(semiBlock != null && !registeredTypes.containsValue(semiBlock.getClass())) throw new IllegalStateException("ISemiBlock \"" + semiBlock + "\" was not registered!");
        if(semiBlock != null) {
            semiBlock.initialize(world, pos);
            addingBlocks.add(semiBlock);
        } else {
            ISemiBlock removedBlock = getOrCreateMap(chunk).get(pos);
            if(removedBlock != null) {
                removedBlock.invalidate();
                for(EntityPlayer player : syncList.get(chunk)) {
                    NetworkHandler.sendTo(new PacketSetSemiBlock(pos, null), (EntityPlayerMP)player);
                }
            }
        }
        chunk.func_76630_e();
    }

    public ISemiBlock getSemiBlock(World world, BlockPos pos){
        for(ISemiBlock semiBlock : addingBlocks) {
            if(semiBlock.getWorld() == world && semiBlock.getPos().equals(pos)) return semiBlock;
        }

        Chunk chunk = world.func_175726_f(pos);
        Map<BlockPos, ISemiBlock> map = semiBlocks.get(chunk);
        if(map != null) {
            return map.get(pos);
        } else {
            return null;
        }
    }

    public Map<Chunk, Map<BlockPos, ISemiBlock>> getSemiBlocks(){
        return semiBlocks;
    }
}
