package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Random;
import java.util.Set;
import java.util.UUID;

import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityBoat;
import net.minecraft.entity.item.EntityExpBottle;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.item.EntityMinecartChest;
import net.minecraft.entity.item.EntityMinecartEmpty;
import net.minecraft.entity.item.EntityMinecartFurnace;
import net.minecraft.entity.item.EntityMinecartHopper;
import net.minecraft.entity.item.EntityMinecartTNT;
import net.minecraft.entity.item.EntityTNTPrimed;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.projectile.EntityArrow;
import net.minecraft.entity.projectile.EntityEgg;
import net.minecraft.entity.projectile.EntityFireball;
import net.minecraft.entity.projectile.EntityPotion;
import net.minecraft.entity.projectile.EntitySnowball;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemMonsterPlacer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraftforge.fml.common.network.NetworkRegistry.TargetPoint;

import org.apache.commons.lang3.tuple.Pair;

import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.tileentity.IAirHandler;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.item.Itemss;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.network.LazySynced;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketPlaySound;
import pneumaticCraft.common.network.PacketSetEntityMotion;
import pneumaticCraft.common.network.PacketSpawnParticle;
import pneumaticCraft.common.thirdparty.computercraft.LuaConstant;
import pneumaticCraft.common.thirdparty.computercraft.LuaMethod;
import pneumaticCraft.common.util.IOHelper;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.PneumaticValues;
import pneumaticCraft.lib.Sounds;
import pneumaticCraft.lib.TileEntityConstants;

public class TileEntityAirCannon extends TileEntityPneumaticBase implements ISidedInventory, IInventory,
        IMinWorkingPressure, IRedstoneControl{

    private ItemStack[] inventory;
    private final Random rand = new Random();
    @DescSynced
    @LazySynced
    public float rotationAngle;
    @DescSynced
    @LazySynced
    public float heightAngle;
    @DescSynced
    public float targetRotationAngle;
    @DescSynced
    public float targetHeightAngle;
    @GuiSynced
    public boolean doneTurning = false;
    @GuiSynced
    public int gpsX;
    @GuiSynced
    public int gpsY;
    @GuiSynced
    public int gpsZ;
    @GuiSynced
    public boolean coordWithinReach;
    @GuiSynced
    public int redstoneMode;
    private int oldRangeUpgrades;
    private boolean externalControl;//used in the CC API, to disallow the Cannon to update its angles when things like range upgrades / GPS Tool have changed.
    private boolean entityUpgradeInserted, dispenserUpgradeInserted;
    private final List<EntityItem> trackedItems = new ArrayList<EntityItem>();//Items that are being checked to be hoppering into inventories.
    private Set<UUID> trackedItemIds;
    private BlockPos lastInsertingInventory; //Last coordinate where the item went into the inventory (as a result of the Block Tracker upgrade).
    private EnumFacing lastInsertingInventorySide;
    @GuiSynced
    public boolean insertingInventoryHasSpace = true;

    private final int INVENTORY_SIZE = 6;
    public final int CANNON_SLOT = 0;
    public final int GPS_SLOT = 1;
    public static final int UPGRADE_SLOT_1 = 2;
    public static final int UPGRADE_SLOT_2 = 3;
    public static final int UPGRADE_SLOT_3 = 4;
    public static final int UPGRADE_SLOT_4 = 5;

    public TileEntityAirCannon(){
        super(PneumaticValues.DANGER_PRESSURE_AIR_CANNON, PneumaticValues.MAX_PRESSURE_AIR_CANNON, PneumaticValues.VOLUME_AIR_CANNON, UPGRADE_SLOT_1, UPGRADE_SLOT_2, UPGRADE_SLOT_3, UPGRADE_SLOT_4);
        inventory = new ItemStack[INVENTORY_SIZE];
        addApplicableUpgrade(EnumUpgrade.RANGE, EnumUpgrade.SPEED, EnumUpgrade.DISPENSER, EnumUpgrade.ENTITY_TRACKER, EnumUpgrade.BLOCK_TRACKER, EnumUpgrade.ITEM_LIFE);
    }

    @Override
    public void func_73660_a(){
        // GPS Tool read
        if(inventory[1] != null && inventory[1].func_77973_b() == Itemss.GPSTool && !externalControl) {
            if(inventory[1].func_77978_p() != null) {

                NBTTagCompound gpsTag = inventory[1].func_77978_p();
                int destinationX = gpsTag.func_74762_e("x");
                int destinationY = gpsTag.func_74762_e("y");
                int destinationZ = gpsTag.func_74762_e("z");

                if(destinationX != gpsX || destinationY != gpsY || destinationZ != gpsZ) {

                    gpsX = destinationX;
                    gpsY = destinationY;
                    gpsZ = destinationZ;
                    updateDestination();
                }
            }
        }

        int curRangeUpgrades = Math.min(8, getUpgrades(EnumUpgrade.RANGE));
        if(curRangeUpgrades != oldRangeUpgrades) {
            oldRangeUpgrades = curRangeUpgrades;
            if(!externalControl) updateDestination();
        }

        if(field_145850_b.func_82737_E() % 40 == 0) {
            boolean isDispenserUpgradeInserted = getUpgrades(EnumUpgrade.DISPENSER) > 0;
            boolean isEntityTrackerUpgradeInserted = getUpgrades(EnumUpgrade.ENTITY_TRACKER) > 0;
            if(dispenserUpgradeInserted != isDispenserUpgradeInserted || entityUpgradeInserted != isEntityTrackerUpgradeInserted) {
                dispenserUpgradeInserted = isDispenserUpgradeInserted;
                entityUpgradeInserted = isEntityTrackerUpgradeInserted;
                updateDestination();
            }
        }

        // update angles
        doneTurning = true;
        float speedMultiplier = getSpeedMultiplierFromUpgrades();
        if(rotationAngle < targetRotationAngle) {
            if(rotationAngle < targetRotationAngle - TileEntityConstants.CANNON_SLOW_ANGLE) {
                rotationAngle += TileEntityConstants.CANNON_TURN_HIGH_SPEED * speedMultiplier;
            } else {
                rotationAngle += TileEntityConstants.CANNON_TURN_LOW_SPEED * speedMultiplier;
            }
            if(rotationAngle > targetRotationAngle) rotationAngle = targetRotationAngle;
            doneTurning = false;
        }
        if(rotationAngle > targetRotationAngle) {
            if(rotationAngle > targetRotationAngle + TileEntityConstants.CANNON_SLOW_ANGLE) {
                rotationAngle -= TileEntityConstants.CANNON_TURN_HIGH_SPEED * speedMultiplier;
            } else {
                rotationAngle -= TileEntityConstants.CANNON_TURN_LOW_SPEED * speedMultiplier;
            }
            if(rotationAngle < targetRotationAngle) rotationAngle = targetRotationAngle;
            doneTurning = false;
        }
        if(heightAngle < targetHeightAngle) {
            if(heightAngle < targetHeightAngle - TileEntityConstants.CANNON_SLOW_ANGLE) {
                heightAngle += TileEntityConstants.CANNON_TURN_HIGH_SPEED * speedMultiplier;
            } else {
                heightAngle += TileEntityConstants.CANNON_TURN_LOW_SPEED * speedMultiplier;
            }
            if(heightAngle > targetHeightAngle) heightAngle = targetHeightAngle;
            doneTurning = false;
        }
        if(heightAngle > targetHeightAngle) {
            if(heightAngle > targetHeightAngle + TileEntityConstants.CANNON_SLOW_ANGLE) {
                heightAngle -= TileEntityConstants.CANNON_TURN_HIGH_SPEED * speedMultiplier;
            } else {
                heightAngle -= TileEntityConstants.CANNON_TURN_LOW_SPEED * speedMultiplier;
            }
            if(heightAngle < targetHeightAngle) heightAngle = targetHeightAngle;
            doneTurning = false;
        }

        updateTrackedItems();

        super.func_73660_a();

        if(!field_145850_b.field_72995_K) {
            List<Pair<EnumFacing, IAirHandler>> teList = getAirHandler(null).getConnectedPneumatics();
            if(teList.size() == 0) getAirHandler(null).airLeak(getRotation());
        }
    }

    private void updateTrackedItems(){
        if(trackedItemIds != null) {
            trackedItems.clear();
            for(Entity entity : field_145850_b.field_72996_f) {
                if(trackedItemIds.contains(entity.func_110124_au()) && entity instanceof EntityItem) {
                    trackedItems.add((EntityItem)entity);
                }
            }
            trackedItemIds = null;
        }
        Iterator<EntityItem> iterator = trackedItems.iterator();
        while(iterator.hasNext()) {
            EntityItem item = iterator.next();
            if(item.field_70170_p != field_145850_b || item.field_70128_L) {
                iterator.remove();
            } else {
                Map<BlockPos, EnumFacing> positions = new HashMap<BlockPos, EnumFacing>();
                double range = 0.2;
                for(EnumFacing d : EnumFacing.field_82609_l) {
                    double posX = item.field_70165_t + d.func_82601_c() * range;
                    double posY = item.field_70163_u + d.func_96559_d() * range;
                    double posZ = item.field_70161_v + d.func_82599_e() * range;
                    positions.put(new BlockPos((int)Math.floor(posX), (int)Math.floor(posY), (int)Math.floor(posZ)), d.func_176734_d());
                }
                for(Entry<BlockPos, EnumFacing> entry : positions.entrySet()) {
                    BlockPos pos = entry.getKey();
                    TileEntity te = field_145850_b.func_175625_s(pos);
                    IInventory inv = IOHelper.getInventoryForTE(te);
                    ItemStack remainder = IOHelper.insert(inv, item.func_92059_d(), entry.getValue(), false);
                    if(remainder != null) {
                        item.func_92058_a(remainder);
                    } else {
                        item.func_70106_y();
                        iterator.remove();
                        lastInsertingInventory = te.func_174877_v();
                        lastInsertingInventorySide = entry.getValue();
                        break;
                    }
                }
            }
        }
    }

    // ANGLE METHODS -------------------------------------------------

    private void updateDestination(){
        doneTurning = false;
        // take dispenser upgrade in account
        double payloadFrictionY = 0.98D;// this value will differ when a
                                        // dispenser upgrade is inserted.
        double payloadFrictionX = 0.98D;
        double payloadGravity = 0.04D;
        if(getUpgrades(EnumUpgrade.ENTITY_TRACKER) > 0) {
            payloadFrictionY = 0.98D;
            payloadFrictionX = 0.91D;
            payloadGravity = 0.08D;
        } else if(getUpgrades(EnumUpgrade.DISPENSER) > 0 && inventory[0] != null) {// if
            // there
            // is
            // a
            // dispenser
            // upgrade
            // inserted.
            Item item = inventory[0].func_77973_b();
            if(item == Items.field_151068_bn || item == Items.field_151062_by || item == Items.field_151110_aK || item == Items.field_151126_ay) {// EntityThrowable
                payloadFrictionY = 0.99D;
                payloadGravity = 0.03D;
            } else if(item == Items.field_151032_g) {
                payloadFrictionY = 0.99D;
                payloadGravity = 0.05D;
            } else if(item == Items.field_151143_au || item == Items.field_151108_aI || item == Items.field_151140_bW || item == Items.field_151142_bV || item == Items.field_151109_aJ) {
                payloadFrictionY = 0.95D;
            }
            // else if(itemID == Item.fireballCharge.itemID){
            // payloadGravity = 0.0D;
            // }

            // family items (throwable) which only differ in gravity.
            if(item == Items.field_151068_bn) payloadGravity = 0.05D;
            else if(item == Items.field_151062_by) payloadGravity = 0.07D;

            payloadFrictionX = payloadFrictionY;

            // items which have different frictions for each axis.
            if(item == Items.field_151124_az) {
                payloadFrictionX = 0.99D;
                payloadFrictionY = 0.95D;
            }
            if(item == Items.field_151063_bx) {
                payloadFrictionY = 0.98D;
                payloadFrictionX = 0.91D;
                payloadGravity = 0.08D;
            }
        }

        // calculate the heading.
        double deltaX = gpsX - func_174877_v().func_177958_n();
        double deltaZ = gpsZ - func_174877_v().func_177952_p();
        float calculatedRotationAngle;
        if(deltaX >= 0 && deltaZ < 0) {
            calculatedRotationAngle = (float)(Math.atan(Math.abs(deltaX / deltaZ)) / Math.PI * 180D);
        } else if(deltaX >= 0 && deltaZ >= 0) {
            calculatedRotationAngle = (float)(Math.atan(Math.abs(deltaZ / deltaX)) / Math.PI * 180D) + 90;
        } else if(deltaX < 0 && deltaZ >= 0) {
            calculatedRotationAngle = (float)(Math.atan(Math.abs(deltaX / deltaZ)) / Math.PI * 180D) + 180;
        } else {
            calculatedRotationAngle = (float)(Math.atan(Math.abs(deltaZ / deltaX)) / Math.PI * 180D) + 270;
        }

        // calculate the height angle.
        double distance = Math.sqrt(deltaX * deltaX + deltaZ * deltaZ);
        double deltaY = gpsY - func_174877_v().func_177956_o();
        float calculatedHeightAngle = calculateBestHeightAngle(distance, deltaY, getForce(), payloadGravity, payloadFrictionX, payloadFrictionY);

        setTargetAngles(calculatedRotationAngle, calculatedHeightAngle);
    }

    private float calculateBestHeightAngle(double distance, double deltaY, float force, double payloadGravity, double payloadFrictionX, double payloadFrictionY){
        double bestAngle = 0;
        double bestDistance = Float.MAX_VALUE;
        if(payloadGravity == 0D) {
            return 90F - (float)(Math.atan(deltaY / distance) * 180F / Math.PI);
        }
        for(double i = Math.PI * 0.25D; i < Math.PI * 0.50D; i += 0.001D) {
            double motionX = Math.cos(i) * force;// calculate the x component of
                                                 // the vector
            double motionY = Math.sin(i) * force;// calculate the y component of
                                                 // the vector
            double posX = 0;
            double posY = 0;
            while(posY > deltaY || motionY > 0) { // simulate movement, until we
                                                  // reach the y-level required
                posX += motionX;
                posY += motionY;
                motionY -= payloadGravity;// gravity
                motionX *= payloadFrictionX;// friction
                motionY *= payloadFrictionY;// friction
            }
            double distanceToTarget = Math.abs(distance - posX);// take the
                                                                // distance
            if(distanceToTarget < bestDistance) {// and return the best angle.
                bestDistance = distanceToTarget;
                bestAngle = i;
            }
        }
        coordWithinReach = bestDistance < 1.5D;
        return 90F - (float)(bestAngle * 180D / Math.PI);
    }

    public synchronized void setTargetAngles(float rotationAngle, float heightAngle){
        targetRotationAngle = rotationAngle;
        targetHeightAngle = heightAngle;
        if(!field_145850_b.field_72995_K) scheduleDescriptionPacket();
    }

    // this function calculates with the parsed in X and Z angles and the force
    // the needed, and outputs the X, Y and Z velocities.
    public double[] getVelocityVector(float angleX, float angleZ, float force){
        double[] velocities = new double[3];
        velocities[0] = Math.sin((double)angleZ / 180 * Math.PI);
        velocities[1] = Math.cos((double)angleX / 180 * Math.PI);
        velocities[2] = Math.cos((double)angleZ / 180 * Math.PI) * -1;

        velocities[0] *= Math.sin((double)angleX / 180 * Math.PI);
        velocities[2] *= Math.sin((double)angleX / 180 * Math.PI);
        // calculate the total velocity vector, in relation.
        double vectorTotal = velocities[0] * velocities[0] + velocities[1] * velocities[1] + velocities[2] * velocities[2];
        vectorTotal = force / vectorTotal; // calculate the relation between the
                                           // forces to be shot, and the
                                           // calculated vector (the scale).
        for(int i = 0; i < 3; i++) {
            velocities[i] *= vectorTotal; // scale up the velocities
            // System.out.println("velocities " + i + " = " + velocities[i]);
        }
        return velocities;
    }

    public boolean hasCoordinate(){
        return gpsX != 0 || gpsY != 0 || gpsZ != 0;
    }

    // PNEUMATIC METHODS -----------------------------------------

    @Override
    public boolean isConnectedTo(EnumFacing side){
        return getRotation() == side;
    }

    public float getForce(){
        return 2F + oldRangeUpgrades;
    }

    // INVENTORY METHODS- && NBT
    // ------------------------------------------------------------

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){

        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){

        return inventory[slot];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
    }

    @Override
    public String getName(){
        return Blockss.airCannon.func_149739_a();
    }

    @Override
    public int func_70297_j_(){
        return 64;
    }

    @Override
    public void func_174888_l(){
        inventory = new ItemStack[inventory.length];
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        targetRotationAngle = tag.func_74760_g("targetRotationAngle");
        targetHeightAngle = tag.func_74760_g("targetHeightAngle");
        rotationAngle = tag.func_74760_g("rotationAngle");
        heightAngle = tag.func_74760_g("heightAngle");
        gpsX = tag.func_74762_e("gpsX");
        gpsY = tag.func_74762_e("gpsY");
        gpsZ = tag.func_74762_e("gpsZ");
        redstoneMode = tag.func_74771_c("redstoneMode");

        coordWithinReach = tag.func_74767_n("targetWithinReach");
        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[func_70302_i_()];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }

        trackedItemIds = new HashSet<UUID>();
        tagList = tag.func_150295_c("trackedItems", 10);
        for(int i = 0; i < tagList.func_74745_c(); i++) {
            NBTTagCompound t = tagList.func_150305_b(i);
            trackedItemIds.add(new UUID(t.func_74763_f("UUIDMost"), t.func_74763_f("UUIDLeast")));
        }

        if(tag.func_74764_b("inventoryX")) {
            lastInsertingInventory = new BlockPos(tag.func_74762_e("inventoryX"), tag.func_74762_e("inventoryY"), tag.func_74762_e("inventoryZ"));
            lastInsertingInventorySide = EnumFacing.func_82600_a(tag.func_74771_c("inventorySide"));
        } else {
            lastInsertingInventory = null;
            lastInsertingInventorySide = null;
        }
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        tag.func_74776_a("targetRotationAngle", targetRotationAngle);
        tag.func_74776_a("targetHeightAngle", targetHeightAngle);
        tag.func_74776_a("rotationAngle", rotationAngle);
        tag.func_74776_a("heightAngle", heightAngle);
        tag.func_74768_a("gpsX", gpsX);
        tag.func_74768_a("gpsY", gpsY);
        tag.func_74768_a("gpsZ", gpsZ);
        tag.func_74774_a("redstoneMode", (byte)redstoneMode);
        tag.func_74757_a("targetWithinReach", coordWithinReach);
        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Items", tagList);

        tagList = new NBTTagList();
        for(EntityItem entity : trackedItems) {
            UUID uuid = entity.func_110124_au();
            NBTTagCompound t = new NBTTagCompound();
            t.func_74772_a("UUIDMost", uuid.getMostSignificantBits());
            t.func_74772_a("UUIDLeast", uuid.getLeastSignificantBits());
            tagList.func_74742_a(t);
        }
        tag.func_74782_a("trackedItems", tagList);

        if(lastInsertingInventory != null) {
            tag.func_74768_a("inventoryX", lastInsertingInventory.func_177958_n());
            tag.func_74768_a("inventoryY", lastInsertingInventory.func_177956_o());
            tag.func_74768_a("inventoryZ", lastInsertingInventory.func_177952_p());
            tag.func_74774_a("inventorySide", (byte)lastInsertingInventorySide.ordinal());
        }
    }

    @Override
    public boolean func_94041_b(int i, ItemStack itemstack){
        if(canInsertUpgrade(i, itemstack)) return true;
        if(i == GPS_SLOT && itemstack != null && itemstack.func_77973_b() != Itemss.GPSTool) return false;
        return i == CANNON_SLOT;
    }

    @Override
    public int[] func_180463_a(EnumFacing var1){
        return new int[]{1, 2, 3, 4, 5, 0};
    }

    @Override
    public boolean func_180462_a(int slot, ItemStack itemstack, EnumFacing side){
        return true;
    }

    @Override
    public boolean func_180461_b(int slot, ItemStack itemstack, EnumFacing side){
        return true;
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    // REDSTONE BEHAVIOUR
    // ------------------------------------------------------------

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID == 0) {
            if(++redstoneMode > 2) redstoneMode = 0;
            if(redstoneMode == 2 && getUpgrades(EnumUpgrade.BLOCK_TRACKER) == 0) redstoneMode = 0;
        }
    }

    @Override
    public void onNeighborBlockUpdate(){
        boolean wasPowered = poweredRedstone > 0;
        super.onNeighborBlockUpdate();
        boolean isPowered = poweredRedstone > 0;
        if(isPowered && !wasPowered && (redstoneMode != 0 || doneTurning) && (redstoneMode != 2 || inventoryCanCarry())) {
            fire();
        }
    }

    private boolean inventoryCanCarry(){
        insertingInventoryHasSpace = true;
        if(lastInsertingInventory == null) return true;
        if(inventory[0] == null) return true;
        TileEntity te = field_145850_b.func_175625_s(lastInsertingInventory);
        IInventory inv = IOHelper.getInventoryForTE(te);
        if(inv != null) {
            ItemStack remainder = IOHelper.insert(inv, inventory[0].func_77946_l(), lastInsertingInventorySide, true);
            insertingInventoryHasSpace = remainder == null;
            return insertingInventoryHasSpace;
        } else {
            lastInsertingInventory = null;
            lastInsertingInventorySide = null;
            return true;
        }
    }

    private synchronized boolean fire(){
        Entity itemShot = getCloseEntityIfUpgraded();
        if(getPressure() >= PneumaticValues.MIN_PRESSURE_AIR_CANNON && (itemShot != null || inventory[0] != null)) {
            double[] velocity = getVelocityVector(heightAngle, rotationAngle, getForce());
            addAir((int)(-500 * getForce()));
            boolean shootingInventory = false;
            if(itemShot == null) {
                shootingInventory = true;
                itemShot = getPayloadEntity();

                if(itemShot instanceof EntityItem) {
                    inventory[0] = null;
                    if(getUpgrades(EnumUpgrade.BLOCK_TRACKER) > 0) {
                        trackedItems.add((EntityItem)itemShot);
                    }
                } else {
                    inventory[0].field_77994_a--;
                    if(inventory[0].field_77994_a <= 0) inventory[0] = null;
                }
            } else if(itemShot instanceof EntityPlayer) {
                EntityPlayerMP entityplayermp = (EntityPlayerMP)itemShot;
                if(entityplayermp.field_71135_a.func_147362_b().func_150724_d()) {
                    entityplayermp.func_70634_a(func_174877_v().func_177958_n() + 0.5D, func_174877_v().func_177956_o() + 1.8D, func_174877_v().func_177952_p() + 0.5D);
                }
            }

            if(itemShot.func_70115_ae()) {
                itemShot.func_70078_a((Entity)null);
            }

            itemShot.func_70107_b(func_174877_v().func_177958_n() + 0.5D, func_174877_v().func_177956_o() + 1.8D, func_174877_v().func_177952_p() + 0.5D);
            NetworkHandler.sendToAllAround(new PacketSetEntityMotion(itemShot, velocity[0], velocity[1], velocity[2]), new TargetPoint(field_145850_b.field_73011_w.func_177502_q(), func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), 64));

            if(itemShot instanceof EntityFireball) {
                velocity[0] *= 0.05D;
                velocity[1] *= 0.05D;
                velocity[2] *= 0.05D;
            }

            itemShot.field_70159_w = velocity[0];
            itemShot.field_70181_x = velocity[1];
            itemShot.field_70179_y = velocity[2];

            itemShot.field_70122_E = false;
            itemShot.field_70132_H = false;
            itemShot.field_70123_F = false;
            itemShot.field_70124_G = false;
            if(itemShot instanceof EntityLivingBase) ((EntityLivingBase)itemShot).func_70637_d(true);

            if(shootingInventory && !field_145850_b.field_72995_K) field_145850_b.func_72838_d(itemShot);

            for(int i = 0; i < 10; i++) {
                double velX = velocity[0] * 0.4D + (rand.nextGaussian() - 0.5D) * 0.05D;
                double velY = velocity[1] * 0.4D + (rand.nextGaussian() - 0.5D) * 0.05D;
                double velZ = velocity[2] * 0.4D + (rand.nextGaussian() - 0.5D) * 0.05D;
                NetworkHandler.sendToAllAround(new PacketSpawnParticle(EnumParticleTypes.SMOKE_LARGE, func_174877_v().func_177958_n() + 0.5D, func_174877_v().func_177956_o() + 0.7D, func_174877_v().func_177952_p() + 0.5D, velX, velY, velZ), field_145850_b);
            }
            NetworkHandler.sendToAllAround(new PacketPlaySound(Sounds.CANNON_SOUND, func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), 1.0F, rand.nextFloat() / 4F + 0.75F, true), field_145850_b);
            return true;
        } else {
            return false;
        }
    }

    // warning: no null-check for inventory slot 0
    private Entity getPayloadEntity(){

        if(getUpgrades(EnumUpgrade.DISPENSER) > 0) {
            Item item = inventory[0].func_77973_b();
            if(item == Item.func_150898_a(Blocks.field_150335_W)) {
                EntityTNTPrimed tnt = new EntityTNTPrimed(field_145850_b);
                tnt.field_70516_a = 80;
                return tnt;
            } else if(item == Items.field_151062_by) return new EntityExpBottle(field_145850_b);
            else if(item == Items.field_151068_bn) {
                EntityPotion potion = new EntityPotion(field_145850_b);
                potion.func_82340_a(inventory[0].func_77952_i());
                return potion;
            } else if(item == Items.field_151032_g) return new EntityArrow(field_145850_b);
            else if(item == Items.field_151110_aK) return new EntityEgg(field_145850_b);
            // else if(itemID == Item.fireballCharge) return new
            // EntitySmallFireball(worldObj);
            else if(item == Items.field_151126_ay) return new EntitySnowball(field_145850_b);
            else if(item == Items.field_151063_bx) return ItemMonsterPlacer.func_77840_a(field_145850_b, inventory[0].func_77952_i(), 0, 0, 0);
            else if(item == Items.field_151143_au) return new EntityMinecartEmpty(field_145850_b);
            else if(item == Items.field_151108_aI) return new EntityMinecartChest(field_145850_b);
            else if(item == Items.field_151109_aJ) return new EntityMinecartFurnace(field_145850_b);
            else if(item == Items.field_151140_bW) return new EntityMinecartHopper(field_145850_b);
            else if(item == Items.field_151142_bV) return new EntityMinecartTNT(field_145850_b);
            else if(item == Items.field_151124_az) return new EntityBoat(field_145850_b);

        }
        EntityItem item = new EntityItem(field_145850_b);
        item.func_92058_a(inventory[0].func_77946_l());
        item.func_70288_d(); // 1200 ticks left to live, = 60s.
        item.lifespan += Math.min(getUpgrades(EnumUpgrade.ITEM_LIFE) * 600, 4800); // add
        // 30s
        // for
        // each
        // life
        // upgrade,
        // to
        // the
        // max
        // of
        // 5
        // min.
        return item;

    }

    private Entity getCloseEntityIfUpgraded(){
        int entityUpgrades = getUpgrades(EnumUpgrade.ENTITY_TRACKER);
        if(entityUpgrades > 0) {
            entityUpgrades = Math.min(entityUpgrades, 5);
            List<EntityLivingBase> entities = field_145850_b.func_72872_a(EntityLivingBase.class, new AxisAlignedBB(func_174877_v().func_177982_a(-entityUpgrades, -entityUpgrades, -entityUpgrades), func_174877_v().func_177982_a(1 + entityUpgrades, 1 + entityUpgrades, 1 + entityUpgrades)));
            Entity closestEntity = null;
            for(Entity entity : entities) {
                if(closestEntity == null || PneumaticCraftUtils.distBetween(closestEntity.field_70165_t, closestEntity.field_70163_u, closestEntity.field_70161_v, func_174877_v().func_177958_n() + 0.5, func_174877_v().func_177956_o() + 0.5, func_174877_v().func_177952_p() + 0.5) > PneumaticCraftUtils.distBetween(entity.field_70165_t, entity.field_70163_u, entity.field_70161_v, func_174877_v().func_177958_n() + 0.5, func_174877_v().func_177956_o() + 0.5, func_174877_v().func_177952_p() + 0.5)) {
                    closestEntity = entity;
                }
            }
            return closestEntity;
        }
        return null;
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    /*
     *  COMPUTERCRAFT API
     */

    /*@Override TODO CC dep
    public String getType(){
        return "airCannon";
    }*/

    @Override
    protected void addLuaMethods(){
        super.addLuaMethods();
        luaMethods.add(new LuaConstant("getMinWorkingPressure", PneumaticValues.MIN_PRESSURE_AIR_CANNON));

        luaMethods.add(new LuaMethod("setTargetLocation"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 3) {
                    gpsX = ((Double)args[0]).intValue();
                    gpsY = ((Double)args[1]).intValue();
                    gpsZ = ((Double)args[2]).intValue();
                    updateDestination();
                    return new Object[]{coordWithinReach};
                } else {
                    throw new IllegalArgumentException("setTargetLocation requires 3 parameters (x,y,z)");
                }
            }
        });

        luaMethods.add(new LuaMethod("fire"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 0) {
                    return new Object[]{fire()};//returns true if the fire succeeded.
                } else {
                    throw new IllegalArgumentException("fire doesn't take any arguments!");
                }
            }
        });
        luaMethods.add(new LuaMethod("isDoneTurning"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 0) {
                    return new Object[]{doneTurning};
                } else {
                    throw new IllegalArgumentException("isDoneTurning doesn't take any arguments!");
                }
            }
        });

        luaMethods.add(new LuaMethod("setRotationAngle"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 1) {
                    setTargetAngles(((Double)args[0]).floatValue(), targetHeightAngle);
                    return null;
                } else {
                    throw new IllegalArgumentException("setRotationAngle does take one argument!");
                }
            }
        });

        luaMethods.add(new LuaMethod("setHeightAngle"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 1) {
                    setTargetAngles(targetRotationAngle, 90 - ((Double)args[0]).floatValue());
                    return null;
                } else {
                    throw new IllegalArgumentException("setHeightAngle does take one argument!");
                }
            }
        });

        luaMethods.add(new LuaMethod("setExternalControl"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 1) {
                    externalControl = (Boolean)args[0];
                    return null;
                } else {
                    throw new IllegalArgumentException("setExternalControl does take one argument!");
                }
            }
        });
    }

    @Override
    public float getMinWorkingPressure(){
        return PneumaticValues.MIN_PRESSURE_AIR_CANNON;
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }
}
