package pneumaticCraft.common.tileentity;

import java.util.Arrays;
import java.util.List;
import java.util.Random;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntityFurnace;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.apache.commons.lang3.tuple.Pair;

import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.tileentity.IAirHandler;
import pneumaticCraft.common.block.BlockAirCompressor;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.lib.PneumaticValues;

public class TileEntityAirCompressor extends TileEntityPneumaticBase implements ISidedInventory, IRedstoneControlled{

    private ItemStack[] inventory;

    private final int INVENTORY_SIZE = 5;

    public static final int FUEL_INVENTORY_INDEX = 0;
    public static final int UPGRADE_SLOT_START = 1;
    public static final int UPGRADE_SLOT_END = 4;

    @GuiSynced
    public int burnTime;
    @GuiSynced
    public int maxBurnTime; // in here the total burn time of the current
                            // burning item is stored.
    @GuiSynced
    public int redstoneMode = 0; // determines how the compressor responds to
                                 // redstone.
    @DescSynced
    private boolean isActive;
    @GuiSynced
    public int curFuelUsage;

    public TileEntityAirCompressor(){
        this(PneumaticValues.DANGER_PRESSURE_AIR_COMPRESSOR, PneumaticValues.MAX_PRESSURE_AIR_COMPRESSOR, PneumaticValues.VOLUME_AIR_COMPRESSOR);
    }

    public TileEntityAirCompressor(float dangerPressure, float criticalPressure, int volume){
        super(dangerPressure, criticalPressure, volume, UPGRADE_SLOT_START, 2, 3, UPGRADE_SLOT_END);
        inventory = new ItemStack[INVENTORY_SIZE];
        addApplicableUpgrade(EnumUpgrade.SPEED);
    }

    @Override
    public void func_73660_a(){
        if(!field_145850_b.field_72995_K) {
            if(burnTime < curFuelUsage && inventory[0] != null && TileEntityFurnace.func_145954_b(inventory[0]) && redstoneAllows()) {
                burnTime += TileEntityFurnace.func_145952_a(inventory[0]);
                maxBurnTime = burnTime;

                inventory[0].field_77994_a--;
                if(inventory[0].field_77994_a == 0) {
                    inventory[0] = inventory[0].func_77973_b().getContainerItem(inventory[0]);
                }

            }

            curFuelUsage = (int)(getBaseProduction() * getSpeedUsageMultiplierFromUpgrades() / 10);
            if(burnTime >= curFuelUsage) {
                burnTime -= curFuelUsage;
                if(!field_145850_b.field_72995_K) {
                    addAir((int)(getBaseProduction() * getSpeedMultiplierFromUpgrades() * getEfficiency() / 100D));
                    onFuelBurn(curFuelUsage);
                }
            }
            boolean wasActive = isActive;
            isActive = burnTime > curFuelUsage;
            if(wasActive != isActive) {
                field_145850_b.func_175656_a(func_174877_v(), field_145850_b.func_180495_p(func_174877_v()).func_177226_a(BlockAirCompressor.ON, isActive));
            }
        } else if(isActive) spawnBurningParticle();

        super.func_73660_a();

        if(!field_145850_b.field_72995_K) {
            List<Pair<EnumFacing, IAirHandler>> teList = getAirHandler(null).getConnectedPneumatics();
            if(teList.size() == 0) getAirHandler(null).airLeak(getRotation());
        }
    }

    protected void onFuelBurn(int burnedFuel){}

    public int getEfficiency(){
        return 100;
    }

    public int getBaseProduction(){
        return PneumaticValues.PRODUCTION_COMPRESSOR;
    }

    private void spawnBurningParticle(){
        Random rand = new Random();
        if(rand.nextInt(3) != 0) return;
        float f = func_174877_v().func_177958_n() + 0.5F;
        float f1 = func_174877_v().func_177956_o() + 0.0F + rand.nextFloat() * 6.0F / 16.0F;
        float f2 = func_174877_v().func_177952_p() + 0.5F;
        float f3 = 0.5F;
        float f4 = rand.nextFloat() * 0.4F - 0.2F;
        switch(getRotation()){
            case EAST:
                field_145850_b.func_175688_a(EnumParticleTypes.SMOKE_NORMAL, f - f3, f1, f2 + f4, 0.0D, 0.0D, 0.0D);
                field_145850_b.func_175688_a(EnumParticleTypes.FLAME, f - f3, f1, f2 + f4, 0.0D, 0.0D, 0.0D);
                break;
            case WEST:
                field_145850_b.func_175688_a(EnumParticleTypes.SMOKE_NORMAL, f + f3, f1, f2 + f4, 0.0D, 0.0D, 0.0D);
                field_145850_b.func_175688_a(EnumParticleTypes.FLAME, f + f3, f1, f2 + f4, 0.0D, 0.0D, 0.0D);
                break;
            case SOUTH:
                field_145850_b.func_175688_a(EnumParticleTypes.SMOKE_NORMAL, f + f4, f1, f2 - f3, 0.0D, 0.0D, 0.0D);
                field_145850_b.func_175688_a(EnumParticleTypes.FLAME, f + f4, f1, f2 - f3, 0.0D, 0.0D, 0.0D);
                break;
            case NORTH:
                field_145850_b.func_175688_a(EnumParticleTypes.SMOKE_NORMAL, f + f4, f1, f2 + f3, 0.0D, 0.0D, 0.0D);
                field_145850_b.func_175688_a(EnumParticleTypes.FLAME, f + f4, f1, f2 + f3, 0.0D, 0.0D, 0.0D);
                break;
        }
    }

    @Override
    public boolean isConnectedTo(EnumFacing side){
        return getRotation() == side;
    }

    public int getBurnTimeRemainingScaled(int parts){
        if(maxBurnTime == 0 || burnTime < curFuelUsage) return 0;
        return parts * burnTime / maxBurnTime;
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID == 0) {
            redstoneMode++;
            if(redstoneMode > 2) redstoneMode = 0;
        }
    }

    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox(){
        return new AxisAlignedBB(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), func_174877_v().func_177958_n() + 1, func_174877_v().func_177956_o() + 1, func_174877_v().func_177952_p() + 1);
    }

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){

        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){

        return inventory[slot];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){

        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
    }

    @Override
    public String getName(){

        return Blockss.airCompressor.func_149739_a();
    }

    @Override
    public int func_70297_j_(){

        return 64;
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public void func_145839_a(NBTTagCompound nbtTagCompound){

        super.func_145839_a(nbtTagCompound);
        burnTime = nbtTagCompound.func_74762_e("burnTime");
        maxBurnTime = nbtTagCompound.func_74762_e("maxBurn");
        redstoneMode = nbtTagCompound.func_74762_e("redstoneMode");
        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = nbtTagCompound.func_150295_c("Items", 10);
        inventory = new ItemStack[func_70302_i_()];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }
    }

    @Override
    public void func_145841_b(NBTTagCompound nbtTagCompound){

        super.func_145841_b(nbtTagCompound);

        nbtTagCompound.func_74768_a("burnTime", burnTime);
        nbtTagCompound.func_74768_a("maxBurn", maxBurnTime);
        nbtTagCompound.func_74768_a("redstoneMode", redstoneMode);
        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        nbtTagCompound.func_74782_a("Items", tagList);
    }

    @Override
    public boolean func_94041_b(int i, ItemStack itemstack){
        return i == 0 && TileEntityFurnace.func_145954_b(itemstack) || canInsertUpgrade(i, itemstack);
    }

    @Override
    // upgrades in bottom, fuel in the rest.
    public int[] func_180463_a(EnumFacing var1){
        return new int[]{0};
    }

    @Override
    public boolean func_180462_a(int i, ItemStack itemstack, EnumFacing j){
        return true;
    }

    @Override
    public boolean func_180461_b(int i, ItemStack itemstack, EnumFacing j){
        return true;
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }
}
