package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.apache.commons.lang3.tuple.Pair;

import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.tileentity.IAirHandler;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.item.ItemAssemblyProgram;
import pneumaticCraft.common.item.Itemss;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.recipes.programs.AssemblyProgram;
import pneumaticCraft.common.recipes.programs.AssemblyProgram.EnumMachine;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.GuiConstants;
import pneumaticCraft.lib.PneumaticValues;

public class TileEntityAssemblyController extends TileEntityPneumaticBase implements ISidedInventory, IAssemblyMachine,
        IMinWorkingPressure{
    private ItemStack[] inventory;
    @DescSynced
    public boolean[] sidesConnected = new boolean[6];
    public AssemblyProgram curProgram;
    private final int INVENTORY_SIZE = 5;
    @GuiSynced
    public boolean foundAllMachines;
    @GuiSynced
    private boolean foundDuplicateMachine;
    private boolean goingToHomePosition;
    @DescSynced
    public String displayedText = "";
    public static final int PROGRAM_INVENTORY_INDEX = 0;
    public static final int UPGRADE_SLOT_START = 1;
    public static final int UPGRADE_SLOT_END = 4;
    @DescSynced
    public boolean hasProblem;

    public TileEntityAssemblyController(){
        super(PneumaticValues.DANGER_PRESSURE_ASSEMBLY_CONTROLLER, PneumaticValues.MAX_PRESSURE_ASSEMBLY_CONTROLLER, PneumaticValues.VOLUME_ASSEMBLY_CONTROLLER, UPGRADE_SLOT_START, 2, 3, UPGRADE_SLOT_END);
        inventory = new ItemStack[INVENTORY_SIZE];
        addApplicableUpgrade(EnumUpgrade.SPEED);
    }

    @Override
    public void func_73660_a(){

        if(!field_145850_b.field_72995_K && firstRun) updateConnections();

        // curProgram must be available on the client, or we can't show program-problems in the GUI
        if(curProgram == null && !goingToHomePosition && inventory[PROGRAM_INVENTORY_INDEX] != null && inventory[PROGRAM_INVENTORY_INDEX].func_77973_b() == Itemss.assemblyProgram) {
            AssemblyProgram program = ItemAssemblyProgram.getProgramFromItem(inventory[PROGRAM_INVENTORY_INDEX].func_77952_i());
            curProgram = program;
        } else if(curProgram != null && (inventory[PROGRAM_INVENTORY_INDEX] == null || curProgram.getClass() != ItemAssemblyProgram.getProgramFromItem(inventory[PROGRAM_INVENTORY_INDEX].func_77952_i()).getClass())) {
            curProgram = null;
            if(!field_145850_b.field_72995_K) goingToHomePosition = true;
        }

        if(!field_145850_b.field_72995_K) {
            displayedText = "Standby";
            if(getPressure() >= PneumaticValues.MIN_PRESSURE_ASSEMBLY_CONTROLLER) {
                if(curProgram != null || goingToHomePosition) {
                    List<IAssemblyMachine> machineList = getMachines();
                    EnumMachine[] requiredMachines = curProgram != null ? curProgram.getRequiredMachines() : EnumMachine.values();
                    TileEntityAssemblyDrill drill = null;
                    TileEntityAssemblyLaser laser = null;
                    TileEntityAssemblyIOUnit ioUnitImport = null;
                    TileEntityAssemblyIOUnit ioUnitExport = null;
                    TileEntityAssemblyPlatform platform = null;
                    foundDuplicateMachine = false;
                    boolean foundMachines[] = new boolean[requiredMachines.length];
                    for(IAssemblyMachine machine : machineList) {
                        if(machine != this && machine instanceof TileEntityAssemblyController) foundDuplicateMachine = true;
                        for(int i = 0; i < requiredMachines.length; i++) {
                            switch(requiredMachines[i]){
                                case DRILL:
                                    if(machine instanceof TileEntityAssemblyDrill) {
                                        if(drill != null) foundDuplicateMachine = true;
                                        drill = (TileEntityAssemblyDrill)machine;
                                        foundMachines[i] = true;
                                    }
                                    break;
                                case LASER:
                                    if(machine instanceof TileEntityAssemblyLaser) {
                                        if(laser != null) foundDuplicateMachine = true;
                                        laser = (TileEntityAssemblyLaser)machine;
                                        foundMachines[i] = true;
                                    }
                                    break;
                                case IO_UNIT_IMPORT:
                                    if(machine instanceof TileEntityAssemblyIOUnit && ((TileEntityAssemblyIOUnit)machine).isImportUnit()) {
                                        if(ioUnitImport != null) foundDuplicateMachine = true;
                                        ioUnitImport = (TileEntityAssemblyIOUnit)machine;
                                        foundMachines[i] = true;
                                    }
                                    break;
                                case IO_UNIT_EXPORT:
                                    if(machine instanceof TileEntityAssemblyIOUnit && !((TileEntityAssemblyIOUnit)machine).isImportUnit()) {
                                        if(ioUnitExport != null) foundDuplicateMachine = true;
                                        ioUnitExport = (TileEntityAssemblyIOUnit)machine;
                                        foundMachines[i] = true;
                                    }
                                    break;
                                case PLATFORM:
                                    if(machine instanceof TileEntityAssemblyPlatform) {
                                        if(platform != null) foundDuplicateMachine = true;
                                        platform = (TileEntityAssemblyPlatform)machine;
                                        foundMachines[i] = true;
                                    }
                                    break;
                            }
                        }
                    }

                    foundAllMachines = true;
                    for(boolean foundMachine : foundMachines) {
                        if(!foundMachine) {
                            foundAllMachines = false;
                            break;
                        }
                    }

                    if((foundAllMachines || curProgram == null) && !foundDuplicateMachine) {
                        // if(firstRun || areAllMachinesDone(machineList)) {
                        boolean useAir;
                        if(curProgram != null) {
                            useAir = curProgram.executeStep(this, platform, ioUnitImport, ioUnitExport, drill, laser);
                            if(useAir) displayedText = "Running...";
                        } else {
                            useAir = true;
                            goToHomePosition(platform, ioUnitImport, ioUnitExport, drill, laser);
                            displayedText = "Resetting...";
                        }
                        if(useAir) addAir(-(int)(PneumaticValues.USAGE_ASSEMBLING * getSpeedUsageMultiplierFromUpgrades()));
                        float speedMultiplier = getSpeedMultiplierFromUpgrades();
                        for(IAssemblyMachine machine : machineList) {
                            machine.setSpeed(speedMultiplier);
                        }
                    }
                }
            }
            hasProblem = hasProblem();
        }
        super.func_73660_a();

    }

    private void goToHomePosition(TileEntityAssemblyPlatform platform, TileEntityAssemblyIOUnit ioUnitImport, TileEntityAssemblyIOUnit ioUnitExport, TileEntityAssemblyDrill drill, TileEntityAssemblyLaser laser){

        boolean resetDone = true;

        for(IResettable machine : new IResettable[]{drill, laser, ioUnitImport, platform, ioUnitExport}) {
            if(machine != null && !machine.reset()) {
                resetDone = false;

                if(machine == platform) {
                    if(ioUnitExport != null) ioUnitExport.pickupItem(null);
                }

                break;
            }
        }

        goingToHomePosition = !(foundAllMachines && resetDone);
    }

    public void addProblems(List<String> problemList){
        if(getPressure() < PneumaticValues.MIN_PRESSURE_ASSEMBLY_CONTROLLER) {
            problemList.add(EnumChatFormatting.GRAY + "No sufficient pressure.");
            problemList.add(EnumChatFormatting.BLACK + "Add pressure.");
        }
        if(curProgram == null) {
            problemList.add(EnumChatFormatting.GRAY + "There's no program to run.");
            problemList.add(EnumChatFormatting.BLACK + "Insert an Assembly Program.");
        } else {
            if(foundDuplicateMachine) {
                problemList.addAll(PneumaticCraftUtils.convertStringIntoList(EnumChatFormatting.GRAY + "Controller found a duplicate machine!", GuiConstants.maxCharPerLineLeft));
                problemList.addAll(PneumaticCraftUtils.convertStringIntoList(EnumChatFormatting.BLACK + "Remove it so there is one machine of each type.", GuiConstants.maxCharPerLineLeft));
            } else if(foundAllMachines) {
                curProgram.addProgramProblem(problemList);
            } else {
                problemList.addAll(PneumaticCraftUtils.convertStringIntoList(EnumChatFormatting.GRAY + "Not all machines required for this program are available.", GuiConstants.maxCharPerLineLeft));
                problemList.addAll(PneumaticCraftUtils.convertStringIntoList(EnumChatFormatting.BLACK + "Connect up the other required machines.", GuiConstants.maxCharPerLineLeft));
            }
        }
    }

    public boolean hasProblem(){
        List<String> textList = null;
        if(curProgram != null) {
            textList = new ArrayList<String>();
            curProgram.addProgramProblem(textList);
        }
        return !foundAllMachines || foundDuplicateMachine || getPressure() < PneumaticValues.MIN_PRESSURE_ASSEMBLY_CONTROLLER || curProgram == null || textList.size() > 0;
    }

    public List<IAssemblyMachine> getMachines(){
        List<IAssemblyMachine> machineList = new ArrayList<IAssemblyMachine>();
        getMachines(machineList, func_174877_v());
        return machineList;
    }

    public boolean areAllMachinesDone(List<IAssemblyMachine> machineList){
        for(IAssemblyMachine machine : machineList) {
            if(!machine.isIdle()) return false;
        }
        return true;
    }

    private void getMachines(List<IAssemblyMachine> machineList, BlockPos pos){
        for(EnumFacing dir : EnumFacing.field_176754_o) {
            TileEntity te = field_145850_b.func_175625_s(pos.func_177972_a(dir));
            if(te instanceof IAssemblyMachine && !machineList.contains(te)) {
                machineList.add((IAssemblyMachine)te);
                getMachines(machineList, te.func_174877_v());
            }
        }
    }

    public void updateConnections(){
        List<Pair<EnumFacing, IAirHandler>> connections = getAirHandler(null).getConnectedPneumatics();
        Arrays.fill(sidesConnected, false);
        for(Pair<EnumFacing, IAirHandler> entry : connections) {
            sidesConnected[entry.getKey().ordinal()] = true;
        }
    }

    @Override
    public void onNeighborBlockUpdate(){
        super.onNeighborBlockUpdate();
        updateConnections();
    }

    @Override
    public boolean isConnectedTo(EnumFacing side){
        return side != EnumFacing.UP;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox(){
        return new AxisAlignedBB(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), func_174877_v().func_177958_n() + 1, func_174877_v().func_177956_o() + 1, func_174877_v().func_177952_p() + 1);
    }

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){

        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){

        return inventory[slot];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){

        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
    }

    @Override
    public String getName(){

        return Blockss.assemblyController.func_149739_a();
    }

    @Override
    public int func_70297_j_(){

        return 64;
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){

        super.func_145839_a(tag);
        goingToHomePosition = tag.func_74767_n("goingToHomePosition");
        foundAllMachines = tag.func_74767_n("foundAllMachines");
        foundDuplicateMachine = tag.func_74767_n("foundDuplicate");
        displayedText = tag.func_74779_i("displayedText");
        for(int i = 0; i < 6; i++) {
            sidesConnected[i] = tag.func_74767_n("sideConnected" + i);
        }
        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[func_70302_i_()];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }
        if(inventory[PROGRAM_INVENTORY_INDEX] != null) {
            curProgram = ItemAssemblyProgram.getProgramFromItem(inventory[PROGRAM_INVENTORY_INDEX].func_77952_i());
            if(curProgram != null) curProgram.readFromNBT(tag);
        }
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){

        super.func_145841_b(tag);
        tag.func_74757_a("goingToHomePosition", goingToHomePosition);
        tag.func_74757_a("foundAllMachines", foundAllMachines);
        tag.func_74757_a("foundDuplicate", foundDuplicateMachine);
        tag.func_74778_a("displayedText", displayedText);
        if(curProgram != null) curProgram.writeToNBT(tag);
        for(int i = 0; i < 6; i++) {
            tag.func_74757_a("sideConnected" + i, sidesConnected[i]);
        }
        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Items", tagList);
    }

    @Override
    public boolean func_94041_b(int i, ItemStack itemstack){
        if(i > 0) {
            return canInsertUpgrade(i, itemstack);
        } else {
            return itemstack != null && itemstack.func_77973_b() == Itemss.assemblyProgram;
        }
    }

    @Override
    // upgrades in bottom, fuel in the rest.
    public int[] func_180463_a(EnumFacing var1){
        if(var1 == EnumFacing.DOWN) return new int[]{1, 2, 3, 4};
        return new int[]{0};
    }

    @Override
    public boolean func_180462_a(int i, ItemStack itemstack, EnumFacing j){
        return true;
    }

    @Override
    public boolean func_180461_b(int i, ItemStack itemstack, EnumFacing j){
        return true;
    }

    @Override
    public boolean isIdle(){
        return true;
    }

    @Override
    public void setSpeed(float speed){}

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public float getMinWorkingPressure(){
        return PneumaticValues.MIN_PRESSURE_ASSEMBLY_CONTROLLER;
    }
}
