package pneumaticCraft.common.tileentity;

import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.LazySynced;
import pneumaticCraft.common.recipes.AssemblyRecipe;
import pneumaticCraft.common.recipes.programs.AssemblyProgram;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.TileEntityConstants;

public class TileEntityAssemblyDrill extends TileEntityAssemblyRobot{
    @DescSynced
    public boolean isDrillOn;
    @DescSynced
    @LazySynced
    private float drillSpeed;
    public float drillRotation;
    public float oldDrillRotation;
    private int drillStep;

    @Override
    public void func_73660_a(){
        oldDrillRotation = drillRotation;
        super.func_73660_a();
        if(isDrillOn) {
            drillSpeed = Math.min(drillSpeed + TileEntityConstants.ASSEMBLY_DRILL_ACCELERATION * speed, TileEntityConstants.ASSEMBLY_DRILL_MAX_SPEED);
        } else {
            drillSpeed = Math.max(drillSpeed - TileEntityConstants.ASSEMBLY_DRILL_ACCELERATION, 0);
        }
        drillRotation += drillSpeed;
        while(drillRotation >= 360) {
            drillRotation -= 360;
        }

        if(!field_145850_b.field_72995_K && drillStep > 0) {
            EnumFacing[] platformDirection = getPlatformDirection();
            if(platformDirection == null) drillStep = 1;
            switch(drillStep){
                case 1:
                    slowMode = false;
                    gotoHomePosition();
                    break;
                case 2:
                    hoverOverNeighbour(platformDirection[0], platformDirection[1]);
                    break;
                case 3:
                    isDrillOn = true;
                    break;
                case 4:
                    slowMode = true;
                    gotoNeighbour(platformDirection[0], platformDirection[1]);
                    break;
                case 5:
                    hoverOverNeighbour(platformDirection[0], platformDirection[1]);
                    isDrillOn = false;
                    TileEntity te = getTileEntityForCurrentDirection();
                    if(te instanceof TileEntityAssemblyPlatform) {
                        TileEntityAssemblyPlatform platform = (TileEntityAssemblyPlatform)te;
                        platform.hasDrilledStack = true;
                        ItemStack output = getDrilledOutputForItem(platform.getHeldStack());
                        if(output != null) {
                            platform.setHeldStack(output);
                        }
                    }
                    break;
                case 6:
                    slowMode = false;
                    gotoHomePosition();
                    break;
            }
            if(isDoneInternal()) {
                drillStep++;
                if(drillStep > 6) drillStep = 0;
            }
        }

    }

    public void goDrilling(){
        if(drillStep == 0) {
            drillStep = 1;
        }
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        tag.func_74757_a("drill", isDrillOn);
        tag.func_74776_a("drillSpeed", drillSpeed);
        tag.func_74768_a("drillStep", drillStep);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        isDrillOn = tag.func_74767_n("drill");
        drillSpeed = tag.func_74760_g("drillSpeed");
        drillStep = tag.func_74762_e("drillStep");
    }

    @Override
    public boolean isIdle(){
        return drillStep == 0 && isDoneInternal();
    }

    private boolean isDoneInternal(){
        if(super.isDoneMoving()) {
            return isDrillOn ? drillSpeed > TileEntityConstants.ASSEMBLY_DRILL_MAX_SPEED - 1F : PneumaticCraftUtils.areFloatsEqual(drillSpeed, 0F);
        } else {
            return false;
        }
    }

    @Override
    public boolean canMoveToDiagonalNeighbours(){
        return false;
    }

    public static ItemStack getDrilledOutputForItem(ItemStack input){
        for(AssemblyRecipe recipe : AssemblyRecipe.drillRecipes) {
            if(AssemblyProgram.isValidInput(recipe, input)) return recipe.getOutput().func_77946_l();
        }
        return null;
    }

    @Override
    public boolean reset(){
        if(isIdle()) return true;
        else {
            isDrillOn = false;
            drillStep = 6;
            return false;
        }
    }

}
