package pneumaticCraft.common.tileentity;

import java.util.List;

import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.LazySynced;
import pneumaticCraft.common.recipes.AssemblyRecipe;
import pneumaticCraft.common.util.IOHelper;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.TileEntityConstants;

public class TileEntityAssemblyIOUnit extends TileEntityAssemblyRobot{
    @DescSynced
    public boolean shouldClawClose;
    @DescSynced
    @LazySynced
    public float clawProgress;
    public float oldClawProgress;
    @DescSynced
    public ItemStack[] inventory = new ItemStack[1];
    private List<AssemblyRecipe> recipeList;
    private ItemStack searchedItemStack;
    private byte state = 0;
    private byte tickCounter = 0;
    private boolean hasSwitchedThisTick;
    @DescSynced
    private boolean exporting;

    private final static byte SLEEP_TICKS = 50;

    private final static byte STATE_IDLE = 0;
    private final static byte STATE_SEARCH_SRC = 1;
    private final static byte STATE_CLOSECLAW_AFTER_PICKUP = 5;
    private final static byte STATE_RESET_CLOSECLAW_AFTER_PICKUP = 20;
    private final static byte STATE_RESET_GOTO_IDLE = 26;
    private final static byte STATE_MAX = 127;

    @Override
    public void func_73660_a(){
        super.func_73660_a();
        hasSwitchedThisTick = false;
        if(field_145850_b.field_72995_K) {
            if(!isClawDone()) moveClaw();
        } else {

            slowMode = false;

            switch(state){

                case STATE_IDLE:
                    break;
                case STATE_SEARCH_SRC:
                    if(findPickupLocation()) state++;
                    break;
                // rise to the right height for target location
                case 2: // for pickup
                case 7: // for drop-off
                case 22: // for reset
                    if(hoverOverTarget()) state++;
                    break;
                // turn and move to target
                case 3: // for pickup
                case 8: // for drop-off
                case 23: // for reset
                    slowMode = true;
                    if(gotoTarget()) state++;
                    break;
                case 4: // pickup item - need to pick up before closeClaw; claw needs to know item size to 'grab' it!
                    if(getItemFromCurrentDirection()) state++;
                    break;
                case STATE_CLOSECLAW_AFTER_PICKUP:
                case STATE_RESET_CLOSECLAW_AFTER_PICKUP:
                    if(closeClaw()) state++;
                    break;
                case 6:
                case 21:
                    if(findDropOffLocation()) state++;
                    break;
                case 9:
                case 24:
                    if(openClaw()) state++;
                    break;
                case 10: // drop off item
                case 25:
                    if(putItemToCurrentDirection()) state++;
                    break;
                case 11:
                case STATE_RESET_GOTO_IDLE:
                    if(gotoIdlePos()) state = 0;
                case STATE_MAX: // this will be set if we encounter an unknown state; prevents log-spam that would result from default-case
                    break;
                default:
                    System.out.printf("unexpected state: %d%n", state);
                    state = STATE_MAX;
                    break;
            }
        }
    }

    @Override
    public boolean reset(){
        if(state >= STATE_RESET_CLOSECLAW_AFTER_PICKUP) return false;
        else if(inventory[0] != null) {
            state = STATE_RESET_CLOSECLAW_AFTER_PICKUP;
            return false;
        } else if(state == STATE_IDLE) {
            return true;
        } else {
            state = STATE_RESET_GOTO_IDLE;
            return isIdle();
        }
    }

    /**
     * @return true if the controller should use air and display 'running'
     */
    public boolean pickupItem(List<AssemblyRecipe> list){
        recipeList = list;

        if(state == STATE_IDLE) state++;

        return state > STATE_IDLE && !isSleeping() // will not use air while waiting for item/inventory to be available
                && state < STATE_MAX;
    }

    private boolean gotoIdlePos(){
        gotoHomePosition();
        return isDoneInternal();
    }

    private boolean findPickupLocation(){
        if(shouldSleep()) return false;

        EnumFacing[] inventoryDir = null;

        if(isImportUnit()) {
            searchedItemStack = null;
            if(recipeList != null) {
                for(AssemblyRecipe recipe : recipeList) {
                    inventoryDir = getInventoryDirectionForItem(recipe.getInput());
                    if(inventoryDir != null) {
                        searchedItemStack = recipe.getInput();
                        break;
                    }
                }
            }
        } else {
            inventoryDir = getPlatformDirection();
        }

        targetDirection = inventoryDir;

        if(targetDirection == null) {
            sleepBeforeNextSearch();

            return false;
        } else return true;
    }

    private boolean isSleeping(){
        return tickCounter > 0;
    }

    private boolean shouldSleep(){
        if(tickCounter > 0 && tickCounter++ < SLEEP_TICKS) {
            return true;
        } else {
            tickCounter = 0;
            return false;
        }
    }

    private void sleepBeforeNextSearch(){
        tickCounter = 1;
    }

    private boolean findDropOffLocation(){
        if(shouldSleep()) return false;

        EnumFacing[] inventoryDir = null;

        if(isImportUnit()) {
            inventoryDir = getPlatformDirection();
        } else {
            inventoryDir = getExportLocationForItem(inventory[0]);
        }

        targetDirection = inventoryDir;

        if(targetDirection == null) {
            sleepBeforeNextSearch();

            return false;
        } else return true;
    }

    private boolean getItemFromCurrentDirection(){
        TileEntity tile = getTileEntityForCurrentDirection();

        boolean extracted = false;

        /*
         * we must not .reset here because we might inadvertently change this.state right before this.state++
         *
        if((tile == null) || !(tile instanceof IInventory)) // TE / inventory is gone
        	reset();
        */

        if(isImportUnit()) {
            if(searchedItemStack == null) { // we don't know what we're supposed to pick up
                reset();
            } else if(tile instanceof IInventory) {
                IInventory inv = (IInventory)tile;

                int oldStackSize = inventory[0] == null ? 0 : inventory[0].field_77994_a;

                for(int i = 0; i < inv.func_70302_i_(); i++) {
                    if(inv.func_70301_a(i) != null) {
                        if(inventory[0] == null) {
                            if(inv.func_70301_a(i).func_77969_a(searchedItemStack)) {
                                inventory[0] = inv.func_70298_a(i, 1);
                            }
                        } else if(inv.func_70301_a(i).func_77969_a(inventory[0])) {
                            inv.func_70298_a(i, 1);
                            inventory[0].field_77994_a++;
                        }
                        extracted = (inventory[0] == null ? 0 : inventory[0].field_77994_a) == searchedItemStack.field_77994_a; // we might need to pickup more than 1 item
                        if(extracted) break;
                    }
                }

                if(oldStackSize == (inventory[0] == null ? 0 : inventory[0].field_77994_a)) // nothing picked up, search for different inventory
                state = STATE_SEARCH_SRC;

            } else state = STATE_SEARCH_SRC; // inventory gone
        } else {
            if(tile instanceof TileEntityAssemblyPlatform) {

                TileEntityAssemblyPlatform plat = (TileEntityAssemblyPlatform)tile;

                if(plat.openClaw()) {
                    inventory[0] = plat.getHeldStack();
                    plat.setHeldStack(null);
                    extracted = inventory[0] != null;

                    if(!extracted) // something went wrong - either the platform is gone altogether, or the item is not there anymore
                    state = STATE_SEARCH_SRC;
                }
            }
        }

        return extracted;
    }

    private boolean putItemToCurrentDirection(){
        if(isImportUnit()) {
            TileEntity tile = getTileEntityForCurrentDirection();
            if(tile instanceof TileEntityAssemblyPlatform) {

                TileEntityAssemblyPlatform plat = (TileEntityAssemblyPlatform)tile;

                if(inventory[0] == null) return plat.closeClaw();

                if(plat.isIdle()) {
                    plat.setHeldStack(inventory[0]);
                    inventory[0] = null;
                    return plat.closeClaw();
                }
            } else repeatDropOffSearch(); // platform gone; close claw and search new drop-off-location
        } else {
            IInventory inv = getInventoryForCurrentDirection();
            if(inv == null) repeatDropOffSearch(); // inventory gone; close claw and search new drop-off-location
            else {
                int startSize = inventory[0].field_77994_a;
                for(int i = 0; i < 6; i++) {
                    inventory[0] = PneumaticCraftUtils.exportStackToInventory(inv, inventory[0], EnumFacing.func_82600_a(i));
                    if(inventory[0] == null) break;
                }
                if(inventory[0] == null || startSize != inventory[0].field_77994_a) sendDescriptionPacket(); // TODO - is this still needed? Shouldn't @DescSynced on inventory take care of this?

                if(inventory[0] != null && startSize == inventory[0].field_77994_a) repeatDropOffSearch(); // target-inventory full or unavailable
            }

            return inventory[0] == null;
        }

        return false;
    }

    private void repeatDropOffSearch(){
        state = state >= STATE_RESET_CLOSECLAW_AFTER_PICKUP ? STATE_RESET_CLOSECLAW_AFTER_PICKUP : STATE_CLOSECLAW_AFTER_PICKUP;
    }

    private boolean closeClaw(){
        shouldClawClose = true;
        return moveClaw();
    }

    private boolean openClaw(){
        shouldClawClose = false;
        return moveClaw();
    }

    private boolean moveClaw(){
        oldClawProgress = clawProgress;

        if(!shouldClawClose && clawProgress > 0F) {
            clawProgress = Math.max(clawProgress - TileEntityConstants.ASSEMBLY_IO_UNIT_CLAW_SPEED * speed, 0);
        } else if(shouldClawClose && clawProgress < 1F) {
            clawProgress = Math.min(clawProgress + TileEntityConstants.ASSEMBLY_IO_UNIT_CLAW_SPEED * speed, 1);
        }

        return isClawDone();
    }

    private boolean isClawDone(){
        // need to make sure that clawProgress and oldClawProgress are the same, or we will get rendering artifacts
        return clawProgress == oldClawProgress && clawProgress == (shouldClawClose ? 1F : 0F);
    }

    public boolean isImportUnit(){
        return !exporting;
    }

    public IInventory getInventoryForCurrentDirection(){
        TileEntity te = getTileEntityForCurrentDirection();
        if(te instanceof IInventory) return (IInventory)te;
        return null;
    }

    public boolean switchMode(){
        if(state <= STATE_SEARCH_SRC) {
            if(!hasSwitchedThisTick) {
                exporting = !exporting;
                hasSwitchedThisTick = true;
            }
            return true;
        } else {
            return false;
        }

        //PacketDispatcher.sendPacketToAllPlayers(getDescriptionPacket());
    }

    @Override
    public void gotoHomePosition(){
        super.gotoHomePosition();

        if(isClawDone()) openClaw();
    }

    @Override
    public boolean isIdle(){
        return state == STATE_IDLE;
    }

    private boolean isDoneInternal(){
        return super.isDoneMoving();
        /*
        if(super.isDone()) {
            boolean searchDone = feedPlatformStep != 4 || searchedItemStack != null && inventory[0] != null && searchedItemStack.isItemEqual(inventory[0]) && inventory[0].stackSize == searchedItemStack.stackSize;
            return clawProgress == (shouldClawClose ? 1F : 0F) && searchDone;
        } else {
            return false;
        }
        */
    }

    public EnumFacing[] getInventoryDirectionForItem(ItemStack searchedItem){
        if(searchedItem != null && (inventory[0] == null || inventory[0].func_77969_a(searchedItem))) {
            for(EnumFacing dir : EnumFacing.field_176754_o) {
                TileEntity te = field_145850_b.func_175625_s(func_174877_v().func_177972_a(dir));
                if(te instanceof IInventory) {
                    if(IOHelper.extract(te, EnumFacing.UP, searchedItem, true, true) != null) return new EnumFacing[]{dir, null};
                }
            }
            if(canMoveToDiagonalNeighbours()) {
                for(EnumFacing secDir : new EnumFacing[]{EnumFacing.WEST, EnumFacing.EAST}) {
                    for(EnumFacing primDir : new EnumFacing[]{EnumFacing.NORTH, EnumFacing.SOUTH}) {
                        TileEntity te = field_145850_b.func_175625_s(func_174877_v().func_177972_a(primDir).func_177972_a(secDir));
                        if(te instanceof IInventory) {
                            if(IOHelper.extract(te, EnumFacing.UP, searchedItem, true, true) != null) return new EnumFacing[]{primDir, secDir};
                        }
                    }
                }
            }
        }
        return null;
    }

    /**
     * Returns the inventory the IOUnit can point to for the given item. This should only be invoked when you're sure that there is
     * an accessible inventory with the item surrounding this TE.
     * @param searchedItem
     * @return
     */
    public IInventory getInventory(ItemStack searchedItem){
        EnumFacing[] inventoryDir = getInventoryDirectionForItem(searchedItem);
        return (IInventory)field_145850_b.func_175625_s(func_174877_v().func_177972_a(inventoryDir[0]).func_177967_a(inventoryDir[1], inventoryDir[1] != null ? 1 : 0));
    }

    public EnumFacing[] getExportLocationForItem(ItemStack exportedItem){
        if(exportedItem != null) {
            for(EnumFacing dir : EnumFacing.field_82609_l) {
                if(dir != EnumFacing.UP && dir != EnumFacing.DOWN) {
                    TileEntity te = field_145850_b.func_175625_s(func_174877_v().func_177972_a(dir));
                    if(te instanceof IInventory) {
                        int slot = getInventoryPlaceLocation(exportedItem, (IInventory)te);
                        if(slot >= 0) return new EnumFacing[]{dir, null};
                    }
                }
            }
            if(canMoveToDiagonalNeighbours()) {
                for(EnumFacing secDir : new EnumFacing[]{EnumFacing.WEST, EnumFacing.EAST}) {
                    for(EnumFacing primDir : new EnumFacing[]{EnumFacing.NORTH, EnumFacing.SOUTH}) {
                        TileEntity te = field_145850_b.func_175625_s(func_174877_v().func_177972_a(primDir).func_177972_a(secDir));
                        if(te instanceof IInventory) {
                            int slot = getInventoryPlaceLocation(exportedItem, (IInventory)te);
                            if(slot >= 0) return new EnumFacing[]{primDir, secDir};
                        }
                    }
                }
            }
        }
        return null;
    }

    /**
     *
     * @param exported
     * @param inventory where the item is being tried to be placed in the top (respects ISidedInventory)
     * @return returns -1 when the item can't be placed / accessed
     */
    public static int getInventoryPlaceLocation(ItemStack exportedItem, IInventory inventory){
        if(inventory instanceof ISidedInventory) {
            int[] slotsInTop = ((ISidedInventory)inventory).func_180463_a(EnumFacing.UP);
            for(int slot : slotsInTop) {
                if(inventory.func_94041_b(slot, exportedItem)) {
                    ItemStack stack = inventory.func_70301_a(slot);
                    if(stack == null || stack.func_77969_a(exportedItem) && stack.func_77976_d() > stack.field_77994_a) return slot;
                }
            }
        } else {
            for(int slot = 0; slot < inventory.func_70302_i_(); slot++) {
                if(inventory.func_94041_b(slot, exportedItem)) {
                    ItemStack stack = inventory.func_70301_a(slot);
                    if(stack == null || stack.func_77969_a(exportedItem) && stack.func_77976_d() > stack.field_77994_a) return slot;
                }
            }
        }
        return -1;
    }

    /**
     * Returns the inventory the IOUnit can point to export the given item. This should only be invoked when you're sure that there is
     * an accessible inventory with the item surrounding this TE.
     * @param exportedItem
     * @return
     */
    public IInventory getExportInventory(ItemStack exportedItem){
        EnumFacing[] inventoryDir = getExportLocationForItem(exportedItem);
        return (IInventory)field_145850_b.func_175625_s(func_174877_v().func_177972_a(inventoryDir[0]).func_177967_a(inventoryDir[1], inventoryDir[1] != null ? 1 : 0));
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        clawProgress = tag.func_74760_g("clawProgress");
        shouldClawClose = tag.func_74767_n("clawClosing");
        state = tag.func_74771_c("state");
        exporting = tag.func_74767_n("exporting");
        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[1];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        tag.func_74776_a("clawProgress", clawProgress);
        tag.func_74757_a("clawClosing", shouldClawClose);
        tag.func_74774_a("state", state);
        tag.func_74757_a("exporting", exporting);
        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Items", tagList);
    }

    @Override
    public boolean canMoveToDiagonalNeighbours(){
        return true;
    }

}
