package pneumaticCraft.common.tileentity;

import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.LazySynced;
import pneumaticCraft.lib.TileEntityConstants;

public class TileEntityAssemblyPlatform extends TileEntityBase implements IAssemblyMachine, IResettable{
    @DescSynced
    private boolean shouldClawClose;
    @DescSynced
    @LazySynced
    public float clawProgress;
    public float oldClawProgress;
    @DescSynced
    private ItemStack[] inventory = new ItemStack[1];
    private float speed = 1.0F;
    public boolean hasDrilledStack;
    public boolean hasLaseredStack;

    @Override
    public void func_73660_a(){
        super.func_73660_a();
        oldClawProgress = clawProgress;
        if(!shouldClawClose && clawProgress > 0F) {
            clawProgress = Math.max(clawProgress - TileEntityConstants.ASSEMBLY_IO_UNIT_CLAW_SPEED * speed, 0);
        } else if(shouldClawClose && clawProgress < 1F) {
            clawProgress = Math.min(clawProgress + TileEntityConstants.ASSEMBLY_IO_UNIT_CLAW_SPEED * speed, 1);
        }
    }

    private boolean isClawDone(){
        return clawProgress == (shouldClawClose ? 1F : 0F);
    }

    @Override
    public boolean isIdle(){
        return !shouldClawClose && isClawDone() && inventory[0] == null;
    }

    @Override
    public boolean reset(){
        openClaw();
        return isIdle();
    }

    public boolean closeClaw(){
        hasDrilledStack = false;
        hasLaseredStack = false;
        shouldClawClose = true;
        return isClawDone();
    }

    public boolean openClaw(){
        shouldClawClose = false;
        return isClawDone();
    }

    public ItemStack getHeldStack(){
        return inventory[0];
    }

    public void setHeldStack(ItemStack stack){
        if(stack == null) {
            hasDrilledStack = false;
            hasLaseredStack = false;
        }
        inventory[0] = stack;
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        tag.func_74757_a("clawClosing", shouldClawClose);
        tag.func_74776_a("clawProgress", clawProgress);
        tag.func_74776_a("speed", speed);
        tag.func_74757_a("drilled", hasDrilledStack);
        tag.func_74757_a("lasered", hasLaseredStack);
        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Items", tagList);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        shouldClawClose = tag.func_74767_n("clawClosing");
        clawProgress = tag.func_74760_g("clawProgress");
        speed = tag.func_74760_g("speed");
        hasDrilledStack = tag.func_74767_n("drilled");
        hasLaseredStack = tag.func_74767_n("lasered");
        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[1];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }
    }

    @Override
    public void setSpeed(float speed){
        this.speed = speed;
    }

}
