package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.network.Packet;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.ChatComponentTranslation;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.IChatComponent;
import net.minecraft.util.ITickable;
import net.minecraft.world.World;
import net.minecraftforge.fluids.FluidContainerRegistry;
import net.minecraftforge.fluids.FluidContainerRegistry.FluidContainerData;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidTankInfo;
import net.minecraftforge.fluids.IFluidContainerItem;
import net.minecraftforge.fluids.IFluidHandler;
import net.minecraftforge.fml.common.network.NetworkRegistry.TargetPoint;
import pneumaticCraft.api.heat.IHeatExchangerLogic;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.item.IUpgradeAcceptor;
import pneumaticCraft.api.tileentity.IHeatExchanger;
import pneumaticCraft.common.block.BlockPneumaticCraft;
import pneumaticCraft.common.inventory.SyncedField;
import pneumaticCraft.common.item.Itemss;
import pneumaticCraft.common.network.DescPacketHandler;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.IDescSynced;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.NetworkUtils;
import pneumaticCraft.common.network.PacketDescription;
import pneumaticCraft.common.thirdparty.computercraft.ILuaMethod;
import pneumaticCraft.common.thirdparty.computercraft.LuaMethod;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.common.util.TileEntityCache;
import pneumaticCraft.lib.PneumaticValues;

//TODO Computercraft dep @Optional.InterfaceList({@Optional.Interface(iface = "dan200.computercraft.api.peripheral.IPeripheral", modid = ModIds.COMPUTERCRAFT)})
public class TileEntityBase extends TileEntity implements IGUIButtonSensitive, IDescSynced, ITickable, IUpgradeAcceptor/*, IPeripheral*/{
    /**
     * True only the first time updateEntity invokes in a session.
     */
    protected boolean firstRun = true;
    private final int[] upgradeSlots;
    private boolean descriptionPacketScheduled;
    private List<SyncedField> descriptionFields;
    protected int poweredRedstone; //The redstone strength currently applied to the block.
    private TileEntityCache[] tileCache;
    protected List<ILuaMethod> luaMethods = new ArrayList<ILuaMethod>();
    private IBlockState cachedBlockState;
    private final Set<Item> applicableUpgrades = new HashSet<Item>();

    public TileEntityBase(int... upgradeSlots){
        addLuaMethods();
        this.upgradeSlots = upgradeSlots;
    }

    protected void addApplicableUpgrade(EnumUpgrade... upgrades){
        for(EnumUpgrade upgrade : upgrades)
            addApplicableUpgrade(Itemss.upgrades.get(upgrade));
    }

    protected void addApplicableUpgrade(Item upgrade){
        applicableUpgrades.add(upgrade);
    }

    @Override
    public Packet func_145844_m(){
        return DescPacketHandler.getPacket(new PacketDescription(this));
    }

    @Override
    public BlockPos getPosition(){
        return func_174877_v();
    }

    protected double getPacketDistance(){
        return 64;
    }

    @Override
    public List<SyncedField> getDescriptionFields(){
        if(descriptionFields == null) {
            descriptionFields = NetworkUtils.getSyncedFields(this, DescSynced.class);
            for(SyncedField field : descriptionFields) {
                field.update();
            }
        }
        return descriptionFields;
    }

    public void sendDescriptionPacket(){
        field_145850_b.func_175689_h(func_174877_v());
    }

    /**
     * A way to safely mark a block for an update from another thread (like the CC Lua thread).
     */
    protected void scheduleDescriptionPacket(){
        descriptionPacketScheduled = true;
    }

    public void sendDescPacket(double maxPacketDistance){
        NetworkHandler.sendToAllAround(new PacketDescription(this), new TargetPoint(field_145850_b.field_73011_w.func_177502_q(), func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), maxPacketDistance));
    }

    @Override
    public void func_73660_a(){
        if(firstRun && !field_145850_b.field_72995_K) {
            //firstRun = false;
            onFirstServerUpdate();
            onNeighborTileUpdate();
            onNeighborBlockUpdate();
        }
        firstRun = false;

        if(!field_145850_b.field_72995_K) {
            if(this instanceof IHeatExchanger) {
                ((IHeatExchanger)this).getHeatExchangerLogic(null).update();
            }

            if(descriptionFields == null) descriptionPacketScheduled = true;
            for(SyncedField field : getDescriptionFields()) {
                if(field.update()) {
                    descriptionPacketScheduled = true;
                }
            }

            if(descriptionPacketScheduled) {
                descriptionPacketScheduled = false;
                sendDescriptionPacket();
            }
        }
    }

    protected void onFirstServerUpdate(){
        initializeIfHeatExchanger();
    }

    protected void updateNeighbours(){
        field_145850_b.func_175685_c(func_174877_v(), func_145838_q());
    }

    public void onBlockRotated(){}

    protected void rerenderChunk(){
        field_145850_b.func_147458_c(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p());
    }

    protected boolean shouldRerenderChunkOnDescUpdate(){
        return false;
    }

    /**
     * Encoded into the description packet. Also is included in the world save.
     * Used as last resort, using @DescSynced is preferred.
     * @param tag
     */
    @Override
    public void writeToPacket(NBTTagCompound tag){}

    /**
     * Encoded into the description packet. Also is included in the world save.
     * Used as last resort, using @DescSynced is preferred.
     * @param tag
     */
    @Override
    public void readFromPacket(NBTTagCompound tag){}

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        writeToPacket(tag);
        if(this instanceof IHeatExchanger) {
            ((IHeatExchanger)this).getHeatExchangerLogic(null).writeToNBT(tag);
        }
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        readFromPacket(tag);
        if(this instanceof IHeatExchanger) {
            ((IHeatExchanger)this).getHeatExchangerLogic(null).readFromNBT(tag);
        }
    }

    @Override
    public void func_145829_t(){
        super.func_145829_t();
        scheduleDescriptionPacket();
    }

    @Override
    public void onDescUpdate(){
        if(shouldRerenderChunkOnDescUpdate()) rerenderChunk();
    }

    /**
     * Called when a key is synced in the container.
     */
    public void onGuiUpdate(){}

    public EnumFacing getRotation(){
        if(cachedBlockState == null) {
            cachedBlockState = field_145850_b.func_180495_p(func_174877_v());
        }
        return cachedBlockState.func_177229_b(BlockPneumaticCraft.ROTATION);
    }

    @Override
    public void func_145836_u(){
        cachedBlockState = null;
        super.func_145836_u();
    }

    public int getUpgrades(EnumUpgrade upgrade){
        return getUpgrades(upgrade, upgradeSlots);
    }

    protected int getUpgrades(EnumUpgrade upgrade, int... upgradeSlots){
        return getUpgrades((IInventory)this, upgrade, upgradeSlots);
    }

    public int getUpgrades(Item upgrade){
        return getUpgrades((IInventory)this, upgrade, upgradeSlots);
    }

    public static int getUpgrades(IInventory inv, EnumUpgrade upgrade, int... upgradeSlots){
        return getUpgrades(inv, Itemss.upgrades.get(upgrade), upgradeSlots);
    }

    public static int getUpgrades(IInventory inv, Item upgrade, int... upgradeSlots){
        int upgrades = 0;
        for(int i : upgradeSlots) {
            if(inv.func_70301_a(i) != null && inv.func_70301_a(i).func_77973_b() == upgrade) {
                upgrades += inv.func_70301_a(i).field_77994_a;
            }
        }
        return upgrades;
    }

    public float getSpeedMultiplierFromUpgrades(){
        return getSpeedMultiplierFromUpgrades(upgradeSlots);
    }

    public float getSpeedUsageMultiplierFromUpgrades(){
        return getSpeedUsageMultiplierFromUpgrades(upgradeSlots);
    }

    public float getSpeedMultiplierFromUpgrades(int[] upgradeSlots){
        return (float)Math.pow(PneumaticValues.SPEED_UPGRADE_MULTIPLIER, Math.min(10, getUpgrades(EnumUpgrade.SPEED, upgradeSlots)));
    }

    protected float getSpeedUsageMultiplierFromUpgrades(int[] upgradeSlots){
        return (float)Math.pow(PneumaticValues.SPEED_UPGRADE_USAGE_MULTIPLIER, Math.min(10, getUpgrades(EnumUpgrade.SPEED, upgradeSlots)));
    }

    /* TODO IC2 dep @Optional.Method(modid = ModIds.INDUSTRIALCRAFT)
     protected int getIC2Upgrades(String ic2ItemKey, int[] upgradeSlots){
         ItemStack itemStack = IC2Items.getItem(ic2ItemKey);
         if(itemStack == null) return 0;
         int upgrades = 0;
         if(this instanceof IInventory) {// this always should be true.
             IInventory inv = (IInventory)this;
             for(int i : upgradeSlots) {
                 if(inv.getStackInSlot(i) != null && inv.getStackInSlot(i).getItem() == itemStack.getItem()) {
                     upgrades += inv.getStackInSlot(i).stackSize;
                 }
             }
         }
         return upgrades;
     }*/

    @Override
    public void handleGUIButtonPress(int guiID, EntityPlayer player){}

    public boolean isGuiUseableByPlayer(EntityPlayer par1EntityPlayer){
        return field_145850_b.func_175625_s(func_174877_v()) != this ? false : par1EntityPlayer.func_70092_e(func_174877_v().func_177958_n() + 0.5D, func_174877_v().func_177956_o() + 0.5D, func_174877_v().func_177952_p() + 0.5D) <= 64.0D;
    }

    public static void writeInventoryToNBT(NBTTagCompound tag, ItemStack[] stacks){
        writeInventoryToNBT(tag, stacks, "Items");
    }

    public static void writeInventoryToNBT(NBTTagCompound tag, IInventory inventory, String tagName){
        ItemStack[] stacks = new ItemStack[inventory.func_70302_i_()];
        for(int i = 0; i < stacks.length; i++) {
            stacks[i] = inventory.func_70301_a(i);
        }
        writeInventoryToNBT(tag, stacks, tagName);
    }

    public static void writeInventoryToNBT(NBTTagCompound tag, ItemStack[] stacks, String tagName){
        NBTTagList tagList = new NBTTagList();
        for(int i = 0; i < stacks.length; i++) {
            if(stacks[i] != null) {
                NBTTagCompound itemTag = new NBTTagCompound();
                stacks[i].func_77955_b(itemTag);
                itemTag.func_74774_a("Slot", (byte)i);
                tagList.func_74742_a(itemTag);
            }
        }
        tag.func_74782_a(tagName, tagList);
    }

    public static void readInventoryFromNBT(NBTTagCompound tag, ItemStack[] stacks){
        readInventoryFromNBT(tag, stacks, "Items");
    }

    public static void readInventoryFromNBT(NBTTagCompound tag, IInventory inventory, String tagName){
        ItemStack[] stacks = new ItemStack[inventory.func_70302_i_()];
        readInventoryFromNBT(tag, stacks, tagName);
        for(int i = 0; i < stacks.length; i++) {
            inventory.func_70299_a(i, stacks[i]);
        }
    }

    public static void readInventoryFromNBT(NBTTagCompound tag, ItemStack[] stacks, String tagName){
        for(int i = 0; i < stacks.length; i++) {
            stacks[i] = null;
        }
        NBTTagList tagList = tag.func_150295_c(tagName, 10);
        for(int i = 0; i < tagList.func_74745_c(); i++) {
            NBTTagCompound itemTag = tagList.func_150305_b(i);
            int slot = itemTag.func_74771_c("Slot");
            if(slot >= 0 && slot < stacks.length) {
                stacks[slot] = ItemStack.func_77949_a(itemTag);
            }
        }
    }

    public void onNeighborTileUpdate(){
        initializeIfHeatExchanger();
        for(TileEntityCache cache : getTileCache()) {
            cache.update();
        }
    }

    public TileEntityCache[] getTileCache(){
        if(tileCache == null) tileCache = TileEntityCache.getDefaultCache(field_145850_b, func_174877_v());
        return tileCache;
    }

    public void onNeighborBlockUpdate(){
        poweredRedstone = PneumaticCraftUtils.getRedstoneLevel(field_145850_b, func_174877_v());
        initializeIfHeatExchanger();
    }

    public boolean redstoneAllows(){
        if(field_145850_b.field_72995_K) onNeighborBlockUpdate();
        switch(((IRedstoneControl)this).getRedstoneMode()){
            case 0:
                return true;
            case 1:
                return poweredRedstone > 0;
            case 2:
                return poweredRedstone == 0;
        }
        return false;
    }

    protected void initializeIfHeatExchanger(){
        if(this instanceof IHeatExchanger) {
            initializeHeatExchanger(((IHeatExchanger)this).getHeatExchangerLogic(null), getConnectedHeatExchangerSides());
        }
    }

    protected void initializeHeatExchanger(IHeatExchangerLogic heatExchanger, EnumFacing... connectedSides){
        heatExchanger.initializeAsHull(field_145850_b, func_174877_v(), connectedSides);
    }

    /**
     * Gets the valid sides for heat exchanging to be allowed. returning an empty array will allow any side.
     * @return
     */
    protected EnumFacing[] getConnectedHeatExchangerSides(){
        return new EnumFacing[0];
    }

    public void autoExportLiquid(){
        FluidStack extractedStack = ((IFluidHandler)this).drain(null, Integer.MAX_VALUE, false);
        if(extractedStack != null) {
            for(EnumFacing d : EnumFacing.field_82609_l) {
                TileEntity te = getTileCache()[d.ordinal()].getTileEntity();
                if(te instanceof IFluidHandler) {
                    if(((IFluidHandler)te).canFill(d.func_176734_d(), extractedStack.getFluid())) {
                        int filledAmount = ((IFluidHandler)te).fill(d.func_176734_d(), extractedStack, true);
                        ((IFluidHandler)this).drain(null, filledAmount, true);
                        extractedStack.amount -= filledAmount;
                        if(extractedStack.amount <= 0) break;
                    }
                }
            }
        }
    }

    @Override
    public Type getSyncType(){
        return Type.TILE_ENTITY;
    }

    protected void processFluidItem(int inputSlot, int outputSlot){
        IInventory inv = (IInventory)this;
        IFluidHandler fluidHandler = (IFluidHandler)this;
        FluidTankInfo tankInfo = fluidHandler.getTankInfo(null)[0];
        if(inv.func_70301_a(inputSlot) != null) {
            ItemStack fluidContainer = inv.func_70301_a(inputSlot);
            if(tankInfo.fluid == null || tankInfo.fluid.isFluidEqual(fluidContainer)) {
                FluidStack fluid = FluidContainerRegistry.getFluidForFilledItem(fluidContainer);
                int amount = FluidContainerRegistry.BUCKET_VOLUME;
                if(fluid == null) {
                    if(fluidContainer.func_77973_b() instanceof IFluidContainerItem) {
                        IFluidContainerItem containerItem = (IFluidContainerItem)fluidContainer.func_77973_b();
                        fluid = containerItem.getFluid(fluidContainer);
                        if(fluid != null && fluidHandler.canFill(null, fluid.getFluid())) {
                            amount = fluid != null ? fluid.amount : 0;
                            int availableSpace = tankInfo.capacity - (tankInfo.fluid != null ? tankInfo.fluid.amount : 0);
                            if(availableSpace >= amount) {
                                ItemStack singleFuelItem = fluidContainer.func_77946_l();
                                singleFuelItem.field_77994_a = 1;
                                FluidStack drainedStack = containerItem.drain(singleFuelItem, availableSpace, true);
                                if(fluidContainer.field_77994_a == 1 || inv.func_70301_a(outputSlot) == null || canStack(singleFuelItem, inv.func_70301_a(outputSlot))) {
                                    fluidHandler.fill(null, drainedStack, true);
                                    if(fluidContainer.field_77994_a == 1) {
                                        inv.func_70299_a(inputSlot, singleFuelItem);
                                    } else {
                                        inv.func_70301_a(inputSlot).field_77994_a--;
                                        if(inv.func_70301_a(outputSlot) == null) {
                                            inv.func_70299_a(outputSlot, singleFuelItem);
                                        } else {
                                            inv.func_70301_a(outputSlot).field_77994_a++;
                                        }
                                    }
                                }
                            }
                        }
                    }
                } else if(fluidHandler.canFill(null, fluid.getFluid())) {
                    if(tankInfo.capacity - (tankInfo.fluid != null ? tankInfo.fluid.amount : 0) >= amount) {
                        ItemStack returnedItem = null;
                        FluidContainerData[] allFluidData = FluidContainerRegistry.getRegisteredFluidContainerData();
                        for(FluidContainerData fluidData : allFluidData) {
                            if(fluidData.filledContainer.func_77969_a(fluidContainer)) {
                                returnedItem = fluidData.emptyContainer;
                                break;
                            }
                        }
                        if(returnedItem == null || inv.func_70301_a(outputSlot) == null || canStack(returnedItem, inv.func_70301_a(outputSlot))) {
                            if(returnedItem != null) {
                                if(inv.func_70301_a(outputSlot) == null) {
                                    inv.func_70299_a(outputSlot, returnedItem.func_77946_l());
                                } else {
                                    inv.func_70301_a(outputSlot).field_77994_a += returnedItem.field_77994_a;
                                }
                            }
                            fluidHandler.fill(null, new FluidStack(fluid.getFluid(), amount, fluid.tag), true);
                            inv.func_70301_a(inputSlot).field_77994_a--;
                            if(inv.func_70301_a(inputSlot).field_77994_a <= 0) inv.func_70299_a(inputSlot, null);
                        }
                    }
                }
            }
            if(fluidContainer.func_77973_b() instanceof IFluidContainerItem) {
                if(((IFluidContainerItem)fluidContainer.func_77973_b()).getFluid(fluidContainer) == null && (inv.func_70301_a(outputSlot) == null || canStack(fluidContainer, inv.func_70301_a(outputSlot)))) {
                    if(inv.func_70301_a(outputSlot) == null) {
                        inv.func_70299_a(outputSlot, fluidContainer);
                    } else {
                        inv.func_70301_a(outputSlot).field_77994_a += fluidContainer.field_77994_a;
                    }
                    inv.func_70299_a(inputSlot, null);
                }
            }
        }
    }

    private boolean canStack(ItemStack stack1, ItemStack stack2){
        return stack1.func_77969_a(stack2) && ItemStack.func_77970_a(stack1, stack2) && stack1.field_77994_a + stack2.field_77994_a <= stack1.func_77976_d();
    }

    public IChatComponent getDisplayName(){
        IInventory inv = (IInventory)this;
        return inv.func_145818_k_() ? new ChatComponentText(inv.func_70005_c_()) : new ChatComponentTranslation(inv.func_70005_c_(), new Object[0]);
    }

    public int getField(int id){
        return 0;
    }

    public void setField(int id, int value){

    }

    public int getFieldCount(){
        return 0;
    }

    @Override
    public Set<Item> getApplicableUpgrades(){
        return applicableUpgrades;
    }

    @Override
    public String getName(){
        return null; //Is called directly from the block instead.
    }

    public boolean canInsertUpgrade(int slot, ItemStack item){
        return item != null ? canInsertUpgrade(slot, item.func_77973_b()) : false;
    }

    protected boolean canInsertUpgrade(int slot, Item item){
        if(!applicableUpgrades.contains(item)) return false;
        for(int upgradeSlot : upgradeSlots) {
            if(upgradeSlot == slot) return true;
        }
        return false;
    }

    @Override
    public boolean shouldRefresh(World world, BlockPos pos, IBlockState oldState, IBlockState newSate){
        return oldState.func_177230_c() != newSate.func_177230_c();
    }

    /*
     * COMPUTERCRAFT API 
     */

    protected void addLuaMethods(){
        if(this instanceof IHeatExchanger) {
            final IHeatExchanger exchanger = (IHeatExchanger)this;
            luaMethods.add(new LuaMethod("getTemperature"){
                @Override
                public Object[] call(Object[] args) throws Exception{
                    if(args.length == 0) {
                        return new Object[]{exchanger.getHeatExchangerLogic(null).getTemperature()};
                    } else if(args.length == 1) {
                        IHeatExchangerLogic logic = exchanger.getHeatExchangerLogic(getDirForString((String)args[0]));
                        return new Object[]{logic != null ? logic.getTemperature() : 0};
                    } else {
                        throw new IllegalArgumentException("getTemperature method requires 0 or 1 argument (direction: up, down, east, west, north, south!");
                    }
                }
            });
        }
    }

    /* @Override
     public String getType(){
         return getBlockType().getUnlocalizedName().substring(5);
     }

     @Override
     public String[] getMethodNames(){
         String[] methodNames = new String[luaMethods.size()];
         for(int i = 0; i < methodNames.length; i++) {
             methodNames[i] = luaMethods.get(i).getMethodName();
         }
         return methodNames;
     }

     public List<ILuaMethod> getLuaMethods(){
         return luaMethods;
     }

     @Override
     @Optional.Method(modid = ModIds.COMPUTERCRAFT)
     public Object[] callMethod(IComputerAccess computer, ILuaContext context, int method, Object[] arguments) throws LuaException, InterruptedException{
         try {
             return luaMethods.get(method).call(arguments);
         } catch(Exception e) {
             throw new LuaException(e.getMessage());
         }
     }

     @Override
     @Optional.Method(modid = ModIds.COMPUTERCRAFT)
     public void attach(IComputerAccess computer){}

     @Override
     @Optional.Method(modid = ModIds.COMPUTERCRAFT)
     public void detach(IComputerAccess computer){}

     @Override
     @Optional.Method(modid = ModIds.COMPUTERCRAFT)
     public boolean equals(IPeripheral other){
         if(other == null) {
             return false;
         }
         if(this == other) {
             return true;
         }
         if(other instanceof TileEntity) {
             TileEntity tother = (TileEntity)other;
             return tother.getWorldObj().equals(worldObj) && tother.getPos().getX() == getPos().getX() && tother.getPos().getY() == getPos().getY() && tother.getPos().getZ() == getPos().getZ();
         }

         return false;
     }*/
}
