package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.apache.commons.lang3.tuple.Pair;

import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.item.IPressurizable;
import pneumaticCraft.api.tileentity.IAirHandler;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.inventory.ContainerChargingStationItemInventory;
import pneumaticCraft.common.inventory.InventoryPneumaticInventoryItem;
import pneumaticCraft.common.item.IChargingStationGUIHolderItem;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.PneumaticValues;
import pneumaticCraft.proxy.CommonProxy.EnumGuiId;

public class TileEntityChargingStation extends TileEntityPneumaticBase implements ISidedInventory, IRedstoneControl{
    @DescSynced
    private ItemStack[] inventory;
    private InventoryPneumaticInventoryItem chargeableInventory;

    private final int INVENTORY_SIZE = 5;

    public static final int CHARGE_INVENTORY_INDEX = 0;
    public static final int UPGRADE_SLOT_START = 1;
    public static final int UPGRADE_SLOT_END = 4;
    public static final float ANIMATION_AIR_SPEED = 0.001F;

    @GuiSynced
    public boolean charging;
    @GuiSynced
    public boolean disCharging;
    @GuiSynced
    public int redstoneMode;
    private boolean oldRedstoneStatus;
    public float renderAirProgress;
    @DescSynced
    private ItemStack camoStack;

    public TileEntityChargingStation(){
        super(PneumaticValues.DANGER_PRESSURE_CHARGING_STATION, PneumaticValues.MAX_PRESSURE_CHARGING_STATION, PneumaticValues.VOLUME_CHARGING_STATION, UPGRADE_SLOT_START, 2, 3, UPGRADE_SLOT_END);
        inventory = new ItemStack[INVENTORY_SIZE];
        addApplicableUpgrade(EnumUpgrade.SPEED, EnumUpgrade.DISPENSER);
    }

    public ItemStack getCamoStack(){
        if(camoStack == null || !(camoStack.func_77973_b() instanceof ItemBlock)) {
            return new ItemStack(Blocks.field_150347_e);
        }
        Block block = ((ItemBlock)camoStack.func_77973_b()).func_179223_d();
        if(PneumaticCraftUtils.isRenderIDCamo(block.func_149645_b())) {
            return camoStack;
        } else {
            return new ItemStack(Blocks.field_150347_e);
        }
    }

    public void setCamoStack(ItemStack stack){
        camoStack = stack;
    }

    @Override
    public void func_73660_a(){
        disCharging = false;
        charging = false;
        List<IPressurizable> chargingItems = new ArrayList<IPressurizable>();
        List<ItemStack> chargedStacks = new ArrayList<ItemStack>();
        if(inventory[CHARGE_INVENTORY_INDEX] != null && inventory[CHARGE_INVENTORY_INDEX].func_77973_b() instanceof IPressurizable) {
            chargingItems.add((IPressurizable)inventory[CHARGE_INVENTORY_INDEX].func_77973_b());
            chargedStacks.add(inventory[CHARGE_INVENTORY_INDEX]);
        }
        if(this.getUpgrades(EnumUpgrade.DISPENSER) > 0) {
            //creating a new word, 'entities padding'.
            List<Entity> entitiesPadding = field_145850_b.func_72872_a(Entity.class, new AxisAlignedBB(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), func_174877_v().func_177958_n() + 1, func_174877_v().func_177956_o() + 2, func_174877_v().func_177952_p() + 1));
            for(Entity entity : entitiesPadding) {
                if(entity instanceof IPressurizable) {
                    chargingItems.add((IPressurizable)entity);
                    chargedStacks.add(null);
                } else if(entity instanceof EntityItem) {
                    ItemStack entityStack = ((EntityItem)entity).func_92059_d();
                    if(entityStack != null && entityStack.func_77973_b() instanceof IPressurizable) {
                        chargingItems.add((IPressurizable)entityStack.func_77973_b());
                        chargedStacks.add(entityStack);
                    }
                } else if(entity instanceof EntityPlayer) {
                    InventoryPlayer inv = ((EntityPlayer)entity).field_71071_by;
                    for(int i = 0; i < inv.func_70302_i_(); i++) {
                        ItemStack stack = inv.func_70301_a(i);
                        if(stack != null && stack.func_77973_b() instanceof IPressurizable) {
                            chargingItems.add((IPressurizable)stack.func_77973_b());
                            chargedStacks.add(stack);
                        }
                    }
                }
            }
        }
        int speedMultiplier = (int)getSpeedMultiplierFromUpgrades();
        for(int i = 0; i < PneumaticValues.CHARGING_STATION_CHARGE_RATE * speedMultiplier; i++) {
            boolean charged = false;
            for(int j = 0; j < chargingItems.size(); j++) {
                IPressurizable chargingItem = chargingItems.get(j);
                ItemStack chargedItem = chargedStacks.get(j);
                if(chargingItem.getPressure(chargedItem) > getPressure() + 0.01F && chargingItem.getPressure(chargedItem) > 0F) {
                    if(!field_145850_b.field_72995_K) {
                        chargingItem.addAir(chargedItem, -1);
                        addAir(1);
                    }
                    disCharging = true;
                    renderAirProgress -= ANIMATION_AIR_SPEED;
                    if(renderAirProgress < 0.0F) {
                        renderAirProgress += 1F;
                    }
                    charged = true;
                } else if(chargingItem.getPressure(chargedItem) < getPressure() - 0.01F && chargingItem.getPressure(chargedItem) < chargingItem.maxPressure(chargedItem)) {// if there is pressure, and the item isn't fully charged yet..
                    if(!field_145850_b.field_72995_K) {
                        chargingItem.addAir(chargedItem, 1);
                        addAir(-1);
                    }
                    charging = true;
                    renderAirProgress += ANIMATION_AIR_SPEED;
                    if(renderAirProgress > 1.0F) {
                        renderAirProgress -= 1F;
                    }
                    charged = true;
                }
            }
            if(!charged) break;
        }

        if(!field_145850_b.field_72995_K && oldRedstoneStatus != shouldEmitRedstone()) {
            oldRedstoneStatus = shouldEmitRedstone();
            updateNeighbours();
        }

        super.func_73660_a();

        if(!field_145850_b.field_72995_K) {
            List<Pair<EnumFacing, IAirHandler>> teList = getAirHandler(null).getConnectedPneumatics();
            if(teList.size() == 0) getAirHandler(null).airLeak(getRotation());
        }
    }

    @Override
    public boolean isConnectedTo(EnumFacing side){
        return getRotation() == side;
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID == 0) {
            redstoneMode++;
            if(redstoneMode > 3) redstoneMode = 0;
            updateNeighbours();
        } else if((buttonID == 1 || buttonID == 2) && inventory[CHARGE_INVENTORY_INDEX] != null && inventory[CHARGE_INVENTORY_INDEX].func_77973_b() instanceof IChargingStationGUIHolderItem) {
            player.openGui(PneumaticCraft.instance, buttonID == 1 ? ((IChargingStationGUIHolderItem)inventory[CHARGE_INVENTORY_INDEX].func_77973_b()).getGuiID().ordinal() : EnumGuiId.CHARGING_STATION.ordinal(), field_145850_b, func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p());
        }
    }

    public boolean shouldEmitRedstone(){
        // if(!worldObj.isRemote) System.out.println("redstone mode: " +
        // redstoneMode + ", charging: " + charging + ",discharging: " +
        // disCharging);

        switch(redstoneMode){
            case 0:
                return false;
            case 1:
                return !charging && !disCharging && inventory[CHARGE_INVENTORY_INDEX] != null && inventory[CHARGE_INVENTORY_INDEX].func_77973_b() instanceof IPressurizable;
            case 2:
                return charging;
            case 3:
                return disCharging;

        }
        return false;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox(){
        return new AxisAlignedBB(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), func_174877_v().func_177958_n() + 1, func_174877_v().func_177956_o() + 1, func_174877_v().func_177952_p() + 1);
    }

    public InventoryPneumaticInventoryItem getChargeableInventory(){
        return field_145850_b.field_72995_K ? new InventoryPneumaticInventoryItem(this) : chargeableInventory;
    }

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){

        return inventory.length + (chargeableInventory != null ? chargeableInventory.func_70302_i_() : 0);
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){
        if(slot < inventory.length) {
            return inventory[slot];
        } else {
            return chargeableInventory.func_70301_a(slot - inventory.length);
        }
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        if(!field_145850_b.field_72995_K && chargeableInventory != null && slot == CHARGE_INVENTORY_INDEX) {
            chargeableInventory.writeToNBT();
            chargeableInventory = null;
        }

        if(slot < inventory.length) {
            inventory[slot] = itemStack;
        } else {
            chargeableInventory.func_70299_a(slot - inventory.length, itemStack);
        }

        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
        if(slot == CHARGE_INVENTORY_INDEX && !field_145850_b.field_72995_K) {
            if(itemStack != null && itemStack.func_77973_b() instanceof IChargingStationGUIHolderItem) {
                chargeableInventory = new InventoryPneumaticInventoryItem(this);
            }
            List<EntityPlayer> players = field_145850_b.field_73010_i;
            for(EntityPlayer player : players) {
                if(player.field_71070_bA instanceof ContainerChargingStationItemInventory && ((ContainerChargingStationItemInventory)player.field_71070_bA).te == this) {
                    if(itemStack != null && itemStack.func_77973_b() instanceof IChargingStationGUIHolderItem) {
                        // player.openGui(PneumaticCraft.instance, CommonProxy.GUI_ID_PNEUMATIC_ARMOR, worldObj, getPos().getX(), getPos().getY(), getPos().getZ());
                    } else {
                        player.openGui(PneumaticCraft.instance, EnumGuiId.CHARGING_STATION.ordinal(), field_145850_b, func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p());
                    }
                }
            }
        }
    }

    @Override
    public String getName(){

        return Blockss.chargingStation.func_149739_a();
    }

    @Override
    public int func_70297_j_(){

        return 64;
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public void func_145839_a(NBTTagCompound tag){

        super.func_145839_a(tag);
        // Read in the ItemStacks in the inventory from NBT

        redstoneMode = tag.func_74762_e("redstoneMode");
        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[func_70302_i_()];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }
        ItemStack chargeSlot = inventory[CHARGE_INVENTORY_INDEX];
        if(chargeSlot != null && chargeSlot.func_77973_b() instanceof IChargingStationGUIHolderItem) {
            chargeableInventory = new InventoryPneumaticInventoryItem(this);
        }

        if(tag.func_74764_b("camoStack")) {
            camoStack = ItemStack.func_77949_a(tag.func_74775_l("camoStack"));
        } else {
            camoStack = null;
        }
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){

        super.func_145841_b(tag);

        if(chargeableInventory != null) chargeableInventory.writeToNBT();

        // Write the ItemStacks in the inventory to NBT
        tag.func_74768_a("redstoneMode", redstoneMode);
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Items", tagList);

        if(camoStack != null) {
            NBTTagCompound subTag = new NBTTagCompound();
            camoStack.func_77955_b(subTag);
            tag.func_74782_a("camoStack", subTag);
        }

    }

    @Override
    public boolean func_94041_b(int slot, ItemStack itemstack){
        if(itemstack == null) return true;
        if(slot < inventory.length) {
            if(slot == 0) {
                return itemstack.func_77973_b() instanceof IPressurizable;
            } else {
                return canInsertUpgrade(slot, itemstack);
            }
        } else {
            return chargeableInventory.func_94041_b(slot - inventory.length, itemstack);
        }
    }

    @Override
    // upgrades in bottom, fuel in the rest.
    public int[] func_180463_a(EnumFacing var1){
        if(chargeableInventory != null) {
            int[] slots = new int[chargeableInventory.func_70302_i_() + 1];
            slots[0] = 0;
            for(int i = 0; i < chargeableInventory.func_70302_i_(); i++) {
                slots[i + 1] = i + inventory.length;
            }
            return slots;
        } else {
            return new int[]{0};
        }
    }

    @Override
    public boolean func_180462_a(int i, ItemStack itemstack, EnumFacing j){
        return true;
    }

    @Override
    public boolean func_180461_b(int i, ItemStack itemstack, EnumFacing j){
        return true;
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public void func_70296_d(){
        super.func_70296_d();
        if(chargeableInventory != null) chargeableInventory.func_70296_d();
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }

    @Override
    protected boolean shouldRerenderChunkOnDescUpdate(){
        return true;
    }
}
