package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Stack;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.apache.commons.lang3.tuple.Pair;

import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.tileentity.IAirHandler;
import pneumaticCraft.api.tileentity.IAirListener;
import pneumaticCraft.common.block.BlockElevatorBase;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.config.Config;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.network.LazySynced;
import pneumaticCraft.common.network.PacketServerTickTime;
import pneumaticCraft.common.thirdparty.computercraft.LuaConstant;
import pneumaticCraft.common.thirdparty.computercraft.LuaMethod;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.Log;
import pneumaticCraft.lib.PneumaticValues;
import pneumaticCraft.lib.Sounds;
import pneumaticCraft.lib.TileEntityConstants;

public class TileEntityElevatorBase extends TileEntityPneumaticBase implements IInventory, IGUITextFieldSensitive,
        IRedstoneControlled, IMinWorkingPressure, IAirListener{
    @DescSynced
    public boolean[] sidesConnected = new boolean[6];
    public float oldExtension;
    @DescSynced
    @LazySynced
    public float extension;
    @DescSynced
    public float targetExtension;
    private int soundCounter;
    private boolean isStopped; //used for sounds
    private TileEntityElevatorBase coreElevator;
    private List<TileEntityElevatorBase> multiElevators;//initialized when multiple elevators are connected in a multiblock manner.
    @GuiSynced
    public int redstoneMode;
    public int[] floorHeights = new int[0];//list of every floor of Elevator Callers.
    private HashMap<Integer, String> floorNames = new HashMap<Integer, String>();
    @GuiSynced
    private int maxFloorHeight;
    private int redstoneInputLevel;//current redstone input level

    @DescSynced
    private ItemStack[] inventory = new ItemStack[6];
    public Block baseCamo, frameCamo;
    public static final int UPGRADE_SLOT_1 = 0;
    public static final int UPGRADE_SLOT_4 = 3;

    public TileEntityElevatorBase(){
        super(PneumaticValues.DANGER_PRESSURE_ELEVATOR, PneumaticValues.MAX_PRESSURE_ELEVATOR, PneumaticValues.VOLUME_ELEVATOR, UPGRADE_SLOT_1, 1, 2, UPGRADE_SLOT_4);
        addApplicableUpgrade(EnumUpgrade.SPEED);
    }

    @Override
    public void func_73660_a(){
        oldExtension = extension;
        if(field_145850_b.field_72995_K && field_145850_b.func_82737_E() % 60 == 0) coreElevator = null;//reset this because the client doesn't get notified of neighbor block updates.
        if(isCoreElevator()) {
            super.func_73660_a();
            if(!field_145850_b.field_72995_K && isControlledByRedstone()) {
                float oldTargetExtension = targetExtension;
                float maxExtension = getMaxElevatorHeight();

                int redstoneInput = redstoneInputLevel;
                if(multiElevators != null) {
                    for(TileEntityElevatorBase base : multiElevators) {
                        redstoneInput = Math.max(redstoneInputLevel, base.redstoneInputLevel);
                    }
                }

                targetExtension = redstoneInput * maxExtension / 15;
                if(targetExtension > oldExtension && getPressure() < PneumaticValues.MIN_PRESSURE_ELEVATOR) targetExtension = oldExtension; // only ascent when there's enough pressure
                if(oldTargetExtension != targetExtension) sendDescPacketFromAllElevators();
            }
            float speedMultiplier = getSpeedMultiplierFromUpgrades();
            if(field_145850_b.field_72995_K) {
                speedMultiplier = (float)(speedMultiplier * PacketServerTickTime.tickTimeMultiplier);
            }

            String soundName = null;
            if(extension < targetExtension) {
                if(!field_145850_b.field_72995_K && getPressure() < PneumaticValues.MIN_PRESSURE_ELEVATOR) {
                    targetExtension = extension;
                    sendDescPacket(256D);
                }
                soundName = Sounds.ELEVATOR_MOVING;

                float moveBy;

                if(extension < targetExtension - TileEntityConstants.ELEVATOR_SLOW_EXTENSION) {
                    moveBy = TileEntityConstants.ELEVATOR_SPEED_FAST * speedMultiplier;
                } else {
                    moveBy = TileEntityConstants.ELEVATOR_SPEED_SLOW * speedMultiplier;
                }
                if(extension + moveBy > targetExtension) {
                    extension = targetExtension;
                    if(!field_145850_b.field_72995_K) updateFloors();
                }
                if(isStopped) {
                    soundName = Sounds.ELEVATOR_START;
                    isStopped = false;
                }
                float startingExtension = extension;

                while(extension < startingExtension + moveBy) {
                    extension += TileEntityConstants.ELEVATOR_SPEED_SLOW;
                    /*
                    if(extension > startingExtension + moveBy) {
                        extension = startingExtension + moveBy;
                    }
                    */
                    // moveEntities(TileEntityConstants.ELEVATOR_SPEED_SLOW);
                }
                addAir((int)((oldExtension - extension) * PneumaticValues.USAGE_ELEVATOR * (getSpeedUsageMultiplierFromUpgrades() / speedMultiplier)));// substract the ascended distance from the air reservoir.
            }
            if(extension > targetExtension) {
                soundName = Sounds.ELEVATOR_MOVING;
                if(extension > targetExtension + TileEntityConstants.ELEVATOR_SLOW_EXTENSION) {
                    extension -= TileEntityConstants.ELEVATOR_SPEED_FAST * speedMultiplier;
                } else {
                    extension -= TileEntityConstants.ELEVATOR_SPEED_SLOW * speedMultiplier;
                }
                if(extension < targetExtension) {
                    extension = targetExtension;
                    if(!field_145850_b.field_72995_K) updateFloors();
                }
                if(isStopped) {
                    soundName = Sounds.ELEVATOR_START;
                    isStopped = false;
                }
                //  movePlayerDown();
            }
            if(oldExtension == extension && !isStopped) {
                soundName = Sounds.ELEVATOR_STOP;
                isStopped = true;
                soundCounter = 0;
            }

            if(soundCounter > 0) soundCounter--;
            if(soundName != null && field_145850_b.field_72995_K && soundCounter == 0) {
                field_145850_b.func_72980_b(func_174877_v().func_177958_n() + 0.5, func_174877_v().func_177956_o() + 0.5, func_174877_v().func_177952_p() + 0.5, soundName, 0.1F, 1.0F, true);
                soundCounter = 10;
            }

        } else {
            extension = 0;
        }
        if(!field_145850_b.field_72995_K && oldExtension != extension) {
            sendDescPacket(256);
        }
    }

    private void movePlayerDown(){
        if(!field_145850_b.field_72995_K) return;

        AxisAlignedBB aabb = new AxisAlignedBB(func_174877_v().func_177958_n(), func_174877_v().func_177956_o() + 1, func_174877_v().func_177952_p(), func_174877_v().func_177958_n() + 1, func_174877_v().func_177956_o() + oldExtension + 1.05F, func_174877_v().func_177952_p() + 1);
        List<Entity> entityList = field_145850_b.func_72839_b(null, aabb);
        for(Entity entity : entityList) {
            if(entity instanceof EntityPlayer) {
                //   moveEntityToCenter(entity);
                double posX = entity.field_70165_t;
                double posZ = entity.field_70161_v;
                if(posX >= func_174877_v().func_177958_n() && posX < func_174877_v().func_177958_n() + 1 && posZ >= func_174877_v().func_177952_p() && posZ < func_174877_v().func_177952_p() + 1) {
                    entity.field_70159_w *= 0.6;
                    entity.field_70179_y *= 0.6;
                    entity.func_70091_d(0, extension - oldExtension + 0.001F, 0);
                }
            }
        }
    }

    private void moveEntities(float moveBy){
        AxisAlignedBB aabb = new AxisAlignedBB(func_174877_v().func_177958_n(), func_174877_v().func_177956_o() + 1, func_174877_v().func_177952_p(), func_174877_v().func_177958_n() + 1, func_174877_v().func_177956_o() + extension + 1, func_174877_v().func_177952_p() + 1);
        List<Entity> entityList = field_145850_b.func_72839_b(null, aabb);
        for(Entity entity : entityList) {
            if(entity instanceof EntityPlayer) {

            } else entity.func_70091_d(0, moveBy + 0.05F, 0);
        }
    }

    private void moveEntityToCenter(Entity entity){
        /*
         * this is for your own protection. you may hurt yourself if you're not standing right, especially
         * on multiblock-elevators (entity will be found by multiple bases, causing problems)
         */
        ((EntityPlayer)entity).func_70107_b(func_174877_v().func_177958_n() + 0.5F, entity.field_70163_u, func_174877_v().func_177952_p() + 0.5F);
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID == 0) {
            redstoneMode++;
            if(redstoneMode > 1) redstoneMode = 0;

            if(multiElevators != null) {
                for(TileEntityElevatorBase base : multiElevators) {
                    while(base.redstoneMode != redstoneMode) {
                        base.handleGUIButtonPress(buttonID, player);
                    }
                }
            }

            int i = -1;
            TileEntity te = field_145850_b.func_175625_s(func_174877_v().func_177972_a(EnumFacing.DOWN));
            while(te instanceof TileEntityElevatorBase) {
                ((TileEntityElevatorBase)te).redstoneMode = redstoneMode;
                i--;
                te = field_145850_b.func_175625_s(func_174877_v().func_177982_a(0, i, 0));
            }
        }
    }

    private boolean isControlledByRedstone(){
        return redstoneMode == 0;
    }

    public void updateRedstoneInputLevel(){
        if(multiElevators == null) return;

        int maxRedstone = 0;
        for(TileEntityElevatorBase base : multiElevators) {
            int i = 0;
            while(field_145850_b.func_180495_p(base.func_174877_v().func_177982_a(0, i, 0)).func_177230_c() == Blockss.elevatorBase) {
                maxRedstone = Math.max(maxRedstone, PneumaticCraftUtils.getRedstoneLevel(field_145850_b, base.func_174877_v().func_177982_a(0, i, 0)));
                i--;
            }
        }
        for(TileEntityElevatorBase base : multiElevators) {
            base.redstoneInputLevel = maxRedstone;
        }
    }

    public float getMaxElevatorHeight(){
        int max = maxFloorHeight;
        if(multiElevators != null) {
            for(TileEntityElevatorBase base : multiElevators) {
                max = Math.max(max, base.maxFloorHeight);
            }
        }
        return max;
    }

    public void updateMaxElevatorHeight(){
        int i = -1;
        do {
            i++;
        } while(field_145850_b.func_180495_p(func_174877_v().func_177982_a(0, i + 1, 0)).func_177230_c() == Blockss.elevatorFrame);
        int elevatorBases = 0;
        do {
            elevatorBases++;
        } while(field_145850_b.func_180495_p(func_174877_v().func_177982_a(0, -elevatorBases, 0)).func_177230_c() == Blockss.elevatorBase);

        maxFloorHeight = Math.min(i, elevatorBases * Config.elevatorBaseBlocksPerBase);
    }

    // NBT methods-----------------------------------------------
    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        extension = tag.func_74760_g("extension");
        targetExtension = tag.func_74760_g("targetExtension");
        redstoneMode = tag.func_74762_e("redstoneMode");
        if(!tag.func_74764_b("maxFloorHeight")) {//backwards compatibility implementation.
            updateMaxElevatorHeight();
        } else {
            maxFloorHeight = tag.func_74762_e("maxFloorHeight");
        }
        for(int i = 0; i < 6; i++) {
            sidesConnected[i] = tag.func_74767_n("sideConnected" + i);
        }

        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[inventory.length];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        tag.func_74776_a("extension", extension);
        tag.func_74776_a("targetExtension", targetExtension);
        tag.func_74768_a("redstoneMode", redstoneMode);
        tag.func_74768_a("maxFloorHeight", maxFloorHeight);
        for(int i = 0; i < 6; i++) {
            tag.func_74757_a("sideConnected" + i, sidesConnected[i]);
        }

        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Items", tagList);

    }

    @Override
    public void readFromPacket(NBTTagCompound tag){
        super.readFromPacket(tag);
        floorHeights = tag.func_74759_k("floorHeights");

        floorNames.clear();
        NBTTagList floorNameList = tag.func_150295_c("floorNames", 10);
        for(int i = 0; i < floorNameList.func_74745_c(); i++) {
            NBTTagCompound floorName = floorNameList.func_150305_b(i);
            floorNames.put(floorName.func_74762_e("floorHeight"), floorName.func_74779_i("floorName"));
        }
    }

    @Override
    public void writeToPacket(NBTTagCompound tag){
        super.writeToPacket(tag);
        tag.func_74783_a("floorHeights", floorHeights);

        NBTTagList floorNameList = new NBTTagList();
        for(int key : floorNames.keySet()) {
            NBTTagCompound floorNameTag = new NBTTagCompound();
            floorNameTag.func_74768_a("floorHeight", key);
            floorNameTag.func_74778_a("floorName", floorNames.get(key));
            floorNameList.func_74742_a(floorNameTag);
        }
        tag.func_74782_a("floorNames", floorNameList);
    }

    @Override
    public void onNeighborTileUpdate(){
        super.onNeighborTileUpdate();
        updateConnections();
        connectAsMultiblock();
    }

    private void connectAsMultiblock(){
        multiElevators = null;
        if(isCoreElevator()) {
            multiElevators = new ArrayList<TileEntityElevatorBase>();
            Stack<TileEntityElevatorBase> todo = new Stack<TileEntityElevatorBase>();
            todo.add(this);
            while(!todo.isEmpty()) {
                TileEntityElevatorBase curElevator = todo.pop();
                if(curElevator.isCoreElevator()) {
                    multiElevators.add(curElevator);
                    curElevator.multiElevators = multiElevators;
                    for(int i = 2; i < 6; i++) {
                        TileEntity te = curElevator.getTileCache()[i].getTileEntity();
                        if(!multiElevators.contains(te) && te instanceof TileEntityElevatorBase) {
                            todo.push((TileEntityElevatorBase)te);
                        }
                    }
                }
            }
        }
    }

    @Override
    public void onNeighborBlockUpdate(){
        super.onNeighborBlockUpdate();
        getCoreElevator().updateRedstoneInputLevel();
    }

    public void updateConnections(){
        List<Pair<EnumFacing, IAirHandler>> connections = getAirHandler(null).getConnectedPneumatics();
        Arrays.fill(sidesConnected, false);
        for(Pair<EnumFacing, IAirHandler> entry : connections) {
            sidesConnected[entry.getKey().ordinal()] = true;
        }

        if(field_145850_b.func_180495_p(func_174877_v().func_177972_a(EnumFacing.UP)) != Blockss.elevatorBase) {
            coreElevator = this;
            int i = -1;
            TileEntity te = field_145850_b.func_175625_s(func_174877_v().func_177972_a(EnumFacing.DOWN));
            while(te instanceof TileEntityElevatorBase) {
                ((TileEntityElevatorBase)te).coreElevator = this;
                i--;
                te = field_145850_b.func_175625_s(func_174877_v().func_177982_a(0, i, 0));
            }
        }
    }

    public void moveInventoryToThis(){
        TileEntity te = field_145850_b.func_175625_s(func_174877_v().func_177972_a(EnumFacing.UP));
        if(te instanceof TileEntityElevatorBase) {
            for(int i = 0; i < func_70302_i_(); i++) {
                inventory[i] = ((TileEntityElevatorBase)te).inventory[i];
                ((TileEntityElevatorBase)te).inventory[i] = null;
            }
        }
    }

    public void updateFloors(){
        List<Integer> floorList = new ArrayList<Integer>();
        List<BlockPos> callerList = new ArrayList<BlockPos>();

        if(multiElevators != null) {
            int i = 0;
            boolean shouldBreak = false;
            while(!shouldBreak) {
                boolean registeredThisFloor = false;
                for(TileEntityElevatorBase base : multiElevators) {
                    for(EnumFacing dir : EnumFacing.field_176754_o) {
                        if(base.field_145850_b.func_180495_p(base.func_174877_v().func_177972_a(dir).func_177982_a(0, i + 2, 0)).func_177230_c() == Blockss.elevatorCaller) {
                            callerList.add(new BlockPos(base.func_174877_v().func_177958_n() + dir.func_82601_c(), base.func_174877_v().func_177956_o() + i + 2, base.func_174877_v().func_177952_p() + dir.func_82599_e()));
                            if(!registeredThisFloor) floorList.add(i);
                            registeredThisFloor = true;
                        }
                    }
                }

                i++;
                for(TileEntityElevatorBase base : multiElevators) {
                    if(base.field_145850_b.func_180495_p(base.func_174877_v().func_177982_a(0, i, 0)).func_177230_c() != Blockss.elevatorFrame) {
                        shouldBreak = true;
                        break;
                    }
                }
            }

            for(TileEntityElevatorBase base : multiElevators) {
                base.floorHeights = new int[floorList.size()];
                for(i = 0; i < base.floorHeights.length; i++) {
                    base.floorHeights[i] = floorList.get(i);
                }
            }
        }

        double buttonHeight = 0.06D;
        double buttonSpacing = 0.02D;
        TileEntityElevatorCaller.ElevatorButton[] elevatorButtons = new TileEntityElevatorCaller.ElevatorButton[floorHeights.length];
        int columns = (elevatorButtons.length - 1) / 12 + 1;
        for(int j = 0; j < columns; j++) {
            for(int i = j * 12; i < floorHeights.length && i < j * 12 + 12; i++) {
                elevatorButtons[i] = new TileEntityElevatorCaller.ElevatorButton(0.2D + 0.6D / columns * j, 0.5D + (Math.min(floorHeights.length, 12) - 2) * (buttonSpacing + buttonHeight) / 2 - i % 12 * (buttonHeight + buttonSpacing), 0.58D / columns, buttonHeight, i, floorHeights[i]);
                elevatorButtons[i].setColor(floorHeights[i] == targetExtension ? 0 : 1, 1, floorHeights[i] == targetExtension ? 0 : 1);
                String floorName = floorNames.get(floorHeights[i]);
                if(floorName != null) {
                    elevatorButtons[i].buttonText = floorName;
                } else {
                    floorNames.put(floorHeights[i], elevatorButtons[i].buttonText);
                }
            }
        }

        if(multiElevators != null) {
            for(TileEntityElevatorBase base : multiElevators) {
                base.floorNames = new HashMap<Integer, String>(floorNames);
            }
        }

        for(BlockPos p : callerList) {
            TileEntity te = field_145850_b.func_175625_s(p);
            if(te instanceof TileEntityElevatorCaller) {
                int callerFloorHeight = p.func_177956_o() - func_174877_v().func_177956_o() - 2;
                int callerFloor = -1;
                for(TileEntityElevatorCaller.ElevatorButton floor : elevatorButtons) {
                    if(floor.floorHeight == callerFloorHeight) {
                        callerFloor = floor.floorNumber;
                        break;
                    }
                }
                if(callerFloor == -1) {
                    Log.error("Error while updating elevator floors! This will cause a indexOutOfBoundsException, index = -1");
                }
                ((TileEntityElevatorCaller)te).setEmittingRedstone(PneumaticCraftUtils.areFloatsEqual(targetExtension, extension, 0.1F) && PneumaticCraftUtils.areFloatsEqual(extension, callerFloorHeight, 0.1F));
                ((TileEntityElevatorCaller)te).setFloors(elevatorButtons, callerFloor);
            }
        }
    }

    public void goToFloor(int floor){
        if(getCoreElevator().isControlledByRedstone()) getCoreElevator().handleGUIButtonPress(0, null);
        if(floor >= 0 && floor < floorHeights.length) setTargetHeight(floorHeights[floor]);
        updateFloors();
        sendDescPacketFromAllElevators();
    }

    private void setTargetHeight(float height){
        height = Math.min(height, getMaxElevatorHeight());
        if(multiElevators != null) {
            for(TileEntityElevatorBase base : multiElevators) {
                base.targetExtension = height;
            }
        }
    }

    @Override
    public void onDescUpdate(){
        baseCamo = inventory[4] != null && inventory[4].func_77973_b() instanceof ItemBlock ? ((ItemBlock)inventory[4].func_77973_b()).func_179223_d() : null;
        Block newFrameCamo = inventory[5] != null && inventory[5].func_77973_b() instanceof ItemBlock ? ((ItemBlock)inventory[5].func_77973_b()).func_179223_d() : null;

        if(newFrameCamo != frameCamo) {
            frameCamo = newFrameCamo;
            rerenderChunk();
        }
    }

    private void sendDescPacketFromAllElevators(){
        if(multiElevators != null) {
            for(TileEntityElevatorBase base : multiElevators) {
                base.sendDescPacket(256);
            }
        } else {
            sendDescPacket(256);
        }
    }

    // INVENTORY METHODS-
    // ------------------------------------------------------------

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){
        return getCoreElevator().inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){
        return getCoreElevator().inventory[slot];
    }

    public ItemStack getRealStackInSlot(int slot){
        return inventory[slot];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        // super.setInventorySlotContents(slot, itemStack);
        getCoreElevator().inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
    }

    @Override
    public int func_70297_j_(){

        return 64;
    }

    @Override
    public String getName(){
        return Blockss.elevatorBase.func_149739_a();
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public boolean func_94041_b(int i, ItemStack itemstack){
        return canInsertUpgrade(i, itemstack) || itemstack.func_77973_b() instanceof ItemBlock && i >= 4;
    }

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    @Override
    public AxisAlignedBB getRenderBoundingBox(){
        return new AxisAlignedBB(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), func_174877_v().func_177958_n() + 1, func_174877_v().func_177956_o() + 1 + extension, func_174877_v().func_177952_p() + 1);
    }

    @Override
    protected double getPacketDistance(){
        return 256;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public double func_145833_n(){
        return 65536D;
    }

    private TileEntityElevatorBase getCoreElevator(){
        if(coreElevator == null) {
            coreElevator = BlockElevatorBase.getCoreTileEntity(field_145850_b, func_174877_v());
        }
        return coreElevator;
    }

    public boolean isCoreElevator(){
        return getCoreElevator() == this;
    }

    @Override
    public boolean isConnectedTo(EnumFacing side){
        return side != EnumFacing.UP && side != EnumFacing.DOWN || field_145850_b.func_180495_p(func_174877_v().func_177972_a(side)).func_177230_c() != Blockss.elevatorBase;
    }

    @Override
    public IAirHandler getAirHandler(EnumFacing sideRequested){
        if(isCoreElevator()) {
            return super.getAirHandler(sideRequested);
        } else {
            return getCoreElevator().getAirHandler(sideRequested);
        }
    }

    @Override
    public void addConnectedPneumatics(List<Pair<EnumFacing, IAirHandler>> connectedMachines){
        TileEntity te = getTileCache()[EnumFacing.DOWN.ordinal()].getTileEntity();
        if(te instanceof TileEntityElevatorBase) {
            connectedMachines.addAll(((TileEntityElevatorBase)te).airHandler.getConnectedPneumatics());
        }
    }

    @Override
    public void onAirDispersion(IAirHandler handler, EnumFacing dir, int airAdded){}

    @Override
    public int getMaxDispersion(IAirHandler handler, EnumFacing dir){
        return Integer.MAX_VALUE;
    }

    @Override
    public void setText(int textFieldID, String text){
        setFloorName(textFieldID, text);
    }

    @Override
    public String getText(int textFieldID){
        return getFloorName(textFieldID);
    }

    public String getFloorName(int floor){
        return floor < floorHeights.length ? floorNames.get(floorHeights[floor]) : "";
    }

    public void setFloorName(int floor, String name){
        if(floor < floorHeights.length) {
            floorNames.put(floorHeights[floor], name);
            updateFloors();
        }
    }

    @Override
    public boolean isGuiUseableByPlayer(EntityPlayer par1EntityPlayer){
        return field_145850_b.func_175625_s(func_174877_v()) == this;
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    /*
     * COMPUTERCRAFT API
     */

    /* TODO CC dep @Override
     public String getType(){
         return "elevator";
     }*/

    @Override
    protected void addLuaMethods(){
        super.addLuaMethods();
        luaMethods.add(new LuaConstant("getMinWorkingPressure", PneumaticValues.MIN_PRESSURE_ELEVATOR));
        luaMethods.add(new LuaMethod("setHeight"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 1) {
                    setTargetHeight(((Double)args[0]).floatValue());
                    if(getCoreElevator().isControlledByRedstone()) getCoreElevator().handleGUIButtonPress(0, null);
                    getCoreElevator().sendDescPacketFromAllElevators();
                    return null;
                } else {
                    throw new IllegalArgumentException("setHeight does take one argument (height)");
                }
            }
        });

        luaMethods.add(new LuaMethod("setExternalControl"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 1) {
                    if((Boolean)args[0] && getCoreElevator().isControlledByRedstone() || !(Boolean)args[0] && !getCoreElevator().isControlledByRedstone()) {
                        getCoreElevator().handleGUIButtonPress(0, null);
                    }
                    return null;
                } else {
                    throw new IllegalArgumentException("setExternalControl does take one argument! (bool)");
                }
            }
        });
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }

    @Override
    public float getMinWorkingPressure(){
        return PneumaticValues.MIN_PRESSURE_ELEVATOR;
    }
}
