package pneumaticCraft.common.tileentity;

import net.minecraft.block.Block;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.AxisAlignedBB;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.common.block.BlockElevatorCaller;
import pneumaticCraft.common.network.DescSynced;

public class TileEntityElevatorCaller extends TileEntityBase{
    private ElevatorButton[] floors = new ElevatorButton[0];
    public int thisFloor;
    private boolean emittingRedstone;
    private boolean shouldUpdateNeighbors;
    @DescSynced
    public ItemStack camoStack;
    public Block camoBlock;

    public void setEmittingRedstone(boolean emittingRedstone){
        if(emittingRedstone != this.emittingRedstone) {
            this.emittingRedstone = emittingRedstone;
            shouldUpdateNeighbors = true;
        }
    }

    @Override
    public void func_73660_a(){
        super.func_73660_a();
        if(shouldUpdateNeighbors) {
            updateNeighbours();
            shouldUpdateNeighbors = false;
        }
    }

    public boolean getEmittingRedstone(){
        return emittingRedstone;
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        emittingRedstone = tag.func_74767_n("emittingRedstone");
        thisFloor = tag.func_74762_e("thisFloor");
        camoStack = tag.func_74764_b("camoStack") ? ItemStack.func_77949_a(tag.func_74775_l("camoStack")) : null;
        shouldUpdateNeighbors = tag.func_74767_n("shouldUpdateNeighbors");
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        tag.func_74757_a("emittingRedstone", emittingRedstone);
        tag.func_74768_a("thisFloor", thisFloor);
        if(camoStack != null) {
            NBTTagCompound camoTag = new NBTTagCompound();
            camoStack.func_77955_b(camoTag);
            tag.func_74782_a("camoStack", camoTag);
        }
        tag.func_74757_a("shouldUpdateNeighbors", shouldUpdateNeighbors);
    }

    @Override
    public void readFromPacket(NBTTagCompound tag){
        super.readFromPacket(tag);
        int floorAmount = tag.func_74762_e("floors");
        floors = new ElevatorButton[floorAmount];
        for(int i = 0; i < floorAmount; i++) {
            NBTTagCompound buttonTag = tag.func_74775_l("floor" + i);
            floors[i] = new ElevatorButton();
            floors[i].readFromNBT(buttonTag);
        }
    }

    @Override
    public void writeToPacket(NBTTagCompound tag){
        super.writeToPacket(tag);
        tag.func_74768_a("floors", floors.length);
        for(ElevatorButton floor : floors) {
            NBTTagCompound buttonTag = new NBTTagCompound();
            floor.writeToNBT(buttonTag);
            tag.func_74782_a("floor" + floor.floorNumber, buttonTag);
        }
    }

    @Override
    public void onDescUpdate(){
        Block newCamo = camoStack != null && camoStack.func_77973_b() instanceof ItemBlock ? ((ItemBlock)camoStack.func_77973_b()).func_179223_d() : null;

        if(newCamo != camoBlock) {
            camoBlock = newCamo;
            rerenderChunk();
        }
    }

    @Override
    public void onNeighborBlockUpdate(){
        boolean wasPowered = poweredRedstone > 0;
        super.onNeighborBlockUpdate();
        if(poweredRedstone > 0 && !wasPowered) {
            BlockElevatorCaller.setSurroundingElevators(field_145850_b, func_174877_v(), thisFloor);
        }
    }

    public void setFloors(ElevatorButton[] floors, int thisFloorLevel){
        this.floors = floors;
        thisFloor = thisFloorLevel;
        sendDescriptionPacket();
    }

    public ElevatorButton[] getFloors(){
        return floors;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox(){
        return new AxisAlignedBB(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), func_174877_v().func_177958_n() + 1, func_174877_v().func_177956_o() + 1, func_174877_v().func_177952_p() + 1);
    }

    public static class ElevatorButton{
        public double posX, posY, width, height;
        public float red, green, blue;
        public String buttonText = "";
        public int floorNumber;
        public int floorHeight;

        public ElevatorButton(double posX, double posY, double width, double height, int floorNumber, int floorHeight){
            this.posX = posX;
            this.posY = posY;
            this.width = width;
            this.height = height;
            this.floorNumber = floorNumber;
            this.floorHeight = floorHeight;
            buttonText = floorNumber + 1 + "";
        }

        public ElevatorButton(){}

        public void setColor(float red, float green, float blue){
            this.red = red;
            this.green = green;
            this.blue = blue;
        }

        public void writeToNBT(NBTTagCompound tag){
            tag.func_74780_a("posX", posX);
            tag.func_74780_a("posY", posY);
            tag.func_74780_a("width", width);
            tag.func_74780_a("height", height);
            tag.func_74778_a("buttonText", buttonText);
            tag.func_74768_a("floorNumber", floorNumber);
            tag.func_74768_a("floorHeight", floorHeight);
            tag.func_74776_a("red", red);
            tag.func_74776_a("green", green);
            tag.func_74776_a("blue", blue);
        }

        public void readFromNBT(NBTTagCompound tag){
            posX = tag.func_74769_h("posX");
            posY = tag.func_74769_h("posY");
            width = tag.func_74769_h("width");
            height = tag.func_74769_h("height");
            buttonText = tag.func_74779_i("buttonText");
            floorNumber = tag.func_74762_e("floorNumber");
            floorHeight = tag.func_74762_e("floorHeight");
            red = tag.func_74760_g("red");
            green = tag.func_74760_g("green");
            blue = tag.func_74760_g("blue");
        }

        /*
        @Override
        public boolean equals(Object button){
            if(!(button instanceof ElevatorButton)) return false;
            ElevatorButton b = (ElevatorButton)button;
            return posX == b.posX && posY == b.posY && width == b.width && height == b.height && buttonText.equals(b.buttonText) && floorNumber == b.floorNumber && floorHeight == b.floorHeight && red == b.red && green == ;
        }

        @Override
        public int hashCode(){
            return (int)(posX * 191 + posY * 281 + width * 342 + height * 425) + buttonText.hashCode() + floorNumber * 1041 + floorHeight * 1024;
        }*/

    }
}
