package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Stack;

import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidRegistry;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidTank;
import net.minecraftforge.fluids.FluidTankInfo;
import net.minecraftforge.fluids.IFluidHandler;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.apache.commons.lang3.tuple.Pair;

import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.tileentity.IAirHandler;
import pneumaticCraft.common.ai.ChunkPositionSorter;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.util.FluidUtils;
import pneumaticCraft.common.util.IOHelper;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.PneumaticValues;

public class TileEntityGasLift extends TileEntityPneumaticBase implements IMinWorkingPressure, IRedstoneControlled,
        IFluidHandler, IInventory{
    @GuiSynced
    private final FluidTank tank = new FluidTank(PneumaticValues.NORMAL_TANK_CAPACITY);
    private final ItemStack[] inventory = new ItemStack[5];
    @GuiSynced
    public int currentDepth;
    @GuiSynced
    public int redstoneMode, status, mode;
    @DescSynced
    public boolean[] sidesConnected = new boolean[6];
    private int workTimer;
    private int ticker;
    private List<BlockPos> pumpingLake;
    private static final int MAX_PUMP_RANGE = 15;

    public TileEntityGasLift(){
        super(5, 7, 3000, 0, 1, 2, 3);
        addApplicableUpgrade(EnumUpgrade.SPEED);
    }

    @Override
    public boolean isConnectedTo(EnumFacing d){
        return d != EnumFacing.DOWN;
    }

    @Override
    public void onNeighborTileUpdate(){
        super.onNeighborTileUpdate();
        List<Pair<EnumFacing, IAirHandler>> connections = getAirHandler(null).getConnectedPneumatics();
        Arrays.fill(sidesConnected, false);
        for(Pair<EnumFacing, IAirHandler> entry : connections) {
            sidesConnected[entry.getKey().ordinal()] = true;
        }
    }

    @Override
    public void func_73660_a(){
        super.func_73660_a();
        if(!field_145850_b.field_72995_K) {
            ticker++;
            if(currentDepth > 0) {
                int curCheckingPipe = ticker % currentDepth;
                if(curCheckingPipe > 0 && !isPipe(func_174877_v().func_177982_a(0, -curCheckingPipe, 0))) currentDepth = curCheckingPipe - 1;
            }
            if(ticker % 400 == 0) pumpingLake = null;

            if(redstoneAllows() && getPressure() >= getMinWorkingPressure()) {
                workTimer += this.getSpeedMultiplierFromUpgrades();
                while(workTimer > 20) {
                    workTimer -= 20;
                    status = 0;
                    if(mode == 2) {
                        if(currentDepth > 0) {
                            status = 3;
                            if(isPipe(func_174877_v().func_177982_a(0, -currentDepth, 0))) {
                                if(IOHelper.insert((IInventory)this, new ItemStack(Blockss.pressureTube), null, false) == null) {
                                    field_145850_b.func_175655_b(func_174877_v().func_177982_a(0, -currentDepth, 0), false);
                                    addAir(-100);
                                    currentDepth--;
                                } else {
                                    status = 0;
                                }
                            } else {
                                currentDepth--;
                            }
                        }
                    } else {
                        if(!suckLiquid()) {
                            if(func_174877_v().func_177956_o() - currentDepth >= 0 && !isUnbreakable(func_174877_v().func_177982_a(0, -currentDepth - 1, 0))) {
                                status = 2;
                                currentDepth++;
                                if(!isPipe(func_174877_v().func_177982_a(0, -currentDepth, 0))) {
                                    if(inventory[4] != null) {
                                        func_70298_a(4, 1);
                                        field_145850_b.func_175655_b(func_174877_v().func_177982_a(0, -currentDepth, 0), false);
                                        field_145850_b.func_175656_a(func_174877_v().func_177982_a(0, -currentDepth, 0), Blockss.pressureTube.func_176223_P());
                                        addAir(-100);
                                    } else {
                                        status = 0;
                                        currentDepth--;
                                        break;
                                    }
                                }
                            } else {
                                break;
                            }
                        }
                    }
                }
            } else {
                status = 0;
            }
            if(getUpgrades(EnumUpgrade.DISPENSER) > 0) {
                autoExportLiquid();
            }
        }
    }

    private boolean isPipe(BlockPos pos){
        return field_145850_b.func_180495_p(pos).func_177230_c() == Blockss.pressureTube;
    }

    private boolean isUnbreakable(BlockPos pos){
        return field_145850_b.func_180495_p(pos).func_177230_c().func_176195_g(field_145850_b, pos) < 0;
    }

    public boolean suckLiquid(){
        Block block = field_145850_b.func_180495_p(func_174877_v().func_177982_a(0, -currentDepth - 1, 0)).func_177230_c();
        Fluid fluid = FluidRegistry.lookupFluidForBlock(block);
        if(fluid != null) {
            if(fill(null, new FluidStack(fluid, 1000), false) == 1000) {
                if(pumpingLake == null) {
                    pumpingLake = new ArrayList<BlockPos>();
                    Stack<BlockPos> pendingPositions = new Stack<BlockPos>();
                    BlockPos thisPos = new BlockPos(func_174877_v().func_177958_n(), func_174877_v().func_177956_o() - currentDepth - 1, func_174877_v().func_177952_p());
                    pendingPositions.add(thisPos);
                    pumpingLake.add(thisPos);
                    while(!pendingPositions.empty()) {
                        BlockPos checkingPos = pendingPositions.pop();
                        for(EnumFacing d : EnumFacing.field_82609_l) {
                            if(d == EnumFacing.DOWN) continue;
                            BlockPos newPos = new BlockPos(checkingPos.func_177958_n() + d.func_82601_c(), checkingPos.func_177956_o() + d.func_96559_d(), checkingPos.func_177952_p() + d.func_82599_e());
                            if(PneumaticCraftUtils.distBetween(newPos, func_174877_v().func_177958_n() + 0.5, func_174877_v().func_177956_o() - currentDepth - 1, func_174877_v().func_177952_p() + 0.5) <= MAX_PUMP_RANGE && field_145850_b.func_180495_p(newPos).func_177230_c() == block && !pumpingLake.contains(newPos)) {
                                pendingPositions.add(newPos);
                                pumpingLake.add(newPos);
                            }
                        }
                    }
                    Collections.sort(pumpingLake, new ChunkPositionSorter(func_174877_v().func_177958_n() + 0.5, func_174877_v().func_177956_o() - currentDepth - 1, func_174877_v().func_177952_p() + 0.5));
                    Collections.reverse(pumpingLake);
                }
                BlockPos curPos = null;
                boolean foundSource = false;
                while(pumpingLake.size() > 0) {
                    curPos = pumpingLake.get(0);
                    if(field_145850_b.func_180495_p(curPos).func_177230_c() == block && FluidUtils.isSourceBlock(field_145850_b, curPos)) {
                        foundSource = true;
                        break;
                    }
                    pumpingLake.remove(0);
                }
                if(pumpingLake.size() == 0) {
                    pumpingLake = null;
                } else if(foundSource) {
                    field_145850_b.func_175698_g(curPos);
                    fill(null, new FluidStack(fluid, 1000), true);
                    addAir(-100);
                    status = 1;
                }
            }
            return true;
        } else {
            pumpingLake = null;
            return false;
        }
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID == 0) {
            redstoneMode++;
            if(redstoneMode > 2) redstoneMode = 0;
        } else if(buttonID > 0 && buttonID < 4) {
            mode = buttonID - 1;
        }
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }

    @Override
    public float getMinWorkingPressure(){
        return 0.5F + currentDepth * 0.05F;
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        writeInventoryToNBT(tag, inventory);
        tag.func_74774_a("redstoneMode", (byte)redstoneMode);
        tag.func_74774_a("mode", (byte)mode);

        NBTTagCompound tankTag = new NBTTagCompound();
        tank.writeToNBT(tankTag);
        tag.func_74782_a("tank", tankTag);
        tag.func_74768_a("currentDepth", currentDepth);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        readInventoryFromNBT(tag, inventory);
        redstoneMode = tag.func_74771_c("redstoneMode");
        mode = tag.func_74771_c("mode");
        tank.readFromNBT(tag.func_74775_l("tank"));
        currentDepth = tag.func_74762_e("currentDepth");
    }

    @Override
    public int fill(EnumFacing from, FluidStack resource, boolean doFill){
        return tank.fill(resource, doFill);
    }

    @Override
    public FluidStack drain(EnumFacing from, FluidStack resource, boolean doDrain){
        return tank.getFluid() != null && tank.getFluid().isFluidEqual(resource) ? drain(null, resource.amount, doDrain) : null;
    }

    @Override
    public FluidStack drain(EnumFacing from, int maxDrain, boolean doDrain){
        return mode == 0 ? tank.drain(maxDrain, doDrain) : tank.drain(Math.min(tank.getFluidAmount() - 1000, maxDrain), doDrain);
    }

    @Override
    public boolean canFill(EnumFacing from, Fluid fluid){
        return false;
    }

    @Override
    public boolean canDrain(EnumFacing from, Fluid fluid){
        return true;
    }

    @Override
    public FluidTankInfo[] getTankInfo(EnumFacing from){
        return new FluidTankInfo[]{new FluidTankInfo(tank)};
    }

    @SideOnly(Side.CLIENT)
    public FluidTank getTank(){
        return tank;
    }

    /**
     * Returns the name of the inventory.
     */
    @Override
    public String getName(){
        return Blockss.gasLift.func_149739_a();
    }

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){
        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int par1){
        return inventory[par1];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }
        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
    }

    @Override
    public boolean func_94041_b(int par1, ItemStack stack){
        return stack != null && (canInsertUpgrade(par1, stack) || par1 == 4 && stack.func_77969_a(new ItemStack(Blockss.pressureTube)));
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public int func_70297_j_(){
        return 64;
    }

    @Override
    public boolean func_70300_a(EntityPlayer p_70300_1_){
        return isGuiUseableByPlayer(p_70300_1_);
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

}
