package pneumaticCraft.common.tileentity;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.util.Vec3;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidContainerRegistry;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidTank;
import net.minecraftforge.fluids.FluidTankInfo;
import net.minecraftforge.fluids.IFluidContainerItem;
import net.minecraftforge.fluids.IFluidHandler;
import net.minecraftforge.fluids.IFluidTank;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.fluid.Fluids;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.util.PneumaticCraftUtils;

public class TileEntityKeroseneLamp extends TileEntityBase implements IFluidHandler, IRedstoneControlled,
        ISidedInventory{
    private final Set<BlockPos> managingLights = new HashSet<BlockPos>();
    @DescSynced
    private boolean isOn;
    @GuiSynced
    private int range;
    @GuiSynced
    private int targetRange = 10;
    @GuiSynced
    private int redstoneMode;
    @GuiSynced
    private int fuel;
    private static final int LIGHT_SPACING = 3;
    public static final int FUEL_PER_MB = 10000;
    private int checkingX, checkingY, checkingZ;
    @DescSynced
    private EnumFacing sideConnected = EnumFacing.DOWN;

    @DescSynced
    private final FluidTank tank = new FluidTank(1000);

    private final ItemStack[] inventory = new ItemStack[2];

    @Override
    public void func_73660_a(){
        super.func_73660_a();
        if(!field_145850_b.field_72995_K) {
            processFluidItem(0, 1);
            if(field_145850_b.func_82737_E() % 5 == 0) {
                int realTargetRange = redstoneAllows() ? targetRange : 0;
                if(redstoneMode == 3) realTargetRange = (int)(poweredRedstone / 15D * targetRange);
                updateRange(Math.min(realTargetRange, tank.getFluidAmount())); //Fade out the lamp when almost empty.
                updateLights();
                useFuel();
            }
        } else {
            if(isOn && field_145850_b.func_82737_E() % 5 == 0) {
                field_145850_b.func_175688_a(EnumParticleTypes.FLAME, func_174877_v().func_177958_n() + 0.4 + 0.2 * field_145850_b.field_73012_v.nextDouble(), func_174877_v().func_177956_o() + 0.2 + tank.getFluidAmount() / 1000D * 3 / 16D, func_174877_v().func_177952_p() + 0.4 + 0.2 * field_145850_b.field_73012_v.nextDouble(), 0, 0, 0);
            }
        }
    }

    private void useFuel(){
        fuel -= Math.pow(range, 3);
        if(fuel < 0 && tank.drain(1, true) != null) {
            fuel += FUEL_PER_MB;
        }
        if(fuel < 0) fuel = 0;
    }

    @Override
    public void func_145829_t(){
        super.func_145829_t();
        checkingX = func_174877_v().func_177958_n();
        checkingY = func_174877_v().func_177956_o();
        checkingZ = func_174877_v().func_177952_p();
    }

    @Override
    public void func_145843_s(){
        super.func_145843_s();
        for(BlockPos pos : managingLights) {
            if(isLampLight(pos)) {
                field_145850_b.func_175698_g(pos);
            }
        }
    }

    private boolean isLampLight(BlockPos pos){
        return field_145850_b.func_180495_p(pos).func_177230_c() == Blockss.keroseneLampLight;
    }

    private void updateLights(){
        int roundedRange = range / LIGHT_SPACING * LIGHT_SPACING;
        checkingX += LIGHT_SPACING;
        if(checkingX > func_174877_v().func_177958_n() + roundedRange) {
            checkingX = func_174877_v().func_177958_n() - roundedRange;
            checkingY += LIGHT_SPACING;
            if(checkingY > func_174877_v().func_177956_o() + roundedRange) {
                checkingY = func_174877_v().func_177956_o() - roundedRange;
                checkingZ += LIGHT_SPACING;
                if(checkingZ > func_174877_v().func_177952_p() + roundedRange) checkingZ = func_174877_v().func_177952_p() - roundedRange;
            }
        }
        BlockPos pos = new BlockPos(checkingX, checkingY, checkingZ);
        BlockPos lampPos = new BlockPos(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p());
        if(managingLights.contains(pos)) {
            if(isLampLight(pos)) {
                if(!passesRaytraceTest(pos, lampPos)) {
                    field_145850_b.func_175698_g(pos);
                    managingLights.remove(pos);
                }
            } else {
                managingLights.remove(pos);
            }
        } else {
            tryAddLight(pos, lampPos);
        }
    }

    private void updateRange(int targetRange){
        if(targetRange > range) {
            range++;
            BlockPos lampPos = new BlockPos(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p());
            int roundedRange = range / LIGHT_SPACING * LIGHT_SPACING;
            for(int x = -roundedRange; x <= roundedRange; x += LIGHT_SPACING) {
                for(int y = -roundedRange; y <= roundedRange; y += LIGHT_SPACING) {
                    for(int z = -roundedRange; z <= roundedRange; z += LIGHT_SPACING) {
                        BlockPos pos = new BlockPos(x + func_174877_v().func_177958_n(), y + func_174877_v().func_177956_o(), z + func_174877_v().func_177952_p());
                        if(!managingLights.contains(pos)) {
                            tryAddLight(pos, lampPos);
                        }
                    }
                }
            }
        } else if(targetRange < range) {
            range--;
            Iterator<BlockPos> iterator = managingLights.iterator();
            BlockPos lampPos = new BlockPos(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p());
            while(iterator.hasNext()) {
                BlockPos pos = iterator.next();
                if(!isLampLight(pos)) {
                    iterator.remove();
                } else if(PneumaticCraftUtils.distBetween(pos, lampPos) > range) {
                    field_145850_b.func_175698_g(pos);
                    iterator.remove();
                }
            }
        }
        isOn = range > 0;
    }

    private boolean passesRaytraceTest(BlockPos pos, BlockPos lampPos){
        MovingObjectPosition mop = field_145850_b.func_72933_a(new Vec3(pos.func_177958_n() + 0.5, pos.func_177956_o() + 0.5, pos.func_177952_p() + 0.5), new Vec3(lampPos.func_177958_n() + 0.5, lampPos.func_177956_o() + 0.5, lampPos.func_177952_p() + 0.5));
        return mop != null && lampPos.equals(mop.func_178782_a());
    }

    private boolean tryAddLight(BlockPos pos, BlockPos lampPos){
        if(PneumaticCraftUtils.distBetween(pos, lampPos) <= range) {
            if(field_145850_b.func_175623_d(pos) && !isLampLight(pos)) {
                if(passesRaytraceTest(pos, lampPos)) {
                    field_145850_b.func_175656_a(pos, Blockss.keroseneLampLight.func_176223_P());
                    managingLights.add(pos);
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public void onNeighborBlockUpdate(){
        super.onNeighborBlockUpdate();
        sideConnected = EnumFacing.DOWN;
        for(EnumFacing d : EnumFacing.field_82609_l) {
            BlockPos neighborPos = func_174877_v().func_177972_a(d);
            Block block = field_145850_b.func_180495_p(neighborPos).func_177230_c();
            if(block.isSideSolid(field_145850_b, neighborPos, d.func_176734_d())) {
                sideConnected = d;
                break;
            }
        }
    }

    @Override
    public int fill(EnumFacing from, FluidStack resource, boolean doFill){
        return canFill(from, resource.getFluid()) ? tank.fill(resource, doFill) : 0;
    }

    @Override
    public FluidStack drain(EnumFacing from, FluidStack resource, boolean doDrain){
        return tank.getFluid() != null && tank.getFluid().isFluidEqual(resource) ? drain(null, resource.amount, doDrain) : null;
    }

    @Override
    public FluidStack drain(EnumFacing from, int maxDrain, boolean doDrain){
        return tank.drain(maxDrain, doDrain);
    }

    @Override
    public boolean canFill(EnumFacing from, Fluid fluid){
        return Fluids.areFluidsEqual(fluid, Fluids.kerosene);
    }

    @Override
    public boolean canDrain(EnumFacing from, Fluid fluid){
        return true;
    }

    @Override
    public FluidTankInfo[] getTankInfo(EnumFacing from){
        return new FluidTankInfo[]{new FluidTankInfo(tank)};
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        NBTTagList lights = new NBTTagList();
        for(BlockPos pos : managingLights) {
            NBTTagCompound t = new NBTTagCompound();
            t.func_74768_a("x", pos.func_177958_n());
            t.func_74768_a("y", pos.func_177956_o());
            t.func_74768_a("z", pos.func_177952_p());
            lights.func_74742_a(t);
        }
        tag.func_74782_a("lights", lights);

        NBTTagCompound tankTag = new NBTTagCompound();
        tank.writeToNBT(tankTag);
        tag.func_74782_a("tank", tankTag);
        tag.func_74774_a("redstoneMode", (byte)redstoneMode);
        tag.func_74774_a("targetRange", (byte)targetRange);
        tag.func_74774_a("range", (byte)range);
        tag.func_74774_a("sideConnected", (byte)sideConnected.ordinal());
        writeInventoryToNBT(tag, inventory);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        managingLights.clear();
        NBTTagList lights = tag.func_150295_c("lights", 10);
        for(int i = 0; i < lights.func_74745_c(); i++) {
            NBTTagCompound t = lights.func_150305_b(i);
            managingLights.add(new BlockPos(t.func_74762_e("x"), t.func_74762_e("y"), t.func_74762_e("z")));
        }
        tank.readFromNBT(tag.func_74775_l("tank"));
        redstoneMode = tag.func_74771_c("redstoneMode");
        targetRange = tag.func_74771_c("targetRange");
        range = tag.func_74771_c("range");
        sideConnected = EnumFacing.func_82600_a(tag.func_74771_c("sideConnected"));
        readInventoryFromNBT(tag, inventory);
    }

    @Override
    public boolean redstoneAllows(){
        if(redstoneMode == 3) return true;
        return super.redstoneAllows();
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID == 0) {
            redstoneMode++;
            if(redstoneMode > 3) redstoneMode = 0;
        } else if(buttonID > 0 && buttonID <= 30) {
            targetRange = buttonID;
        }
    }

    @SideOnly(Side.CLIENT)
    public IFluidTank getTank(){
        return tank;
    }

    public int getRange(){
        return range;
    }

    public int getTargetRange(){
        return targetRange;
    }

    public int getFuel(){
        return fuel;
    }

    public EnumFacing getSideConnected(){
        return sideConnected;
    }

    /*
     * ---------------IInventory---------------------
     */

    /**
     * Returns the name of the inventory.
     */
    @Override
    public String getName(){
        return Blockss.keroseneLamp.func_149739_a();
    }

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){
        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int par1){
        return inventory[par1];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }
        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
    }

    @Override
    public boolean func_94041_b(int slot, ItemStack stack){
        return slot == 1 ? false : stack != null && (FluidContainerRegistry.getFluidForFilledItem(stack) != null || stack.func_77973_b() instanceof IFluidContainerItem && ((IFluidContainerItem)stack.func_77973_b()).getFluid(stack) != null);
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public int func_70297_j_(){
        return 64;
    }

    @Override
    public boolean func_70300_a(EntityPlayer p_70300_1_){
        return isGuiUseableByPlayer(p_70300_1_);
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public int[] func_180463_a(EnumFacing p_94128_1_){
        return new int[]{0, 1};
    }

    @Override
    public boolean func_180462_a(int slot, ItemStack stack, EnumFacing side){
        return func_94041_b(slot, stack);
    }

    @Override
    public boolean func_180461_b(int slot, ItemStack stack, EnumFacing side){
        return slot == 1;
    }

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }
}
