package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidRegistry;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidTank;
import net.minecraftforge.fluids.FluidTankInfo;
import net.minecraftforge.fluids.IFluidHandler;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.util.FluidUtils;
import pneumaticCraft.common.util.IOHelper;
import pneumaticCraft.lib.PneumaticValues;

public class TileEntityLiquidHopper extends TileEntityOmnidirectionalHopper implements IFluidHandler{

    @DescSynced
    private final FluidTank tank = new FluidTank(PneumaticValues.NORMAL_TANK_CAPACITY);

    public TileEntityLiquidHopper(){
        super(0, 1, 2, 3);
        addApplicableUpgrade(EnumUpgrade.DISPENSER);
    }

    @Override
    protected int getInvSize(){
        return 4;
    }

    @Override
    public String getName(){
        return Blockss.liquidHopper.func_149739_a();
    }

    @Override
    public int[] func_180463_a(EnumFacing var1){
        return new int[]{0, 1, 2, 3};
    }

    @Override
    public boolean func_94041_b(int slot, ItemStack stack){
        return canInsertUpgrade(slot, stack);
    }

    @Override
    protected boolean exportItem(int maxItems){
        EnumFacing dir = getRotation();
        if(tank.getFluid() != null) {
            TileEntity neighbor = IOHelper.getNeighbor(this, dir);
            if(neighbor instanceof IFluidHandler) {
                IFluidHandler fluidHandler = (IFluidHandler)neighbor;
                if(fluidHandler.canFill(dir.func_176734_d(), tank.getFluid().getFluid())) {
                    FluidStack fluid = tank.getFluid().copy();
                    fluid.amount = Math.min(maxItems * 100, tank.getFluid().amount - (leaveMaterial ? 1000 : 0));
                    if(fluid.amount > 0) {
                        tank.getFluid().amount -= fluidHandler.fill(dir.func_176734_d(), fluid, true);
                        if(tank.getFluidAmount() <= 0) tank.setFluid(null);
                        return true;
                    }
                }
            }
        }

        if(field_145850_b.func_175623_d(func_174877_v().func_177972_a(dir))) {
            for(EntityItem entity : getNeighborItems(this, dir)) {
                if(!entity.field_70128_L) {
                    List<ItemStack> returnedItems = new ArrayList<ItemStack>();
                    if(FluidUtils.tryExtractingLiquid(this, entity.func_92059_d(), returnedItems)) {
                        if(entity.func_92059_d().field_77994_a <= 0) entity.func_70106_y();
                        for(ItemStack stack : returnedItems) {
                            EntityItem item = new EntityItem(field_145850_b, entity.field_70165_t, entity.field_70163_u, entity.field_70161_v, stack);
                            item.field_70159_w = entity.field_70159_w;
                            item.field_70181_x = entity.field_70181_x;
                            item.field_70179_y = entity.field_70179_y;
                            field_145850_b.func_72838_d(item);
                        }
                        return true;
                    }
                }
            }
        }

        if(getUpgrades(EnumUpgrade.DISPENSER) > 0) {
            if(field_145850_b.func_175623_d(func_174877_v().func_177972_a(dir))) {
                FluidStack extractedFluid = drain(null, 1000, false);
                if(extractedFluid != null && extractedFluid.amount == 1000) {
                    Block fluidBlock = extractedFluid.getFluid().getBlock();
                    if(fluidBlock != null) {
                        drain(null, 1000, true);
                        field_145850_b.func_175656_a(func_174877_v().func_177972_a(dir), fluidBlock.func_176223_P());
                    }
                }
            }
        }

        return false;
    }

    @Override
    protected boolean suckInItem(int maxItems){
        TileEntity inputInv = IOHelper.getNeighbor(this, inputDir);
        if(inputInv instanceof IFluidHandler) {
            IFluidHandler fluidHandler = (IFluidHandler)inputInv;

            FluidStack fluid = fluidHandler.drain(inputDir.func_176734_d(), maxItems * 100, false);
            if(fluid != null && fluidHandler.canDrain(inputDir.func_176734_d(), fluid.getFluid())) {
                int filledFluid = fill(inputDir, fluid, true);
                if(filledFluid > 0) {
                    fluidHandler.drain(inputDir.func_176734_d(), filledFluid, true);
                    return true;
                }
            }
        }

        if(field_145850_b.func_175623_d(func_174877_v().func_177972_a(inputDir))) {
            for(EntityItem entity : getNeighborItems(this, inputDir)) {
                if(!entity.field_70128_L) {
                    List<ItemStack> returnedItems = new ArrayList<ItemStack>();
                    if(FluidUtils.tryInsertingLiquid(this, entity.func_92059_d(), false, returnedItems)) {
                        if(entity.func_92059_d().field_77994_a <= 0) entity.func_70106_y();
                        for(ItemStack stack : returnedItems) {
                            EntityItem item = new EntityItem(field_145850_b, entity.field_70165_t, entity.field_70163_u, entity.field_70161_v, stack);
                            item.field_70159_w = entity.field_70159_w;
                            item.field_70181_x = entity.field_70181_x;
                            item.field_70179_y = entity.field_70179_y;
                            field_145850_b.func_72838_d(item);
                        }
                        return true;
                    }
                }
            }
        }

        if(getUpgrades(EnumUpgrade.DISPENSER) > 0) {
            BlockPos neighborPos = func_174877_v().func_177972_a(inputDir);
            Fluid fluid = FluidRegistry.lookupFluidForBlock(field_145850_b.func_180495_p(neighborPos).func_177230_c());
            if(fluid != null && FluidUtils.isSourceBlock(field_145850_b, neighborPos)) {
                if(fill(null, new FluidStack(fluid, 1000), false) == 1000) {
                    fill(null, new FluidStack(fluid, 1000), true);
                    field_145850_b.func_175698_g(neighborPos);
                    return true;
                }
            }
        }

        return false;
    }

    @Override
    public int fill(EnumFacing from, FluidStack resource, boolean doFill){
        return tank.fill(resource, doFill);
    }

    @Override
    public FluidStack drain(EnumFacing from, FluidStack resource, boolean doDrain){
        return tank.getFluid() != null && tank.getFluid().isFluidEqual(resource) ? drain(null, resource.amount, doDrain) : null;
    }

    @Override
    public FluidStack drain(EnumFacing from, int maxDrain, boolean doDrain){
        return tank.drain(leaveMaterial ? Math.min(maxDrain, tank.getFluidAmount() - 1000) : maxDrain, doDrain);
    }

    @Override
    public boolean canFill(EnumFacing from, Fluid fluid){
        return true;
    }

    @Override
    public boolean canDrain(EnumFacing from, Fluid fluid){
        return true;
    }

    @Override
    public FluidTankInfo[] getTankInfo(EnumFacing from){
        return new FluidTankInfo[]{new FluidTankInfo(tank)};
    }

    @SideOnly(Side.CLIENT)
    public FluidTank getTank(){
        return tank;
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);

        NBTTagCompound tankTag = new NBTTagCompound();
        tank.writeToNBT(tankTag);
        tag.func_74782_a("tank", tankTag);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        tank.readFromNBT(tag.func_74775_l("tank"));
    }
}
