package pneumaticCraft.common.tileentity;

import java.util.Arrays;
import java.util.List;

import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.EnumFacing;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.util.IOHelper;

public class TileEntityOmnidirectionalHopper extends TileEntityBase implements ISidedInventory, IRedstoneControlled{
    @DescSynced
    protected EnumFacing inputDir = EnumFacing.UP;
    @DescSynced
    protected EnumFacing outputDir = EnumFacing.UP;
    private ItemStack[] inventory = new ItemStack[getInvSize()];
    @GuiSynced
    public int redstoneMode;
    private int cooldown;
    @GuiSynced
    protected boolean leaveMaterial;//leave items/liquids (used as filter)

    public TileEntityOmnidirectionalHopper(){
        this(5, 6, 7, 8);
    }

    public TileEntityOmnidirectionalHopper(int... upgradeSlots){
        super(upgradeSlots);
        addApplicableUpgrade(EnumUpgrade.SPEED);
    }

    protected int getInvSize(){
        return 9;
    }

    @Override
    protected boolean shouldRerenderChunkOnDescUpdate(){
        return true;
    }

    @Override
    public void func_73660_a(){
        super.func_73660_a();
        if(!func_145831_w().field_72995_K && --cooldown <= 0 && redstoneAllows()) {
            int maxItems = getMaxItems();
            boolean success = suckInItem(maxItems);
            success |= exportItem(maxItems);

            if(!success) {
                cooldown = 8;//When insertion failed, do a long cooldown as penalty for performance.
            } else {
                cooldown = getItemTransferInterval();
            }
        }
    }

    protected boolean exportItem(int maxItems){
        EnumFacing dir = getRotation();
        TileEntity neighbor = IOHelper.getNeighbor(this, dir);
        for(int i = 0; i < 5; i++) {
            ItemStack stack = inventory[i];
            if(stack != null && (!leaveMaterial || stack.field_77994_a > 1)) {
                ItemStack exportedStack = stack.func_77946_l();
                if(leaveMaterial) exportedStack.field_77994_a--;
                if(exportedStack.field_77994_a > maxItems) exportedStack.field_77994_a = maxItems;
                int count = exportedStack.field_77994_a;

                ItemStack remainder = IOHelper.insert(neighbor, exportedStack, dir.func_176734_d(), false);
                int exportedItems = count - (remainder == null ? 0 : remainder.field_77994_a);

                stack.field_77994_a -= exportedItems;
                if(stack.field_77994_a <= 0) {
                    func_70299_a(i, null);
                }
                maxItems -= exportedItems;
                if(maxItems <= 0) return true;
            }
        }
        return false;
    }

    protected boolean suckInItem(int maxItems){
        TileEntity inputInv = IOHelper.getNeighbor(this, inputDir);
        boolean success = false;

        //Suck from input inventory.
        for(int i = 0; i < maxItems; i++) {
            if(hasEmptySlot()) {
                ItemStack extracted = IOHelper.extractOneItem(inputInv, inputDir.func_176734_d(), true);//simulate extraction from the neighbor.
                if(extracted != null) {
                    ItemStack inserted = IOHelper.insert((IInventory)this, extracted, null, false);//if we can insert the item in this hopper.
                    if(inserted == null) {
                        IOHelper.extractOneItem(inputInv, inputDir.func_176734_d(), false); //actually retrieve it from the neighbor.
                        success = true;
                    } else {
                        break;
                    }
                } else {
                    break;
                }
            } else {
                for(int slot = 0; slot < 5; slot++) {
                    ItemStack stack = inventory[slot];
                    stack = stack.func_77946_l();
                    stack.field_77994_a = 1;
                    ItemStack extracted = IOHelper.extract(inputInv, inputDir.func_176734_d(), stack, true, true);//simulate extraction from the neighbor.
                    if(extracted != null) {
                        ItemStack inserted = IOHelper.insert((IInventory)this, extracted, null, false);//if we can insert the item in this hopper.
                        if(inserted == null) {
                            IOHelper.extract(inputInv, inputDir.func_176734_d(), stack, true, false); //actually retrieve it from the neighbor.
                            success = true;
                            break;
                        }
                    }
                }
                if(!success) break;
            }
        }

        for(EntityItem entity : getNeighborItems(this, inputDir)) {
            if(!entity.field_70128_L) {
                ItemStack remainder = IOHelper.insert((IInventory)this, entity.func_92059_d(), null, false);
                if(remainder == null) {
                    entity.func_70106_y();
                    success = true;//Don't set true when the stack could not be fully consumes, as that means next insertion there won't be any room.
                }
            }
        }

        return success;
    }

    private boolean hasEmptySlot(){
        for(int i = 0; i < 5; i++) {
            if(inventory[i] == null) return true;
        }
        return false;
    }

    public static List<EntityItem> getNeighborItems(TileEntity te, EnumFacing inputDir){
        AxisAlignedBB box = new AxisAlignedBB(te.func_174877_v().func_177972_a(inputDir), te.func_174877_v().func_177972_a(inputDir).func_177982_a(1, 1, 1));
        return te.func_145831_w().func_72872_a(EntityItem.class, box);
    }

    public int getMaxItems(){
        int upgrades = getUpgrades(EnumUpgrade.SPEED);
        if(upgrades > 3) {
            return Math.min((int)Math.pow(2, upgrades - 3), 256);
        } else {
            return 1;
        }
    }

    public int getItemTransferInterval(){
        return 8 / (int)Math.pow(2, getUpgrades(EnumUpgrade.SPEED));
    }

    public void setDirection(EnumFacing dir){
        inputDir = dir;
    }

    public EnumFacing getDirection(){
        return inputDir;
    }

    @Override
    public EnumFacing getRotation(){
        return outputDir;
    }

    public void setRotation(EnumFacing rotation){
        outputDir = rotation;
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        tag.func_74768_a("inputDir", inputDir.ordinal());
        tag.func_74768_a("outputDir", outputDir.ordinal());
        tag.func_74768_a("redstoneMode", redstoneMode);
        tag.func_74757_a("leaveMaterial", leaveMaterial);

        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Inventory", tagList);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        inputDir = EnumFacing.func_82600_a(tag.func_74762_e("inputDir"));
        outputDir = EnumFacing.func_82600_a(tag.func_74762_e("outputDir"));
        redstoneMode = tag.func_74762_e("redstoneMode");
        leaveMaterial = tag.func_74767_n("leaveMaterial");

        NBTTagList tagList = tag.func_150295_c("Inventory", 10);
        inventory = new ItemStack[inventory.length];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }
    }

    /**
     * Returns the name of the inventory.
     */
    @Override
    public String getName(){
        return Blockss.omnidirectionalHopper.func_149739_a();
    }

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){
        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int par1){
        return inventory[par1];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }
        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID == 0) {
            redstoneMode++;
            if(redstoneMode > 2) redstoneMode = 0;
        } else if(buttonID == 1) {
            leaveMaterial = false;
        } else if(buttonID == 2) {
            leaveMaterial = true;
        }
    }

    @Override
    public boolean func_94041_b(int par1, ItemStack par2ItemStack){
        return par1 < 5;
    }

    private static final int[] accessibleSlots = {0, 1, 2, 3, 4};

    @Override
    public int[] func_180463_a(EnumFacing var1){
        return accessibleSlots;
    }

    @Override
    public boolean func_180462_a(int var1, ItemStack var2, EnumFacing var3){
        return var1 < 5;
    }

    @Override
    public boolean func_180461_b(int var1, ItemStack var2, EnumFacing var3){
        return var1 < 5;
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public int func_70297_j_(){
        return 64;
    }

    @Override
    public boolean func_70300_a(EntityPlayer p_70300_1_){
        return isGuiUseableByPlayer(p_70300_1_);
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }

    public boolean doesLeaveMaterial(){
        return leaveMaterial;
    }
}
