package pneumaticCraft.common.tileentity;

import java.util.Arrays;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemDye;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidRegistry;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidTank;
import net.minecraftforge.fluids.FluidTankInfo;
import net.minecraftforge.fluids.IFluidHandler;
import net.minecraftforge.fluids.IFluidTank;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.oredict.OreDictionary;
import pneumaticCraft.api.PneumaticRegistry;
import pneumaticCraft.api.heat.IHeatExchangerLogic;
import pneumaticCraft.api.tileentity.IHeatExchanger;
import pneumaticCraft.common.AchievementHandler;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.fluid.FluidPlastic;
import pneumaticCraft.common.fluid.Fluids;
import pneumaticCraft.common.item.Itemss;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.thirdparty.computercraft.LuaMethod;
import pneumaticCraft.lib.PneumaticValues;

public class TileEntityPlasticMixer extends TileEntityBase implements IFluidHandler, ISidedInventory, IHeatExchanger,
        IRedstoneControlled{
    private final FluidTank tank = new FluidTank(PneumaticValues.NORMAL_TANK_CAPACITY);
    private final ItemStack[] inventory = new ItemStack[9];
    private int lastTickInventoryStacksize;
    private static int BASE_TEMPERATURE = FluidRegistry.WATER.getTemperature();
    @GuiSynced
    private final IHeatExchangerLogic hullLogic = PneumaticRegistry.getInstance().getHeatRegistry().getHeatExchangerLogic();
    @GuiSynced
    private final IHeatExchangerLogic itemLogic = PneumaticRegistry.getInstance().getHeatRegistry().getHeatExchangerLogic();
    @GuiSynced
    public int selectedPlastic = -1;
    @GuiSynced
    private int redstoneMode;
    @GuiSynced
    public boolean lockSelection;
    @GuiSynced
    public int[] dyeBuffers = new int[3];
    public static final int DYE_PER_DYE = 0xFF * 10;
    public static final int DYE_BUFFER_MAX = 0xFF * 2 * PneumaticValues.NORMAL_TANK_CAPACITY / 1000;

    public static final String[] DYES = {"dyeBlack", "dyeRed", "dyeGreen", "dyeBrown", "dyeBlue", "dyePurple", "dyeCyan", "dyeLightGray", "dyeGray", "dyePink", "dyeLime", "dyeYellow", "dyeLightBlue", "dyeMagenta", "dyeOrange", "dyeWhite"};
    private static final int[] SLOTS = {0, 1, 2, 3, 4, 5, 6, 7, 8};
    public static final int INV_INPUT = 4, INV_OUTPUT = 5, INV_DYE_RED = 6, INV_DYE_GREEN = 7, INV_DYE_BLUE = 8;

    public TileEntityPlasticMixer(){
        super(0, 1, 2, 3);
        hullLogic.addConnectedExchanger(itemLogic);
        hullLogic.setThermalCapacity(100);
    }

    @SideOnly(Side.CLIENT)
    public IHeatExchangerLogic getLogic(int index){
        switch(index){
            case 0:
                return hullLogic;
            case 1:
                return itemLogic;
        }
        throw new IllegalArgumentException("Invalid index: " + index);
    }

    @SideOnly(Side.CLIENT)
    public IFluidTank getFluidTank(){
        return tank;
    }

    @Override
    public void func_73660_a(){
        super.func_73660_a();
        if(!field_145850_b.field_72995_K) {
            refillDyeBuffers();
            itemLogic.update();
            if(field_145850_b.func_82737_E() % 20 == 0) {//We don't need to run _that_ often.
                if(inventory[INV_INPUT] != null && inventory[INV_INPUT].field_77994_a > lastTickInventoryStacksize) {
                    int stackIncrease = inventory[INV_INPUT].field_77994_a - lastTickInventoryStacksize;
                    double ratio = (double)inventory[INV_INPUT].field_77994_a / (inventory[INV_INPUT].field_77994_a + stackIncrease);
                    itemLogic.setTemperature((int)(ratio * itemLogic.getTemperature() + (1 - ratio) * BASE_TEMPERATURE));
                } else if(inventory[INV_INPUT] == null) {
                    itemLogic.setTemperature(BASE_TEMPERATURE);
                }

                if(itemLogic.getTemperature() >= PneumaticValues.PLASTIC_MIXER_MELTING_TEMP) {
                    FluidStack moltenPlastic = new FluidStack(Fluids.plastic, inventory[INV_INPUT].field_77994_a * 1000);
                    int maxFill = fill(null, moltenPlastic, false) / 1000;
                    if(maxFill > 0) {
                        inventory[INV_INPUT].field_77994_a -= maxFill;
                        if(inventory[INV_INPUT].field_77994_a <= 0) inventory[INV_INPUT] = null;
                        fill(null, new FluidStack(moltenPlastic, maxFill * 1000), true);
                    }
                }

                lastTickInventoryStacksize = inventory[INV_INPUT] != null ? inventory[INV_INPUT].field_77994_a : 0;

                itemLogic.setThermalCapacity(inventory[INV_INPUT] == null ? 0 : inventory[INV_INPUT].field_77994_a);
            }
            if(tank.getFluid() != null && selectedPlastic >= 0 && redstoneAllows()) {
                ItemStack solidifiedStack = new ItemStack(Itemss.plastic, tank.getFluid().amount / 1000, selectedPlastic);
                if(solidifiedStack.field_77994_a > 0) {
                    solidifiedStack.field_77994_a = 1;
                    if(inventory[INV_OUTPUT] == null) {
                        solidifiedStack.field_77994_a = useDye(solidifiedStack.field_77994_a);
                        if(solidifiedStack.field_77994_a > 0) {
                            inventory[INV_OUTPUT] = solidifiedStack;
                            tank.drain(inventory[INV_OUTPUT].field_77994_a * 1000, true);
                            sendDescriptionPacket();
                        }
                    } else if(solidifiedStack.func_77969_a(inventory[INV_OUTPUT])) {
                        int solidifiedItems = Math.min(64 - inventory[INV_OUTPUT].field_77994_a, solidifiedStack.field_77994_a);
                        solidifiedItems = useDye(solidifiedItems);
                        inventory[INV_OUTPUT].field_77994_a += solidifiedItems;
                        tank.drain(solidifiedItems * 1000, true);
                        sendDescriptionPacket();
                    }
                }
            }
            if(!lockSelection) selectedPlastic = -1;
            if(redstoneMode == 3) selectedPlastic = poweredRedstone;
        }
    }

    private void refillDyeBuffers(){
        for(int i = 0; i < 3; i++) {
            if(func_70301_a(INV_DYE_RED + i) != null && dyeBuffers[i] <= DYE_BUFFER_MAX - DYE_PER_DYE) {
                func_70298_a(INV_DYE_RED + i, 1);
                dyeBuffers[i] += DYE_PER_DYE;
            }
        }
    }

    private int useDye(int maxItems){
        int desiredColor = ItemDye.field_150922_c[selectedPlastic];
        if(selectedPlastic == 15) return maxItems;//Converting to white plastic is free.
        for(int i = 0; i < 3; i++) {
            int colorComponent = desiredColor >> 8 * i & 0xFF;
            colorComponent = 0xFF - colorComponent;//Invert, because we start out with white, and we darken the plastic.
            if(colorComponent > 0) {
                maxItems = Math.min(maxItems, dyeBuffers[i] / colorComponent);
            }
        }
        for(int i = 0; i < 3; i++) {
            int colorComponent = desiredColor >> 8 * i & 0xFF;
            colorComponent = 0xFF - colorComponent;//Invert, because we start out with white, and we darken the plastic.
            dyeBuffers[i] -= colorComponent * maxItems;
        }
        return maxItems;
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        readInventoryFromNBT(tag, inventory, "Items");
        lastTickInventoryStacksize = tag.func_74762_e("lastTickInventoryStacksize");
        selectedPlastic = tag.func_74762_e("selectedPlastic");
        lockSelection = tag.func_74767_n("lockSelection");
        dyeBuffers[0] = tag.func_74762_e("dyeBuffer0");
        dyeBuffers[1] = tag.func_74762_e("dyeBuffer1");
        dyeBuffers[2] = tag.func_74762_e("dyeBuffer2");
        redstoneMode = tag.func_74762_e("redstoneMode");

        itemLogic.readFromNBT(tag.func_74775_l("itemLogic"));
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        writeInventoryToNBT(tag, inventory, "Items");
        tag.func_74768_a("lastTickInventoryStacksize", lastTickInventoryStacksize);
        tag.func_74768_a("selectedPlastic", selectedPlastic);
        tag.func_74757_a("lockSelection", lockSelection);
        tag.func_74768_a("dyeBuffer0", dyeBuffers[0]);
        tag.func_74768_a("dyeBuffer1", dyeBuffers[1]);
        tag.func_74768_a("dyeBuffer2", dyeBuffers[2]);
        tag.func_74768_a("redstoneMode", redstoneMode);

        NBTTagCompound heatTag = new NBTTagCompound();
        itemLogic.writeToNBT(heatTag);
        tag.func_74782_a("itemLogic", heatTag);
    }

    @Override
    public void readFromPacket(NBTTagCompound tag){
        super.readFromPacket(tag);
        tank.setFluid(null);
        tank.readFromNBT(tag.func_74775_l("fluid"));
    }

    @Override
    public void writeToPacket(NBTTagCompound tag){
        super.writeToPacket(tag);
        NBTTagCompound tankTag = new NBTTagCompound();
        tank.writeToNBT(tankTag);
        tag.func_74782_a("fluid", tankTag);
    }

    /******************* Tank methods *******************/

    @Override
    public int fill(EnumFacing from, FluidStack resource, boolean doFill){
        if(resource == null || !Fluids.areFluidsEqual(resource.getFluid(), Fluids.plastic)) return 0;
        int fillingAmount = Math.min(tank.getCapacity() - tank.getFluidAmount(), resource.amount);
        if(doFill && fillingAmount > 0) {
            tank.setFluid(FluidPlastic.mixFluid(tank.getFluid(), new FluidStack(resource, fillingAmount)));
            sendDescriptionPacket();
        }
        return fillingAmount;
    }

    @Override
    public FluidStack drain(EnumFacing from, FluidStack resource, boolean doDrain){
        if(resource == null || Fluids.areFluidsEqual(resource.getFluid(), Fluids.plastic)) return drain(from, PneumaticValues.MAX_DRAIN, doDrain);
        else return null;
    }

    @Override
    public FluidStack drain(EnumFacing from, int maxDrain, boolean doDrain){
        FluidStack drained = tank.drain(Math.min(PneumaticValues.MAX_DRAIN, maxDrain), doDrain);
        if(doDrain && drained != null && drained.getFluid() != null) sendDescriptionPacket();
        return drained;
    }

    @Override
    public boolean canFill(EnumFacing from, Fluid fluid){
        return Fluids.areFluidsEqual(fluid, Fluids.plastic);
    }

    @Override
    public boolean canDrain(EnumFacing from, Fluid fluid){
        return canFill(from, fluid);
    }

    @Override
    public FluidTankInfo[] getTankInfo(EnumFacing from){
        return new FluidTankInfo[]{tank.getInfo()};
    }

    /****************** End Tank methods *******************/

    /****************** IInventory *********************/

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){
        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){
        return inventory[slot];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
    }

    @Override
    public String getName(){
        return Blockss.plasticMixer.func_149739_a();
    }

    @Override
    public int func_70297_j_(){
        return 64;
    }

    @Override
    public boolean func_94041_b(int i, ItemStack itemstack){
        if(itemstack == null) return true;
        switch(i){
            case INV_INPUT:
                return itemstack.func_77973_b() == Itemss.plastic;
            case INV_OUTPUT:
                return false;
            case INV_DYE_RED:
                return getDyeIndex(itemstack) == 1;
            case INV_DYE_GREEN:
                return getDyeIndex(itemstack) == 2;
            case INV_DYE_BLUE:
                return getDyeIndex(itemstack) == 4;
            default:
                return canInsertUpgrade(i, itemstack);
        }
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    @Override
    public IHeatExchangerLogic getHeatExchangerLogic(EnumFacing side){
        return hullLogic;
    }

    public static int getDyeIndex(ItemStack stack){
        int[] ids = OreDictionary.getOreIDs(stack);
        for(int id : ids) {
            String name = OreDictionary.getOreName(id);
            for(int i = 0; i < DYES.length; i++) {
                if(DYES[i].equals(name)) return i;
            }
        }
        return -1;
    }

    @Override
    public int[] func_180463_a(EnumFacing side){
        return SLOTS;
    }

    @Override
    public boolean func_180462_a(int slot, ItemStack stack, EnumFacing side){
        return func_94041_b(slot, stack);
    }

    @Override
    public boolean func_180461_b(int slot, ItemStack stack, EnumFacing side){
        return slot == INV_OUTPUT;
    }

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public void handleGUIButtonPress(int guiID, EntityPlayer player){
        super.handleGUIButtonPress(guiID, player);
        if(guiID == 0) {
            if(++redstoneMode > 3) {
                redstoneMode = 0;
            }
        } else if(guiID >= 1 && guiID < 17) {
            if(selectedPlastic != guiID) {
                selectedPlastic = guiID - 1;
                if(tank.getFluidAmount() >= 1000) {
                    AchievementHandler.giveAchievement(player, new ItemStack(Itemss.plastic));
                }
            } else {
                selectedPlastic = -1;
            }
        } else if(guiID == 17) {
            lockSelection = !lockSelection;
        }
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }

    @Override
    public boolean redstoneAllows(){
        return redstoneMode == 3 ? true : super.redstoneAllows();
    }

    @Override
    protected void addLuaMethods(){
        super.addLuaMethods();
        luaMethods.add(new LuaMethod("selectColor"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 1) {
                    int selection = ((Double)args[0]).intValue();
                    if(selection >= 0 && selection <= 16) {
                        selectedPlastic = selection - 1;
                        return null;
                    } else {
                        throw new IllegalArgumentException("selectColor method only accepts a value ranging from 0-16. The value passed was: " + selection);
                    }
                } else {
                    throw new IllegalArgumentException("selectColor method requires 1 argument (int color index, with 0 being no color");
                }
            }
        });
    }
}
