package pneumaticCraft.common.tileentity;

import java.util.Arrays;
import java.util.List;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.common.block.BlockPneumaticDoor;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.network.LazySynced;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.PneumaticValues;
import pneumaticCraft.lib.TileEntityConstants;

public class TileEntityPneumaticDoorBase extends TileEntityPneumaticBase implements IInventory, IRedstoneControl,
        IMinWorkingPressure{
    private TileEntityPneumaticDoor door;
    private TileEntityPneumaticDoorBase doubleDoor;
    @DescSynced
    public boolean rightGoing;
    public float oldProgress;
    @DescSynced
    @LazySynced
    public float progress;
    @DescSynced
    private boolean opening;
    public boolean wasPowered;
    @DescSynced
    private ItemStack[] inventory = new ItemStack[5];
    @GuiSynced
    public int redstoneMode;
    public static final int UPGRADE_SLOT_1 = 0;
    public static final int UPGRADE_SLOT_4 = 3;
    public static final int CAMO_SLOT = 4;
    private ItemStack oldCamo;

    public TileEntityPneumaticDoorBase(){
        super(PneumaticValues.DANGER_PRESSURE_PNEUMATIC_DOOR, PneumaticValues.MAX_PRESSURE_PNEUMATIC_DOOR, PneumaticValues.VOLUME_PNEUMATIC_DOOR, UPGRADE_SLOT_1, 1, 2, UPGRADE_SLOT_4);
        addApplicableUpgrade(EnumUpgrade.SPEED, EnumUpgrade.RANGE);
    }

    @Override
    public void func_73660_a(){
        super.func_73660_a();
        oldProgress = progress;
        if(!field_145850_b.field_72995_K) {
            if(getPressure() >= PneumaticValues.MIN_PRESSURE_PNEUMATIC_DOOR) {
                if(field_145850_b.func_82737_E() % 60 == 0) {
                    TileEntity te = field_145850_b.func_175625_s(func_174877_v().func_177967_a(getRotation(), 3));
                    if(te instanceof TileEntityPneumaticDoorBase) {
                        doubleDoor = (TileEntityPneumaticDoorBase)te;
                    } else {
                        doubleDoor = null;
                    }
                }
                setOpening(shouldOpen() || isNeighborOpening());
                setNeighborOpening(isOpening());
            } else {
                setOpening(true);
            }
        }
        float targetProgress = opening ? 1F : 0F;
        float speedMultiplier = getSpeedMultiplierFromUpgrades();
        if(progress < targetProgress) {
            if(progress < targetProgress - TileEntityConstants.PNEUMATIC_DOOR_EXTENSION) {
                progress += TileEntityConstants.PNEUMATIC_DOOR_SPEED_FAST * speedMultiplier;
            } else {
                progress += TileEntityConstants.PNEUMATIC_DOOR_SPEED_SLOW * speedMultiplier;
            }
            if(progress > targetProgress) progress = targetProgress;
        }
        if(progress > targetProgress) {
            if(progress > targetProgress + TileEntityConstants.PNEUMATIC_DOOR_EXTENSION) {
                progress -= TileEntityConstants.PNEUMATIC_DOOR_SPEED_FAST * speedMultiplier;
            } else {
                progress -= TileEntityConstants.PNEUMATIC_DOOR_SPEED_SLOW * speedMultiplier;
            }
            if(progress < targetProgress) progress = targetProgress;
        }
        if(!field_145850_b.field_72995_K) addAir((int)(-Math.abs(oldProgress - progress) * PneumaticValues.USAGE_PNEUMATIC_DOOR * (getSpeedUsageMultiplierFromUpgrades() / speedMultiplier)));

        // if(worldObj.isRemote) System.out.println("progress: " + progress);
        door = getDoor();
        if(door != null) {
            door.setRotation(progress * 90);
            if(!field_145850_b.field_72995_K) rightGoing = door.rightGoing;
        }

        if(oldCamo != inventory[CAMO_SLOT]) {
            oldCamo = inventory[CAMO_SLOT];
            //TODO 1.8 fix camo meta     worldObj.setBlockMetadataWithNotify(getPos().getX(), getPos().getY(), getPos().getZ(), inventory[CAMO_SLOT] != null ? inventory[CAMO_SLOT].getItemDamage() % 16 : 0, 2);
            rerenderChunk();
        }
    }

    private boolean shouldOpen(){
        switch(redstoneMode){
            case 0:
            case 1:
                int range = TileEntityConstants.RANGE_PNEUMATIC_DOOR_BASE + this.getUpgrades(EnumUpgrade.RANGE);
                AxisAlignedBB aabb = new AxisAlignedBB(func_174877_v().func_177958_n() - range, func_174877_v().func_177956_o() - range, func_174877_v().func_177952_p() - range, func_174877_v().func_177958_n() + range + 1, func_174877_v().func_177956_o() + range + 1, func_174877_v().func_177952_p() + range + 1);
                List<EntityPlayer> players = field_145850_b.func_72872_a(EntityPlayer.class, aabb);
                for(EntityPlayer player : players) {
                    if(PneumaticCraftUtils.getProtectingSecurityStations(field_145850_b, func_174877_v(), player, false, false) == 0) {
                        if(redstoneMode == 0) {
                            return true;
                        } else {
                            ((BlockPneumaticDoor)Blockss.pneumaticDoor).isTrackingPlayerEye = true;
                            BlockPos lookedPosition = PneumaticCraftUtils.getEntityLookedBlock(player, range * 1.41F); //max range = range * sqrt(2).
                            ((BlockPneumaticDoor)Blockss.pneumaticDoor).isTrackingPlayerEye = false;
                            if(lookedPosition != null) {
                                if(lookedPosition.equals(new BlockPos(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p()))) {
                                    return true;
                                } else {
                                    if(door != null) {
                                        if(lookedPosition.equals(new BlockPos(door.func_174877_v().func_177958_n(), door.func_174877_v().func_177956_o(), door.func_174877_v().func_177952_p()))) return true;
                                        if(lookedPosition.equals(new BlockPos(door.func_174877_v().func_177958_n(), door.func_174877_v().func_177956_o() + (door.isTopDoor() ? -1 : 1), door.func_174877_v().func_177952_p()))) return true;
                                    }
                                }
                            }
                        }
                    }
                }
                return false;
            case 2:
                return opening;
        }
        return false;
    }

    public void setOpening(boolean opening){
        this.opening = opening;
    }

    public boolean isOpening(){
        return opening;
    }

    private boolean isNeighborOpening(){
        return doubleDoor != null ? doubleDoor.shouldOpen() : false;
    }

    public void setNeighborOpening(boolean opening){
        if(doubleDoor != null && doubleDoor.getPressure() >= PneumaticValues.MIN_PRESSURE_PNEUMATIC_DOOR) {
            doubleDoor.setOpening(opening);
        }
    }

    @Override
    public boolean isConnectedTo(EnumFacing side){
        return side != EnumFacing.UP;
    }

    private TileEntityPneumaticDoor getDoor(){
        TileEntity te = field_145850_b.func_175625_s(func_174877_v().func_177972_a(getRotation()).func_177982_a(0, -1, 0));
        if(te instanceof TileEntityPneumaticDoor) {
            if(getRotation().func_176746_e() == getRotation() && !((TileEntityPneumaticDoor)te).rightGoing) {
                return (TileEntityPneumaticDoor)te;
            } else if(getRotation().func_176735_f() == getRotation() && ((TileEntityPneumaticDoor)te).rightGoing) {
                return (TileEntityPneumaticDoor)te;
            }
        }
        return null;
    }

    // NBT methods-----------------------------------------------
    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        progress = tag.func_74760_g("extension");
        opening = tag.func_74767_n("opening");
        redstoneMode = tag.func_74762_e("redstoneMode");
        rightGoing = tag.func_74767_n("rightGoing");
        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[func_70302_i_()];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        tag.func_74776_a("extension", progress);
        tag.func_74757_a("opening", opening);
        tag.func_74768_a("redstoneMode", redstoneMode);
        tag.func_74757_a("rightGoing", rightGoing);
        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Items", tagList);

    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID == 0) {
            redstoneMode++;
            if(redstoneMode > 2) redstoneMode = 0;
        }
    }

    // INVENTORY METHODS-
    // ------------------------------------------------------------

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){

        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){

        return inventory[slot];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        // super.setInventorySlotContents(slot, itemStack);
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
    }

    @Override
    public int func_70297_j_(){
        return 64;
    }

    @Override
    public String getName(){
        return Blockss.pneumaticDoorBase.func_149739_a();
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public boolean func_94041_b(int i, ItemStack itemstack){
        return i == CAMO_SLOT || canInsertUpgrade(i, itemstack);
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    @Override
    public float getMinWorkingPressure(){
        return PneumaticValues.MIN_PRESSURE_PNEUMATIC_DOOR;
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }
}
