package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.FilteredSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.network.LazySynced;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.GuiConstants;
import pneumaticCraft.lib.PneumaticValues;
import pneumaticCraft.lib.Sounds;

public class TileEntityPressureChamberInterface extends TileEntityPressureChamberWall implements ISidedInventory,
        IGUITextFieldSensitive, IRedstoneControlled{
    @DescSynced
    @FilteredSynced(index = 0)
    private ItemStack[] inventory = new ItemStack[14];
    @DescSynced
    @LazySynced
    public int inputProgress;
    public int oldInputProgress;
    @DescSynced
    @LazySynced
    public int outputProgress;
    public int oldOutputProgress;
    public static final int MAX_PROGRESS = 40;
    private static final int UPGRADE_SLOT_START = 1;
    private static final int UPGRADE_SLOT_END = 4;
    @GuiSynced
    public EnumInterfaceMode interfaceMode = EnumInterfaceMode.NONE;
    @GuiSynced
    private boolean enoughAir = true;
    @DescSynced
    public EnumFilterMode filterMode = EnumFilterMode.ITEM;
    @GuiSynced
    public int creativeTabID;
    @DescSynced
    public String itemNameFilter = "";
    private boolean isOpeningI;//used to determine sounds.
    private boolean isOpeningO;//used to determine sounds.
    @DescSynced
    private boolean shouldOpenInput, shouldOpenOutput;
    @GuiSynced
    public int redstoneMode;
    private int inputTimeOut;
    private int oldItemCount;

    public enum EnumInterfaceMode{
        NONE, IMPORT, EXPORT;
    }

    public enum EnumFilterMode{
        ITEM, CREATIVE_TAB, NAME_BEGINS, NAME_CONTAINS;
    }

    public TileEntityPressureChamberInterface(){
        super(UPGRADE_SLOT_START, 2, 3, UPGRADE_SLOT_END);
        addApplicableUpgrade(EnumUpgrade.SPEED);
    }

    @Override
    public void func_73660_a(){
        super.func_73660_a();

        boolean wasOpeningI = isOpeningI;
        boolean wasOpeningO = isOpeningO;
        oldInputProgress = inputProgress;
        oldOutputProgress = outputProgress;
        TileEntityPressureChamberValve core = getCore();

        if(!field_145850_b.field_72995_K) {
            int itemCount = inventory[0] != null ? inventory[0].field_77994_a : 0;
            if(oldItemCount != itemCount) {
                oldItemCount = itemCount;
                inputTimeOut = 0;
            }

            interfaceMode = getInterfaceMode(core);
            enoughAir = true;

            if(interfaceMode != EnumInterfaceMode.NONE) {
                if(inventory[0] != null && ++inputTimeOut > 10) {
                    shouldOpenInput = false;
                    if(inputProgress == 0) {
                        shouldOpenOutput = true;
                        if(outputProgress == MAX_PROGRESS) {
                            if(interfaceMode == EnumInterfaceMode.IMPORT) {
                                outputInChamber();
                            } else {
                                exportToInventory();
                            }
                        }
                    }
                } else {
                    shouldOpenOutput = false;
                    if(outputProgress == 0) {
                        shouldOpenInput = true;
                        if(interfaceMode == EnumInterfaceMode.EXPORT && inputProgress == MAX_PROGRESS && redstoneAllows()) {
                            importFromChamber(core);
                        }
                    }
                }
            } else {
                shouldOpenInput = false;
                shouldOpenOutput = false;
            }
        }

        int speed = (int)getSpeedMultiplierFromUpgrades();

        if(shouldOpenInput) {
            inputProgress = Math.min(inputProgress + speed, MAX_PROGRESS);
            isOpeningI = true;
        } else {
            inputProgress = Math.max(inputProgress - speed, 0);
            isOpeningI = false;
        }

        if(shouldOpenOutput) {
            outputProgress = Math.min(outputProgress + speed, MAX_PROGRESS);
            isOpeningO = true;
        } else {
            outputProgress = Math.max(outputProgress - speed, 0);
            isOpeningO = false;
        }

        if(field_145850_b.field_72995_K && (wasOpeningI != isOpeningI || wasOpeningO != isOpeningO)) {
            field_145850_b.func_72980_b(func_174877_v().func_177958_n() + 0.5, func_174877_v().func_177956_o() + 0.5, func_174877_v().func_177952_p() + 0.5, Sounds.INTERFACE_DOOR, 0.1F, 1.0F, true);
        }
    }

    private void exportToInventory(){
        EnumFacing facing = getRotation();
        TileEntity te = field_145850_b.func_175625_s(func_174877_v().func_177972_a(facing));
        if(te != null) {
            EnumFacing side = facing.func_176734_d();
            ItemStack leftoverStack = PneumaticCraftUtils.exportStackToInventory(te, inventory[0], side);
            if(leftoverStack == null || leftoverStack.field_77994_a == 0) {
                inventory[0] = null;
            }
        }
    }

    private void importFromChamber(TileEntityPressureChamberValve core){
        ItemStack[] chamberStacks = core.getStacksInChamber();
        for(ItemStack chamberStack : chamberStacks) {
            if((inventory[0] == null || inventory[0].func_77969_a(chamberStack)) && func_94041_b(0, chamberStack)) {
                int maxAllowedItems = Math.abs(core.getAirHandler(null).getAir()) / PneumaticValues.USAGE_CHAMBER_INTERFACE;
                if(maxAllowedItems > 0) {
                    if(inventory[0] != null) maxAllowedItems = Math.min(maxAllowedItems, chamberStack.func_77976_d() - inventory[0].field_77994_a);
                    int transferedItems = Math.min(chamberStack.field_77994_a, maxAllowedItems);
                    core.addAir((core.getAirHandler(null).getAir() > 0 ? -1 : 1) * transferedItems * PneumaticValues.USAGE_CHAMBER_INTERFACE);
                    ItemStack transferedStack = chamberStack.func_77946_l().func_77979_a(transferedItems);
                    ItemStack insertedStack = transferedStack.func_77946_l();
                    if(inventory[0] != null) insertedStack.field_77994_a += inventory[0].field_77994_a;
                    func_70299_a(0, insertedStack);
                    core.clearStacksInChamber(transferedStack);
                }
            }
        }
    }

    private void outputInChamber(){
        TileEntityPressureChamberValve valve = getCore();
        if(valve != null) {
            for(EnumFacing d : EnumFacing.field_82609_l) {
                BlockPos neighborPos = func_174877_v().func_177972_a(d);
                if(valve.isCoordWithinChamber(field_145850_b, neighborPos)) {
                    enoughAir = Math.abs(valve.getAirHandler(null).getAir()) > inventory[0].field_77994_a * PneumaticValues.USAGE_CHAMBER_INTERFACE;
                    if(enoughAir) {
                        valve.addAir((valve.getAirHandler(null).getAir() > 0 ? -1 : 1) * inventory[0].field_77994_a * PneumaticValues.USAGE_CHAMBER_INTERFACE);
                        EntityItem item = new EntityItem(field_145850_b, neighborPos.func_177958_n() + 0.5, neighborPos.func_177956_o() + 0.5, neighborPos.func_177952_p() + 0.5D, inventory[0].func_77946_l());
                        field_145850_b.func_72838_d(item);
                        func_70299_a(0, null);
                        break;
                    }
                }
            }
        }
    }

    /*
     * public void setCore(TileEntityPressureChamberValve core){ boolean
     * wasNotNull = teValve != null; super.setCore(core); if(worldObj.isRemote)
     * return; if(core != null) modeNeedsChecking = true; else if(wasNotNull)
     * modeNeedsChecking = true; }
     */
    // figure out whether the Interface is exporting or importing.
    private EnumInterfaceMode getInterfaceMode(TileEntityPressureChamberValve core){
        if(core != null) {
            boolean xMid = func_174877_v().func_177958_n() != core.multiBlockX && func_174877_v().func_177958_n() != core.multiBlockX + core.multiBlockSize - 1;
            boolean yMid = func_174877_v().func_177956_o() != core.multiBlockY && func_174877_v().func_177956_o() != core.multiBlockY + core.multiBlockSize - 1;
            boolean zMid = func_174877_v().func_177952_p() != core.multiBlockZ && func_174877_v().func_177952_p() != core.multiBlockZ + core.multiBlockSize - 1;
            EnumFacing rotation = getRotation();
            if(xMid && yMid && rotation == EnumFacing.NORTH || xMid && zMid && rotation == EnumFacing.DOWN || yMid && zMid && rotation == EnumFacing.WEST) {
                if(func_174877_v().func_177958_n() == core.multiBlockX || func_174877_v().func_177956_o() == core.multiBlockY || func_174877_v().func_177952_p() == core.multiBlockZ) {
                    return EnumInterfaceMode.EXPORT;
                } else {
                    return EnumInterfaceMode.IMPORT;
                }
            } else if(xMid && yMid && rotation == EnumFacing.SOUTH || xMid && zMid && rotation == EnumFacing.UP || yMid && zMid && rotation == EnumFacing.EAST) {
                if(func_174877_v().func_177958_n() == core.multiBlockX || func_174877_v().func_177956_o() == core.multiBlockY || func_174877_v().func_177952_p() == core.multiBlockZ) {
                    return EnumInterfaceMode.IMPORT;
                } else {
                    return EnumInterfaceMode.EXPORT;
                }
            }
        }
        return EnumInterfaceMode.NONE;
    }

    public List<String> getProblemStat(){
        List<String> textList = new ArrayList<String>();
        if(interfaceMode == EnumInterfaceMode.NONE) {
            textList.addAll(PneumaticCraftUtils.convertStringIntoList("\u00a77The Interface can't work.", GuiConstants.maxCharPerLineLeft));
            textList.addAll(PneumaticCraftUtils.convertStringIntoList("\u00a70-The Interface is not in a properly formed Pressure Chamber, and/or", GuiConstants.maxCharPerLineLeft));
            textList.addAll(PneumaticCraftUtils.convertStringIntoList("\u00a70-The Interface is not adjacent to an air block of the Pressure Chamber, and/or", GuiConstants.maxCharPerLineLeft));
            textList.addAll(PneumaticCraftUtils.convertStringIntoList("\u00a70-The Interface isn't orientated right.", GuiConstants.maxCharPerLineLeft));
        } else if(!redstoneAllows()) {
            textList.add("gui.tab.problems.redstoneDisallows");
        } else if(!enoughAir) {
            textList.addAll(PneumaticCraftUtils.convertStringIntoList("\u00a77There's not enough pressure in the Pressure Chamber to move the items.", GuiConstants.maxCharPerLineLeft));
            textList.addAll(PneumaticCraftUtils.convertStringIntoList("\u00a70Apply more pressure to the Pressure Chamber. The required pressure is dependent on the amount of items being transported.", GuiConstants.maxCharPerLineLeft));
        }
        return textList;
    }

    public boolean hasEnoughPressure(){
        return enoughAir;
    }

    // INVENTORY METHODS- && NBT
    // ------------------------------------------------------------

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);

        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[func_70302_i_()];
        for(int i = 0; i < tagList.func_74745_c(); i++) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }

        outputProgress = tag.func_74762_e("outputProgress");
        inputProgress = tag.func_74762_e("inputProgress");
        interfaceMode = EnumInterfaceMode.values()[tag.func_74762_e("interfaceMode")];
        filterMode = EnumFilterMode.values()[tag.func_74762_e("filterMode")];
        creativeTabID = tag.func_74762_e("creativeTabID");
        itemNameFilter = tag.func_74779_i("itemNameFilter");
        redstoneMode = tag.func_74762_e("redstoneMode");
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);

        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int i = 0; i < inventory.length; i++)
            if(inventory[i] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)i);
                inventory[i].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        tag.func_74782_a("Items", tagList);
        tag.func_74768_a("outputProgress", outputProgress);
        tag.func_74768_a("inputProgress", inputProgress);
        tag.func_74768_a("interfaceMode", interfaceMode.ordinal());
        tag.func_74768_a("filterMode", filterMode.ordinal());
        tag.func_74768_a("creativeTabID", creativeTabID);
        tag.func_74778_a("itemNameFilter", itemNameFilter);
        tag.func_74768_a("redstoneMode", redstoneMode);
    }

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){

        return 14;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){
        return inventory[slot];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
        if(!field_145850_b.field_72995_K && slot == 0) {
            sendDescriptionPacket();
        }
    }

    @Override
    public int func_70297_j_(){
        return 64;
    }

    @Override
    public String getName(){
        return Blockss.pressureChamberInterface.func_149739_a();
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public boolean func_94041_b(int slot, ItemStack iStack){
        if(canInsertUpgrade(slot, iStack)) return true;
        if(iStack == null || slot >= UPGRADE_SLOT_START && slot <= UPGRADE_SLOT_END) return false;
        switch(filterMode){
            case ITEM:
                boolean filterEmpty = true;
                for(int i = 0; i < 9; i++) {
                    ItemStack filterStack = func_70301_a(i + 5);
                    if(filterStack != null) {
                        filterEmpty = false;
                        if(iStack.func_77969_a(filterStack)) {
                            return true;
                        }
                    }
                }
                return filterEmpty;
            case CREATIVE_TAB:
                try {
                    int itemCreativeTabIndex = iStack.func_77973_b().func_77640_w() != null ? iStack.func_77973_b().func_77640_w().func_78021_a() : -1;
                    if(itemCreativeTabIndex == creativeTabID) {
                        return true;
                    }
                } catch(Throwable e) {//when we are SMP getCreativeTab() is client only.
                    filterMode = EnumFilterMode.NAME_BEGINS;
                }
                return false;
            case NAME_BEGINS:
                return iStack.func_82833_r().toLowerCase().startsWith(itemNameFilter.toLowerCase());
            case NAME_CONTAINS:
                return iStack.func_82833_r().toLowerCase().contains(itemNameFilter.toLowerCase());
        }
        return false;
    }

    @Override
    public boolean func_70300_a(EntityPlayer entityplayer){
        return true;
    }

    @Override
    public boolean isGuiUseableByPlayer(EntityPlayer par1EntityPlayer){
        return field_145850_b.func_175625_s(func_174877_v()) != this ? false : par1EntityPlayer.func_70092_e(func_174877_v().func_177958_n() + 0.5D, func_174877_v().func_177956_o() + 0.5D, func_174877_v().func_177952_p() + 0.5D) <= 64.0D;
    }

    @Override
    public int[] func_180463_a(EnumFacing var1){
        return new int[]{0};
    }

    @Override
    public boolean func_180462_a(int i, ItemStack itemstack, EnumFacing j){
        return inputProgress == MAX_PROGRESS && j == getRotation().func_176734_d() && redstoneAllows();
    }

    @Override
    public boolean func_180461_b(int i, ItemStack itemstack, EnumFacing j){
        return outputProgress == MAX_PROGRESS && j == getRotation();
    }

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public void handleGUIButtonPress(int guiID, EntityPlayer player){
        if(guiID == 1) {
            if(filterMode.ordinal() >= EnumFilterMode.values().length - 1) {
                filterMode = EnumFilterMode.ITEM;
            } else {
                filterMode = EnumFilterMode.values()[filterMode.ordinal() + 1];
            }
            func_94041_b(0, new ItemStack(Items.field_151055_y));//when an SideOnly exception is thrown this method automatically will set the filter mode to Item.

        } else if(guiID == 2) {
            creativeTabID++;
            if(creativeTabID == 5 || creativeTabID == 11) creativeTabID++;
            if(creativeTabID >= CreativeTabs.field_78032_a.length) {
                creativeTabID = 0;
            }
        } else if(guiID == 0) {
            redstoneMode++;
            if(redstoneMode > 2) redstoneMode = 0;
        }
    }

    @Override
    public void setText(int textFieldID, String text){
        itemNameFilter = text;
    }

    @Override
    public String getText(int textFieldID){
        return itemNameFilter;
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }
}
