package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.passive.EntityVillager;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.world.World;

import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.Pair;

import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.recipe.IPressureChamberRecipe;
import pneumaticCraft.api.tileentity.IAirHandler;
import pneumaticCraft.api.tileentity.IAirListener;
import pneumaticCraft.common.AchievementHandler;
import pneumaticCraft.common.DamageSourcePneumaticCraft;
import pneumaticCraft.common.NBTUtil;
import pneumaticCraft.common.block.BlockPressureChamberValve;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.block.IBlockPressureChamber;
import pneumaticCraft.common.config.Config;
import pneumaticCraft.common.fluid.Fluids;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.recipes.PneumaticRecipeRegistry;
import pneumaticCraft.common.recipes.PressureChamberRecipe;
import pneumaticCraft.lib.PneumaticValues;

public class TileEntityPressureChamberValve extends TileEntityPneumaticBase implements IInventory, IMinWorkingPressure,
        IAirListener{
    public int multiBlockX;
    public int multiBlockY;
    public int multiBlockZ;
    @GuiSynced
    public int multiBlockSize;
    public List<TileEntityPressureChamberValve> accessoryValves;
    private final List<BlockPos> nbtValveList;
    private boolean readNBT = false;
    @GuiSynced
    public boolean isValidRecipeInChamber;
    @GuiSynced
    public boolean isSufficientPressureInChamber;
    @GuiSynced
    public boolean areEntitiesDoneMoving;
    @GuiSynced
    public float recipePressure;
    private final Item etchingAcid = Fluids.getBucket(Fluids.etchingAcid);

    private ItemStack[] inventory = new ItemStack[4];
    public static final int UPGRADE_SLOT_1 = 0;
    public static final int UPGRADE_SLOT_4 = 3;
    private final Random rand = new Random();

    // private float oldPressure;
    // private int pressureUpdateTimer = 60;

    public TileEntityPressureChamberValve(){
        super(PneumaticValues.DANGER_PRESSURE_PRESSURE_CHAMBER, PneumaticValues.MAX_PRESSURE_PRESSURE_CHAMBER, PneumaticValues.VOLUME_PRESSURE_CHAMBER, UPGRADE_SLOT_1, 1, 2, UPGRADE_SLOT_4);
        accessoryValves = new ArrayList<TileEntityPressureChamberValve>();
        nbtValveList = new ArrayList<BlockPos>();
        addApplicableUpgrade(EnumUpgrade.ITEM_LIFE);
    }

    @Override
    protected boolean shouldRerenderChunkOnDescUpdate(){
        return true;
    }

    // pneumatic methods-------------------------------------------------------
    // used in the air dispersion methods.
    @Override
    public boolean isConnectedTo(EnumFacing side){
        switch(getRotation()){
            case UP:
            case DOWN:
                return side == EnumFacing.UP || side == EnumFacing.DOWN;
            case NORTH:
            case SOUTH:
                return side == EnumFacing.NORTH || side == EnumFacing.SOUTH;
            case EAST:
            case WEST:
                return side == EnumFacing.EAST || side == EnumFacing.WEST;
        }
        return false;
    }

    @Override
    public void addConnectedPneumatics(List<Pair<EnumFacing, IAirHandler>> teList){
        if(accessoryValves != null) {
            for(TileEntityPressureChamberValve valve : accessoryValves) {
                if(valve != this) teList.add(new ImmutablePair(null, valve.getAirHandler(null)));
            }
        }
    }

    @Override
    public void onAirDispersion(IAirHandler handler, EnumFacing dir, int airAdded){}

    @Override
    public int getMaxDispersion(IAirHandler handler, EnumFacing dir){
        return Integer.MAX_VALUE;
    }

    // main update method
    @Override
    public void func_73660_a(){
        if(readNBT) {// this way of doing this is needed because other TE's are
                     // loaded after the reading of the NBT of this TE.
            readNBT = false;
            accessoryValves.clear();
            for(BlockPos valve : nbtValveList) {
                TileEntity te = field_145850_b.func_175625_s(valve);
                if(te instanceof TileEntityPressureChamberValve) {

                    accessoryValves.add((TileEntityPressureChamberValve)te);
                }
            }
            if(accessoryValves.isEmpty()) {//Hacky solution for an unexplainable bug.
                invalidateMultiBlock();
                checkIfProperlyFormed(field_145850_b, func_174877_v());
            }
            if(field_145850_b.field_72995_K) field_145850_b.func_147458_c(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p());
        }
        if(!field_145850_b.field_72995_K) {
            //code to check if we need to leak air.
            boolean[] connected = new boolean[]{true, true, true, true, true, true};//assume we are not leaking at any side
            switch(getRotation()){//take off the sides that tubes can connect to.
                case UP:
                case DOWN:
                    connected[EnumFacing.UP.ordinal()] = false;
                    connected[EnumFacing.DOWN.ordinal()] = false;
                    break;
                case NORTH:
                case SOUTH:
                    connected[EnumFacing.NORTH.ordinal()] = false;
                    connected[EnumFacing.SOUTH.ordinal()] = false;
                    break;
                case EAST:
                case WEST:
                    connected[EnumFacing.EAST.ordinal()] = false;
                    connected[EnumFacing.WEST.ordinal()] = false;
                    break;
            }
            List<Pair<EnumFacing, IAirHandler>> teList = getAirHandler(null).getConnectedPneumatics();//we need the super method, as the overridden method adds the other valves.
            for(Pair<EnumFacing, IAirHandler> entry : teList) {
                if(entry.getKey() != null) connected[entry.getKey().ordinal()] = true;
            }
            //retrieve the valve that is controlling the (potential) chamber.
            TileEntityPressureChamberValve baseValve = null;
            for(TileEntityPressureChamberValve valve : accessoryValves) {
                if(valve.multiBlockSize > 0) {
                    baseValve = valve;
                    break;
                }
            }
            //if we found one, we can scratch one side to be leaking air.
            if(baseValve != null) {
                switch(getRotation()){
                    case UP:
                    case DOWN:
                        if(baseValve.multiBlockY == func_174877_v().func_177956_o()) connected[EnumFacing.UP.ordinal()] = true;
                        else connected[EnumFacing.DOWN.ordinal()] = true;
                        break;
                    case NORTH:
                    case SOUTH:
                        if(baseValve.multiBlockZ == func_174877_v().func_177952_p()) connected[EnumFacing.SOUTH.ordinal()] = true;
                        else connected[EnumFacing.NORTH.ordinal()] = true;
                        break;
                    case EAST:
                    case WEST:
                        if(baseValve.multiBlockX == func_174877_v().func_177958_n()) connected[EnumFacing.EAST.ordinal()] = true;
                        else connected[EnumFacing.WEST.ordinal()] = true;
                        break;
                }
            }
            for(int i = 0; i < 6; i++)
                if(!connected[i]) getAirHandler(null).airLeak(EnumFacing.func_82600_a(i));
        }
        super.func_73660_a();

        /*
         * FIXME pressure chamber particle effects not updating properly
         * if(!worldObj.isRemote){ pressureUpdateTimer++;
         * System.out.println("timer: " + pressureUpdateTimer);
         * if(pressureUpdateTimer > 60){ pressureUpdateTimer = 0;
         * updatePressuresToClient(); } }
         */

        if(multiBlockSize != 0 && !field_145850_b.field_72995_K) {
            ItemStack[] stacksInChamber = getStacksInChamber();
            isValidRecipeInChamber = false;
            isSufficientPressureInChamber = false;
            recipePressure = Float.MAX_VALUE;
            //simple recipes
            for(PressureChamberRecipe recipe : PressureChamberRecipe.chamberRecipes) {
                boolean isValidRecipeInChamberFlag = canBeCompressed(recipe, stacksInChamber);
                boolean isSufficientPressureInChamberFlag = recipe.pressure <= getPressure() && recipe.pressure > 0F || recipe.pressure >= getPressure() && recipe.pressure < 0F;
                if(isValidRecipeInChamberFlag) {
                    isValidRecipeInChamber = true;
                    if(Math.abs(recipe.pressure) < Math.abs(recipePressure)) {
                        recipePressure = recipe.pressure;
                    }
                }
                if(isSufficientPressureInChamberFlag) isSufficientPressureInChamber = true;
                if(isValidRecipeInChamberFlag && isSufficientPressureInChamberFlag && areEntitiesDoneMoving) {
                    double[] outputPosition = clearStacksInChamber(recipe.input);
                    giveOutput(recipe.output, outputPosition);
                }
            }
            //special recipes
            for(IPressureChamberRecipe recipe : PressureChamberRecipe.specialRecipes) {
                ItemStack[] removedStacks = recipe.isValidRecipe(stacksInChamber);
                boolean isValidRecipeInChamberFlag = removedStacks != null;
                boolean isSufficientPressureInChamberFlag = recipe.getCraftingPressure() <= getPressure() && recipe.getCraftingPressure() > 0F || recipe.getCraftingPressure() >= getPressure() && recipe.getCraftingPressure() < 0F;
                if(isValidRecipeInChamberFlag) {
                    isValidRecipeInChamber = true;
                    if(Math.abs(recipe.getCraftingPressure()) < Math.abs(recipePressure)) {
                        recipePressure = recipe.getCraftingPressure();
                    }
                }
                if(isSufficientPressureInChamberFlag) isSufficientPressureInChamber = true;
                if(isValidRecipeInChamberFlag && isSufficientPressureInChamberFlag && areEntitiesDoneMoving) {
                    double[] outputPosition = clearStacksInChamber(removedStacks);
                    giveOutput(recipe.craftRecipe(stacksInChamber, removedStacks), outputPosition);
                }
            }

            if(getPressure() > PneumaticValues.MAX_PRESSURE_LIVING_ENTITY) {
                AxisAlignedBB bbBox = new AxisAlignedBB(multiBlockX + 1, multiBlockY + 1, multiBlockZ + 1, multiBlockX + multiBlockSize - 1, multiBlockY + multiBlockSize - 1, multiBlockZ + multiBlockSize - 1);
                List<EntityLivingBase> entities = field_145850_b.func_72872_a(EntityLivingBase.class, bbBox);
                for(EntityLivingBase entity : entities) {
                    if(entity instanceof EntityVillager) {
                        EntityVillager villager = (EntityVillager)entity;
                        if(villager.func_70946_n() != Config.villagerMechanicID) {
                            villager.func_70938_b(Config.villagerMechanicID);
                            NBTTagCompound tag = new NBTTagCompound();
                            villager.func_70014_b(tag);
                            if(tag.func_74764_b("Offers")) {//reset the trade list
                                tag.func_82580_o("Offers");
                                villager.func_70037_a(tag);
                            }
                        }
                    }
                    entity.func_70097_a(DamageSourcePneumaticCraft.pressure, (int)(getPressure() * 2D));
                }
            }
        }

        // move entities to eachother.
        AxisAlignedBB bbBox = new AxisAlignedBB(multiBlockX, multiBlockY, multiBlockZ, multiBlockX + multiBlockSize, multiBlockY + multiBlockSize, multiBlockZ + multiBlockSize);
        List<EntityItem> entities = field_145850_b.func_72872_a(EntityItem.class, bbBox);
        areEntitiesDoneMoving = true; // set to true, set to false when one of
                                      // the entities is moving.
        for(int i = 0; i < entities.size() - 1; i++) {
            EntityItem lastEntity = entities.get(i);
            EntityItem entity = entities.get(i + 1);
            // XP Orb code snippet
            double d0 = 8.0D;
            double d1 = (lastEntity.field_70165_t - entity.field_70165_t) / d0;
            double d3 = (lastEntity.field_70161_v - entity.field_70161_v) / d0;
            double d4 = Math.sqrt(d1 * d1 + d3 * d3);
            double d5 = 1.0D - d4;

            if(d5 > 0.0D && d4 > 0.02D) {
                d5 *= d5;
                entity.field_70159_w += d1 / d4 * d5 * 0.01D;
                entity.field_70179_y += d3 / d4 * d5 * 0.01D;
                lastEntity.field_70159_w -= d1 / d4 * d5 * 0.01D;
                lastEntity.field_70179_y -= d3 / d4 * d5 * 0.01D;
                areEntitiesDoneMoving = false;
            }
        }

        boolean lifeUpgrade = getUpgrades(EnumUpgrade.ITEM_LIFE) > 0;
        if(lifeUpgrade && !field_145850_b.field_72995_K) {
            for(EntityItem entity : entities) {
                entity.field_70292_b--;
            }
        }

        // particles
        if(field_145850_b.field_72995_K && getPressure() > 0.2D) {
            int particles = (int)Math.pow(multiBlockSize - 2, 3);
            for(int i = 0; i < particles; i++) {
                if(rand.nextInt(Math.max(1, 8 - (int)(getPressure() * 2D))) == 0) {
                    double posX = multiBlockX + 1D + rand.nextDouble() * (multiBlockSize - 2D);
                    double posY = multiBlockY + 1D + rand.nextDouble() * (multiBlockSize - 2D);
                    double posZ = multiBlockZ + 1D + rand.nextDouble() * (multiBlockSize - 2D);
                    field_145850_b.func_175688_a(EnumParticleTypes.EXPLOSION_NORMAL, posX, posY, posZ, 0D, 0D, 0D);
                }
            }
        }

    }

    private boolean canBeCompressed(PressureChamberRecipe recipe, ItemStack[] items){
        for(Object in : recipe.input) {
            if(in != null) {
                int amount = 0;
                for(ItemStack item : items) {
                    if(item != null && PneumaticRecipeRegistry.isItemEqual(in, item)) amount += item.field_77994_a;
                }
                if(amount < PneumaticRecipeRegistry.getItemAmount(in)) return false;
            }
        }
        return true;
    }

    public ItemStack[] getStacksInChamber(){
        List<ItemStack> stackList = new ArrayList<ItemStack>(); // first make a
                                                                // list, convert
                                                                // it to an
                                                                // array
                                                                // afterwards.
        /*for(int i = 0; i < multiBlockSize - 2; i++) {
            for(int j = 0; j < multiBlockSize - 2; j++) {
                for(int k = 0; k < multiBlockSize - 2; k++) {
                    Block blockID = worldObj.getBlock(i + multiBlockX + 1, j + multiBlockY + 1, k + multiBlockZ + 1);
                    if(!blockID.isAir(worldObj, i + multiBlockX + 1, j + multiBlockY + 1, k + multiBlockZ + 1)) {
                        ItemStack stack = new ItemStack(blockID, 1, worldObj.getBlockMetadata(i + multiBlockX + 1, j + multiBlockY + 1, k + multiBlockZ + 1));
                        if(stack.getItem() != null) addStackToList(stackList, stack);//getItem() shouldn't ever be able to be null, but still some defence cuz https://github.com/MineMaarten/PneumaticCraft-API/issues/60
                    }
                }
            }
        }*/

        // add Item entities lying on the ground.
        AxisAlignedBB bbBox = new AxisAlignedBB(multiBlockX, multiBlockY, multiBlockZ, multiBlockX + multiBlockSize, multiBlockY + multiBlockSize, multiBlockZ + multiBlockSize);
        List<EntityItem> entities = field_145850_b.func_72872_a(EntityItem.class, bbBox);
        for(EntityItem entity : entities) {
            if(entity.field_70128_L) continue;
            stackList.add(entity.func_92059_d());
        }

        return stackList.toArray(new ItemStack[stackList.size()]); // return an
                                                                   // into an
                                                                   // array
                                                                   // converted
                                                                   // list
    }

    public double[] clearStacksInChamber(Object... stacksToClear){
        int[] stackSizes = new int[stacksToClear.length];
        for(int i = 0; i < stacksToClear.length; i++) {
            stackSizes[i] = PneumaticRecipeRegistry.getItemAmount(stacksToClear[i]);
        }
        // default the output position to the middle of the chamber.
        double[] outputPosition = new double[]{multiBlockX + multiBlockSize / 2D, multiBlockY + 1.2D, multiBlockZ + multiBlockSize / 2D};

        // get the in world EntityItems
        AxisAlignedBB bbBox = new AxisAlignedBB(multiBlockX, multiBlockY, multiBlockZ, multiBlockX + multiBlockSize, multiBlockY + multiBlockSize, multiBlockZ + multiBlockSize);
        List<EntityItem> entities = field_145850_b.func_72872_a(EntityItem.class, bbBox);
        for(EntityItem entity : entities) {
            if(entity.field_70128_L) continue;
            ItemStack entityStack = entity.func_92059_d();
            for(int l = 0; l < stacksToClear.length; l++) {
                if(PneumaticRecipeRegistry.isItemEqual(stacksToClear[l], entityStack) && stackSizes[l] > 0) {
                    outputPosition[0] = entity.field_70165_t;
                    outputPosition[1] = entity.field_70163_u;
                    outputPosition[2] = entity.field_70161_v;
                    int removedItems = Math.min(stackSizes[l], entityStack.field_77994_a);
                    stackSizes[l] -= removedItems;
                    entityStack.field_77994_a -= removedItems;
                    if(entityStack.field_77994_a <= 0) entity.func_70106_y();
                    break;
                }
            }
        }
        return outputPosition;
    }

    private void giveOutput(ItemStack[] output, double[] outputPosition){
        for(ItemStack iStack : output) {
            if(iStack.func_77973_b() == etchingAcid) {
                for(EntityPlayer player : field_145850_b.func_72872_a(EntityPlayer.class, new AxisAlignedBB(func_174877_v().func_177958_n() - 32, func_174877_v().func_177956_o() - 32, func_174877_v().func_177952_p() - 32, func_174877_v().func_177958_n() + 32, func_174877_v().func_177956_o() + 32, func_174877_v().func_177952_p() + 32))) {
                    AchievementHandler.giveAchievement(player, new ItemStack(etchingAcid));
                }
            }
            EntityItem item = new EntityItem(field_145850_b, outputPosition[0], outputPosition[1], outputPosition[2], iStack.func_77946_l());
            field_145850_b.func_72838_d(item);
        }
    }

    // NBT methods-----------------------------------------------
    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        setMultiBlockCoords(tag.func_74762_e("multiBlockSize"), tag.func_74762_e("multiBlockX"), tag.func_74762_e("multiBlockY"), tag.func_74762_e("multiBlockZ"));
        isSufficientPressureInChamber = tag.func_74767_n("sufPressure");
        isValidRecipeInChamber = tag.func_74767_n("validRecipe");
        recipePressure = tag.func_74760_g("recipePressure");
        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[func_70302_i_()];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }

        // Read in the accessory valves from NBT
        NBTTagList tagList2 = tag.func_150295_c("Valves", 10);
        nbtValveList.clear();
        for(int i = 0; i < tagList2.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList2.func_150305_b(i);
            nbtValveList.add(NBTUtil.getPos(tagCompound));
        }
        readNBT = true;
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        tag.func_74768_a("multiBlockX", multiBlockX);
        tag.func_74768_a("multiBlockY", multiBlockY);
        tag.func_74768_a("multiBlockZ", multiBlockZ);
        tag.func_74768_a("multiBlockSize", multiBlockSize);
        tag.func_74757_a("sufPressure", isSufficientPressureInChamber);
        tag.func_74757_a("validRecipe", isValidRecipeInChamber);
        tag.func_74776_a("recipePressure", recipePressure);
        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Items", tagList);

        // Write the accessory valve to NBT
        NBTTagList tagList2 = new NBTTagList();
        for(TileEntityPressureChamberValve valve : accessoryValves) {
            NBTTagCompound tagCompound = new NBTTagCompound();
            tagCompound.func_74768_a("getPos().getX()", valve.func_174877_v().func_177958_n());
            tagCompound.func_74768_a("getPos().getY()", valve.func_174877_v().func_177956_o());
            tagCompound.func_74768_a("getPos().getZ()", valve.func_174877_v().func_177952_p());
            tagList2.func_74742_a(tagCompound);
        }

        tag.func_74782_a("Valves", tagList2);
    }

    // Multi-block structure methods
    // --------------------------------------------------------

    public void onMultiBlockBreak(){
        dropInventory(field_145850_b, multiBlockX + multiBlockSize / 2D, multiBlockY + multiBlockSize / 2D, multiBlockZ + multiBlockSize / 2D);
        invalidateMultiBlock();
    }

    private void dropInventory(World world, double x, double y, double z){

        IInventory inventory = this;
        Random rand = new Random();
        for(int i = 0; i < inventory.func_70302_i_(); i++) {

            ItemStack itemStack = inventory.func_70301_a(i);

            if(itemStack != null && itemStack.field_77994_a > 0) {
                float dX = rand.nextFloat() * 0.8F - 0.4F;
                float dY = rand.nextFloat() * 0.8F - 0.4F;
                float dZ = rand.nextFloat() * 0.8F - 0.4F;

                EntityItem entityItem = new EntityItem(world, x + dX, y + dY, z + dZ, new ItemStack(itemStack.func_77973_b(), itemStack.field_77994_a, itemStack.func_77952_i()));

                if(itemStack.func_77942_o()) {
                    entityItem.func_92059_d().func_77982_d((NBTTagCompound)itemStack.func_77978_p().func_74737_b());
                }

                float factor = 0.05F;
                entityItem.field_70159_w = rand.nextGaussian() * factor;
                entityItem.field_70181_x = rand.nextGaussian() * factor + 0.2F;
                entityItem.field_70179_y = rand.nextGaussian() * factor;
                world.func_72838_d(entityItem);
                // itemStack.stackSize = 0;
            }
        }
        this.inventory = new ItemStack[4];
    }

    private void invalidateMultiBlock(){
        for(int x = 0; x < multiBlockSize; x++) {
            for(int y = 0; y < multiBlockSize; y++) {
                for(int z = 0; z < multiBlockSize; z++) {
                    TileEntity te = field_145850_b.func_175625_s(new BlockPos(x + multiBlockX, y + multiBlockY, z + multiBlockZ));
                    if(te instanceof TileEntityPressureChamberWall) {
                        TileEntityPressureChamberWall teWall = (TileEntityPressureChamberWall)te;
                        teWall.setCore(null); // Clear the base TE's, so that
                                              // the walls can be used in a new
                                              // MultiBlock
                    }
                }
            }
        }
        if(accessoryValves != null) {
            for(TileEntityPressureChamberValve valve : accessoryValves) {
                valve.setMultiBlockCoords(0, 0, 0, 0);
                if(valve != this) {
                    valve.accessoryValves.clear();
                    valve.sendDescriptionPacket();
                }
            }
            accessoryValves.clear();
        }
        sendDescriptionPacket();
    }

    public void setMultiBlockCoords(int size, int baseX, int baseY, int baseZ){
        multiBlockSize = size;
        multiBlockX = baseX;
        multiBlockY = baseY;
        multiBlockZ = baseZ;
        getAirHandler(null).setDefaultVolume(PneumaticValues.VOLUME_PRESSURE_CHAMBER + (multiBlockSize > 0 ? (int)Math.pow(multiBlockSize - 2, 3) * PneumaticValues.VOLUME_PRESSURE_CHAMBER_PER_EMPTY : 0));
        if(field_145850_b != null) {
            IBlockState state = field_145850_b.func_180495_p(func_174877_v());
            if(state.func_177230_c() == Blockss.pressureChamberValve) field_145850_b.func_180501_a(func_174877_v(), state.func_177226_a(BlockPressureChamberValve.FORMED, multiBlockSize > 0), 2);
        }

    }

    public static boolean checkIfProperlyFormed(World world, BlockPos pos){
        // ArrayList<int[]> blockList = getAccessoryTiles(new
        // ArrayList<int[]>(), world, x,y,z);

        for(int i = 3; i < 6; i++) {
            if(checkForShiftedCubeOfSize(i, world, pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p())) {
                return true;
            }
        }
        return false;
    }

    private static boolean checkForShiftedCubeOfSize(int size, World world, int baseX, int baseY, int baseZ){
        for(int wallX = 0; wallX < size; wallX++) {
            for(int wallY = 0; wallY < size; wallY++) {
                // check every possible configuration the block can be in.
                if(checkForCubeOfSize(size, world, baseX, baseY - wallY, baseZ - wallX)) return true;
                if(checkForCubeOfSize(size, world, baseX, baseY + wallY, baseZ + wallX)) return true;
                if(checkForCubeOfSize(size, world, baseX - wallX, baseY - wallY, baseZ)) return true;
                if(checkForCubeOfSize(size, world, baseX + wallX, baseY + wallY, baseZ)) return true;
                if(checkForCubeOfSize(size, world, baseX - wallX, baseY, baseZ - wallY)) return true;
                if(checkForCubeOfSize(size, world, baseX + wallX, baseY, baseZ + wallY)) return true;

                if(checkForCubeOfSize(size, world, baseX - size + 1, baseY - wallY, baseZ - wallX)) return true;
                if(checkForCubeOfSize(size, world, baseX - size + 1, baseY + wallY, baseZ + wallX)) return true;
                if(checkForCubeOfSize(size, world, baseX - wallX, baseY - wallY, baseZ - size + 1)) return true;
                if(checkForCubeOfSize(size, world, baseX + wallX, baseY + wallY, baseZ - size + 1)) return true;
                if(checkForCubeOfSize(size, world, baseX - wallX, baseY - size + 1, baseZ - wallY)) return true;
                if(checkForCubeOfSize(size, world, baseX + wallX, baseY - size + 1, baseZ + wallY)) return true;
            }
        }
        return false;
    }

    private static boolean checkForCubeOfSize(int size, World world, int baseX, int baseY, int baseZ){
        boolean validValveFound = false;
        for(int x = 0; x < size; x++) {
            for(int y = 0; y < size; y++) {
                for(int z = 0; z < size; z++) {
                    if(x != 0 && x != size - 1 && y != 0 && y != size - 1 && z != 0 && z != size - 1) continue;
                    BlockPos pos = new BlockPos(x + baseX, y + baseY, z + baseZ);
                    Block block = world.func_180495_p(pos).func_177230_c();
                    if(!(block instanceof IBlockPressureChamber)) {
                        return false;
                    } else if(block == Blockss.pressureChamberValve) {
                        boolean xMid = x != 0 && x != size - 1;
                        boolean yMid = y != 0 && y != size - 1;
                        boolean zMid = z != 0 && z != size - 1;
                        EnumFacing facing = ((TileEntityBase)world.func_175625_s(pos)).getRotation();
                        if(xMid && yMid && (facing == EnumFacing.NORTH || facing == EnumFacing.SOUTH) || xMid && zMid && (facing == EnumFacing.UP || facing == EnumFacing.DOWN) || yMid && zMid && (facing == EnumFacing.EAST || facing == EnumFacing.WEST)) {
                            validValveFound = true;
                        } else {
                            return false;
                        }
                    } else {// when blockID == wall/interface
                        TileEntity te = world.func_175625_s(pos);
                        if(te instanceof TileEntityPressureChamberWall && ((TileEntityPressureChamberWall)te).getCore() != null) {
                            return false;
                        }
                    }
                }
            }
        }

        // when the code makes it to here we've found a valid structure. it only
        // depends on whether there is a valid valve in the structure now.
        if(!validValveFound) return false;

        TileEntityPressureChamberValve teValve = null;
        List<TileEntityPressureChamberValve> valveList = new ArrayList<TileEntityPressureChamberValve>();
        for(int x = 0; x < size; x++) {
            for(int y = 0; y < size; y++) {
                for(int z = 0; z < size; z++) {
                    BlockPos pos = new BlockPos(x + baseX, y + baseY, z + baseZ);
                    TileEntity te = world.func_175625_s(pos);
                    if(te instanceof TileEntityPressureChamberValve) {
                        boolean xMid = x != 0 && x != size - 1;
                        boolean yMid = y != 0 && y != size - 1;
                        boolean zMid = z != 0 && z != size - 1;
                        EnumFacing facing = ((TileEntityBase)world.func_175625_s(pos)).getRotation();
                        if(xMid && yMid && (facing == EnumFacing.NORTH || facing == EnumFacing.SOUTH) || xMid && zMid && (facing == EnumFacing.UP || facing == EnumFacing.DOWN) || yMid && zMid && (facing == EnumFacing.EAST || facing == EnumFacing.WEST)) {
                            teValve = (TileEntityPressureChamberValve)te;
                            valveList.add(teValve);
                        }
                    }
                }
            }
        }
        if(teValve == null) return false;// this line shouldn't be triggered
                                         // ever.

        // put the list of all the valves in the structure in all the valve
        // TE's.
        for(TileEntityPressureChamberValve valve : valveList) {
            valve.accessoryValves = new ArrayList<TileEntityPressureChamberValve>(valveList);
            valve.sendDescriptionPacket();
        }

        // set the multi-block coords in the valve TE
        teValve.setMultiBlockCoords(size, baseX, baseY, baseZ);

        // set the redirections of right clicking and breaking a wall block to
        // the valve.
        for(int x = 0; x < size; x++) {
            for(int y = 0; y < size; y++) {
                for(int z = 0; z < size; z++) {
                    TileEntity te = world.func_175625_s(new BlockPos(x + baseX, y + baseY, z + baseZ));
                    if(te instanceof TileEntityPressureChamberWall) {
                        TileEntityPressureChamberWall teWall = (TileEntityPressureChamberWall)te;
                        teWall.setCore(teValve); // set base TE to the valve
                                                 // found.
                    }
                }
            }
        }

        teValve.sendDescriptionPacket();
        return true;
    }

    public boolean isCoordWithinChamber(World world, BlockPos pos){
        int x = pos.func_177958_n();
        int y = pos.func_177956_o();
        int z = pos.func_177952_p();
        if(x > multiBlockX && x < multiBlockX + multiBlockSize - 1 && y > multiBlockY && y < multiBlockY + multiBlockSize - 1 && z > multiBlockZ && z < multiBlockZ + multiBlockSize - 1) {
            return true;
        }
        return false;
    }

    // INVENTORY METHODS- && NBT
    // ------------------------------------------------------------

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){

        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){

        return inventory[slot];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        // super.setInventorySlotContents(slot, itemStack);
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
    }

    @Override
    public int func_70297_j_(){

        return 64;
    }

    @Override
    public String getName(){

        return Blockss.pressureChamberValve.func_149739_a();
    }

    @Override
    public boolean func_94041_b(int i, ItemStack itemstack){
        return canInsertUpgrade(i, itemstack);
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public float getMinWorkingPressure(){
        return isValidRecipeInChamber ? recipePressure : -1;
    }

}
