package pneumaticCraft.common.tileentity;

import java.util.Arrays;
import java.util.List;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.MovingObjectPosition;
import net.minecraftforge.fml.common.Optional;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.apache.commons.lang3.tuple.Pair;

import pneumaticCraft.api.tileentity.IAirHandler;
import pneumaticCraft.api.tileentity.IAirListener;
import pneumaticCraft.api.tileentity.IManoMeasurable;
import pneumaticCraft.common.block.tubes.IInfluenceDispersing;
import pneumaticCraft.common.block.tubes.ModuleRegistrator;
import pneumaticCraft.common.block.tubes.TubeModule;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.thirdparty.ModInteractionUtils;
import pneumaticCraft.common.thirdparty.mcmultipart.PartPressureTube;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.ModIds;
import pneumaticCraft.lib.PneumaticValues;

public class TileEntityPressureTube extends TileEntityPneumaticBase implements IAirListener, IManoMeasurable{
    @DescSynced
    public boolean[] sidesConnected = new boolean[6];
    public TubeModule[] modules = new TubeModule[6];

    private Object part;

    public TileEntityPressureTube(){
        super(PneumaticValues.DANGER_PRESSURE_PRESSURE_TUBE, PneumaticValues.MAX_PRESSURE_PRESSURE_TUBE, PneumaticValues.VOLUME_PRESSURE_TUBE);
    }

    public TileEntityPressureTube(float dangerPressurePressureTube, float maxPressurePressureTube,
            int volumePressureTube){
        super(dangerPressurePressureTube, maxPressurePressureTube, volumePressureTube);
    }

    public TileEntityPressureTube setPart(Object part){
        this.part = part;
        for(TubeModule module : modules) {
            if(module != null) module.shouldDrop = false;
        }
        return this;
    }

    @Override
    public void func_145839_a(NBTTagCompound nbt){
        super.func_145839_a(nbt);
        for(int i = 0; i < 6; i++) {
            sidesConnected[i] = nbt.func_74767_n("sideConnected" + i);
        }
    }

    @Override
    public void func_145841_b(NBTTagCompound nbt){
        super.func_145841_b(nbt);
        for(int i = 0; i < 6; i++) {
            nbt.func_74757_a("sideConnected" + i, sidesConnected[i]);
        }
    }

    @Override
    public void writeToPacket(NBTTagCompound tag){
        super.writeToPacket(tag);
        writeModulesToNBT(tag);
    }

    public void writeModulesToNBT(NBTTagCompound tag){
        NBTTagList moduleList = new NBTTagList();
        for(int i = 0; i < modules.length; i++) {
            if(modules[i] != null) {
                NBTTagCompound moduleTag = new NBTTagCompound();
                moduleTag.func_74778_a("type", modules[i].getType());
                modules[i].writeToNBT(moduleTag);
                moduleTag.func_74768_a("side", i);
                moduleList.func_74742_a(moduleTag);
            }
        }
        tag.func_74782_a("modules", moduleList);
    }

    @Override
    public void readFromPacket(NBTTagCompound tag){
        super.readFromPacket(tag);
        modules = new TubeModule[6];
        NBTTagList moduleList = tag.func_150295_c("modules", 10);
        for(int i = 0; i < moduleList.func_74745_c(); i++) {
            NBTTagCompound moduleTag = moduleList.func_150305_b(i);
            TubeModule module = ModuleRegistrator.getModule(moduleTag.func_74779_i("type"));
            module.readFromNBT(moduleTag);
            setModule(module, EnumFacing.func_82600_a(moduleTag.func_74762_e("side")));
        }
        if(field_145850_b != null && field_145850_b.field_72995_K) {
            rerenderChunk();
        }
    }

    @Override
    public void func_73660_a(){
        super.func_73660_a();

        for(TubeModule module : modules) {
            if(module != null) {
                module.shouldDrop = true;
                module.update();
            }
        }

        List<Pair<EnumFacing, IAirHandler>> teList = getAirHandler(null).getConnectedPneumatics();

        boolean hasModules = false;
        for(TubeModule module : modules) {
            if(module != null) {
                hasModules = true;
                break;
            }
        }
        if(!hasModules && teList.size() == 1 && !field_145850_b.field_72995_K) {
            for(Pair<EnumFacing, IAirHandler> entry : teList) {
                if(entry.getKey() != null && modules[entry.getKey().func_176734_d().ordinal()] == null && isConnectedTo(entry.getKey().func_176734_d())) getAirHandler(null).airLeak(entry.getKey().func_176734_d());
            }
        }
    }

    @Override
    public void onAirDispersion(IAirHandler handler, EnumFacing side, int amount){
        if(side != null) {
            int intSide = side/*.getOpposite()*/.ordinal();
            if(modules[intSide] instanceof IInfluenceDispersing) {
                ((IInfluenceDispersing)modules[intSide]).onAirDispersion(amount);
            }
        }
    }

    @Override
    public int getMaxDispersion(IAirHandler handler, EnumFacing side){
        if(side != null) {
            int intSide = side/*.getOpposite()*/.ordinal();
            if(modules[intSide] instanceof IInfluenceDispersing) {
                return ((IInfluenceDispersing)modules[intSide]).getMaxDispersion();
            }
        }
        return Integer.MAX_VALUE;
    }

    @Override
    public void addConnectedPneumatics(List<Pair<EnumFacing, IAirHandler>> pneumatics){}

    public void setModule(TubeModule module, EnumFacing side){
        if(module != null) {
            module.setDirection(side);
            module.setTube(this);
        }
        modules[side.ordinal()] = module;
        if(field_145850_b != null && !field_145850_b.field_72995_K) {
            if(part != null) updatePart();
            sendDescriptionPacket();
        }
    }

    @Override
    public boolean isConnectedTo(EnumFacing side){
        return (modules[side.ordinal()] == null || modules[side.ordinal()].isInline()) && (part == null || ModInteractionUtils.getInstance().isMultipartWiseConnected(part, side));
    }

    @Override
    public void onNeighborTileUpdate(){
        super.onNeighborTileUpdate();
        updateConnections();
        for(TubeModule module : modules) {
            if(module != null) module.onNeighborTileUpdate();
        }
    }

    @Override
    public void onNeighborBlockUpdate(){
        super.onNeighborBlockUpdate();
        updateConnections();
        for(TubeModule module : modules) {
            if(module != null) module.onNeighborBlockUpdate();
        }
    }

    public void updateConnections(){
        sidesConnected = new boolean[6];

        List<Pair<EnumFacing, IAirHandler>> connections = getAirHandler(null).getConnectedPneumatics();
        Arrays.fill(sidesConnected, false);
        for(Pair<EnumFacing, IAirHandler> entry : connections) {
            sidesConnected[entry.getKey().ordinal()] = true;
        }

        boolean hasModule = false;
        for(int i = 0; i < 6; i++) {
            if(modules[i] != null) {
                hasModule = true;
                break;
            }
        }

        int sidesCount = 0;
        for(boolean bool : sidesConnected) {
            if(bool) sidesCount++;
        }
        if(sidesCount == 1 && !hasModule) {
            for(int i = 0; i < 6; i++) {
                if(sidesConnected[i]) {
                    EnumFacing opposite = EnumFacing.func_82600_a(i).func_176734_d();
                    if(isConnectedTo(opposite)) sidesConnected[opposite.ordinal()] = true;
                    break;
                }
            }
        }
        for(int i = 0; i < 6; i++) {
            if(modules[i] != null && modules[i].isInline()) sidesConnected[i] = false;
        }
    }

    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox(){
        return new AxisAlignedBB(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), func_174877_v().func_177958_n() + 1, func_174877_v().func_177956_o() + 1, func_174877_v().func_177952_p() + 1);
    }

    @Override
    public void printManometerMessage(EntityPlayer player, List<String> text){
        MovingObjectPosition mop = PneumaticCraftUtils.getEntityLookedObject(player);
        if(mop != null && mop.hitInfo instanceof EnumFacing) {
            EnumFacing dir = (EnumFacing)mop.hitInfo;
            if(dir != null && modules[dir.ordinal()] != null) {
                modules[dir.ordinal()].addInfo(text);
            }
        }
    }

    @Override
    @Optional.Method(modid = ModIds.MCMP)
    public void sendDescriptionPacket(){
        if(part != null && !field_145850_b.field_72995_K) {
            ((PartPressureTube)part).sendUpdatePacket();
        }
        super.sendDescriptionPacket();
    }

    @Optional.Method(modid = ModIds.FMP)
    public void updatePart(){
        ((PartPressureTube)part).onNeighborTileChange(null);
    }
}
