package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import net.minecraft.entity.Entity;
import net.minecraft.entity.ai.EntityAITasks;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.network.EnumPacketDirection;
import net.minecraft.network.NetHandlerPlayServer;
import net.minecraft.network.NetworkManager;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;
import net.minecraft.world.WorldServer;
import net.minecraftforge.common.IExtendedEntityProperties;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidTank;
import net.minecraftforge.fluids.FluidTankInfo;
import net.minecraftforge.fluids.IFluidHandler;
import net.minecraftforge.fluids.IFluidTank;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.api.drone.DroneConstructingEvent;
import pneumaticCraft.api.drone.IPathNavigator;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.item.IProgrammable;
import pneumaticCraft.common.ai.DroneAIManager;
import pneumaticCraft.common.ai.FakePlayerItemInWorldManager;
import pneumaticCraft.common.ai.IDroneBase;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.entity.EntityProgrammableController;
import pneumaticCraft.common.entity.living.EntityDrone.DroneFakePlayer;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.LazySynced;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketSpawnParticle;
import pneumaticCraft.common.progwidgets.IProgWidget;
import pneumaticCraft.common.progwidgets.ProgWidgetDroneConditionEntity;
import pneumaticCraft.common.progwidgets.ProgWidgetEntityAttack;
import pneumaticCraft.common.progwidgets.ProgWidgetEntityExport;
import pneumaticCraft.common.progwidgets.ProgWidgetEntityImport;
import pneumaticCraft.common.progwidgets.ProgWidgetStandby;
import pneumaticCraft.common.progwidgets.ProgWidgetSuicide;
import pneumaticCraft.common.progwidgets.ProgWidgetTeleport;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.Log;

import com.mojang.authlib.GameProfile;

public class TileEntityProgrammableController extends TileEntityPneumaticBase implements ISidedInventory,
        IFluidHandler, IMinWorkingPressure, IDroneBase{
    private ItemStack[] inventory;

    private final int INVENTORY_SIZE = 5;

    private final FluidTank tank = new FluidTank(16000);
    private DroneAIManager aiManager;
    private DroneFakePlayer fakePlayer;
    private ItemStack[] droneItems;
    private final Map<String, IExtendedEntityProperties> properties = new HashMap<String, IExtendedEntityProperties>();
    private List<IProgWidget> progWidgets = new ArrayList<IProgWidget>();
    private final int[] redstoneLevels = new int[6];
    private String droneName = "";
    @DescSynced
    private double targetX, targetY, targetZ;
    @DescSynced
    @LazySynced
    private double curX, curY, curZ;
    public double oldCurX, oldCurY, oldCurZ;
    private EntityProgrammableController drone;
    @DescSynced
    private int diggingX, diggingY, diggingZ;
    private int dispenserUpgrades, speedUpgrades;

    private static final Set<Class<? extends IProgWidget>> WIDGET_BLACKLIST = new HashSet<Class<? extends IProgWidget>>();

    static {
        //TODO CC dep    WIDGET_BLACKLIST.add(ProgWidgetCC.class);
        WIDGET_BLACKLIST.add(ProgWidgetEntityAttack.class);
        WIDGET_BLACKLIST.add(ProgWidgetDroneConditionEntity.class);
        WIDGET_BLACKLIST.add(ProgWidgetStandby.class);
        WIDGET_BLACKLIST.add(ProgWidgetSuicide.class);
        WIDGET_BLACKLIST.add(ProgWidgetTeleport.class);
        WIDGET_BLACKLIST.add(ProgWidgetEntityExport.class);
        WIDGET_BLACKLIST.add(ProgWidgetEntityImport.class);
    }

    public TileEntityProgrammableController(){
        super(5, 7, 5000, 1, 2, 3, 4);
        inventory = new ItemStack[INVENTORY_SIZE];
        addApplicableUpgrade(EnumUpgrade.SPEED, EnumUpgrade.DISPENSER);
        MinecraftForge.EVENT_BUS.post(new DroneConstructingEvent(this));
    }

    @Override
    public void func_73660_a(){

        super.func_73660_a();

        oldCurX = curX;
        oldCurY = curY;
        oldCurZ = curZ;
        if(PneumaticCraftUtils.distBetween(func_174877_v(), targetX, targetY, targetZ) <= getSpeed()) {
            curX = targetX;
            curY = targetY;
            curZ = targetZ;
        } else {
            Vec3 vec = new Vec3(targetX - curX, targetY - curY, targetZ - curZ).func_72432_b();
            curX += vec.field_72450_a * getSpeed();
            curY += vec.field_72448_b * getSpeed();
            curZ += vec.field_72449_c * getSpeed();
        }

        if(!field_145850_b.field_72995_K) {
            getAIManager();
            if(field_145850_b.func_82737_E() % 40 == 0) {
                dispenserUpgrades = getUpgrades(EnumUpgrade.DISPENSER);
                speedUpgrades = getUpgrades(EnumUpgrade.SPEED);

                for(int i = getDroneSlots(); i < 36; i++) {
                    ItemStack stack = getFakePlayer().field_71071_by.func_70301_a(i);
                    if(stack != null) {
                        field_145850_b.func_72838_d(new EntityItem(field_145850_b, func_174877_v().func_177958_n() + 0.5, func_174877_v().func_177956_o() + 1.5, func_174877_v().func_177952_p() + 0.5, stack));
                        getFakePlayer().field_71071_by.func_70299_a(i, null);
                    }
                }

                tank.setCapacity((dispenserUpgrades + 1) * 16000);
                if(tank.getFluidAmount() > tank.getCapacity()) {
                    tank.getFluid().amount = tank.getCapacity();
                }
            }
            for(int i = 0; i < 4; i++) {
                getFakePlayer().field_71134_c.func_73075_a();
            }
            if(getPressure() >= getMinWorkingPressure()) {
                if(!aiManager.isIdling()) addAir(-10);
                aiManager.onUpdateTasks();
            }
        } else {
            if(drone == null || drone.field_70128_L) {
                drone = new EntityProgrammableController(field_145850_b, this);
                drone.field_70165_t = curX;
                drone.field_70163_u = curY;
                drone.field_70161_v = curZ;
                field_145850_b.func_72838_d(drone);
            }
            drone.func_70107_b(curX, curY, curZ);
            // drone.getMoveHelper().setMoveTo(curX, curY, curZ, 0);
            /*   drone.prevPosX = oldCurX;
            drone.prevPosY = oldCurY;
            drone.prevPosZ = oldCurZ;*/
            //drone.getMoveHelper().setMoveTo(curX, curY, curZ, getSpeed());
        }
    }

    @Override
    public void onDescUpdate(){
        super.onDescUpdate();
        if(drone != null) {
            drone.func_70106_y();
        }
    }

    private double getSpeed(){
        return Math.min(10, speedUpgrades) * 0.1 + 0.1;
    }

    private void initializeFakePlayer(){
        String playerUUID = null;
        String playerName = "Drone";
        fakePlayer = new DroneFakePlayer((WorldServer)field_145850_b, new GameProfile(playerUUID != null ? UUID.fromString(playerUUID) : null, playerName), new FakePlayerItemInWorldManager(field_145850_b, fakePlayer, this), this);
        fakePlayer.field_71135_a = new NetHandlerPlayServer(MinecraftServer.func_71276_C(), new NetworkManager(EnumPacketDirection.SERVERBOUND), fakePlayer);
        fakePlayer.field_71071_by = new InventoryPlayer(fakePlayer){
            private ItemStack oldStack;

            @Override
            public int func_70302_i_(){
                return getDroneSlots();
            }

            @Override
            public void func_70299_a(int slot, ItemStack stack){
                super.func_70299_a(slot, stack);
                if(slot == 0) {
                    if(oldStack != null) {
                        getFakePlayer().func_110140_aT().func_111148_a(oldStack.func_111283_C());
                    }

                    if(stack != null) {
                        getFakePlayer().func_110140_aT().func_111147_b(stack.func_111283_C());
                    }
                    oldStack = stack;
                }
            }
        };
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){

    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){
        return inventory.length + getDroneSlots();
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){
        return slot < 5 ? inventory[slot] : getFakePlayer().field_71071_by.func_70301_a(slot - 5);
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){

        if(slot < 5) {
            inventory[slot] = itemStack;
            if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
                itemStack.field_77994_a = func_70297_j_();
            }

            if(slot == 0) {
                if(itemStack != null && isProgrammableAndValidForDrone(this, itemStack)) {
                    progWidgets = TileEntityProgrammer.getProgWidgets(itemStack);
                    if(!field_145850_b.field_72995_K) getAIManager().setWidgets(progWidgets);
                } else {
                    progWidgets.clear();
                    targetX = func_174877_v().func_177958_n() + 0.5;
                    targetY = func_174877_v().func_177956_o() + 0.6;
                    targetZ = func_174877_v().func_177952_p() + 0.5;
                    boolean updateNeighbours = false;
                    for(int i = 0; i < redstoneLevels.length; i++) {
                        if(redstoneLevels[i] > 0) {
                            redstoneLevels[i] = 0;
                            updateNeighbours = true;
                        }
                    }
                    if(updateNeighbours) updateNeighbours();
                }
                getAIManager();
            }
        } else {
            getFakePlayer().field_71071_by.func_70299_a(slot - 5, itemStack);
        }
    }

    @Override
    public String getName(){

        return Blockss.programmableController.func_149739_a();
    }

    @Override
    public int func_70297_j_(){

        return 64;
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){

        super.func_145839_a(tag);

        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[INVENTORY_SIZE];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }

        tank.readFromNBT(tag.func_74775_l("tank"));

        NBTTagList droneItemTag = tag.func_150295_c("droneItems", 10);
        droneItems = new ItemStack[getDroneSlots()];
        for(int i = 0; i < droneItemTag.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = droneItemTag.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < droneItems.length) {
                droneItems[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }

        NBTTagList extendedList = tag.func_150295_c("extendedProperties", 10);
        for(int i = 0; i < extendedList.func_74745_c(); ++i) {
            NBTTagCompound propertyTag = extendedList.func_150305_b(i);
            String key = propertyTag.func_74779_i("key");
            IExtendedEntityProperties property = properties.get(key);
            if(property != null) {
                property.loadNBTData(propertyTag);
            } else {
                Log.warning("Extended entity property \"" + key + "\" doesn't exist in a Programmable Controller");
            }
        }
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){

        super.func_145841_b(tag);

        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Items", tagList);

        NBTTagCompound tankTag = new NBTTagCompound();
        tank.writeToNBT(tankTag);
        tag.func_74782_a("tank", tankTag);

        NBTTagList droneItems = new NBTTagList();
        for(int currentIndex = 0; currentIndex < getDroneSlots(); ++currentIndex) {
            if(getFakePlayer().field_71071_by.func_70301_a(currentIndex) != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                getFakePlayer().field_71071_by.func_70301_a(currentIndex).func_77955_b(tagCompound);
                droneItems.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("droneItems", droneItems);

        NBTTagList extendedList = new NBTTagList();
        for(Map.Entry<String, IExtendedEntityProperties> entry : properties.entrySet()) {
            NBTTagCompound propertyTag = new NBTTagCompound();
            propertyTag.func_74778_a("key", entry.getKey());
            entry.getValue().saveNBTData(propertyTag);
            extendedList.func_74742_a(propertyTag);
        }
        tag.func_74782_a("extendedProperties", extendedList);
    }

    @Override
    protected void onFirstServerUpdate(){
        super.onFirstServerUpdate();
        func_70299_a(0, func_70301_a(0));
    }

    private int getDroneSlots(){
        return field_145850_b != null && field_145850_b.field_72995_K ? 0 : Math.min(36, 1 + dispenserUpgrades);
    }

    @Override
    public boolean func_94041_b(int i, ItemStack itemstack){
        return i == 0 && isProgrammableAndValidForDrone(this, itemstack) || i != 0 && canInsertUpgrade(i, itemstack);
    }

    public static boolean isProgrammableAndValidForDrone(IDroneBase drone, ItemStack programmable){
        if(programmable != null && programmable.func_77973_b() instanceof IProgrammable && ((IProgrammable)programmable.func_77973_b()).canProgram(programmable) && ((IProgrammable)programmable.func_77973_b()).usesPieces(programmable)) {
            List<IProgWidget> widgets = TileEntityProgrammer.getProgWidgets(programmable);
            for(IProgWidget widget : widgets) {
                if(!drone.isProgramApplicable(widget)) return false;
            }
            return true;
        }
        return false;
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public int[] func_180463_a(EnumFacing var1){
        if(var1 == EnumFacing.UP) {
            return new int[]{0};
        } else {
            if(field_145850_b.field_72995_K) return new int[0];
            int[] mainInv = new int[func_70302_i_()];
            for(int i = 0; i < func_70302_i_(); i++) {
                mainInv[i] = i + 5;
            }
            return mainInv;
        }
    }

    @Override
    public boolean func_180462_a(int slot, ItemStack stack, EnumFacing p_102007_3_){
        return func_94041_b(slot, stack);
    }

    @Override
    public boolean func_180461_b(int p_102008_1_, ItemStack p_102008_2_, EnumFacing p_102008_3_){
        return true;
    }

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public float getMinWorkingPressure(){
        return 3;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox(){
        return INFINITE_EXTENT_AABB;
    }

    /*
     *                IDrone
     */

    @Override
    public float getPressure(ItemStack iStack){
        return getPressure();
    }

    @Override
    public void addAir(ItemStack iStack, int amount){
        addAir(amount);
    }

    @Override
    public float maxPressure(ItemStack iStack){
        return 7;
    }

    @Override
    public World func_145831_w(){
        return field_145850_b;
    }

    @Override
    public IFluidTank getTank(){
        return tank;
    }

    @Override
    public IInventory getInv(){
        return getFakePlayer().field_71071_by;
    }

    @Override
    public Vec3 getDronePos(){
        if(curX == 0 && curY == 0 && curZ == 0) {
            curX = func_174877_v().func_177958_n() + 0.5;
            curY = func_174877_v().func_177956_o() + 0.6;
            curZ = func_174877_v().func_177952_p() + 0.5;
            targetX = curX;
            targetY = curY;
            targetZ = curZ;
        }
        return new Vec3(curX, curY, curZ);
    }

    @Override
    public IPathNavigator getPathNavigator(){
        return new IPathNavigator(){

            @Override
            public boolean moveToXYZ(double x, double y, double z){
                if(isBlockValidPathfindBlock(new BlockPos(x, y, z))) {
                    targetX = x + 0.5;
                    targetY = y - 0.3;
                    targetZ = z + 0.5;
                    return true;
                } else {
                    return false;
                }
            }

            @Override
            public boolean moveToEntity(Entity entity){
                return moveToXYZ(entity.field_70165_t, entity.field_70163_u + 0.3, entity.field_70161_v);
            }

            @Override
            public boolean hasNoPath(){
                return targetX == curX && targetY == curY && targetZ == curZ;
            }

            @Override
            public boolean isGoingToTeleport(){
                return false;
            }

        };
    }

    @Override
    public void sendWireframeToClient(BlockPos pos){}

    @Override
    public EntityPlayerMP getFakePlayer(){
        if(fakePlayer == null) initializeFakePlayer();
        if(droneItems != null) {
            ItemStack[] copyDroneItems = droneItems;
            droneItems = null;
            for(int i = 0; i < copyDroneItems.length; i++) {
                fakePlayer.field_71071_by.func_70299_a(i, copyDroneItems[i]);
            }
        }
        return fakePlayer;
    }

    @Override
    public boolean isBlockValidPathfindBlock(BlockPos pos){
        return field_145850_b.func_175623_d(pos);
    }

    @Override
    public void dropItem(ItemStack stack){
        Vec3 pos = getDronePos();
        field_145850_b.func_72838_d(new EntityItem(field_145850_b, pos.field_72450_a, pos.field_72448_b, pos.field_72449_c, stack));
    }

    @Override
    public void setDugBlock(BlockPos pos){
        if(pos != null) {
            diggingX = pos.func_177958_n();
            diggingY = pos.func_177956_o();
            diggingZ = pos.func_177952_p();
        } else {
            diggingX = diggingY = diggingZ = 0;
        }
    }

    public BlockPos getDugPosition(){
        return diggingX != 0 || diggingY != 0 || diggingZ != 0 ? new BlockPos(diggingX, diggingY, diggingZ) : null;
    }

    @Override
    public List<IProgWidget> getProgWidgets(){
        return progWidgets;
    }

    @Override
    public void setActiveProgram(IProgWidget widget){}

    @Override
    public boolean isProgramApplicable(IProgWidget widget){
        return !WIDGET_BLACKLIST.contains(widget.getClass());
    }

    @Override
    public EntityAITasks getTargetAI(){
        return null;
    }

    @Override
    public IExtendedEntityProperties getProperty(String key){
        return properties.get(key);
    }

    @Override
    public void setProperty(String key, IExtendedEntityProperties property){
        properties.put(key, property);
    }

    @Override
    public void setEmittingRedstone(EnumFacing orientation, int emittingRedstone){
        redstoneLevels[orientation.ordinal()] = emittingRedstone;
        updateNeighbours();
    }

    public int getEmittingRedstone(EnumFacing direction){
        return redstoneLevels[direction.ordinal()];
    }

    @Override
    public void setName(String string){
        droneName = string;
        if(drone != null) drone.func_96094_a(droneName);
        if(inventory[0] != null) inventory[0].func_151001_c(string);
    }

    @Override
    public void setCarryingEntity(Entity entity){
        Log.warning("Drone AI setting carrying entity. However a Programmable Controller can't carry entities!");
        new Throwable().printStackTrace();
    }

    @Override
    public Entity getCarryingEntity(){
        return null;
    }

    @Override
    public boolean isAIOverriden(){
        return false;
    }

    @Override
    public void onItemPickupEvent(EntityItem curPickingUpEntity, int stackSize){

    }

    /*
     * Liquid handling
     */

    @Override
    public int fill(EnumFacing from, FluidStack resource, boolean doFill){
        return tank.fill(resource, doFill);
    }

    @Override
    public FluidStack drain(EnumFacing from, FluidStack resource, boolean doDrain){
        return tank.getFluid() != null && tank.getFluid().isFluidEqual(resource) ? tank.drain(resource.amount, doDrain) : null;
    }

    @Override
    public FluidStack drain(EnumFacing from, int maxDrain, boolean doDrain){
        return tank.drain(maxDrain, doDrain);
    }

    @Override
    public boolean canFill(EnumFacing from, Fluid fluid){
        return true;
    }

    @Override
    public boolean canDrain(EnumFacing from, Fluid fluid){
        return true;
    }

    @Override
    public FluidTankInfo[] getTankInfo(EnumFacing from){
        return new FluidTankInfo[]{new FluidTankInfo(tank)};
    }

    @Override
    public void overload(){
        for(int i = 0; i < 10; i++) {
            NetworkHandler.sendToAllAround(new PacketSpawnParticle(EnumParticleTypes.SMOKE_LARGE, func_174877_v().func_177958_n() + field_145850_b.field_73012_v.nextDouble(), func_174877_v().func_177956_o() + 1, func_174877_v().func_177952_p() + field_145850_b.field_73012_v.nextDouble(), 0, 0, 0), field_145850_b);
        }
    }

    @Override
    public DroneAIManager getAIManager(){
        if(aiManager == null && field_145850_b != null && !field_145850_b.field_72995_K) {
            aiManager = new DroneAIManager(this, new ArrayList<IProgWidget>());
            aiManager.setWidgets(getProgWidgets());
            aiManager.dontStopWhenEndReached();
        }
        return aiManager;
    }

    @Override
    public void updateLabel(){}

    @Override
    public void addDebugEntry(String message){

    }

    @Override
    public void addDebugEntry(String message, BlockPos pos){

    }
}
