package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import pneumaticCraft.api.item.IProgrammable;
import pneumaticCraft.client.AreaShowManager;
import pneumaticCraft.common.NBTUtil;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.item.ItemProgrammingPuzzle;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.progwidgets.IAreaProvider;
import pneumaticCraft.common.progwidgets.IProgWidget;
import pneumaticCraft.common.progwidgets.IVariableWidget;
import pneumaticCraft.common.progwidgets.WidgetRegistrator;
import pneumaticCraft.common.util.IOHelper;
import pneumaticCraft.common.util.PneumaticCraftUtils;

public class TileEntityProgrammer extends TileEntityBase implements IInventory, IGUITextFieldSensitive{
    public final List<IProgWidget> progWidgets = new ArrayList<IProgWidget>();
    @GuiSynced
    public int redstoneMode;
    private ItemStack[] inventory = new ItemStack[1];

    public static final int PROGRAM_SLOT = 0;

    //Client side variables that are used to prevent resetting.
    public int translatedX, translatedY, zoomState;
    public boolean showInfo = true, showFlow = true;
    @GuiSynced
    public boolean canUndo, canRedo;
    private NBTTagList history = new NBTTagList();//Used to undo/redo.
    private int historyIndex;

    public TileEntityProgrammer(){
        saveToHistory();
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        redstoneMode = tag.func_74762_e("redstoneMode");

        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[func_70302_i_()];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }
        history = tag.func_150295_c("history", 10);
        if(history.func_74745_c() == 0) saveToHistory();
    }

    @Override
    public void readFromPacket(NBTTagCompound tag){
        super.readFromPacket(tag);
        readProgWidgetsFromNBT(tag);
    }

    @Override
    public void func_145843_s(){
        super.func_145843_s();
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        tag.func_74768_a("redstoneMode", redstoneMode);

        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Items", tagList);
        tag.func_74782_a("history", history);
    }

    @Override
    public void writeToPacket(NBTTagCompound tag){
        super.writeToPacket(tag);
        writeProgWidgetsToNBT(tag);
    }

    public void readProgWidgetsFromNBT(NBTTagCompound tag){
        progWidgets.clear();
        getWidgetsFromNBT(tag, progWidgets);
    }

    public void writeProgWidgetsToNBT(NBTTagCompound tag){
        setWidgetsToNBT(progWidgets, tag);
    }

    public static List<IProgWidget> getWidgetsFromNBT(NBTTagCompound tag){
        List<IProgWidget> progWidgets = new ArrayList<IProgWidget>();
        getWidgetsFromNBT(tag, progWidgets);
        return progWidgets;
    }

    public static void getWidgetsFromNBT(NBTTagCompound tag, List<IProgWidget> progWidgets){
        NBTTagList widgetTags = tag.func_150295_c("widgets", 10);
        for(int i = 0; i < widgetTags.func_74745_c(); i++) {
            NBTTagCompound widgetTag = widgetTags.func_150305_b(i);
            String widgetName = widgetTag.func_74779_i("name");
            for(IProgWidget widget : WidgetRegistrator.registeredWidgets) {
                if(widgetName.equals(widget.getWidgetString())) {//create the right progWidget for the given id tag.
                    IProgWidget addedWidget = widget.copy();
                    addedWidget.readFromNBT(widgetTag);
                    progWidgets.add(addedWidget);
                    break;
                }
            }
        }
        updatePuzzleConnections(progWidgets);
    }

    public static void setWidgetsToNBT(List<IProgWidget> widgets, NBTTagCompound tag){
        NBTTagList widgetTags = new NBTTagList();
        for(IProgWidget widget : widgets) {
            NBTTagCompound widgetTag = new NBTTagCompound();
            widget.writeToNBT(widgetTag);
            widgetTags.func_74742_a(widgetTag);
        }
        tag.func_74782_a("widgets", widgetTags);
    }

    public static void updatePuzzleConnections(List<IProgWidget> progWidgets){
        for(IProgWidget widget : progWidgets) {
            widget.setParent(null);
            Class<? extends IProgWidget>[] parameters = widget.getParameters();
            if(parameters != null) {
                for(int i = 0; i < parameters.length * 2; i++) {
                    widget.setParameter(i, null);
                }
            }
            if(widget.hasStepOutput()) widget.setOutputWidget(null);
        }
        for(IProgWidget checkedWidget : progWidgets) {
            //check for connection to the right of the checked widget.
            Class<? extends IProgWidget>[] parameters = checkedWidget.getParameters();
            if(parameters != null) {
                for(IProgWidget widget : progWidgets) {
                    if(widget != checkedWidget && checkedWidget.getX() + checkedWidget.getWidth() / 2 == widget.getX()) {
                        for(int i = 0; i < parameters.length; i++) {
                            if(checkedWidget.canSetParameter(i) && parameters[i] == widget.returnType() && checkedWidget.getY() + i * 11 == widget.getY()) {
                                checkedWidget.setParameter(i, widget);
                                widget.setParent(checkedWidget);
                            }
                        }
                    }
                }
            }

            //check for connection to the bottom of the checked widget.
            if(checkedWidget.hasStepOutput()) {
                for(IProgWidget widget : progWidgets) {
                    if(widget.hasStepInput() && widget.getX() == checkedWidget.getX() && widget.getY() == checkedWidget.getY() + checkedWidget.getHeight() / 2) {
                        checkedWidget.setOutputWidget(widget);
                    }
                }
            }
        }

        //go again for the blacklist (as those are mirrored)
        for(IProgWidget checkedWidget : progWidgets) {
            if(checkedWidget.returnType() == null) { //if it's a program (import/export inventory, attack entity) rather than a parameter (area, item filter).
                Class<? extends IProgWidget>[] parameters = checkedWidget.getParameters();
                if(parameters != null) {
                    for(int i = 0; i < parameters.length; i++) {
                        if(checkedWidget.canSetParameter(i)) {
                            for(IProgWidget widget : progWidgets) {
                                if(parameters[i] == widget.returnType()) {
                                    if(widget != checkedWidget && widget.getX() + widget.getWidth() / 2 == checkedWidget.getX() && widget.getY() == checkedWidget.getY() + i * 11) {
                                        IProgWidget root = widget;
                                        while(root.getParent() != null) {
                                            root = root.getParent();
                                        }
                                        checkedWidget.setParameter(i + parameters.length, root);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        switch(buttonID){
            case 0:
                if(++redstoneMode > 1) redstoneMode = 0;
                break;
            case 1:
                NBTTagCompound tag = inventory[PROGRAM_SLOT] != null ? inventory[PROGRAM_SLOT].func_77978_p() : null;
                if(tag != null) readProgWidgetsFromNBT(tag);
                else progWidgets.clear();
                break;
            case 2:
                tryProgramDrone(player);
                break;
            case 9:
                undo();
                break;
            case 10:
                redo();
                break;
        }
        sendDescriptionPacket();
    }

    @Override
    public void setText(int textFieldID, String text){
        if(textFieldID == 0 && inventory[PROGRAM_SLOT] != null) {
            inventory[PROGRAM_SLOT].func_151001_c(text);
        }
    }

    @Override
    public String getText(int textFieldID){
        return inventory[PROGRAM_SLOT] != null ? inventory[PROGRAM_SLOT].func_82833_r() : "";
    }

    private void tryProgramDrone(EntityPlayer player){
        if(inventory[PROGRAM_SLOT] != null) {
            if(player == null || !player.field_71075_bZ.field_75098_d) {
                List<ItemStack> requiredStacks = getRequiredPuzzleStacks();
                for(ItemStack stack : requiredStacks) {
                    if(!hasEnoughPuzzleStacks(player, stack)) return;
                }
                for(ItemStack stack : requiredStacks) {
                    int left = stack.field_77994_a;
                    if(player != null) {
                        for(int i = 0; i < player.field_71071_by.func_70302_i_(); i++) {
                            if(PneumaticCraftUtils.areStacksEqual(stack, player.field_71071_by.func_70301_a(i), true, true, false, false)) {
                                left -= player.field_71071_by.func_70298_a(i, left).field_77994_a;
                                if(left <= 0) break;
                            }
                        }
                    }
                    if(left > 0) {
                        for(EnumFacing d : EnumFacing.field_82609_l) {
                            IInventory neighbor = IOHelper.getInventoryForTE(func_145831_w().func_175625_s(func_174877_v().func_177972_a(d)));
                            for(int slot : IOHelper.getAccessibleSlotsForInventory(neighbor, d.func_176734_d())) {
                                if(IOHelper.canExtractItemFromInventory(neighbor, stack, slot, d.func_176734_d())) {
                                    ItemStack neighborStack = neighbor.func_70301_a(slot);
                                    if(PneumaticCraftUtils.areStacksEqual(neighborStack, stack, true, true, false, false)) {
                                        left -= neighbor.func_70298_a(slot, left).field_77994_a;
                                        if(left <= 0) break;
                                    }
                                }
                            }
                        }
                    }
                }
                List<ItemStack> returnedStacks = getReturnedPuzzleStacks();
                for(ItemStack stack : returnedStacks) {
                    for(EnumFacing d : EnumFacing.field_82609_l) {
                        IInventory neighbor = IOHelper.getInventoryForTE(func_145831_w().func_175625_s(func_174877_v().func_177972_a(d)));
                        stack = IOHelper.insert(neighbor, stack, d.func_176734_d(), false);
                        if(stack == null) break;
                    }
                    if(player != null && stack != null) {
                        if(!player.field_71071_by.func_70441_a(stack)) {
                            player.func_71019_a(stack.func_77946_l(), false);
                            stack = null;
                        }
                    }
                    if(stack != null) {
                        field_145850_b.func_72838_d(new EntityItem(field_145850_b, func_174877_v().func_177958_n() + 0.5, func_174877_v().func_177956_o() + 1.5, func_174877_v().func_177952_p() + 0.5, stack));
                    }
                }
            }
            NBTTagCompound tag = inventory[PROGRAM_SLOT].func_77978_p();
            if(tag == null) {
                tag = new NBTTagCompound();
                inventory[PROGRAM_SLOT].func_77982_d(tag);
            }
            writeProgWidgetsToNBT(tag);
        }
    }

    public List<ItemStack> getRequiredPuzzleStacks(){
        List<ItemStack> stacks = new ArrayList<ItemStack>();
        if(((IProgrammable)inventory[PROGRAM_SLOT].func_77973_b()).usesPieces(inventory[PROGRAM_SLOT])) {
            Map<Integer, Integer> tePieces = getPuzzleSummary(progWidgets);
            Map<Integer, Integer> dronePieces = getPuzzleSummary(getProgWidgets(inventory[PROGRAM_SLOT]));
            for(Integer includedWidget : tePieces.keySet()) {
                Integer existingWidgets = dronePieces.get(includedWidget);
                if(existingWidgets != null) {
                    Integer neededWidgets = tePieces.get(includedWidget);
                    if(neededWidgets > existingWidgets) {
                        ItemStack stack = ItemProgrammingPuzzle.getStackForColor(includedWidget);
                        stack.field_77994_a = (neededWidgets - existingWidgets) * inventory[PROGRAM_SLOT].field_77994_a;
                        stacks.add(stack);
                    }
                } else {
                    ItemStack stack = ItemProgrammingPuzzle.getStackForColor(includedWidget);
                    stack.field_77994_a = tePieces.get(includedWidget) * inventory[PROGRAM_SLOT].field_77994_a;
                    stacks.add(stack);
                }
            }
        }
        return stacks;
    }

    public List<ItemStack> getReturnedPuzzleStacks(){
        List<ItemStack> stacks = new ArrayList<ItemStack>();
        if(((IProgrammable)inventory[PROGRAM_SLOT].func_77973_b()).usesPieces(inventory[PROGRAM_SLOT])) {
            Map<Integer, Integer> tePieces = getPuzzleSummary(progWidgets);
            Map<Integer, Integer> dronePieces = getPuzzleSummary(getProgWidgets(inventory[PROGRAM_SLOT]));

            for(Integer availableWidget : dronePieces.keySet()) {
                Integer requiredWidget = tePieces.get(availableWidget);
                if(requiredWidget != null) {
                    Integer availableWidgets = dronePieces.get(availableWidget);
                    if(availableWidgets > requiredWidget) {
                        ItemStack stack = ItemProgrammingPuzzle.getStackForColor(availableWidget);
                        stack.field_77994_a = (availableWidgets - requiredWidget) * inventory[PROGRAM_SLOT].field_77994_a;
                        while(stack.field_77994_a > stack.func_77976_d()) {
                            stacks.add(stack.func_77979_a(stack.func_77976_d()));
                        }
                        stacks.add(stack);
                    }
                } else {
                    ItemStack stack = ItemProgrammingPuzzle.getStackForColor(availableWidget);
                    stack.field_77994_a = dronePieces.get(availableWidget) * inventory[PROGRAM_SLOT].field_77994_a;
                    while(stack.field_77994_a > stack.func_77976_d()) {
                        stacks.add(stack.func_77979_a(stack.func_77976_d()));
                    }
                    stacks.add(stack);
                }
            }
        }
        return stacks;
    }

    public static List<IProgWidget> getProgWidgets(ItemStack iStack){
        if(NBTUtil.hasTag(iStack, "widgets")) {
            return TileEntityProgrammer.getWidgetsFromNBT(iStack.func_77978_p());
        } else {
            return new ArrayList<IProgWidget>();
        }
    }

    public boolean hasEnoughPuzzleStacks(EntityPlayer player, ItemStack stack){
        int amountLeft = stack.field_77994_a;
        if(player != null) {
            for(int i = 0; i < player.field_71071_by.func_70302_i_(); i++) {
                ItemStack playerStack = player.field_71071_by.func_70301_a(i);
                if(PneumaticCraftUtils.areStacksEqual(playerStack, stack, true, true, false, false)) {
                    amountLeft -= playerStack.field_77994_a;
                    if(amountLeft <= 0) return true;
                }
            }
        }

        for(EnumFacing d : EnumFacing.field_82609_l) {
            IInventory neighbor = IOHelper.getInventoryForTE(func_145831_w().func_175625_s(func_174877_v().func_177972_a(d)));
            for(int slot : IOHelper.getAccessibleSlotsForInventory(neighbor, d.func_176734_d())) {
                if(IOHelper.canExtractItemFromInventory(neighbor, stack, slot, d.func_176734_d())) {
                    ItemStack neighborStack = neighbor.func_70301_a(slot);
                    if(PneumaticCraftUtils.areStacksEqual(neighborStack, stack, true, true, false, false)) {
                        amountLeft -= neighborStack.field_77994_a;
                        if(amountLeft <= 0) return true;
                    }
                }
            }
        }

        return false;
    }

    public static Map<Integer, Integer> getPuzzleSummary(List<IProgWidget> widgets){
        Map<Integer, Integer> map = new HashMap<Integer, Integer>();
        for(IProgWidget widget : widgets) {
            if(widget.getCraftingColorIndex() != -1) {
                if(!map.containsKey(widget.getCraftingColorIndex())) {
                    map.put(widget.getCraftingColorIndex(), 1);
                } else {
                    map.put(widget.getCraftingColorIndex(), map.get(widget.getCraftingColorIndex()) + 1);
                }
            }
        }
        return map;
    }

    /**
     * Returns a set with all variables that are used in the program.
     * @return
     */
    public Set<String> getAllVariables(){
        Set<String> variables = new HashSet<String>();
        for(IProgWidget widget : progWidgets) {
            if(widget instanceof IVariableWidget) ((IVariableWidget)widget).addVariables(variables);
        }
        variables.remove("");
        return variables;
    }

    // INVENTORY METHODS

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){
        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){
        return inventory[slot];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
        if(redstoneMode == 1 && slot == 0 && itemStack != null) {
            tryProgramDrone(null);
        }
    }

    @Override
    public String getName(){
        return Blockss.programmer.func_149739_a();
    }

    @Override
    public int func_70297_j_(){

        return 64;
    }

    @Override
    public boolean func_94041_b(int i, ItemStack itemstack){
        if(i == PROGRAM_SLOT && itemstack != null && (!(itemstack.func_77973_b() instanceof IProgrammable) || !((IProgrammable)itemstack.func_77973_b()).canProgram(itemstack))) return false;
        return true;
    }

    @Override
    public void func_73660_a(){
        super.func_73660_a();
    }

    @Override
    protected void onFirstServerUpdate(){
        super.onFirstServerUpdate();

    }

    public boolean previewArea(int widgetX, int widgetY){
        for(IProgWidget w : progWidgets) {
            if(w.getX() == widgetX && w.getY() == widgetY && w instanceof IAreaProvider) {
                Set<BlockPos> area = new HashSet<BlockPos>();
                ((IAreaProvider)w).getArea(area);
                AreaShowManager.getInstance().showArea(area, 0x00FF00, this);
            }
        }
        return true;
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    public void saveToHistory(){
        NBTTagCompound tag = new NBTTagCompound();
        writeProgWidgetsToNBT(tag);
        if(history.func_74745_c() == 0 || !history.func_150305_b(historyIndex).equals(tag)) {
            while(history.func_74745_c() > historyIndex + 1) {
                history.func_74744_a(historyIndex + 1);
            }
            history.func_74742_a(tag);
            if(history.func_74745_c() > 20) history.func_74744_a(0);//Only save up to 20 steps back. 
            historyIndex = history.func_74745_c() - 1;
            updateUndoRedoState();
        }
    }

    public void undo(){
        if(canUndo) {
            historyIndex--;
            readProgWidgetsFromNBT(history.func_150305_b(historyIndex));
            updateUndoRedoState();
        }
    }

    public void redo(){
        if(canRedo) {
            historyIndex++;
            readProgWidgetsFromNBT(history.func_150305_b(historyIndex));
            updateUndoRedoState();
        }
    }

    private void updateUndoRedoState(){
        canUndo = historyIndex > 0;
        canRedo = historyIndex < history.func_74745_c() - 1;
    }
}
