package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidTank;
import net.minecraftforge.fluids.FluidTankInfo;
import net.minecraftforge.fluids.IFluidHandler;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.api.PneumaticRegistry;
import pneumaticCraft.api.heat.IHeatExchangerLogic;
import pneumaticCraft.api.tileentity.IHeatExchanger;
import pneumaticCraft.common.fluid.Fluids;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.network.LazySynced;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketSpawnParticle;
import pneumaticCraft.lib.PneumaticValues;

public class TileEntityRefinery extends TileEntityBase implements IFluidHandler, IHeatExchanger, IRedstoneControlled,
        IComparatorSupport{

    @GuiSynced
    @DescSynced
    @LazySynced
    private final FluidTank oilTank = new FluidTank(PneumaticValues.NORMAL_TANK_CAPACITY);
    @GuiSynced
    @DescSynced
    @LazySynced
    private final FluidTank outputTank = new FluidTank(PneumaticValues.NORMAL_TANK_CAPACITY);
    @GuiSynced
    private final IHeatExchangerLogic heatExchanger = PneumaticRegistry.getInstance().getHeatRegistry().getHeatExchangerLogic();
    @DescSynced
    private int oilTankAmount, outputTankAmount;//amount divided by 100 to decrease network load.
    @GuiSynced
    private int redstoneMode;
    private int workTimer = 0;
    private int comparatorValue;

    /**
     * The amounts of LPG, Gasoline, Kerosine and Diesel produced per 10mL Oil, depending on how many refineries are stacked on top of eachother.
     * Type \ Refineries | 2 | 3 | 4
     * ------------------------------
     * LPG               | 2 | 2 | 2
     * Gasoline          | - | - | 3
     * Kerosine          | - | 3 | 3
     * Diesel            | 4 | 2 | 2
     */
    public static final int[][] REFINING_TABLE = new int[][]{{4, 0, 0, 2}, {2, 3, 0, 2}, {2, 3, 3, 2}};
    private final Fluid[] refiningFluids = getRefiningFluids();

    public TileEntityRefinery(){

    }

    public static Fluid[] getRefiningFluids(){
        return new Fluid[]{Fluids.diesel, Fluids.kerosene, Fluids.gasoline, Fluids.lpg};
    }

    @Override
    public void func_73660_a(){
        super.func_73660_a();
        if(!field_145850_b.field_72995_K) {
            oilTankAmount = oilTank.getFluidAmount() / 100;
            outputTankAmount = outputTank.getFluidAmount() / 100;

            if(isMaster()) {
                List<TileEntityRefinery> refineries = getRefineries();
                if(redstoneAllows() && oilTank.getFluidAmount() >= 10) {
                    if(refineries.size() > 1 && refineries.size() <= refiningFluids.length && refine(refineries, true)) {
                        int progress = Math.max(0, ((int)heatExchanger.getTemperature() - 343) / 30);
                        progress = Math.min(5, progress);
                        heatExchanger.addHeat(-progress * 1);
                        workTimer += progress;
                        while(workTimer >= 20 && oilTank.getFluidAmount() >= 10) {
                            workTimer -= 20;

                            refine(refineries, false);
                            oilTank.drain(10, true);
                            for(int i = 0; i < 5; i++)
                                NetworkHandler.sendToAllAround(new PacketSpawnParticle(EnumParticleTypes.SMOKE_LARGE, func_174877_v().func_177958_n() + field_145850_b.field_73012_v.nextDouble(), func_174877_v().func_177956_o() + refineries.size(), func_174877_v().func_177952_p() + field_145850_b.field_73012_v.nextDouble(), 0, 0, 0), field_145850_b);

                        }
                    } else {
                        workTimer = 0;
                    }
                }
                updateComparatorValue(refineries);
            }
        }
    }

    private List<TileEntityRefinery> getRefineries(){
        List<TileEntityRefinery> refineries = new ArrayList<TileEntityRefinery>();
        refineries.add(this);
        TileEntityRefinery refinery = this;
        while(refinery.getTileCache()[EnumFacing.UP.ordinal()].getTileEntity() instanceof TileEntityRefinery) {
            refinery = (TileEntityRefinery)refinery.getTileCache()[EnumFacing.UP.ordinal()].getTileEntity();
            refineries.add(refinery);
        }
        return refineries;
    }

    public boolean refine(List<TileEntityRefinery> refineries, boolean simulate){
        int[] outputTable = REFINING_TABLE[refineries.size() - 2];

        int i = 0;
        for(TileEntityRefinery refinery : refineries) {
            while(outputTable[i] == 0)
                i++;
            if(outputTable[i] != refinery.outputTank.fill(new FluidStack(refiningFluids[i], outputTable[i]), !simulate)) return false;
            i++;
        }
        return true;
    }

    public TileEntityRefinery getMasterRefinery(){
        TileEntityRefinery master = this;
        while(master.getTileCache()[EnumFacing.DOWN.ordinal()].getTileEntity() instanceof TileEntityRefinery) {
            master = (TileEntityRefinery)master.getTileCache()[EnumFacing.DOWN.ordinal()].getTileEntity();
        }
        return master;
    }

    private boolean isMaster(){
        return getMasterRefinery() == this;
    }

    @Override
    public boolean redstoneAllows(){
        if(field_145850_b.field_72995_K) onNeighborBlockUpdate();
        boolean isPoweredByRedstone = poweredRedstone > 0;

        TileEntityRefinery refinery = this;
        while(poweredRedstone == 0 && refinery.getTileCache()[EnumFacing.UP.ordinal()].getTileEntity() instanceof TileEntityRefinery) {
            refinery = (TileEntityRefinery)refinery.getTileCache()[EnumFacing.UP.ordinal()].getTileEntity();
            refinery.onNeighborBlockUpdate();
            isPoweredByRedstone = refinery.poweredRedstone > 0;
        }

        switch(((IRedstoneControl)this).getRedstoneMode()){
            case 0:
                return true;
            case 1:
                return isPoweredByRedstone;
            case 2:
                return !isPoweredByRedstone;
        }
        return false;
    }

    @Override
    public int fill(EnumFacing from, FluidStack resource, boolean doFill){
        if(canFill(from, resource != null ? resource.getFluid() : null)) {
            if(isMaster()) {
                return oilTank.fill(resource, doFill);
            } else {
                return getMasterRefinery().fill(from, resource, doFill);
            }
        } else {
            return 0;
        }
    }

    @Override
    public FluidStack drain(EnumFacing from, FluidStack resource, boolean doDrain){
        return outputTank.getFluid() != null && outputTank.getFluid().isFluidEqual(resource) ? outputTank.drain(resource.amount, doDrain) : null;
    }

    @Override
    public FluidStack drain(EnumFacing from, int maxDrain, boolean doDrain){
        return outputTank.drain(maxDrain, doDrain);
    }

    @Override
    public boolean canFill(EnumFacing from, Fluid fluid){
        return Fluids.areFluidsEqual(fluid, Fluids.oil);
    }

    @Override
    public boolean canDrain(EnumFacing from, Fluid fluid){
        return true;
    }

    @Override
    public FluidTankInfo[] getTankInfo(EnumFacing from){
        return new FluidTankInfo[]{new FluidTankInfo(getMasterRefinery().oilTank), new FluidTankInfo(outputTank)};
    }

    @SideOnly(Side.CLIENT)
    public FluidTank getOilTank(){
        return oilTank;
    }

    @SideOnly(Side.CLIENT)
    public FluidTank getOutputTank(){
        return outputTank;
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);

        NBTTagCompound tankTag = new NBTTagCompound();
        oilTank.writeToNBT(tankTag);
        tag.func_74782_a("oilTank", tankTag);

        tankTag = new NBTTagCompound();
        outputTank.writeToNBT(tankTag);
        tag.func_74782_a("outputTank", tankTag);

        tag.func_74774_a("redstoneMode", (byte)redstoneMode);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        oilTank.readFromNBT(tag.func_74775_l("oilTank"));
        outputTank.readFromNBT(tag.func_74775_l("outputTank"));
        redstoneMode = tag.func_74771_c("redstoneMode");
    }

    @Override
    public IHeatExchangerLogic getHeatExchangerLogic(EnumFacing side){
        return heatExchanger;
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID == 0) {
            redstoneMode++;
            if(redstoneMode > 2) redstoneMode = 0;
        }
    }

    public void updateComparatorValue(List<TileEntityRefinery> refineries){
        int value;
        if(oilTank.getFluidAmount() < 10 || refineries.size() < 2 || refineries.size() > refiningFluids.length) {
            value = 0;
        } else {
            value = refine(refineries, true) ? 15 : 0;
        }
        if(value != comparatorValue) {
            comparatorValue = value;
            updateNeighbours();
        }
    }

    @Override
    public int getComparatorValue(){
        return getMasterRefinery().comparatorValue;
    }
}
