package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.ChatComponentTranslation;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import pneumaticCraft.PneumaticCraft;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.client.render.RenderRangeLines;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.item.ItemNetworkComponents;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketRenderRangeLines;
import pneumaticCraft.lib.Log;
import pneumaticCraft.lib.TileEntityConstants;
import pneumaticCraft.proxy.CommonProxy.EnumGuiId;

import com.mojang.authlib.GameProfile;

public class TileEntitySecurityStation extends TileEntityBase implements ISidedInventory, IGUITextFieldSensitive,
        IRangeLineShower, IRedstoneControl{
    private ItemStack[] inventory;
    private final int INVENTORY_SIZE = 39;
    public static final int UPGRADE_SLOT_START = 35;
    public static final int UPGRADE_SLOT_END = 38;

    public final List<GameProfile> hackedUsers = new ArrayList<GameProfile>(); //Stores all the users that have hacked this Security Station.
    public final List<GameProfile> sharedUsers = new ArrayList<GameProfile>(); //Stores all the users that have been allowed by the stationOwner.
    @GuiSynced
    private int rebootTimer;//When the player decides to reset the station, this variable will hold the remaining reboot time.
    @GuiSynced
    public String textFieldText = "";
    private int securityRange;
    private int oldSecurityRange; //range used by the range line renderer, to figure out if the range has been changed.
    private final RenderRangeLines rangeLineRenderer = new RenderRangeLines(0x33FF0000);

    @GuiSynced
    public int redstoneMode;
    public boolean oldRedstoneStatus;

    private boolean validNetwork;

    public TileEntitySecurityStation(){
        super(UPGRADE_SLOT_START, 36, 37, UPGRADE_SLOT_END);
        inventory = new ItemStack[INVENTORY_SIZE];
        addApplicableUpgrade(EnumUpgrade.ENTITY_TRACKER, EnumUpgrade.SECURITY, EnumUpgrade.RANGE);
    }

    @Override
    public void func_73660_a(){
        if(rebootTimer > 0) {
            rebootTimer--;
            if(!field_145850_b.field_72995_K) {
                if(rebootTimer == 0) {
                    hackedUsers.clear();
                }
            }
        }
        if(field_145850_b.field_72995_K && !firstRun) {
            if(oldSecurityRange != getSecurityRange() || oldSecurityRange == 0) {
                rangeLineRenderer.resetRendering(getSecurityRange());
                oldSecurityRange = getSecurityRange();
            }
            rangeLineRenderer.update();
        }
        if(/* !worldObj.isRemote && */oldRedstoneStatus != shouldEmitRedstone()) {
            oldRedstoneStatus = shouldEmitRedstone();
            updateNeighbours();
        }

        securityRange = Math.min(2 + getUpgrades(EnumUpgrade.RANGE), TileEntityConstants.SECURITY_STATION_MAX_RANGE);

        super.func_73660_a();

    }

    public void rebootStation(){
        rebootTimer = TileEntityConstants.SECURITY_STATION_REBOOT_TIME;
    }

    public int getRebootTime(){
        return rebootTimer;
    }

    /**
     * Will initiate the wireframe rendering. When invoked on the server, it sends a packet to every client to render the box.
     */
    @Override
    public void showRangeLines(){
        if(field_145850_b.field_72995_K) {
            rangeLineRenderer.resetRendering(getSecurityRange());
        } else {
            NetworkHandler.sendToAllAround(new PacketRenderRangeLines(this), field_145850_b, TileEntityConstants.PACKET_UPDATE_DISTANCE + getSecurityRange());
        }
    }

    @SideOnly(Side.CLIENT)
    public void renderRangeLines(){
        rangeLineRenderer.render();
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID == 0) {
            redstoneMode++;
            if(redstoneMode > 2) redstoneMode = 0;
            updateNeighbours();
        } else if(buttonID == 2) {
            rebootStation();
        } else if(buttonID == 3) {
            if(!hasValidNetwork()) {
                player.func_146105_b(new ChatComponentTranslation(EnumChatFormatting.GREEN + "This Security Station is out of order: Its network hasn't been properly configured."));
            } else {
                player.openGui(PneumaticCraft.instance, EnumGuiId.HACKING.ordinal(), field_145850_b, func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p());
            }
        } else if(buttonID > 3 && buttonID - 4 < sharedUsers.size()) {
            sharedUsers.remove(buttonID - 4);
        }
        sendDescriptionPacket();
    }

    public void addSharedUser(GameProfile user){
        for(GameProfile sharedUser : sharedUsers) {
            if(gameProfileEquals(sharedUser, user)) return;
        }
        sharedUsers.add(user);
        sendDescriptionPacket();
    }

    public void addHacker(GameProfile user){
        for(GameProfile hackedUser : hackedUsers) {
            if(gameProfileEquals(hackedUser, user)) {
                return;
            }
        }
        for(GameProfile sharedUser : sharedUsers) {
            if(gameProfileEquals(sharedUser, user)) return;
        }
        hackedUsers.add(user);
        sendDescriptionPacket();
    }

    private boolean gameProfileEquals(GameProfile profile1, GameProfile profile2){
        return profile1.getId() != null && profile2.getId() != null ? profile1.getId().equals(profile2.getId()) : profile1.getName().equals(profile2.getName());
    }

    public boolean shouldEmitRedstone(){
        switch(redstoneMode){
            case 0:
                return false;
            case 1:
                return isHacked();
            case 2:
                return getRebootTime() <= 0;
        }
        return false;
    }

    public boolean isHacked(){
        return hackedUsers.size() > 0;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox(){
        if(!rangeLineRenderer.isCurrentlyRendering()) return super.getRenderBoundingBox();
        int range = getSecurityRange();
        return new AxisAlignedBB(func_174877_v().func_177958_n() - range, func_174877_v().func_177956_o() - range, func_174877_v().func_177952_p() - range, func_174877_v().func_177958_n() + 1 + range, func_174877_v().func_177956_o() + 1 + range, func_174877_v().func_177952_p() + 1 + range);
    }

    public int getSecurityRange(){
        return securityRange;
    }

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){
        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){
        return inventory[slot];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
        checkForNetworkValidity();
    }

    @Override
    public String getName(){
        return Blockss.securityStation.func_149739_a();
    }

    @Override
    public int func_70297_j_(){
        return 64;
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){

        super.func_145839_a(tag);
        redstoneMode = tag.func_74762_e("redstoneMode");
        rebootTimer = tag.func_74762_e("startupTimer");
        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[func_70302_i_()];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }
        checkForNetworkValidity();
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){

        super.func_145841_b(tag);
        tag.func_74768_a("redstoneMode", redstoneMode);
        tag.func_74768_a("startupTimer", rebootTimer);
        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Items", tagList);
    }

    @Override
    public void writeToPacket(NBTTagCompound tag){
        super.writeToPacket(tag);
        NBTTagList sharedList = new NBTTagList();
        for(int i = 0; i < sharedUsers.size(); i++) {
            NBTTagCompound tagCompound = new NBTTagCompound();
            tagCompound.func_74778_a("name", sharedUsers.get(i).getName());
            if(sharedUsers.get(i).getId() != null) tagCompound.func_74778_a("uuid", sharedUsers.get(i).getId().toString());
            sharedList.func_74742_a(tagCompound);
        }
        tag.func_74782_a("SharedUsers", sharedList);

        NBTTagList hackedList = new NBTTagList();
        for(int i = 0; i < hackedUsers.size(); i++) {
            NBTTagCompound tagCompound = new NBTTagCompound();
            tagCompound.func_74778_a("name", hackedUsers.get(i).getName());
            if(hackedUsers.get(i).getId() != null) tagCompound.func_74778_a("uuid", hackedUsers.get(i).getId().toString());
            hackedList.func_74742_a(tagCompound);
        }
        tag.func_74782_a("HackedUsers", hackedList);
    }

    @Override
    public void readFromPacket(NBTTagCompound tag){
        super.readFromPacket(tag);
        sharedUsers.clear();
        NBTTagList sharedList = tag.func_150295_c("SharedUsers", 10);
        for(int i = 0; i < sharedList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = sharedList.func_150305_b(i);
            sharedUsers.add(new GameProfile(tagCompound.func_74764_b("uuid") ? UUID.fromString(tagCompound.func_74779_i("uuid")) : null, tagCompound.func_74779_i("name")));
        }

        hackedUsers.clear();
        NBTTagList hackedList = tag.func_150295_c("HackedUsers", 10);
        for(int i = 0; i < hackedList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = hackedList.func_150305_b(i);
            hackedUsers.add(new GameProfile(tagCompound.func_74764_b("uuid") ? UUID.fromString(tagCompound.func_74779_i("uuid")) : null, tagCompound.func_74779_i("name")));
        }
    }

    @Override
    public boolean func_94041_b(int i, ItemStack itemstack){
        return false;
    }

    @Override
    // upgrades in bottom, fuel in the rest.
    public int[] func_180463_a(EnumFacing var1){
        return new int[0];
    }

    @Override
    public boolean func_180462_a(int i, ItemStack itemstack, EnumFacing j){
        return false;
    }

    @Override
    public boolean func_180461_b(int i, ItemStack itemstack, EnumFacing j){
        return false;
    }

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public void setText(int textFieldID, String text){
        textFieldText = text;
    }

    @Override
    public String getText(int textFieldID){
        return textFieldText;
    }

    /**
     * Returns true if the given player is allowed to interact with the covered area of this Security Station.
     * @param player
     * @return
     */
    public boolean doesAllowPlayer(EntityPlayer player){
        return rebootTimer > 0 || isPlayerOnWhiteList(player) || hasPlayerHacked(player);
    }

    public boolean isPlayerOnWhiteList(EntityPlayer player){
        for(int i = 0; i < sharedUsers.size(); i++) {
            GameProfile user = sharedUsers.get(i);
            if(gameProfileEquals(user, player.func_146103_bH())) {
                if(user.getId() == null && player.func_146103_bH().getId() != null) {
                    sharedUsers.set(i, player.func_146103_bH());
                    Log.info("Legacy conversion: Security Station shared username '" + player.func_70005_c_() + "' is now using UUID '" + player.func_146103_bH().getId() + "'.");
                }
                return true;
            }
        }
        return false;
    }

    public boolean hasPlayerHacked(EntityPlayer player){
        for(int i = 0; i < hackedUsers.size(); i++) {
            GameProfile user = hackedUsers.get(i);
            if(gameProfileEquals(user, player.func_146103_bH())) {
                if(user.getId() == null && player.func_146103_bH().getId() != null) {
                    hackedUsers.set(i, player.func_146103_bH());
                    Log.info("Legacy conversion: Security Station hacked username '" + player.func_70005_c_() + "' is now using UUID '" + player.func_146103_bH().getId() + "'.");
                }
                return true;
            }
        }
        return false;
    }

    /**
     * Returns true if the given slots are connected in the network. For this to be true both slots need to have a network component stored as well.
     * @param firstSlot
     * @param secondSlot
     * @return
     */
    public boolean connects(int firstSlot, int secondSlot){
        if(firstSlot < 0 || secondSlot < 0 || firstSlot >= 35 || secondSlot >= 35 || firstSlot == secondSlot || func_70301_a(firstSlot) == null || func_70301_a(secondSlot) == null) return false;
        for(int column = -1; column <= 1; column++) {
            for(int row = -1; row <= 1; row++) {
                if(firstSlot + row * 5 + column == secondSlot) {
                    if(firstSlot % 5 > 0 && firstSlot % 5 < 4 || secondSlot % 5 > 0 && secondSlot % 5 < 4 || secondSlot % 5 == firstSlot % 5) return true;
                }
            }
        }
        return false;
    }

    public boolean hasValidNetwork(){
        return validNetwork;
    }

    public enum EnumNetworkValidityProblem{
        NONE, NO_SUBROUTINE, NO_IO_PORT, NO_REGISTRY, TOO_MANY_SUBROUTINES, TOO_MANY_IO_PORTS, TOO_MANY_REGISTRIES, NO_CONNECTION_SUB_AND_IO_PORT, NO_CONNECTION_IO_PORT_AND_REGISTRY
    }

    /**
     * Method used to update the check of the validity of the network.
     * @return optional problem enum
     */
    public EnumNetworkValidityProblem checkForNetworkValidity(){
        validNetwork = false;
        int ioPortSlot = -1;
        int registrySlot = -1;
        int subroutineSlot = -1;
        for(int i = 0; i < 35; i++) {
            if(func_70301_a(i) != null) {
                switch(func_70301_a(i).func_77952_i()){
                    case ItemNetworkComponents.DIAGNOSTIC_SUBROUTINE:
                        if(subroutineSlot != -1) return EnumNetworkValidityProblem.TOO_MANY_SUBROUTINES;//only one subroutine per network
                        subroutineSlot = i;
                        break;
                    case ItemNetworkComponents.NETWORK_IO_PORT:
                        if(ioPortSlot != -1) return EnumNetworkValidityProblem.TOO_MANY_IO_PORTS;//only one subroutine per network
                        ioPortSlot = i;
                        break;
                    case ItemNetworkComponents.NETWORK_REGISTRY:
                        if(registrySlot != -1) return EnumNetworkValidityProblem.TOO_MANY_REGISTRIES;//only one subroutine per network
                        registrySlot = i;
                        break;
                }
            }
        }
        if(subroutineSlot == -1) return EnumNetworkValidityProblem.NO_SUBROUTINE;
        if(ioPortSlot == -1) return EnumNetworkValidityProblem.NO_IO_PORT;
        if(registrySlot == -1) return EnumNetworkValidityProblem.NO_REGISTRY;
        if(!traceComponent(subroutineSlot, ioPortSlot, new boolean[35])) return EnumNetworkValidityProblem.NO_CONNECTION_SUB_AND_IO_PORT;//check if there's a valid route between the subroutine/ioPort
        if(!traceComponent(ioPortSlot, registrySlot, new boolean[35])) return EnumNetworkValidityProblem.NO_CONNECTION_IO_PORT_AND_REGISTRY; // and ioPort/registry.
        validNetwork = true;
        return EnumNetworkValidityProblem.NONE;
    }

    private boolean traceComponent(int startSlot, int targetSlot, boolean[] slotsDone){
        for(int i = 0; i < 35; i++) {
            if(!slotsDone[i] && connects(startSlot, i)) {
                if(i == targetSlot) return true;
                slotsDone[i] = true;
                if(traceComponent(i, targetSlot, slotsDone)) return true;
            }
        }
        return false;
    }

    public int getDetectionChance(){
        return Math.min(100, 20 + 20 * getUpgrades(EnumUpgrade.ENTITY_TRACKER));
    }

    public int getSecurityLevel(){
        return 1 + getUpgrades(EnumUpgrade.SECURITY);
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    @Override
    public boolean isGuiUseableByPlayer(EntityPlayer par1EntityPlayer){
        return field_145850_b.func_175625_s(func_174877_v()) == this;
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }
}
