package pneumaticCraft.common.tileentity;

import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.util.Vec3;
import net.minecraft.world.WorldServer;
import net.minecraftforge.common.util.FakePlayerFactory;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.common.ai.StringFilterEntitySelector;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.item.Itemss;
import pneumaticCraft.common.minigun.Minigun;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.util.PneumaticCraftUtils;

import com.mojang.authlib.GameProfile;

public class TileEntitySentryTurret extends TileEntityBase implements IRedstoneControlled, ISidedInventory,
        IGUITextFieldSensitive{

    private final ItemStack[] inventory = new ItemStack[8];
    @GuiSynced
    private String entityFilter = "";
    @GuiSynced
    private int redstoneMode;
    @DescSynced
    private int range;
    @DescSynced
    private boolean activated;
    @DescSynced
    private ItemStack minigunColorStack;
    private Minigun minigun;
    @DescSynced
    private int targetEntityId = -1;
    @DescSynced
    private boolean sweeping;
    private final SentryTurretEntitySelector entitySelector = new SentryTurretEntitySelector();

    public TileEntitySentryTurret(){
        super(0, 1, 2, 3);
        addApplicableUpgrade(EnumUpgrade.RANGE);
    }

    @Override
    public void func_73660_a(){
        super.func_73660_a();
        if(!field_145850_b.field_72995_K) {
            range = 16 + Math.min(16, getUpgrades(EnumUpgrade.RANGE));
            if(getMinigun().getAttackTarget() == null && redstoneAllows()) {
                getMinigun().setSweeping(true);
                if(field_145850_b.func_82737_E() % 20 == 0) {
                    List<EntityLivingBase> entities = field_145850_b.func_175647_a(EntityLivingBase.class, getTargetingBoundingBox(), entitySelector);
                    if(entities.size() > 0) {
                        Collections.sort(entities, new TargetSorter());
                        getMinigun().setAttackTarget(entities.get(0));
                        targetEntityId = entities.get(0).func_145782_y();
                    }
                }
            } else {
                getMinigun().setSweeping(false);
            }
            EntityLivingBase target = getMinigun().getAttackTarget();
            if(target != null) {
                if(!redstoneAllows() || !entitySelector.apply(target)) {
                    getMinigun().setAttackTarget(null);
                    targetEntityId = -1;
                } else {
                    if(field_145850_b.func_82737_E() % 5 == 0) {
                        getFakePlayer().func_70107_b(func_174877_v().func_177958_n() + 0.5, func_174877_v().func_177956_o() + 0.5, func_174877_v().func_177952_p() + 0.5); //Make sure the knockback has the right direction.
                        boolean usedAmmo = getMinigun().tryFireMinigun(target);
                        if(usedAmmo) {
                            for(int i = 4; i < inventory.length; i++) {
                                if(inventory[i] != null) {
                                    func_70299_a(i, null);
                                    break;
                                }
                            }
                        }
                    }
                }
            }
        }
        getMinigun().update(func_174877_v().func_177958_n() + 0.5, func_174877_v().func_177956_o() + 0.5, func_174877_v().func_177952_p() + 0.5);
    }

    private boolean canSeeEntity(Entity entity){
        Vec3 entityVec = new Vec3(entity.field_70165_t + entity.field_70130_N / 2, entity.field_70163_u + entity.field_70131_O / 2, entity.field_70161_v + entity.field_70130_N / 2);
        Vec3 tileVec = new Vec3(func_174877_v().func_177958_n() + 0.5, func_174877_v().func_177956_o() + 0.5, func_174877_v().func_177952_p() + 0.5);
        MovingObjectPosition trace = field_145850_b.func_72933_a(entityVec, tileVec);
        return trace != null && trace.func_178782_a().equals(func_174877_v());
    }

    private AxisAlignedBB getTargetingBoundingBox(){
        return new AxisAlignedBB(func_174877_v().func_177958_n() - range, func_174877_v().func_177956_o() - range, func_174877_v().func_177952_p() - range, func_174877_v().func_177958_n() + range + 1, func_174877_v().func_177956_o() + range + 1, func_174877_v().func_177952_p() + range + 1);
    }

    @Override
    protected void onFirstServerUpdate(){
        super.onFirstServerUpdate();
        updateAmmo();
    }

    @Override
    public void onDescUpdate(){
        super.onDescUpdate();
        Entity entity = field_145850_b.func_73045_a(targetEntityId);
        if(entity instanceof EntityLivingBase) {
            getMinigun().setAttackTarget((EntityLivingBase)entity);
        } else {
            getMinigun().setAttackTarget(null);
        }
    }

    public Minigun getMinigun(){
        if(minigun == null) {
            minigun = new MinigunSentryTurret();
            minigun.setWorld(field_145850_b);
            if(field_145850_b != null && !field_145850_b.field_72995_K) {
                minigun.setPlayer(getFakePlayer());
            }
        }
        return minigun;
    }

    private EntityPlayer getFakePlayer(){
        return FakePlayerFactory.get((WorldServer)field_145850_b, new GameProfile(null, "Sentry Turret"));
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        writeInventoryToNBT(tag, inventory);
        tag.func_74774_a("redstoneMode", (byte)redstoneMode);
        tag.func_74778_a("entityFilter", entityFilter);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        readInventoryFromNBT(tag, inventory);
        redstoneMode = tag.func_74771_c("redstoneMode");
        setText(0, tag.func_74779_i("entityFilter"));
    }

    @Override
    public boolean redstoneAllows(){
        if(redstoneMode == 3) return true;
        return super.redstoneAllows();
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID == 0) {
            redstoneMode++;
            if(redstoneMode > 2) redstoneMode = 0;
        }
    }

    /*
     * ---------------IInventory---------------------
     */

    /**
     * Returns the name of the inventory.
     */
    @Override
    public String getName(){
        return Blockss.sentryTurret.func_149739_a();
    }

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){
        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int par1){
        return inventory[par1];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }
        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
        if(slot >= 4) {
            updateAmmo();
        }
    }

    private void updateAmmo(){
        ItemStack ammo = null;
        for(int i = 4; i < inventory.length; i++) {
            if(inventory[i] != null) {
                ammo = inventory[i];
                break;
            }
        }
        getMinigun().setAmmo(ammo);
    }

    @Override
    public boolean func_94041_b(int slot, ItemStack stack){
        if(slot < 4) {
            return canInsertUpgrade(slot, stack);
        } else {
            return stack != null && stack.func_77973_b() == Itemss.gunAmmo;
        }
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public int func_70297_j_(){
        return 64;
    }

    @Override
    public boolean func_70300_a(EntityPlayer p_70300_1_){
        return isGuiUseableByPlayer(p_70300_1_);
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public int[] func_180463_a(EnumFacing p_94128_1_){
        return new int[]{0, 1, 2, 3, 4, 5, 6, 7};
    }

    @Override
    public boolean func_180462_a(int slot, ItemStack stack, EnumFacing side){
        return func_94041_b(slot, stack);
    }

    @Override
    public boolean func_180461_b(int slot, ItemStack stack, EnumFacing side){
        return true;
    }

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    private class MinigunSentryTurret extends Minigun{

        public MinigunSentryTurret(){
            super(true);
        }

        @Override
        public boolean isMinigunActivated(){
            return activated;
        }

        @Override
        public void setMinigunActivated(boolean activated){
            TileEntitySentryTurret.this.activated = activated;
        }

        @Override
        public void setAmmoColorStack(ItemStack ammo){
            minigunColorStack = ammo;
        }

        @Override
        public int getAmmoColor(){
            return getAmmoColor(minigunColorStack);
        }

        @Override
        public void playSound(String soundName, float volume, float pitch){
            field_145850_b.func_72908_a(func_174877_v().func_177958_n() + 0.5, func_174877_v().func_177956_o() + 0.5, func_174877_v().func_177952_p() + 0.5, soundName, volume, pitch);
        }

        @Override
        public void setSweeping(boolean sweeping){
            TileEntitySentryTurret.this.sweeping = sweeping;
        }

        @Override
        public boolean isSweeping(){
            return sweeping;
        }
    }

    private class TargetSorter implements Comparator<Entity>{

        private final BlockPos pos;

        public TargetSorter(){
            pos = new BlockPos(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p());
        }

        @Override
        public int compare(Entity arg0, Entity arg1){
            double dist1 = PneumaticCraftUtils.distBetween(pos, arg0.field_70165_t, arg0.field_70163_u, arg0.field_70161_v);
            double dist2 = PneumaticCraftUtils.distBetween(pos, arg1.field_70165_t, arg1.field_70163_u, arg1.field_70161_v);
            return Double.compare(dist1, dist2);
        }
    }

    private class SentryTurretEntitySelector extends StringFilterEntitySelector{

        @Override
        public boolean apply(Entity entity){
            if(entity instanceof EntityPlayer) {
                EntityPlayer player = (EntityPlayer)entity;
                if(player.field_71075_bZ.field_75098_d || isExcludedBySecurityStations(player)) return false;
            }
            return super.apply(entity) && inRange(entity) && canSeeEntity(entity);
        }

        private boolean inRange(Entity entity){
            return PneumaticCraftUtils.distBetween(new BlockPos(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p()), entity.field_70165_t, entity.field_70163_u, entity.field_70161_v) <= range;
        }

        private boolean isExcludedBySecurityStations(EntityPlayer player){
            Iterator<TileEntitySecurityStation> iterator = PneumaticCraftUtils.getSecurityStations(field_145850_b, func_174877_v(), false).iterator();
            if(iterator.hasNext()) { //When there are Security Stations, all stations need to be allowing the player.
                while(iterator.hasNext()) {
                    if(!iterator.next().doesAllowPlayer(player)) return false;
                }
                return true;
            } else {
                return false; //When there are no Security Stations at all, the player isn't automatically 'allowed to live'.
            }
        }
    }

    @Override
    public void setText(int textFieldID, String text){
        entityFilter = text;
        entitySelector.setFilter(text);
        if(minigun != null) minigun.setAttackTarget(null);
    }

    @Override
    public String getText(int textFieldID){
        return entityFilter;
    }

    @Override
    public AxisAlignedBB getRenderBoundingBox(){
        return getTargetingBoundingBox();
    }
}
