package pneumaticCraft.common.tileentity;

import java.util.Arrays;
import java.util.List;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.EnumFacing;

import org.apache.commons.lang3.tuple.Pair;

import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.tileentity.IAirHandler;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.item.Itemss;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.lib.PneumaticValues;
import pneumaticCraft.lib.TileEntityConstants;

public class TileEntityUVLightBox extends TileEntityPneumaticBase implements ISidedInventory, IMinWorkingPressure,
        IRedstoneControl{
    @DescSynced
    public boolean leftConnected;
    @DescSynced
    public boolean rightConnected;
    @DescSynced
    public boolean areLightsOn;
    @GuiSynced
    public int redstoneMode;
    @DescSynced
    public ItemStack[] inventory = new ItemStack[INVENTORY_SIZE];
    public int ticksExisted;
    public static final int INVENTORY_SIZE = 5;
    public static final int PCB_INDEX = 0;
    public static final int UPGRADE_SLOT_START = 1;
    public static final int UPGRADE_SLOT_END = 4;
    private boolean oldRedstoneStatus;

    public TileEntityUVLightBox(){
        super(PneumaticValues.DANGER_PRESSURE_UV_LIGHTBOX, PneumaticValues.MAX_PRESSURE_UV_LIGHTBOX, PneumaticValues.VOLUME_UV_LIGHTBOX, UPGRADE_SLOT_START, 2, 3, UPGRADE_SLOT_END);
        addApplicableUpgrade(EnumUpgrade.SPEED);
    }

    @Override
    public void func_145839_a(NBTTagCompound nbt){
        super.func_145839_a(nbt);
        redstoneMode = nbt.func_74762_e("redstoneMode");

        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = nbt.func_150295_c("Items", 10);
        inventory = new ItemStack[func_70302_i_()];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }
        //  if(worldObj.isRemote) System.out.println("reading from NBT, got a packet?");
    }

    @Override
    public void func_145841_b(NBTTagCompound nbt){
        super.func_145841_b(nbt);
        nbt.func_74768_a("redstoneMode", redstoneMode);

        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        nbt.func_74782_a("Items", tagList);
    }

    @Override
    public void func_73660_a(){
        super.func_73660_a();
        if(!field_145850_b.field_72995_K) {
            ticksExisted++;
            if(getPressure() >= PneumaticValues.MIN_PRESSURE_UV_LIGHTBOX && inventory[0] != null && inventory[0].func_77973_b() == Itemss.emptyPCB && inventory[0].func_77952_i() > 0) {

                addAir((int)(-PneumaticValues.USAGE_UV_LIGHTBOX * getSpeedUsageMultiplierFromUpgrades()));
                if(ticksExisted % Math.max(1, (int)(TileEntityConstants.LIGHT_BOX_0_100_TIME / (5 * getSpeedMultiplierFromUpgrades()))) == 0) {
                    if(!areLightsOn) {
                        areLightsOn = true;
                        updateNeighbours();
                    }
                    inventory[0].func_77964_b(Math.max(0, inventory[0].func_77952_i() - 1));
                }
            } else if(areLightsOn) {
                areLightsOn = false;
                updateNeighbours();
            }
            if(oldRedstoneStatus != shouldEmitRedstone()) {
                oldRedstoneStatus = !oldRedstoneStatus;
                updateNeighbours();
            }
        }
    }

    @Override
    protected boolean shouldRerenderChunkOnDescUpdate(){
        return true;
    }

    @Override
    public void onNeighborTileUpdate(){
        super.onNeighborTileUpdate();
        updateConnections();
    }

    public int getLightLevel(){
        return areLightsOn ? Math.min(5, getUpgrades(EnumUpgrade.SPEED) * 2) + 10 : 0;
    }

    // used in the air dispersion methods.
    @Override
    public boolean isConnectedTo(EnumFacing side){
        return side != EnumFacing.UP && side != getRotation() && side != getRotation().func_176734_d();
    }

    public void updateConnections(){
        leftConnected = false;
        rightConnected = false;

        List<Pair<EnumFacing, IAirHandler>> connections = getAirHandler(null).getConnectedPneumatics();
        for(Pair<EnumFacing, IAirHandler> entry : connections) {
            if(entry.getKey() == getRotation().func_176746_e()) { //TODO 1.8 test
                leftConnected = true;
            } else if(entry.getKey() == getRotation().func_176735_f()) {
                rightConnected = true;
            }
        }
    }

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){

        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){

        return inventory[slot];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){
        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
    }

    @Override
    public String getName(){
        return Blockss.uvLightBox.func_149739_a();
    }

    @Override
    public int func_70297_j_(){
        return 64;
    }

    @Override
    public boolean func_94041_b(int i, ItemStack itemstack){
        return i == 0 || canInsertUpgrade(i, itemstack);
    }

    @Override
    // upgrades in bottom, fuel in the rest.
    public int[] func_180463_a(EnumFacing var1){
        return new int[]{0};
    }

    @Override
    public boolean func_180462_a(int i, ItemStack itemstack, EnumFacing j){
        return true;
    }

    @Override
    public boolean func_180461_b(int i, ItemStack itemstack, EnumFacing j){
        return true;
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID == 0) {
            redstoneMode++;
            if(redstoneMode > 4) redstoneMode = 0;
            updateNeighbours();
        }
    }

    public boolean shouldEmitRedstone(){
        if(redstoneMode == 0 || inventory[0] == null || inventory[0].func_77973_b() != Itemss.emptyPCB) return false;
        switch(redstoneMode){
            case 1:
                return inventory[0].func_77952_i() < 30;
            case 2:
                return inventory[0].func_77952_i() < 20;
            case 3:
                return inventory[0].func_77952_i() < 10;
            case 4:
                return inventory[0].func_77952_i() == 0;
        }
        return false;
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }

    @Override
    public float getMinWorkingPressure(){
        return PneumaticValues.MIN_PRESSURE_UV_LIGHTBOX;
    }
}
