package pneumaticCraft.common.tileentity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.GuiScreen;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.fml.common.Optional;
import net.minecraftforge.fml.common.eventhandler.Event;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.apache.commons.lang3.tuple.Pair;

import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.tileentity.IAirHandler;
import pneumaticCraft.api.universalSensor.IEventSensorSetting;
import pneumaticCraft.api.universalSensor.IPollSensorSetting;
import pneumaticCraft.api.universalSensor.ISensorSetting;
import pneumaticCraft.client.gui.GuiUniversalSensor;
import pneumaticCraft.client.render.RenderRangeLines;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.item.ItemGPSTool;
import pneumaticCraft.common.item.Itemss;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.network.NetworkHandler;
import pneumaticCraft.common.network.PacketRenderRangeLines;
import pneumaticCraft.common.sensor.SensorHandler;
import pneumaticCraft.common.thirdparty.ThirdPartyManager;
import pneumaticCraft.common.thirdparty.computercraft.LuaConstant;
import pneumaticCraft.common.thirdparty.computercraft.LuaMethod;
import pneumaticCraft.lib.ModIds;
import pneumaticCraft.lib.PneumaticValues;
import pneumaticCraft.lib.TileEntityConstants;

public class TileEntityUniversalSensor extends TileEntityPneumaticBase implements IInventory, IRangeLineShower,
        IGUITextFieldSensitive, IMinWorkingPressure, IRedstoneControl{

    @DescSynced
    public boolean[] sidesConnected = new boolean[6];
    private ItemStack[] inventory = new ItemStack[5];
    public static final int UPGRADE_SLOT_1 = 0;
    public static final int UPGRADE_SLOT_4 = 3;
    public static final int INVENTORY_SIZE = 5;

    @GuiSynced
    private String sensorSetting = "";
    private int ticksExisted;
    public int redstoneStrength;
    private int eventTimer;
    public float dishRotation;
    public float oldDishRotation;
    public float dishSpeed;
    @GuiSynced
    public boolean invertedRedstone;
    @DescSynced
    public boolean isSensorActive;
    @GuiSynced
    private String sensorGuiText = ""; //optional parameter text for sensors.
    private boolean requestPollPullEvent;
    private Set<BlockPos> positions;

    private int oldSensorRange; //range used by the range line renderer, to figure out if the range has been changed.
    private final RenderRangeLines rangeLineRenderer = new RenderRangeLines(0x330000FF);

    //  private final List<IComputerAccess> attachedComputers = new ArrayList<IComputerAccess>(); //keep track of the computers so we can raise a os.pullevent.

    public TileEntityUniversalSensor(){
        super(PneumaticValues.DANGER_PRESSURE_UNIVERSAL_SENSOR, PneumaticValues.MAX_PRESSURE_UNIVERSAL_SENSOR, PneumaticValues.VOLUME_UNIVERSAL_SENSOR, UPGRADE_SLOT_1, 1, 2, UPGRADE_SLOT_4);
        for(Item upgrade : SensorHandler.getInstance().getUniversalSensorUpgrades()) {
            addApplicableUpgrade(upgrade);
        }
        addApplicableUpgrade(EnumUpgrade.RANGE);
    }

    @Override
    public void func_73660_a(){
        oldDishRotation = dishRotation;
        if(isSensorActive) {
            dishSpeed = Math.min(dishSpeed + 0.2F, 10);
        } else {
            dishSpeed = Math.max(dishSpeed - 0.2F, 0);
        }
        dishRotation += dishSpeed;

        if(field_145850_b.field_72995_K) {
            int sensorRange = getRange();
            if(oldSensorRange != sensorRange || oldSensorRange == 0) {
                oldSensorRange = sensorRange;
                if(!firstRun) rangeLineRenderer.resetRendering(sensorRange);
            }
            rangeLineRenderer.update();
        }
        super.func_73660_a();

        if(!field_145850_b.field_72995_K) {
            ticksExisted++;
            ISensorSetting sensor = SensorHandler.getInstance().getSensorFromPath(sensorSetting);
            if(sensor != null && getPressure() > PneumaticValues.MIN_PRESSURE_UNIVERSAL_SENSOR) {
                isSensorActive = true;
                addAir(-PneumaticValues.USAGE_UNIVERSAL_SENSOR);
                if(sensor instanceof IPollSensorSetting) {

                    if(ticksExisted % ((IPollSensorSetting)sensor).getPollFrequency(this) == 0) {
                        int newRedstoneStrength = ((IPollSensorSetting)sensor).getRedstoneValue(field_145850_b, func_174877_v(), getRange(), sensorGuiText);
                        if(invertedRedstone) newRedstoneStrength = 15 - newRedstoneStrength;
                        if(newRedstoneStrength != redstoneStrength) {
                            redstoneStrength = newRedstoneStrength;
                            if(requestPollPullEvent) {
                                notifyComputers(redstoneStrength);
                            }
                            updateNeighbours();
                        }
                    }
                    eventTimer = 0;
                } else {
                    if(eventTimer > 0) {
                        eventTimer--;
                        if(eventTimer == 0 && redstoneStrength != (invertedRedstone ? 15 : 0)) {
                            redstoneStrength = invertedRedstone ? 15 : 0;
                            updateNeighbours();
                        }
                    }
                }
            } else {
                isSensorActive = false;
                if(redstoneStrength != (invertedRedstone ? 15 : 0)) {
                    redstoneStrength = invertedRedstone ? 15 : 0;
                    updateNeighbours();
                }
            }
        }
    }

    /**
     * Will initiate the wireframe rendering. When invoked on the server, it sends a packet to every client to render the box.
     */
    @Override
    public void showRangeLines(){
        if(field_145850_b.field_72995_K) {
            rangeLineRenderer.resetRendering(getRange());
        } else {
            NetworkHandler.sendToAllAround(new PacketRenderRangeLines(this), field_145850_b, TileEntityConstants.PACKET_UPDATE_DISTANCE + getRange());
        }
    }

    @SideOnly(Side.CLIENT)
    public void renderRangeLines(){
        rangeLineRenderer.render();
    }

    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox(){
        if(!rangeLineRenderer.isCurrentlyRendering()) return super.getRenderBoundingBox();
        int range = getRange();
        return new AxisAlignedBB(func_174877_v().func_177958_n() - range, func_174877_v().func_177956_o() - range, func_174877_v().func_177952_p() - range, func_174877_v().func_177958_n() + 1 + range, func_174877_v().func_177956_o() + 1 + range, func_174877_v().func_177952_p() + 1 + range);
    }

    public void onEvent(Event event){
        ISensorSetting sensor = SensorHandler.getInstance().getSensorFromPath(sensorSetting);
        if(sensor != null && sensor instanceof IEventSensorSetting && getPressure() > PneumaticValues.MIN_PRESSURE_UNIVERSAL_SENSOR) {
            int newRedstoneStrength = ((IEventSensorSetting)sensor).emitRedstoneOnEvent(event, this, getRange(), sensorGuiText);
            if(newRedstoneStrength != 0) eventTimer = ((IEventSensorSetting)sensor).getRedstonePulseLength();
            if(invertedRedstone) newRedstoneStrength = 15 - newRedstoneStrength;
            if(eventTimer > 0 && ThirdPartyManager.computerCraftLoaded) {
                if(event instanceof PlayerInteractEvent) {
                    PlayerInteractEvent e = (PlayerInteractEvent)event;
                    notifyComputers(newRedstoneStrength, e.pos.func_177958_n(), e.pos.func_177956_o(), e.pos.func_177952_p());
                } else {
                    notifyComputers(newRedstoneStrength);
                }
            }
            if(newRedstoneStrength != redstoneStrength) {
                redstoneStrength = newRedstoneStrength;
                updateNeighbours();
            }
        }
    }

    public int getRange(){
        return getUpgrades(EnumUpgrade.RANGE) + 2;
    }

    public void setSensorSetting(String sensorPath){
        sensorSetting = sensorPath;
        if(field_145850_b != null && field_145850_b.field_72995_K) {
            GuiScreen guiScreen = Minecraft.func_71410_x().field_71462_r;
            if(guiScreen instanceof GuiUniversalSensor) {
                ((GuiUniversalSensor)guiScreen).updateButtons();
            }
        }
    }

    private boolean setSensorSetting(ISensorSetting sensor){
        if(areGivenUpgradesInserted(sensor.getRequiredUpgrades())) {
            setSensorSetting(sensor.getSensorPath());
            return true;
        } else {
            return false;
        }
    }

    public String getSensorSetting(){
        return sensorSetting;
    }

    @Override
    public void onGuiUpdate(){
        setSensorSetting(sensorSetting);
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        tag.func_74778_a("sensorSetting", sensorSetting);
        tag.func_74757_a("invertedRedstone", invertedRedstone);
        tag.func_74776_a("dishSpeed", dishSpeed);
        tag.func_74778_a("sensorText", sensorGuiText);
        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Items", tagList);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        setSensorSetting(tag.func_74779_i("sensorSetting"));
        invertedRedstone = tag.func_74767_n("invertedRedstone");
        dishSpeed = tag.func_74760_g("dishSpeed");
        sensorGuiText = tag.func_74779_i("sensorText");
        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[func_70302_i_()];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }
        positions = getGPSPositionsStatic(this, getRange());
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID >= 10 && buttonID % 10 == 0) {
            String[] directories = SensorHandler.getInstance().getDirectoriesAtLocation(getSensorSetting());
            if(buttonID / 10 <= directories.length) {// <= because of the redstone button being 0.
                if(getSensorSetting().equals("")) {
                    setSensorSetting(directories[buttonID / 10 - 1]);
                } else {
                    setSensorSetting(getSensorSetting() + "/" + directories[buttonID / 10 - 1]);
                }
            }
        } else if(buttonID == 1) {//the 'back' button

            String[] folders = getSensorSetting().split("/");
            String newPath = getSensorSetting().replace(folders[folders.length - 1], "");
            if(newPath.endsWith("/")) {
                newPath = newPath.substring(0, newPath.length() - 1);
            }
            setSensorSetting(newPath);
        } else if(buttonID == 0) {
            invertedRedstone = !invertedRedstone;
            redstoneStrength = 15 - redstoneStrength;
            updateNeighbours();
        }
    }

    public boolean areGivenUpgradesInserted(Set<Item> requiredItems){
        for(Item requiredItem : requiredItems) {
            if(getUpgrades(requiredItem) == 0) {
                return false;
            }
        }
        return true;
    }

    @Override
    public void onNeighborTileUpdate(){
        super.onNeighborTileUpdate();
        List<Pair<EnumFacing, IAirHandler>> connections = getAirHandler(null).getConnectedPneumatics();
        Arrays.fill(sidesConnected, false);
        for(Pair<EnumFacing, IAirHandler> entry : connections) {
            sidesConnected[entry.getKey().ordinal()] = true;
        }
    }

    // INVENTORY METHODS- && NBT
    // ------------------------------------------------------------

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){

        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){

        return inventory[slot];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        // super.setInventorySlotContents(slot, itemStack);
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
        if(!field_145850_b.field_72995_K && !getSensorSetting().equals("") && !areGivenUpgradesInserted(SensorHandler.getInstance().getRequiredStacksFromText(getSensorSetting()))) {
            setSensorSetting("");
        }
        positions = getGPSPositionsStatic(this, getRange());
    }

    public Set<BlockPos> getGPSPositions(){
        return positions;
    }

    private static Set<BlockPos> getGPSPositionsStatic(TileEntityUniversalSensor teUs, int sensorRange){
        List<BlockPos> gpsPositions = new ArrayList<BlockPos>();
        for(int i = TileEntityUniversalSensor.UPGRADE_SLOT_1; i <= TileEntityUniversalSensor.UPGRADE_SLOT_4; i++) {
            ItemStack gps = teUs.func_70301_a(i);
            if(gps != null && gps.func_77973_b() == Itemss.GPSTool) {
                BlockPos pos = ItemGPSTool.getGPSLocation(gps);
                if(pos != null && Math.abs(pos.func_177958_n() - teUs.func_174877_v().func_177958_n()) <= sensorRange && Math.abs(pos.func_177956_o() - teUs.func_174877_v().func_177956_o()) <= sensorRange && Math.abs(pos.func_177952_p() - teUs.func_174877_v().func_177952_p()) <= sensorRange) {
                    gpsPositions.add(pos);
                }
            }
        }
        if(gpsPositions.size() == 1) {
            return new HashSet(gpsPositions);
        } else if(gpsPositions.size() > 1) {
            int minX = Math.min(gpsPositions.get(0).func_177958_n(), gpsPositions.get(1).func_177958_n());
            int minY = Math.min(gpsPositions.get(0).func_177956_o(), gpsPositions.get(1).func_177956_o());
            int minZ = Math.min(gpsPositions.get(0).func_177952_p(), gpsPositions.get(1).func_177952_p());
            int maxX = Math.max(gpsPositions.get(0).func_177958_n(), gpsPositions.get(1).func_177958_n());
            int maxY = Math.max(gpsPositions.get(0).func_177956_o(), gpsPositions.get(1).func_177956_o());
            int maxZ = Math.max(gpsPositions.get(0).func_177952_p(), gpsPositions.get(1).func_177952_p());
            Set<BlockPos> positions = new HashSet<BlockPos>();
            for(int x = minX; x <= maxX; x++) {
                for(int y = Math.min(255, maxY); y >= minY && y >= 0; y--) {
                    for(int z = minZ; z <= maxZ; z++) {
                        positions.add(new BlockPos(x, y, z));
                    }
                }
            }
            return positions;
        }
        return null;
    }

    @Override
    public int func_70297_j_(){
        return 64;
    }

    @Override
    public String getName(){
        return Blockss.universalSensor.func_149739_a();
    }

    @Override
    public boolean func_94041_b(int i, ItemStack itemstack){
        return canInsertUpgrade(i, itemstack);
    }

    @Override
    public void setText(int textFieldID, String text){
        sensorGuiText = text;
        if(!field_145850_b.field_72995_K) scheduleDescriptionPacket();
    }

    @Override
    public String getText(int textFieldID){
        return sensorGuiText;
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    /*
     * COMPUTERCRAFT API
     */
    /*TODO CC dep @Override
     public String getType(){
         return "universalSensor";
     }*/

    @Override
    public void addLuaMethods(){
        super.addLuaMethods();

        luaMethods.add(new LuaMethod("getSensorNames"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 0) {
                    return SensorHandler.getInstance().getSensorNames();
                } else {
                    throw new IllegalArgumentException("getSensorNames doesn't accept any arguments!");
                }
            }
        });

        luaMethods.add(new LuaMethod("setSensor"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 1) {
                    ISensorSetting sensor = null;
                    if(args[0] instanceof String) {
                        sensor = SensorHandler.getInstance().getSensorForName((String)args[0]);
                    } else {
                        sensor = SensorHandler.getInstance().getSensorByIndex(((Double)args[0]).intValue() - 1);
                    }
                    if(sensor != null) return new Object[]{setSensorSetting(sensor)};
                    throw new IllegalArgumentException("Invalid sensor name/index: " + args[0]);
                } else if(args.length == 0) {
                    setSensorSetting("");
                    return new Object[]{true};
                } else {
                    throw new IllegalArgumentException("setSensor needs one argument(a number as index, or a sensor name).");
                }
            }
        });

        luaMethods.add(new LuaMethod("getSensor"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 0) {
                    ISensorSetting curSensor = SensorHandler.getInstance().getSensorFromPath(getSensorSetting());
                    return curSensor == null ? null : new Object[]{getSensorSetting().substring(getSensorSetting().lastIndexOf('/') + 1)};
                } else {
                    throw new IllegalArgumentException("getSensor doesn't take any arguments!");
                }
            }
        });

        luaMethods.add(new LuaMethod("setTextfield"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 1) {
                    setText(0, (String)args[0]);
                    return null;
                } else {
                    throw new IllegalArgumentException("setTextfield takes one argument (string)");
                }
            }
        });

        luaMethods.add(new LuaMethod("getTextfield"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 0) {
                    return new Object[]{getText(0)};
                } else {
                    throw new IllegalArgumentException("getTextfield takes no arguments");
                }
            }
        });

        luaMethods.add(new LuaMethod("isSensorEventBased"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 0) {
                    return new Object[]{SensorHandler.getInstance().getSensorFromPath(getSensorSetting()) instanceof IEventSensorSetting};
                } else {
                    throw new IllegalArgumentException("isSensorEventBased takes no arguments");
                }
            }
        });

        luaMethods.add(new LuaMethod("getSensorValue"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 0) {
                    ISensorSetting s = SensorHandler.getInstance().getSensorFromPath(getSensorSetting());
                    if(s instanceof IPollSensorSetting) {
                        requestPollPullEvent = true;
                        return new Object[]{redstoneStrength};
                    } else if(s != null) {
                        throw new IllegalArgumentException("The selected sensor is pull event based. You can't poll the value.");
                    } else {
                        throw new IllegalArgumentException("There's no sensor selected!");
                    }
                } else {
                    throw new IllegalArgumentException("getSensorValue takes no arguments");
                }
            }
        });
        luaMethods.add(new LuaConstant("getMinWorkingPressure", PneumaticValues.MIN_PRESSURE_UNIVERSAL_SENSOR));

        luaMethods.add(new LuaMethod("setGPSToolCoordinate"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 4) {
                    ItemStack stack = func_70301_a(((Double)args[0]).intValue() - 1); //minus one, as lua is 1-oriented.
                    if(stack != null && stack.func_77973_b() == Itemss.GPSTool) {
                        ItemGPSTool.setGPSLocation(stack, new BlockPos((Double)args[1], (Double)args[2], (Double)args[3]));
                        return new Object[]{true};
                    } else {
                        return new Object[]{false};
                    }
                } else {
                    throw new IllegalArgumentException("setGPSToolCoordinate needs 4 arguments: slot, x, y, z");
                }
            }

        });

        luaMethods.add(new LuaMethod("getGPSToolCoordinate"){
            @Override
            public Object[] call(Object[] args) throws Exception{
                if(args.length == 1) {
                    ItemStack stack = func_70301_a(((Double)args[0]).intValue() - 1); //minus one, as lua is 1-oriented.
                    if(stack != null && stack.func_77973_b() == Itemss.GPSTool) {
                        BlockPos pos = ItemGPSTool.getGPSLocation(stack);
                        if(pos != null) {
                            return new Object[]{pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p()};
                        } else {
                            return new Object[]{0, 0, 0};
                        }
                    } else {
                        return null;
                    }
                } else {
                    throw new IllegalArgumentException("setGPSToolCoordinate needs 1 argument: slot");
                }
            }
        });
    }

    /*@Override
    @Optional.Method(modid = ModIds.COMPUTERCRAFT)
    public void attach(IComputerAccess computer){
        attachedComputers.add(computer);
    }

    @Override
    @Optional.Method(modid = ModIds.COMPUTERCRAFT)
    public void detach(IComputerAccess computer){
        attachedComputers.remove(computer);
    }*/

    /**
     * Called on a event sensor
     * @param newRedstoneStrength
     */
    @Optional.Method(modid = ModIds.COMPUTERCRAFT)
    private void notifyComputers(Object... arguments){
        /*     for(IComputerAccess computer : attachedComputers) {
                 computer.queueEvent(getType(), arguments);
             }*/
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public int getRedstoneMode(){
        return invertedRedstone ? 1 : 0;
    }

    @Override
    public float getMinWorkingPressure(){
        return PneumaticValues.MIN_PRESSURE_UNIVERSAL_SENSOR;
    }

}
