package pneumaticCraft.common.tileentity;

import java.util.Arrays;
import java.util.List;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.apache.commons.lang3.tuple.Pair;

import pneumaticCraft.api.PneumaticRegistry;
import pneumaticCraft.api.item.IItemRegistry.EnumUpgrade;
import pneumaticCraft.api.tileentity.IAirHandler;
import pneumaticCraft.api.tileentity.IManoMeasurable;
import pneumaticCraft.common.block.Blockss;
import pneumaticCraft.common.network.DescSynced;
import pneumaticCraft.common.network.GuiSynced;
import pneumaticCraft.common.util.PneumaticCraftUtils;
import pneumaticCraft.lib.PneumaticValues;

public class TileEntityVacuumPump extends TileEntityPneumaticBase implements IInventory, IRedstoneControlled,
        IManoMeasurable{
    @GuiSynced
    private final IAirHandler vacuumHandler = PneumaticRegistry.getInstance().getAirHandlerSupplier().createTierOneAirHandler(PneumaticValues.VOLUME_VACUUM_PUMP);
    public int rotation;
    public int oldRotation;
    public int turnTimer = -1;
    @DescSynced
    public boolean turning = false;
    public int rotationSpeed;
    @GuiSynced
    public int redstoneMode;

    private ItemStack[] inventory = new ItemStack[4];
    public static final int UPGRADE_SLOT_1 = 0;
    public static final int UPGRADE_SLOT_4 = 3;

    public static final int INVENTORY_SIZE = 4;

    public TileEntityVacuumPump(){
        super(PneumaticValues.DANGER_PRESSURE_VACUUM_PUMP, PneumaticValues.MAX_PRESSURE_VACUUM_PUMP, PneumaticValues.VOLUME_VACUUM_PUMP, UPGRADE_SLOT_1, 1, 2, UPGRADE_SLOT_4);
        vacuumHandler.setUpgradeSlots(UPGRADE_SLOT_1, 1, 2, UPGRADE_SLOT_4);
        addApplicableUpgrade(EnumUpgrade.SPEED);
    }

    @Override
    public IAirHandler getAirHandler(EnumFacing side){
        if(side == null || side == getInputSide()) {
            return super.getAirHandler(side);
        } else if(side == getVacuumSide()) {
            return vacuumHandler;
        } else {
            return null;
        }
    }

    @Override
    public void func_145829_t(){
        super.func_145829_t();
        vacuumHandler.validate(this);
    }

    @Override
    public void onNeighborTileUpdate(){
        super.onNeighborTileUpdate();
        vacuumHandler.onNeighborChange();
    }

    public EnumFacing getInputSide(){
        return getVacuumSide().func_176734_d();
    }

    public EnumFacing getVacuumSide(){
        return getRotation();
    }

    @Override
    public void func_73660_a(){
        if(!field_145850_b.field_72995_K && turnTimer >= 0) turnTimer--;
        if(!field_145850_b.field_72995_K && getAirHandler(getInputSide()).getPressure() > PneumaticValues.MIN_PRESSURE_VACUUM_PUMP && getAirHandler(getVacuumSide()).getPressure() > -1F && redstoneAllows()) {
            if(!field_145850_b.field_72995_K && turnTimer == -1) {
                turning = true;
            }
            getAirHandler(getVacuumSide()).addAir((int)(-PneumaticValues.PRODUCTION_VACUUM_PUMP * getSpeedMultiplierFromUpgrades())); // negative because it's pulling a vacuum.
            getAirHandler(getInputSide()).addAir((int)(-PneumaticValues.USAGE_VACUUM_PUMP * getSpeedUsageMultiplierFromUpgrades()));
            turnTimer = 40;
        }
        if(turnTimer == 0) {
            turning = false;
        }
        oldRotation = rotation;
        if(field_145850_b.field_72995_K) {
            if(turning) {
                rotationSpeed = Math.min(rotationSpeed + 1, 20);
            } else {
                rotationSpeed = Math.max(rotationSpeed - 1, 0);
            }
            rotation += rotationSpeed;
        }

        super.func_73660_a();
        vacuumHandler.update();

        IAirHandler inputHandler = getAirHandler(getInputSide());
        List<Pair<EnumFacing, IAirHandler>> teList = inputHandler.getConnectedPneumatics();
        if(teList.size() == 0) inputHandler.airLeak(getInputSide());
        teList = vacuumHandler.getConnectedPneumatics();
        if(teList.size() == 0) vacuumHandler.airLeak(getVacuumSide());

    }

    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox(){
        return new AxisAlignedBB(func_174877_v().func_177958_n(), func_174877_v().func_177956_o(), func_174877_v().func_177952_p(), func_174877_v().func_177958_n() + 1, func_174877_v().func_177956_o() + 1, func_174877_v().func_177952_p() + 1);
    }

    @Override
    public void func_145841_b(NBTTagCompound tag){
        super.func_145841_b(tag);
        NBTTagCompound vacuum = new NBTTagCompound();
        vacuumHandler.writeToNBT(vacuum);
        tag.func_74782_a("vacuum", vacuum);
        tag.func_74757_a("turning", turning);
        tag.func_74768_a("redstoneMode", redstoneMode);
        // Write the ItemStacks in the inventory to NBT
        NBTTagList tagList = new NBTTagList();
        for(int currentIndex = 0; currentIndex < inventory.length; ++currentIndex) {
            if(inventory[currentIndex] != null) {
                NBTTagCompound tagCompound = new NBTTagCompound();
                tagCompound.func_74774_a("Slot", (byte)currentIndex);
                inventory[currentIndex].func_77955_b(tagCompound);
                tagList.func_74742_a(tagCompound);
            }
        }
        tag.func_74782_a("Items", tagList);
    }

    @Override
    public void func_145839_a(NBTTagCompound tag){
        super.func_145839_a(tag);
        vacuumHandler.readFromNBT(tag.func_74775_l("vacuum"));
        turning = tag.func_74767_n("turning");
        redstoneMode = tag.func_74762_e("redstoneMode");
        // Read in the ItemStacks in the inventory from NBT
        NBTTagList tagList = tag.func_150295_c("Items", 10);
        inventory = new ItemStack[func_70302_i_()];
        for(int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound tagCompound = tagList.func_150305_b(i);
            byte slot = tagCompound.func_74771_c("Slot");
            if(slot >= 0 && slot < inventory.length) {
                inventory[slot] = ItemStack.func_77949_a(tagCompound);
            }
        }
    }

    @Override
    public void handleGUIButtonPress(int buttonID, EntityPlayer player){
        if(buttonID == 0) {
            redstoneMode++;
            if(redstoneMode > 2) redstoneMode = 0;
        }
    }

    @Override
    public void printManometerMessage(EntityPlayer player, List<String> curInfo){
        curInfo.add(EnumChatFormatting.GREEN + "Input pressure: " + PneumaticCraftUtils.roundNumberTo(getAirHandler(getInputSide()).getPressure(), 1) + " bar. Vacuum pressure: " + PneumaticCraftUtils.roundNumberTo(getAirHandler(getVacuumSide()).getPressure(), 1) + " bar.");
    }

    // INVENTORY METHODS- && NBT
    // ------------------------------------------------------------

    /**
     * Returns the number of slots in the inventory.
     */
    @Override
    public int func_70302_i_(){

        return inventory.length;
    }

    /**
     * Returns the stack in slot i
     */
    @Override
    public ItemStack func_70301_a(int slot){

        return inventory[slot];
    }

    @Override
    public ItemStack func_70298_a(int slot, int amount){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            if(itemStack.field_77994_a <= amount) {
                func_70299_a(slot, null);
            } else {
                itemStack = itemStack.func_77979_a(amount);
                if(itemStack.field_77994_a == 0) {
                    func_70299_a(slot, null);
                }
            }
        }

        return itemStack;
    }

    @Override
    public ItemStack func_70304_b(int slot){

        ItemStack itemStack = func_70301_a(slot);
        if(itemStack != null) {
            func_70299_a(slot, null);
        }
        return itemStack;
    }

    @Override
    public void func_70299_a(int slot, ItemStack itemStack){
        // super.setInventorySlotContents(slot, itemStack);
        inventory[slot] = itemStack;
        if(itemStack != null && itemStack.field_77994_a > func_70297_j_()) {
            itemStack.field_77994_a = func_70297_j_();
        }
    }

    @Override
    public int func_70297_j_(){

        return 64;
    }

    @Override
    public String getName(){

        return Blockss.vacuumPump.func_149739_a();
    }

    @Override
    public boolean func_94041_b(int i, ItemStack itemstack){
        return canInsertUpgrade(i, itemstack);
    }

    @Override
    public boolean func_145818_k_(){
        return false;
    }

    @Override
    public boolean func_70300_a(EntityPlayer var1){
        return isGuiUseableByPlayer(var1);
    }

    @Override
    public void func_174889_b(EntityPlayer player){}

    @Override
    public void func_174886_c(EntityPlayer player){}

    @Override
    public void func_174888_l(){
        Arrays.fill(inventory, null);
    }

    @Override
    public int getRedstoneMode(){
        return redstoneMode;
    }

}
