package pneumaticCraft.common.util;

import net.minecraft.block.Block;
import net.minecraft.block.BlockChest;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;

/*
 * This file is part of Blue Power.
 *
 *     Blue Power is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     Blue Power is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with Blue Power.  If not, see <http://www.gnu.org/licenses/>
 */

/**
 * @author MineMaarten
 * @author Dynious
 */
public class IOHelper{

    public static IInventory getInventoryForTE(TileEntity te){

        if(te instanceof IInventory) {
            IInventory inv = (IInventory)te;
            Block block = te.func_145838_q();
            if(block instanceof BlockChest) {
                inv = ((BlockChest)block).func_180676_d(te.func_145831_w(), te.func_174877_v());
            }
            return inv;
        } else {
            return null;
        }
    }

    public static TileEntity getNeighbor(TileEntity te, EnumFacing dir){
        return te.func_145831_w().func_175625_s(te.func_174877_v().func_177972_a(dir));
    }

    /**
     * Extracts an exact amount on all sides
     * @param tile
     * @param itemStack
     * @param simulate
     * @return
     */
    public static ItemStack extract(TileEntity tile, ItemStack itemStack, boolean simulate){
        ItemStack extracted = null;
        for(EnumFacing d : EnumFacing.field_82609_l) {
            extracted = extract(tile, d, itemStack, true, simulate);
            if(extracted != null) return extracted;
        }
        return null;
    }

    public static ItemStack extract(TileEntity inventory, EnumFacing direction, boolean simulate){

        IInventory inv = getInventoryForTE(inventory);
        if(inv != null) return extract(inv, direction, simulate);
        return null;
    }

    public static ItemStack extract(IInventory inventory, EnumFacing direction, boolean simulate){

        if(inventory instanceof ISidedInventory) {
            ISidedInventory isidedinventory = (ISidedInventory)inventory;
            int[] accessibleSlotsFromSide = isidedinventory.func_180463_a(direction);

            for(int anAccessibleSlotsFromSide : accessibleSlotsFromSide) {
                ItemStack stack = extract(inventory, direction, anAccessibleSlotsFromSide, simulate);
                if(stack != null) return stack;
            }
        } else {
            int j = inventory.func_70302_i_();

            for(int k = 0; k < j; ++k) {
                ItemStack stack = extract(inventory, direction, k, simulate);
                if(stack != null) return stack;
            }
        }
        return null;
    }

    public static ItemStack extract(IInventory inventory, EnumFacing direction, int slot, boolean simulate){

        ItemStack itemstack = inventory.func_70301_a(slot);

        if(itemstack != null && canExtractItemFromInventory(inventory, itemstack, slot, direction)) {
            if(!simulate) inventory.func_70299_a(slot, null);
            return itemstack;
        }
        return null;
    }

    public static ItemStack extract(TileEntity tile, EnumFacing direction, ItemStack requestedStack, boolean useItemCount, boolean simulate){

        return extract(tile, direction, requestedStack, useItemCount, simulate, 0);
    }

    public static int[] getAccessibleSlotsForInventory(IInventory inv, EnumFacing side){

        int[] accessibleSlots;
        if(inv != null) {
            if(inv instanceof ISidedInventory) {
                accessibleSlots = ((ISidedInventory)inv).func_180463_a(side);
            } else {
                accessibleSlots = new int[inv.func_70302_i_()];
                for(int i = 0; i < accessibleSlots.length; i++)
                    accessibleSlots[i] = i;
            }
            return accessibleSlots;
        } else {
            return new int[0];
        }
    }

    /**
     * Retrieves an item from the specified inventory. This item can be specified.
     * 
     * @param tile
     * @param direction
     * @param requestedStack
     * @param useItemCount
     *            if true, it'll only retrieve the stack of the exact item count given. it'll look in multiple slots of the inventory. if false, the
     *            first matching stack, ignoring item count, will be returned.
     * @param simulate
     * @param fuzzySetting
     *            ,
     * @return
     */
    public static ItemStack extract(TileEntity tile, EnumFacing direction, ItemStack requestedStack, boolean useItemCount, boolean simulate, int fuzzySetting){

        if(requestedStack == null) return requestedStack;
        IInventory inv = getInventoryForTE(tile);
        if(inv != null) {
            int[] accessibleSlots;
            if(inv instanceof ISidedInventory) {
                accessibleSlots = ((ISidedInventory)inv).func_180463_a(direction);
            } else {
                accessibleSlots = new int[inv.func_70302_i_()];
                for(int i = 0; i < accessibleSlots.length; i++)
                    accessibleSlots[i] = i;
            }
            int itemsFound = 0;
            for(int slot : accessibleSlots) {
                ItemStack stack = inv.func_70301_a(slot);
                if(stack != null && stack.field_77994_a > 0 && stack.func_77969_a(requestedStack) && IOHelper.canExtractItemFromInventory(inv, stack, slot, direction)) {
                    if(!useItemCount) {
                        if(!simulate) {
                            inv.func_70299_a(slot, null);
                        }
                        return stack;
                    }
                    itemsFound += stack.field_77994_a;
                }
            }
            if(itemsFound >= requestedStack.field_77994_a) {
                ItemStack exportedStack = null;
                int itemsNeeded = requestedStack.field_77994_a;
                for(int slot : accessibleSlots) {
                    ItemStack stack = inv.func_70301_a(slot);
                    if(stack != null && stack.func_77969_a(requestedStack) && IOHelper.canExtractItemFromInventory(inv, stack, slot, direction)) {
                        int itemsSubstracted = Math.min(itemsNeeded, stack.field_77994_a);
                        if(itemsSubstracted > 0) exportedStack = stack;
                        itemsNeeded -= itemsSubstracted;
                        if(!simulate) {
                            stack.field_77994_a -= itemsSubstracted;
                            if(stack.field_77994_a == 0) {
                                inv.func_70299_a(slot, null);
                            }
                            tile.func_70296_d();
                        }
                    }
                }
                exportedStack = exportedStack.func_77946_l();
                exportedStack.field_77994_a = requestedStack.field_77994_a;
                return exportedStack;
            }
        }
        return null;

    }

    public static ItemStack extractOneItem(TileEntity tile, EnumFacing dir, boolean simulate){

        IInventory inv = getInventoryForTE(tile);
        if(inv != null) {
            int[] accessibleSlots;
            if(inv instanceof ISidedInventory) {
                accessibleSlots = ((ISidedInventory)inv).func_180463_a(dir);
            } else {
                accessibleSlots = new int[inv.func_70302_i_()];
                for(int i = 0; i < accessibleSlots.length; i++)
                    accessibleSlots[i] = i;
            }
            for(int slot : accessibleSlots) {
                ItemStack stack = inv.func_70301_a(slot);
                if(stack != null && stack.field_77994_a > 0 && IOHelper.canExtractItemFromInventory(inv, stack, slot, dir)) {
                    if(simulate) {
                        ItemStack ret = stack.func_77946_l();
                        ret.field_77994_a = 1;
                        return ret;
                    }
                    ItemStack ret = stack.func_77979_a(1);
                    if(stack.field_77994_a == 0) inv.func_70299_a(slot, null);
                    tile.func_70296_d();
                    return ret;
                }
            }
        }
        return null;
    }

    /**
     * Inserts on all sides
     * @param tile
     * @param itemStack
     * @param simulate
     * @return
     */
    public static ItemStack insert(TileEntity tile, ItemStack itemStack, boolean simulate){
        IInventory inv = getInventoryForTE(tile);
        ItemStack insertingStack = itemStack.func_77946_l();
        for(EnumFacing side : EnumFacing.field_82609_l) {
            insertingStack = insert(inv, insertingStack, side, simulate);
            if(insertingStack == null || insertingStack.field_77994_a != itemStack.field_77994_a) return insertingStack;
        }
        return insertingStack;
    }

    public static ItemStack insert(TileEntity tile, ItemStack itemStack, EnumFacing direction, boolean simulate){

        IInventory inv = getInventoryForTE(tile);
        if(inv != null) return insert(inv, itemStack, direction, simulate);
        return itemStack;
    }

    public static ItemStack insert(IInventory inventory, ItemStack itemStack, EnumFacing side, boolean simulate){

        if(inventory instanceof ISidedInventory && side != null) {
            ISidedInventory isidedinventory = (ISidedInventory)inventory;
            int[] aint = isidedinventory.func_180463_a(side);

            for(int j = 0; j < aint.length && itemStack != null && itemStack.field_77994_a > 0; ++j) {
                itemStack = insert(inventory, itemStack, aint[j], side, simulate);
            }
        } else if(inventory != null) {
            int k = inventory.func_70302_i_();

            for(int l = 0; l < k && itemStack != null && itemStack.field_77994_a > 0; ++l) {
                itemStack = insert(inventory, itemStack, l, side, simulate);
            }
        }

        if(itemStack != null && itemStack.field_77994_a == 0) {
            itemStack = null;
        }

        return itemStack;
    }

    public static ItemStack insert(IInventory inventory, ItemStack itemStack, int slot, EnumFacing side, boolean simulate){

        ItemStack itemstack1 = inventory.func_70301_a(slot);

        if(canInsertItemToInventory(inventory, itemStack, slot, side)) {
            boolean flag = false;

            if(itemstack1 == null) {
                int max = Math.min(itemStack.func_77976_d(), inventory.func_70297_j_());
                if(max >= itemStack.field_77994_a) {
                    if(!simulate) {
                        inventory.func_70299_a(slot, itemStack);
                        flag = true;
                    }
                    itemStack = null;
                } else {
                    if(!simulate) {
                        inventory.func_70299_a(slot, itemStack.func_77979_a(max));
                        flag = true;
                    } else {
                        itemStack.func_77979_a(max);
                    }
                }
            } else if(itemstack1.func_77969_a(itemStack) && ItemStack.func_77970_a(itemstack1, itemStack)) {
                int max = Math.min(itemStack.func_77976_d(), inventory.func_70297_j_());
                if(max > itemstack1.field_77994_a) {
                    int l = Math.min(itemStack.field_77994_a, max - itemstack1.field_77994_a);
                    itemStack.field_77994_a -= l;
                    if(!simulate) {
                        itemstack1.field_77994_a += l;
                        flag = l > 0;
                    }
                }
            }
            if(flag) {
                inventory.func_70296_d();
            }
        }

        return itemStack;
    }

    public static boolean canInsertItemToInventory(IInventory inventory, ItemStack itemStack, int slot, boolean[] sides){
        for(int i = 0; i < sides.length; i++) {
            if(sides[i] && canInsertItemToInventory(inventory, itemStack, slot, EnumFacing.func_82600_a(i))) return true;
        }
        return false;
    }

    public static boolean canExtractItemFromInventory(IInventory inventory, ItemStack itemStack, int slot, boolean[] sides){
        for(int i = 0; i < sides.length; i++) {
            if(sides[i] && canExtractItemFromInventory(inventory, itemStack, slot, EnumFacing.func_82600_a(i))) return true;
        }
        return false;
    }

    public static boolean canInsertItemToInventory(IInventory inventory, ItemStack itemStack, int slot, EnumFacing side){

        return inventory.func_94041_b(slot, itemStack) && (!(inventory instanceof ISidedInventory) || ((ISidedInventory)inventory).func_180462_a(slot, itemStack, side));
    }

    public static boolean canExtractItemFromInventory(IInventory inventory, ItemStack itemStack, int slot, EnumFacing side){

        return !(inventory instanceof ISidedInventory) || ((ISidedInventory)inventory).func_180461_b(slot, itemStack, side);
    }

    public static void dropInventory(World world, BlockPos pos){

        TileEntity tileEntity = world.func_175625_s(pos);

        if(!(tileEntity instanceof IInventory)) {
            return;
        }

        IInventory inventory = (IInventory)tileEntity;

        for(int i = 0; i < inventory.func_70302_i_(); i++) {
            ItemStack itemStack = inventory.func_70301_a(i);

            if(itemStack != null && itemStack.field_77994_a > 0) {
                spawnItemInWorld(world, itemStack, pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p());
            }
        }
    }

    public static void spawnItemInWorld(World world, ItemStack itemStack, double x, double y, double z){

        if(world.field_72995_K) return;
        float dX = world.field_73012_v.nextFloat() * 0.8F + 0.1F;
        float dY = world.field_73012_v.nextFloat() * 0.8F + 0.1F;
        float dZ = world.field_73012_v.nextFloat() * 0.8F + 0.1F;

        EntityItem entityItem = new EntityItem(world, x + dX, y + dY, z + dZ, new ItemStack(itemStack.func_77973_b(), itemStack.field_77994_a, itemStack.func_77952_i()));

        if(itemStack.func_77942_o()) {
            entityItem.func_92059_d().func_77982_d((NBTTagCompound)itemStack.func_77978_p().func_74737_b());
        }

        float factor = 0.05F;
        entityItem.field_70159_w = world.field_73012_v.nextGaussian() * factor;
        entityItem.field_70181_x = world.field_73012_v.nextGaussian() * factor + 0.2F;
        entityItem.field_70179_y = world.field_73012_v.nextGaussian() * factor;
        world.func_72838_d(entityItem);
        itemStack.field_77994_a = 0;
    }

    public static boolean canInterfaceWith(TileEntity tile, EnumFacing direction){
        if(tile instanceof IInventory) {
            return !(tile instanceof ISidedInventory) || ((ISidedInventory)tile).func_180463_a(direction).length > 0;
        }
        return false;
    }
}
